package de.uka.algo.generator.standalone;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;

import de.uka.algo.generator.accessory.StandaloneDCRArguments;
import de.uka.algo.generator.standalone.generators.DCRGenerator;
import de.uka.algo.generator.standalone.graph.DCRGraph;
import de.uka.algo.generator.standalone.graph.io.JournalFileWriter;
import de.uka.algo.generator.standalone.graph.io.VisoneGraphMLWriter;
import de.uka.algo.generator.standalone.graph.journaling.ClusteringJournal;
import de.uka.algo.generator.standalone.graph.journaling.GraphJournal;

public class DCRCommandLine {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		if (args.length == 0) {
			System.out.println("-h for help");
			System.out.println("-g for graph generation");
		} else if (args[0].equals("-h")) {
			System.out.println("See http://i11www.iti.uni-karlsruhe.de/projects/spp1307/dyngen for information on the generator.");
		} else if (args[0].equals("-g")) {
			// parse arguments

			parseAndGenerate(args);

		} else {

		}
	}

	/**
	 * Parse a list "[a,b,...]"
	 * 
	 * @return
	 */
	private static String[] parseList(String ls) {
		String lss = ls.substring(1, ls.length() - 1);
		return lss.split(",");
	}

	public static void parseAndGenerate(String[] args) {
		String[] keys = {"t_max", "n","p_in","p_out", "p_inList", "k", "beta", "D_s", "p_chi", "p_nu", "eta", "p_omega", "p_mu", "sigma", "theta", "enp", "graphml", "binary", "log", "outDir", "fileName"};
		String[] defaultValues = {"100", "60", "0.2", "0.01", "", "2", "1.0", "", "0.5", "0.5", "1", "0.02", "0.5", "2.0", "0.25", "false", "false", "false", "false", "", ""};
		
		HashMap<String,String> argMap = new HashMap<String,String>();
		
		for (int i = 0; i < keys.length; ++i) {
			argMap.put(keys[i],defaultValues[i]);
		} 
		
		
		ArrayList<String> invalidArgs = new ArrayList<String>();
		
		for (int i = 1; i < args.length; ++i) {
			String arg = args[i];
 			String[] kv = arg.split("=");
			if (argMap.containsKey(kv[0])) {
				argMap.put(kv[0], kv[1]);
			} else {
				invalidArgs.add(kv[0]);
			}
		}
		
		if (invalidArgs.size() > 0) {
			System.out.println("the following arguments are not valid: " + invalidArgs);
		}
		
		System.out.println("the following arguments have been supplied: " + argMap);
		
		StandaloneDCRArguments arguments = new StandaloneDCRArguments();

		try {
			arguments.t_max = Integer.parseInt(argMap.get("t_max"));
			arguments.n = Integer.parseInt(argMap.get("n"));
			arguments.p_out = Double.parseDouble(argMap.get("p_out"));
			arguments.k = Integer.parseInt(argMap.get("k"));
			arguments.p_chi = Double.parseDouble(argMap.get("p_chi"));
			arguments.p_nu = Double.parseDouble(argMap.get("p_nu"));
			arguments.eta = Integer.parseInt(argMap.get("eta"));
			arguments.p_omega = Double.parseDouble(argMap.get("p_omega"));
			arguments.p_mu = Double.parseDouble(argMap.get("p_mu"));
			arguments.sigma = Double.parseDouble(argMap.get("sigma"));
			arguments.theta = Double.parseDouble(argMap.get("theta"));	
			arguments.enp = Boolean.parseBoolean(argMap.get("enp"));
		} catch (NumberFormatException e) {
			System.out.println("incorrect arguments supplied: " + e.getCause());
			e.printStackTrace();
			System.exit(1);
		}
		
		
		if (argMap.get("p_inList").equals("")) {
			try {
				arguments.p_in = Double.parseDouble(argMap.get("p_in"));
				arguments.p_inList = null;
			} catch (NumberFormatException ex) {
				ex.printStackTrace();
				System.exit(1);
			}
		} else {
			String[] array = parseList(argMap.get("p_inList"));
			if (array.length == arguments.k) {
				arguments.p_inList = new ArrayList<Double>();
				for (int i = 0; i < array.length; ++i) {
					double p_in = Double.parseDouble(array[i]);
					arguments.p_inList.add(p_in);
				}
			} else {
				System.out.println("invalid number of p_in arguments");
				System.exit(1);
			}
		}
		
		
		if (argMap.get("D_s").equals("")) {
			try {
				arguments.beta = Double.parseDouble(argMap.get("beta"));
				arguments.D_s = null;
			} catch (NumberFormatException ex) {
				ex.printStackTrace();
				System.exit(1);
			}
		} else {
			String[] array = parseList(argMap.get("D_s"));
			if (array.length == arguments.k) {
				arguments.D_s = new ArrayList<Double>();
				for (int i = 0; i < array.length; ++i) {
					double s = Double.parseDouble(array[i]);
					arguments.D_s.add(s);
				}
			} else {
				System.out.println("invalid number of D_s arguments");
				System.exit(1);
			}
		}
		
		// catch extreme p_in / p_out values
		
		double epsilon = 1e-16;
		
		if (arguments.p_out == 0) {
			arguments.p_out = 0 + epsilon;
		} else if (arguments.p_out == 1) {
			arguments.p_out = 1 - epsilon;
		}
		
		if (arguments.p_inList == null) {
			if (arguments.p_in == 1) {
				arguments.p_in = 1 - epsilon;
			} else if (arguments.p_in == 0) {
				arguments.p_in = 1 + epsilon;
			}
		} else {
			for (double p_in : arguments.p_inList) {
				if (p_in == 1) {
					p_in = 1 - epsilon;
				} else if (p_in == 0) {
					p_in = 0 + epsilon;
				}
			}
		}

		boolean enableGraphml = Boolean.parseBoolean(argMap.get("graphml"));
		boolean enableLog = Boolean.parseBoolean(argMap.get("log"));
		boolean enableBinary = Boolean.parseBoolean(argMap.get("binary"));
	
		
		// start generator
		
		if (argMap.get("fileName") == "") {
			// create filename
			final String dateFormat = "yyyy-MM-dd-HH-mm-ss";
		    Calendar cal = Calendar.getInstance();
		    SimpleDateFormat sdf = new SimpleDateFormat(dateFormat);
			String time = sdf.format(cal.getTime());
			String filename = "dcrGraph_" + time;
			argMap.put("fileName", filename);
		}
		
		if (argMap.get("outDir") == "") {
			// use current directory
			argMap.put("outDir", "./output");
		}
		
		System.out.println("generating graph");
		
		
		DCRGenerator dcrGenerator = new DCRGenerator();
		
		
		DCRGraph graph = dcrGenerator.generate(arguments);
		GraphJournal gJournal = graph.getGraphJournal();
		ClusteringJournal cJournal  = graph.getClusteringJournal();
		
		
			
		if (enableGraphml) {
	        try {
	            File file = new File(argMap.get("outDir"), argMap.get("fileName") + ".graphml");
	            
	            // Create file if it does not exist
	            boolean success = file.createNewFile();
	            if (success) {
	            	VisoneGraphMLWriter writer = new VisoneGraphMLWriter();
	                writer.writeGraph(gJournal, new FileOutputStream(file));
	                System.out.println("GraphML file successfully written to " + file.getAbsolutePath());
	            } else {
	                System.out.println("Error: File already exists");
	            }
	        } catch (IOException e) {
	            e.printStackTrace();
	        }
		}
        
        
        
		if (enableLog) {
			StringBuffer logBuffer = graph.getLogBuffer();
			try {
				File logFile = new File(argMap.get("outDir"), argMap.get("fileName") + ".graphlog");
				boolean success = logFile.createNewFile();
				if (success) {
					PrintWriter writer = new PrintWriter(new FileOutputStream(
							logFile));
					writer.println(logBuffer);
					System.out.println("Logfile successfully written");
					writer.close();
				} else {
					System.out.println("Error: Logfile already exists");
				}

			} catch (IOException ex) {
				ex.printStackTrace();
			}
		}
		
		
		if (enableBinary) {
			try {
				File binFile = new File(argMap.get("outDir"), argMap.get("fileName") + ".graphj");
				boolean success = binFile.createNewFile();
				if (success) {
					FileOutputStream fileOutputStream = new FileOutputStream(binFile);
					JournalFileWriter.writeGraph(gJournal, fileOutputStream);
					JournalFileWriter.writeClustering(cJournal, fileOutputStream);
	                System.out.println("binary file successfully written to " + binFile.getAbsolutePath());
				} else {
					System.out.println("Error: File already exists");
				}
			} catch (IOException ex) {
				ex.printStackTrace();
			}
		}
        
	}
}
