package de.uka.algo.generator.standalone.graph;

import java.util.ArrayList;
import java.util.HashMap;

import de.uka.algo.generator.util.DynamicTriangularMatrix;

/**
 * A generic, dynamic upper triangular matrix for
 * associating a pair {x,y} of keys with a value.
 * 
 * @author <christian.staudt @ ira.uka.de>
 *
 * @param <Key>
 * @param <Entry>
 */
public class EdgeCountMatrix {
	
	DynamicTriangularMatrix<Cluster, Integer> matrix;
	
	public EdgeCountMatrix (int dim) {
		this.matrix = new DynamicTriangularMatrix<Cluster, Integer>(dim, 0);
	}
	

	/**
	 * Get the entry for the key pair {x,y}.
	 * @param x
	 * @param y
	 * @return
	 */
	public int get(Cluster c, Cluster d) {
		return this.matrix.get(c, d);
	}
	
	/**
	 * Set the entry for the key pair {x,y}.
	 * @param x
	 * @param y
	 * @param e
	 */
	public void set(Cluster c, Cluster d, int count) {
		this.matrix.set(c, d, count);
	}
	

	
	
	/**
	 * Add a new key, resize if necessary.
	 * @param x
	 */
	public void add(Cluster c) {
		this.matrix.add(c);
	}
	

	
	/**
	 * Remove a key and reset its entries.
	 * @param x
	 */
	public void remove(Cluster c) {
		this.matrix.remove(c);
	}
	
	
	
	/**
	 * Reset all entries to nullEntry.
	 */
	public void resetAll() {
		this.matrix.resetAll();
	}
	
	
	
	public void update(Cluster c, Cluster d, int update) {
		this.set(c, d, this.get(c, d) + update);
		int count = this.get(c, d);
		assert count >= 0;
	}
	
	
	public int getSum() {
		int sum = 0;
		for (int count : this.matrix.getEntries()) {
			sum += count;
		}
		return sum;
	}
	
	
	
	
	
	


}
