package de.uka.algo.generator.standalone.graph;

import java.util.ArrayList;
import java.util.Iterator;

import de.uka.algo.generator.standalone.graph.selecttree.EdgeSelectTree;
import de.uka.algo.generator.standalone.graph.selecttree.EdgeSelectTreeElement;

public class EdgeTreeArray {

	ArrayList<EdgeSelectTree> array;
	DCRGraph graph;
	

	public EdgeTreeArray(DCRGraph graph) {
		this.array = new ArrayList<EdgeSelectTree>();
		this.array.add(null);
		this.graph = graph;
	}

	public void add(EdgeSelectTree est) {
		this.array.add(est);
	}

	/**
	 * Returns the EdgeSelectTree for this node.
	 * 
	 * @param u
	 * @return
	 */
	public EdgeSelectTree getTree(Node u) {
		assert this.array.get(u.id) != null;
		return this.array.get(u.id);
	}

	/**
	 * Deletes the EdgeSelectTree for this node.
	 * 
	 * @param u
	 */
	public void delete(Node u) {
		this.array.set(u.id, null);
	}

	public Iterable<EdgeSelectTree> getEdgeSelectTrees() {

		/**
		 * Wraps the data of this EdgeTreeArray in order to support multiple
		 * iterators.
		 * 
		 */
		class EdgeSelectTreeIterable implements Iterable<EdgeSelectTree> {

			public ArrayList<EdgeSelectTree> array;

			public EdgeSelectTreeIterable(ArrayList<EdgeSelectTree> array) {
				this.array = array;
			}

			public Iterator<EdgeSelectTree> iterator() {

				/**
				 * Iterator for all elements in this EdgeTreeArray
				 * 
				 */
				class EdgeSelectTreeIterator implements
						Iterator<EdgeSelectTree> {

					int index;
					boolean hasNext;
					ArrayList<EdgeSelectTree> array;

					public EdgeSelectTreeIterator(
							ArrayList<EdgeSelectTree> array) {
						this.index = 0;
						this.array = array;
						if (this.array.size() >= 1) {
							this.hasNext = true;
						} else
							this.hasNext = false;
						if (this.array.get(this.index) == null) {
							this.forward();
						}
					}

					public boolean hasNext() {
						return this.hasNext;
					}

					public EdgeSelectTree next() {
						EdgeSelectTree next = this.array.get(this.index);

						this.forward();

						return next;
					}

					@Deprecated
					public void remove() {
						// do nothing
					}

					private void forward() {
						if (this.index == this.array.size() - 1) {
							this.hasNext = false;
						} else {
							do {
								++this.index;
							} while ((this.index < this.array.size() - 1)
									&& (this.array.get(this.index) == null));
							if (this.array.get(this.index) != null) {
								this.hasNext = true;
							} else {
								this.hasNext = false;
							}
						}
					}

				}
				;

				return new EdgeSelectTreeIterator(this.array);
			}

		}
		;

		return new EdgeSelectTreeIterable(this.array);
	}

	/**
	 * Get all incident edges for this node.
	 * 
	 * @param u
	 * @return
	 */
	public Iterable<Edge> getEdges(Node u) {

		class EdgeIterable implements Iterable<Edge> {

			Node sourceNode;
			EdgeTreeArray edgeTreeArray;

			public EdgeIterable(Node sourceNode, EdgeTreeArray edgeTreeArray) {
				this.sourceNode = sourceNode;
				this.edgeTreeArray = edgeTreeArray;
			}

			public Iterator<Edge> iterator() {

				class EdgeIterator implements Iterator<Edge> {

					int index;
					Iterator<EdgeSelectTreeElement> subIterator;
					EdgeTreeArray edgeTreeArray;
					Node sourceNode;

					public EdgeIterator(Node sourceNode,
							EdgeTreeArray edgeTreeArray) {
						EdgeSelectTree est = edgeTreeArray.getTree(sourceNode);
						this.edgeTreeArray = edgeTreeArray;
						this.subIterator = est.iterator();
						this.sourceNode = sourceNode;
					}

					public boolean hasNext() {
						return this.subIterator.hasNext();
					}

					public Edge next() {
						EdgeSelectTreeElement nextElement = this.subIterator
								.next();
						Node targetNode = this.edgeTreeArray
								.getNodeById(nextElement.target);
						return new Edge(this.sourceNode, targetNode,
								nextElement.thisWeight);
					}

					@Deprecated
					public void remove() {
						// do nothing
					}

				}
				;

				return new EdgeIterator(this.sourceNode, this.edgeTreeArray);
			}

		}
		;

		return new EdgeIterable(u, this);
	}

	@Deprecated
	public Iterable<Edge> getAllEdges() {
		//TODO: not yet implemented
		
		class AllEdgesIterable implements Iterable<Edge> {

			public AllEdgesIterable(EdgeTreeArray edgeTreeArray) {
				// TODO Auto-generated constructor stub
			}

			public Iterator<Edge> iterator() {
				// TODO: implement iterator
				
				class AllEdgesIterator implements Iterator<Edge> {
					
					Iterator<EdgeSelectTreeElement> estIterator;
					Iterator<Node> nodeIterator;
					NodeArray nodeArray;
					EdgeTreeArray edgeTreeArray;
					boolean hasNext;

					public boolean hasNext() {
						return this.hasNext;
					}

					public Edge next() {
//						EdgeSelectTreeElement estElement = this.estIterator.next();
//						this.nodeArray.getNodeById()
//						
//						if (this.estIterator.hasNext()) {
//							this.hasNext = true;
//						} else {
//							if (this.nodeIterator.hasNext()) {
//								do {
//									EdgeSelectTree est = this.edgeTreeArray.get(this.nodeIterator.next());
//									this.estIterator = est.iterator();
//								} while (.! this.estIterator.hasNext() && this.nodeIterator.hasNext());
//
//							} else {
//								this.hasNext = false;
//							}
//						}
						return null;
					}

					public void remove() {
						// TODO Auto-generated method stub
						
					}
					
				};
				
				return null;
			}
			
		};
		
		return new AllEdgesIterable(this);
		
	}

	public Node getNodeById(int id) {
		return this.graph.getNodeById(id);
	}

	/**
	 * Only for complement edges: Create complement edges
	 * to a new node.
	 * 
	 */
	public void introduceNode(Node u) {
		for (int i = 1; i < this.array.size(); ++i) {
			EdgeSelectTree vComplementTree = this.array.get(i);
			if (vComplementTree != null) {
				if (i != u.id) {
					Node v = this.graph.getNodeById(i);
					double p = this.graph.getEdgeProbability(u, v);
					vComplementTree.addEdge(u, p);
				}
			}

		}
	}
	
}
