package de.uka.algo.generator.standalone.graph;

import java.io.FileOutputStream;
import java.io.OutputStream;

import de.uka.algo.generator.standalone.graph.clusterevents.ClusterEvent;

public class GraphLog {

	private boolean enabled;
	private int time;
	private StringBuffer buffer;
	
	public GraphLog(boolean enabled) {
		this.enabled = enabled;
		this.time = 0;
		this.buffer = new StringBuffer();
	}
	
	private void append(String s) {
		if (this.enabled) {
			this.buffer.append(s);
		}
	}
	
	public void logCreateNode(int nodeId, int clusterId, int refClusterId) {
		this.append("" + this.time + "\t" + "CN" + "\t" + nodeId + "\t" + clusterId + "\t" + refClusterId + "\n");
	}

	public void logDeleteNode(int nodeId) {
		this.append("" + this.time + "\t" + "DN" + "\t" + nodeId + "\n");
	}
	
	public void logCreateEdge(int sourceId, int targetId) {
		this.append("" + this.time + "\t" + "CE" + "\t" + sourceId + "\t" + targetId + "\n");
	}
	
	public void logDeleteEdge(int sourceId, int targetId) {
		this.append("" + this.time + "\t" + "DE" + "\t" + sourceId + "\t" + targetId + "\n");
	}
	
	public void logMoveNode(int nodeId, int did) {
		this.append("" + this.time + "\t" + "MN" + "\t" + nodeId + "\t" + did + "\n");
	}
	
	public void logMoveRefNode(int nodeId, int did) {
		this.append("" + this.time + "\t" + "MRN" + "\t" + nodeId + "\t" + did + "\n");
	}
	
	public void logSplitCluster(int cid, int did, int eid) {
		this.append("" + this.time + "\t" + "SC" + "\t" + cid + "\t" + did + "\t" + eid + "\n");
	}
	
	public void logSplitRefCluster(int cid) {
		this.append("" + this.time + "\t" + "SRC" + "\t" + cid + "\n");
	}
	
	public void logMergeClusters(int cid, int did, int eid) {
		this.append("" + this.time + "\t" + "MC" + "\t" + cid + "\t" + did + "\t" + eid + "\n");
	}
	
	public void logMergeRefClusters(int cid, int did) {
		this.append("" + this.time + "\t" + "MRC" + "\t" + cid + "\t" + did + "\n");
	}
	
	public void logNextStep() {
		this.time++;
		this.append("\n");
	}
	
	
	public void logMergeCompleteness(ClusterEvent event, boolean complete, int n, int cSize, int dSize, double p_out, double p_in, double a, double b, double count, double theta, double value) {
		if (complete) {
			this.append("" + this.time + "\t" + "testing event " + event.toString() + ": " + " event is complete \n");
		} else {
			this.append("" + this.time + "\t" + "testing event " + event.toString() + ": " + " event is not complete \n");
		}
		this.append("n=" + n + "\t" + "|C|=" + cSize + "\t" + "|D|=" + dSize + "\t" + "p_out=" + p_out + "\t" + "p_in=" + p_in + "\n" + "a=|C|*|D|*p_out=" + a + "\t" + "b=|C|*|D|*p_in=" + b + "\t" + "|E(C,D)|=" + count  + "\t" + "theta=" + theta + "\t" + "theta*a + (1-theta)*b=" + value);
		
		this.append("\n");
	}
	
	public void logSplitCompleteness(ClusterEvent event, boolean complete, int n, int dSize, int eSize, double p_out, double p_in, double a, double b, double count, double theta, double value) {
		if (complete) {
			this.append("" + this.time + "\t" + "testing event " + event.toString() + ": " + " event is complete \n");
		} else {
			this.append("" + this.time + "\t" + "testing event " + event.toString() + ": " + " event is not complete \n");
		}
		this.append("n=" + n + "\t" + "|D|=" + dSize + "\t" + "|E|=" + eSize + "\t" + "p_out=" + p_out + "\t" + "p_in=" + p_in + "\n" + "a=|D|*|E|*p_out=" + a + "\t" + "b=|D|*|E|*p_in=" + b + "\t" + "|E(D,E)|=" + count  + "\t" + "theta=" + theta + "\t" + "theta*b + (1-theta)*a=" + value);
		
		this.append("\n");
	}
	
	
	public StringBuffer getStringBuffer() {
		return this.buffer;
	}
	
}
