package de.uka.algo.generator.standalone.graph;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.Random;

import de.uka.algo.generator.util.Pair;

public class NodeArray implements Iterable<Node> {

	ArrayList<InternalNode> array;
	InternalNode nullNode;
	int size;
	int idCounter;
	Random rand;

	class NodeArrayIterator implements Iterator<Node> {
		
		int index;
		NodeArray nodes;
		boolean hasNext;
		
		public NodeArrayIterator(NodeArray array, int index) {
			this.index = index;
			this.nodes = array;
			if (this.nodes.size() >= 1) {
				this.hasNext = true;
			} else this.hasNext = false;
			if (this.nodes.array.get(this.index).equals(this.nodes.nullNode)) {
				this.forward();
			}
		}

		public boolean hasNext() {
			return this.hasNext;
		}

		public Node next() {
			Node next = new Node(this.index, this.nodes.array.get(this.index));
			
			this.forward();
			
			return next;
		}

		public void remove() {
			//
		}
		
		
		private void forward() {
			if (this.index == this.nodes.array.size() - 1) {
				this.hasNext = false;
			} else {
				do {
					++this.index;
				} while ((this.index < this.nodes.array.size() - 1)
						&& (this.nodes.array.get(this.index).equals(this.nodes.nullNode)));
				if (! this.nodes.array.get(this.index).equals(this.nodes.nullNode)) {
					this.hasNext = true;
				} else {
					this.hasNext = false;
				}
			}
		}
		
	};

	public NodeArray(Pair<Integer> nullData) {
		this.array = new ArrayList<InternalNode>();
		this.nullNode = new InternalNode(nullData);
		this.array.add(this.nullNode);
		this.rand = new Random();
		this.size = 0;
		this.idCounter = 0;
	}

	public Iterator<Node> iterator() {

		return new NodeArrayIterator(this, 1);
	}

	public int size() {
		return this.size;
	}

	public Node createNode(Pair<Integer> data) {
		InternalNode in = new InternalNode(data);
		this.array.add(in);
		this.size++;
		return new Node(++this.idCounter, in);
	}

	public void deleteNode(int nid) {
		this.array.set(nid, this.nullNode);
		this.size--;
	}

	public void updateNodeData(Node u, Pair<Integer> data) {
		this.array.set(u.id, new InternalNode(data));
	}

	public Node getNodeById(int id) {
		if (this.array.get(id) != this.nullNode) {
			return new Node(id, this.array.get(id));
		} else return null;
	}

	/**
	 * Returns a randomly selected node.
	 * 
	 * @return
	 */
	public Node getRandomNode() {
		InternalNode n;
		int i;
		do {
			i = this.rand.nextInt(this.array.size());
			n = this.array.get(i);
		} while (n == this.nullNode);
		return new Node(i, n);
	}

	/**
	 * Returns all node pairs {u,v} with u != v
	 */

	public Collection<Pair<Node>> getNodePairs() {

		ArrayList<Pair<Node>> pairs = new ArrayList<Pair<Node>>();
		int i = 1;
		NodeArrayIterator iterI = new NodeArrayIterator(this, 1);
		while (iterI.hasNext()) {
			Node u = iterI.next();
			if (i < this.size()) {
				NodeArrayIterator iterJ = new NodeArrayIterator(this, i + 1);
				while (iterJ.hasNext()) {
					Node v = iterJ.next();
					pairs.add(new Pair<Node>(u, v));
				}
				++i;
			}
		}
		return pairs;
	}

	public String toString() {
		return this.array.toString();
	}

}
