package de.uka.algo.generator.standalone.graph.io;

import java.io.DataOutputStream;
import java.io.FileOutputStream;
import java.io.IOException;

import de.uka.algo.generator.standalone.graph.journaling.ClusteringJournal;
import de.uka.algo.generator.standalone.graph.journaling.GraphJournal;


/***
 * Writes a GraphJournal to a binary file using DataOutputStream.
 *  
 *  * @author Christian Staudt
 *
 */
public class JournalFileWriter {

	public static void writeGraph(GraphJournal journal, FileOutputStream fStream) {
		DataOutputStream dStream = new DataOutputStream(fStream);
		try {
			int opLength = journal.opCodes.size();
			int argLength = journal.arguments.size();
			
			dStream.writeInt(opLength);
			dStream.writeInt(argLength);
			
			for (Byte b : journal.opCodes) {
				dStream.writeByte(b);
			}
			
			for (Integer i : journal.arguments) {
				dStream.writeInt(i);
			}
		} catch (IOException ex) {
			ex.printStackTrace();
		}

	}
	
	public static void writeClustering(ClusteringJournal journal, FileOutputStream fstream) {
		DataOutputStream dStream = new DataOutputStream(fstream);
		try {
			int opLength = journal.opCodes.size();
			int intArgLength = journal.intArgs.size();
			int doubleArgLength = journal.doubleArgs.size();
			
			dStream.writeInt(opLength);
			dStream.writeInt(intArgLength);
			dStream.writeInt(doubleArgLength);
			
			for (Byte b : journal.opCodes) {
				dStream.writeByte(b);
			}
			
			for (Integer i : journal.intArgs) {
				dStream.writeInt(i);
			}
			
			for (Double x : journal.doubleArgs) {
				dStream.writeDouble(x);
			}
		} catch (IOException ex) {
			ex.printStackTrace();
		}
	}
	
	
}
