package de.uka.algo.generator.standalone.graph.io;

import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.HashMap;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.sun.org.apache.xml.internal.serialize.OutputFormat;
import com.sun.org.apache.xml.internal.serialize.XMLSerializer;


import de.uka.algo.generator.standalone.graph.journaling.GraphJournal;
import de.uka.algo.generator.util.ColorMap;
import de.uka.algo.generator.util.Pair;

public class VisoneGraphMLWriter {

	Document dom;
	ColorMap<Integer> colorMap;

	private void createColorMap() {
		this.colorMap = new ColorMap<Integer>();
	}

	private void createDocument() {
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		DocumentBuilder builder;
		try {
			builder = factory.newDocumentBuilder();
			this.dom = builder.newDocument();
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		}
	}

	private Element createRootElement() {
		Element rootElement = this.dom.createElement("graphml");
		rootElement.setAttribute("xmlns",
				"http://graphml.graphdrawing.org/xmlns");
		rootElement.setAttribute("xmlns:xsi",
				"http://www.w3.org/2001/XMLSchema-instance");
		rootElement
				.setAttribute(
						"xsi:schemaLocation",
						"http://graphml.graphdrawing.org/xmlns http://graphml.graphdrawing.org/xmlns/1.1/graphml.xsd");
		rootElement
				.setAttribute("xmlns:y", "http://www.yworks.com/xml/graphml");
		return rootElement;
	}

	private Element createKeyElement(String name, String type, boolean dynamic,
			String domain, String id) {
		Element keyElement = this.dom.createElement("key");
		if (name != null) {
			keyElement.setAttribute("attr.name", name);
		}
		if (type != null) {
			keyElement.setAttribute("attr.type", type);
		}
		keyElement.setAttribute("dynamic", "" + dynamic);
		if (domain != null) {
			keyElement.setAttribute("for", domain);
		}
		if (id != null) {
			keyElement.setAttribute("id", id);
		}
		return keyElement;
	}

	private Element createGraphElement(String id) {
		Element graphElement = this.dom.createElement("graph");
		graphElement.setAttribute("edgedefault", "undirected");
		graphElement.setAttribute("id", id);
		return graphElement;
	}

	private Element createDataElement(String key, String content) {
		Element dataElement = this.dom.createElement("data");
		dataElement.setAttribute("key", key);
		dataElement.setTextContent(content);
		return dataElement;
	}

	private Element createDynDataElement(String key, String content, int time) {
		Element dataElement = this.dom.createElement("data");
		dataElement.setAttribute("key", key);
		dataElement.setAttribute("time", "" + time);
		dataElement.setTextContent(content);
		return dataElement;
	}

	private Element createNodeElement(int id, int clusterId, int refClusterId,
			int time) {
		Element nodeElement = this.dom.createElement("node");
		nodeElement.setAttribute("id", "n" + id);

		// // set bold border
		// Element nodeGraphicsElement = this.dom.createElement("data");
		// nodeGraphicsElement.setAttribute("key", "d1");
		// Element shapeNodeElement = this.dom.createElement("y:ShapeNode");
		// Element borderStyleElement = this.dom.createElement("y:BorderStyle");
		// borderStyleElement.setAttribute("width", "3.0");
		// shapeNodeElement.appendChild(borderStyleElement);
		// nodeGraphicsElement.appendChild(shapeNodeElement);
		// nodeElement.appendChild(nodeGraphicsElement);

		Element idElement = this.createDataElement("d102", "" + id);
		nodeElement.appendChild(idElement);
		
		Element initExElement = this.createDataElement("d7", "false");
		nodeElement.appendChild(initExElement);
		

		this.appendCluster(nodeElement, time, clusterId);
		this.appendRefCluster(nodeElement, time, refClusterId);

		return nodeElement;
	}

	private Element createEdgeElement(int id, int sourceId, int targetId,
			int time) {
		Element edgeElement = this.dom.createElement("edge");
		edgeElement.setAttribute("id", "e" + id);
		edgeElement.setAttribute("source", "n" + sourceId);
		edgeElement.setAttribute("target", "n" + targetId);

		Element initExElement = this.dom.createElement("data");
		initExElement.setAttribute("key", "d15");
		initExElement.setTextContent("false");
		edgeElement.appendChild(initExElement);

		return edgeElement;
	}

	private void appendNodeExistence(Element nodeElement, int time,
			boolean value) {
		Element exElement = this.dom.createElement("data");
		exElement.setAttribute("key", "d7");
		exElement.setAttribute("time", "" + time);
		exElement.setTextContent("" + value);
		nodeElement.appendChild(exElement);
	}

	private void appendEdgeExistence(Element edgeElement, int time,
			boolean value) {
		Element exElement = this.dom.createElement("data");
		exElement.setAttribute("key", "d15");
		exElement.setAttribute("time", "" + time);
		exElement.setTextContent("" + value);
		edgeElement.appendChild(exElement);
	}

	private void appendCluster(Element nodeElement, int time, int clusterId) {

		Element clusterElement = this.createDynDataElement("d100", ""
				+ clusterId, time);
		nodeElement.appendChild(clusterElement);

		String color = this.colorMap.getHexColor(clusterId);
		Element borderColorElement = this.createDynDataElement("d5", color,
				time);
		nodeElement.appendChild(borderColorElement);
	}

	private void appendRefCluster(Element nodeElement, int time,
			int refClusterId) {

		Element clusterElement = this.createDynDataElement("d101", ""
				+ refClusterId, time);
		nodeElement.appendChild(clusterElement);

		String color = this.colorMap.getHexColor(refClusterId);
		Element colorElement = this.createDynDataElement("d4", color, time);
		nodeElement.appendChild(colorElement);
	}

	public void writeGraph(GraphJournal journal, OutputStream outputStream) {

		this.createColorMap();
		this.createDocument();

		Element root = this.createRootElement();
		this.dom.appendChild(root);

		// // y-Files attribute
		// Element nodeGraphicsKey = this.dom.createElement("key");
		// nodeGraphicsKey.setAttribute("dynamic", "false");
		// nodeGraphicsKey.setAttribute("for", "node");
		// nodeGraphicsKey.setAttribute("id", "d1");
		// nodeGraphicsKey.setAttribute("yfiles.type", "nodegraphics");
		// root.appendChild(nodeGraphicsKey);

		root.appendChild(this.createKeyElement("visone.EXISTENCE", "boolean",
				true, "node", "d7"));
		root.appendChild(this.createKeyElement("visone.EXISTENCE", "boolean",
				true, "edge", "d15"));
		root.appendChild(this.createKeyElement("visone.COLOR", "string", true,
				"node", "d4"));
		root.appendChild(this.createKeyElement("visone.BORDERCOLOR", "string",
				true, "node", "d5"));
		root.appendChild(this.createKeyElement("dcrGenerator.CLUSTER", "int",
				true, "node", "d100"));
		root.appendChild(this.createKeyElement("dcrGenerator.REFERENCECLUSTER",
				"int", true, "node", "d101"));
		root.appendChild(this.createKeyElement("dcrGenerator.ID", "int", false, "node", "d102"));

		Element graphElement = this.createGraphElement("G");
		root.appendChild(graphElement);

		int o = -1;
		int a = -1;
		int nidc = 0; // node id counter
		int eidc = 0; // edge id counter
		int t = 0; // time

//		HashMap<Integer,Element> nodeMap = new HashMap<Integer,Element>();
		ArrayList<Element> nodeList = new ArrayList<Element>();
		nodeList.add(null);
		HashMap<Pair<Integer>, Element> edgeMap = new HashMap<Pair<Integer>, Element>();

		ArrayList<Integer> args = journal.arguments;
		int limit = journal.opCodes.size() - 1;

		while (o < limit) {

			byte opCode = journal.opCodes.get(++o);

			if (opCode == 1) {
				// create node
				int nid = ++nidc;
				Element newNodeElement = this.createNodeElement(nid, args
						.get(++a), args.get(++a), t);
				this.appendNodeExistence(newNodeElement, t, true);
				nodeList.add(newNodeElement);

			} else if (opCode == 2) {
				// delete node
				int nid = args.get(++a);
				Element nodeElement = nodeList.get(nid);
				this.appendNodeExistence(nodeElement, t, false);

			} else if (opCode == 3) {
				// create edge
				int sourceId = args.get(++a);
				int targetId = args.get(++a);
				Pair<Integer> key = new Pair<Integer>(sourceId,targetId);
				Element edgeElement;
				if (edgeMap.containsKey(key)) {
					edgeElement = edgeMap.get(key);
				} else {
					edgeElement = this.createEdgeElement(++eidc, sourceId,targetId, t);
				}

				this.appendEdgeExistence(edgeElement, t, true);
				edgeMap.put(key, edgeElement);

			} else if (opCode == 4) {
				// delete edge
				int sourceId = args.get(++a);
				int targetId = args.get(++a);

				assert edgeMap.containsKey(new Pair<Integer>(sourceId, targetId));
				Element edgeElement = edgeMap.get(new Pair<Integer>(sourceId, targetId));
				this.appendEdgeExistence(edgeElement, t, false);

			} else if (opCode == 5) {
				// set cluster
				int nid = args.get(++a);
				Element nodeElement = nodeList.get(nid);
				int clusterId = args.get(++a);
				this.appendCluster(nodeElement, t, clusterId);

			} else if (opCode == 6) {
				// set reference cluster
				int nid = args.get(++a);
				Element nodeElement = nodeList.get(nid);
				int refClusterId = args.get(++a);
				this.appendRefCluster(nodeElement, t, refClusterId);

			} else if (opCode == 7) {
				// next step
				++t;
			}

		}

		for (Element nodeElement : nodeList) {
			if (nodeElement != null) {
				graphElement.appendChild(nodeElement);
			}
		}

		for (Pair<Integer> pair : edgeMap.keySet()) {
			Element edgeElement = edgeMap.get(pair);
			graphElement.appendChild(edgeElement);
		}

		OutputFormat format = new OutputFormat(this.dom);
		format.setIndenting(true);

		try {
			XMLSerializer serializer = new XMLSerializer(outputStream, format);
			serializer.serialize(this.dom);
		} catch (IOException ex) {
			ex.printStackTrace();
		}

	}

}
