package de.uka.algo.generator.standalone.graph.journaling;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;

public class ClusteringJournal implements Iterable<ClusteringOperation>, Serializable {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = -1068689019254156152L;
	
	public ArrayList<Byte> opCodes;
	public ArrayList<Integer> intArgs;
	public ArrayList<Double> doubleArgs;
	
	public ClusteringJournal() {
		this.opCodes = new ArrayList<Byte>();
		this.intArgs = new ArrayList<Integer>();
		this.doubleArgs = new ArrayList<Double>();
	}
	
	
	public void nextStepOp() {
		this.opCodes.add((byte) 0);
	}
	
	/**
	 * Store a cluster merge event operation.
	 * @param c1
	 * 		merge event: first cluster to be merged
	 * @param c2
	 * 		merge event: second cluster to be merged
	 * @param c3
	 * 		merge event: merge product
	 */
	public void mergeOp(int c1, int c2, int c3) {
		this.opCodes.add((byte) 1);
		this.intArgs.add(c1);
		this.intArgs.add(c2);
		this.intArgs.add(c3);
	}
	
	
	/**
	 * Store a cluster split event operation.
	 * @param c1
	 * 			split event: cluster to be split
	 * @param c2
	 * 			split event: first split product
	 * @param c3
	 * 			split event: second split product
	 */
	public void splitOp(int c1, int c2, int c3) {
		this.opCodes.add((byte) 2);
		this.intArgs.add(c1);
		this.intArgs.add(c2);
		this.intArgs.add(c3);
	}
	
	
	public void mergeDone(int c1, int c2, int c3) {
		this.opCodes.add((byte) 3);
		this.intArgs.add(c1);
		this.intArgs.add(c2);
		this.intArgs.add(c3);
	}
	
	public void splitDone(int c1, int c2, int c3) {
		this.opCodes.add((byte) 4);
		this.intArgs.add(c1);
		this.intArgs.add(c2);
		this.intArgs.add(c3);
	}
	
	
	public Iterator<ClusteringOperation> iterator() {
		
		class ClusteringJournalIterator implements Iterator<ClusteringOperation> {
			
			int opIndex;
			int argIndex;
			ArrayList<Byte> opCodes;
			ArrayList<Integer> intArgs;
			ArrayList<Double> doubleArgs;

			public ClusteringJournalIterator(ClusteringJournal journal) {
				this.opIndex = -1;
				this.argIndex = -1;
				this.opCodes = journal.opCodes;
				this.intArgs = journal.intArgs;
				this.doubleArgs = journal.doubleArgs;
			}
			
			public boolean hasNext() {
				return (opIndex != opCodes.size()-1);
			}

			public ClusteringOperation next() {
				Byte opCode = this.opCodes.get(++opIndex);
				if (opCode == 0) {
					return new ClusteringOperation(opCode);
				} else if (opCode == 1 || opCode == 2 || opCode == 3 || opCode == 4) {
					int arg0 = this.intArgs.get(++argIndex);
					int arg1 = this.intArgs.get(++argIndex);
					int arg2 = this.intArgs.get(++argIndex);
					return new ClusteringOperation(opCode, arg0, arg1, arg2);
				}
				return null;
				
			}

			@Deprecated
			public void remove() {
				// TODO Auto-generated method stub
				
			}
			
		};
		
		return new ClusteringJournalIterator(this);
	}

}
