package de.uka.algo.generator.standalone.graph.journaling;

import java.util.ArrayList;
import java.util.Iterator;

public class GraphJournal implements Iterable<GraphOperation>{

	public ArrayList<Byte> opCodes;
	public ArrayList<Integer> arguments;

	

	public GraphJournal() {
		this.opCodes = new ArrayList<Byte>();
		this.arguments = new ArrayList<Integer>();
	}
	
	// write
	
	public void createNodeOp(int clusterId, int refClusterId) {
		this.opCodes.add((byte) 1);
		this.arguments.add(clusterId);
		this.arguments.add(refClusterId);
	}

	public void removeNodeOp(int nodeId) {
		this.opCodes.add((byte) 2);
		this.arguments.add(nodeId);
	}
	
	public void createEdgeOp(int sourceId, int targetId) {
		this.opCodes.add((byte) 3);
		this.arguments.add(sourceId);
		this.arguments.add(targetId);
	}
	
	public void removeEdgeOp(int sourceId, int targetId) {
		this.opCodes.add((byte) 4);
		this.arguments.add(sourceId);
		this.arguments.add(targetId);
	}
	
	public void setClusterOp(int nodeId, int clusterId) {
		this.opCodes.add((byte) 5);
		this.arguments.add(nodeId);
		this.arguments.add(clusterId);
	}
	
	public void setRefClusterOp(int nodeId, int refClusterId) {
		this.opCodes.add((byte) 6);
		this.arguments.add(nodeId);
		this.arguments.add(refClusterId);
	}
	
	public void nextStepOp() {
		this.opCodes.add((byte) 7);
	}


	

	
	public Iterator<GraphOperation> iterator() {
		
		class JournalIterator implements Iterator<GraphOperation> {
			
			int opIndex;
			int argIndex;
			ArrayList<Byte> opCodes;
			ArrayList<Integer> arguments;

			public JournalIterator(GraphJournal journal) {
				this.opIndex = -1;
				this.argIndex = -1;
				this.opCodes = journal.opCodes;
				this.arguments = journal.arguments;
			}
			
			public boolean hasNext() {
				return (opIndex != opCodes.size()-1);
			}

			public GraphOperation next() {
				Byte opCode = this.opCodes.get(++opIndex);
				if (opCode == 1 || opCode == 3 || opCode == 4 || opCode == 5 || opCode == 6) {
					int arg0 = this.arguments.get(++argIndex);
					int arg1 = this.arguments.get(++argIndex);
					return new GraphOperation(opCode, arg0, arg1);
				} else if (opCode == 2) {
					int arg0 = this.arguments.get(++argIndex);
					return new GraphOperation(opCode, arg0);
				} else if (opCode == 7) {
					return new GraphOperation(opCode);
				} else {
					return null;
				}
				
			}

			@Deprecated
			public void remove() {
				// TODO Auto-generated method stub
				
			}
			
		};
		
		return new JournalIterator(this);
	}
	

	
	

}
