package de.uka.algo.generator.standalone.graph.selecttree;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Random;


import de.uka.algo.generator.standalone.graph.Node;
import de.uka.algo.generator.standalone.graph.NodeArray;

public class EdgeSelectTree implements Iterable<EdgeSelectTreeElement> {

	ArrayList<EdgeSelectTreeElement> array;
	Random rand;
	
	
	public EdgeSelectTree() {
		this.rand = new Random();
		this.array = new ArrayList<EdgeSelectTreeElement>();
		this.array.add(null);
	}

	/**
	 * Returns the total weight of the edges in this tree.
	 * 
	 * @return
	 */
	public double getTotalWeight() {
		if (this.array.size() == 1) {
			return 0;
		} else {
			return this.getWeight(this.array.get(1));
		}
	}

	/**
	 * Recursively explores a tree node;
	 * 
	 * @param index
	 * @param carryover
	 * @return
	 */
	private int explore(int index, double carryover) {
		try {
			EdgeSelectTreeElement e = array.get(index);
			if (carryover <= e.thisWeight) {
				return index;
			} else if (carryover <= e.thisWeight + e.leftWeight) {
				return this.explore(index * 2, carryover - e.thisWeight);
			} else {
				return this.explore(index * 2 + 1, carryover
						- (e.thisWeight + e.leftWeight));
			}
		} catch (IndexOutOfBoundsException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return 0;
		}
	}

	public int select() {
		assert this.array.size() >= 2;
		EdgeSelectTreeElement root = this.array.get(1);
		double x = this.rand.nextDouble() * this.getWeight(root);
		int index = this.explore(1, x);
		return this.array.get(index).target;
	}

	private void updateWeight(int index, double weight) {
		
		EdgeSelectTreeElement e = this.array.get(index);
		e.thisWeight = weight;
		this.propagateChange(e, index);
	}	

	private void removeElement(int index) {
		int j = this.array.size() - 1;
		if (j != index) {
			double tempWeight = this.array.get(j).thisWeight;
			int tempTarget = this.array.get(j).target;
			this.updateWeight(j, 0);
			this.array.remove(j);
			this.array.get(index).target = tempTarget;
			this.updateWeight(index, tempWeight);
		} else {
			this.updateWeight(j, 0);
			this.array.remove(j);
		}
	}

	private double getWeight(EdgeSelectTreeElement e) {
		return e.thisWeight + e.rightWeight + e.leftWeight;
	}

	/**
	 * Recursively propagates weight changes to the root of the tree.
	 * 
	 * @param child
	 * @param indexOfChild
	 */
	private void propagateChange(EdgeSelectTreeElement child, int indexOfChild) {

		if (indexOfChild == 1) {
			// do nothing
		} else {
			int indexOfParent = (int) Math.floor(indexOfChild / 2);
			EdgeSelectTreeElement parent = this.array.get(indexOfParent);
			if (indexOfChild % 2 == 0) {
				parent.leftWeight = this.getWeight(child);
			} else {
				parent.rightWeight = this.getWeight(child);
			}
			this.propagateChange(parent, indexOfParent);
		}

	}

	public String toString() {
		return this.array.toString();
	}

	public void addEdge(Node u, double weight) {
		
		int targetId = u.id;
		EdgeSelectTreeElement newElement = new EdgeSelectTreeElement(weight, targetId);
		int i = this.array.size();
		this.array.add(newElement);
		this.propagateChange(newElement, i);

	}

	public boolean hasEdgeTo(Node v) {
		boolean result = false;
		for (EdgeSelectTreeElement e : this.array) {
			if (e != null) {
				if (e.target == v.id) {
					result = true;
					break;
				}
			}
		}
		return result;
	}

	public void removeEdge(Node u) {
		for (int i = 1; i < this.array.size(); ++i) {
			if (this.array.get(i).target == u.id) {
				this.removeElement(i);
				break;
			}
		}

	}

	public int size() {
		return this.array.size() - 1;
	}

	public Iterator<EdgeSelectTreeElement> iterator() {

		class EdgeSelectTreeIterator implements Iterator<EdgeSelectTreeElement> {


			int index;
			EdgeSelectTree est;
			boolean hasNext;

			public EdgeSelectTreeIterator(EdgeSelectTree est) {
				this.est = est;
				this.index = 1;
				if (this.est.size() >= 1) {
					this.hasNext = true;
				} else {
					this.hasNext = false;
				}
			}

			public boolean hasNext() {
				return this.hasNext;
			}

			public EdgeSelectTreeElement next() {
				EdgeSelectTreeElement e = this.est.array.get(this.index);

				if (this.index < this.est.size()) {
					++this.index;
					this.hasNext = true;
				} else {
					this.hasNext = false;
				}
				return e;

			}

			public void remove() {
				// TODO Auto-generated method stub
			}

		};

		return new EdgeSelectTreeIterator(this);
	}

}
