package de.uka.algo.generator.standalone.graph.selecttree;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Random;

import de.uka.algo.generator.standalone.graph.GraphStructureException;

public class NodeSelectTree {

	public ArrayList<NodeSelectTreeElement> array;
	Random rand;

	public NodeSelectTree() {
		this.rand = new Random();
		this.array = new ArrayList<NodeSelectTreeElement>();
		this.array.add(null);
	}

	/**
	 * Recursively explores a tree node;
	 * 
	 * @param index
	 * @param carryover
	 * @return
	 */
	private int explore(int index, double carryover) {
		NodeSelectTreeElement e = array.get(index);
		if (carryover <= e.thisWeight) {
			return index;
		} else if (carryover <= e.thisWeight + e.leftWeight) {
			return this.explore(index * 2, carryover - e.thisWeight);
		} else {
			return this.explore(index * 2 + 1, carryover
					- (e.thisWeight + e.leftWeight));
		}
	}

	public int select() {
		NodeSelectTreeElement root = this.array.get(1);
		if (this.getWeight(root) == 0) {
			return 0;
		} else {
			double x = this.rand.nextDouble() * this.getWeight(root);
			return this.explore(1, x);
		}
	}

	public void add(double weight) {
		NodeSelectTreeElement newElement = new NodeSelectTreeElement(weight);
		int i = this.array.size();
		this.array.add(newElement);
		this.propagateChange(newElement, i);
	}

	public void updateWeight(int index, double weight) {
		NodeSelectTreeElement e = this.array.get(index);
		e.thisWeight = weight;
		this.propagateChange(e, index);
	}

	public void remove(int index) {
		this.updateWeight(index, 0);
	}

	private double getWeight(NodeSelectTreeElement e) {
		return e.thisWeight + e.rightWeight + e.leftWeight;
	}

	/**
	 * Recursively propagates weight changes to the root of the tree.
	 * 
	 * @param child
	 * @param indexOfChild
	 */
	private void propagateChange(NodeSelectTreeElement child, int indexOfChild) {

		if (indexOfChild == 1) {
			// do nothing
		} else {
			int indexOfParent = (int) Math.floor(indexOfChild / 2);
			NodeSelectTreeElement parent = this.array.get(indexOfParent);
			if (indexOfChild % 2 == 0) {
				parent.leftWeight = this.getWeight(child);
			} else {
				parent.rightWeight = this.getWeight(child);
			}
			this.propagateChange(parent, indexOfParent);
		}

	}
	
	
	public double getTotalWeight() {
		if (this.array.size() == 1) {
			return 0;
		} else {
			return this.getWeight(this.array.get(1));
		}
	}
	
	
	public String toString() {
		return this.array.toString();
	}

}
