package de.uka.algo.generator.util;

import java.util.Collection;
import java.util.HashMap;



public class BidiHashMap<Key,Value> {
	
	HashMap<Key,Value> forward;
	HashMap<Value,Key> reverse;
	
	
	/**
	 * Static factory method.
	 * @param <Key>
	 * @param <Value>
	 * @return
	 */
	public static <Key, Value> BidiHashMap<Key,Value> newInstance() {
		return new BidiHashMap<Key,Value>();
	}
	

	public BidiHashMap() {
		this.forward = new HashMap<Key,Value>();
		this.reverse = new HashMap<Value,Key>();
	}

	public void put(Key k, Value v) {
		this.reverse.remove(this.forward.get(k));
		this.forward.remove(k);
		
		this.forward.put(k, v);
		this.reverse.put(v, k);
	}
	
	public void removeKey(Key k) {
		this.reverse.remove(this.forward.get(k));
		this.forward.remove(k);
	}
	
	
	public Value getValue(Key k) {
		return this.forward.get(k);
	}
	
	public Key getKey(Value v) {
		return this.reverse.get(v);
	}
	
	public Collection<Key> keySet() {
		return this.forward.keySet();
	}
	
	public Collection<Value> valueSet() {
		return this.reverse.keySet();
	}
	
	public boolean containsKey(Key k) {
		return this.forward.containsKey(k);
	}
	
	public boolean containsValue(Value v) {
		return this.reverse.containsKey(v);
	}
	
	
	public String toString() {
		return this.forward.toString() + "\n" + this.reverse.toString();
	}
	

	

}
