package de.uka.algo.generator.util;

import java.util.ArrayList;
import java.util.Collection;

public class Combination<E> extends ArrayList<E> {
    
    /**
     * Unordered collection of elements based on the ArrayList data structure.
     * Facilitates the use of combinatorial objects, such as all possible
     * unordered pairs from a set of elements.
     * Two combinations are equal if and only if they have the same size
     * and contain the same elements. A combination can contain the same element
     * more than once. 
     **/
    
    int capacity;
    boolean complete;
    
    public Combination() {
        // TODO Auto-generated constructor stub
    }

    public Combination(int initialCapacity) {
        super(initialCapacity);
        this.capacity = initialCapacity;
    }

    public Combination(Collection <? extends E> collection) {
        super(collection);
        // TODO Auto-generated constructor stub
    }
    

    
    /* reset capacity
     */
    public void resize(int size) {
        this.ensureCapacity(size);
        this.capacity = size;
    }
    
    
    /*
     * adds an element unless capacity is exceeded
     * @see java.util.ArrayList#add(java.lang.Object)
     */
    @Override
    public boolean add(E object) {
        if (this.size() + 1 <= capacity) {
            super.add(object);
        }
        return true;
    }
    
    /* validates a combination
     */
    public boolean isValid() {
        boolean valid;
        if (this.size() == this.capacity) {
            valid = true;
        } else {
            valid = false;
        }
        return valid;
    }
    
    
 
    /* compares two combinations for equality
     */

    @Override
    public boolean equals(Object other) {
        if (! (other instanceof Combination)) {
            return false;
        } else {
            Combination< ? > other_comb = (Combination< ? >) other;
            boolean equal =  (this.size() == other_comb.size()) && (this.containsAll(other_comb)) && (other_comb.containsAll(this));
            return equal;
        }
        

    }
    
    @Override
    public String toString() {
        String string = "(";
        for (E e : this) {
            string = string.concat(e.toString());
            string = string.concat(",");
        }
        string = string.concat(")");
        return string;
    }
    

    
  
    
}
