package de.uka.algo.generator.util;


import java.util.ArrayList;
import java.util.Collection;
import java.util.Random;

public class Combinatorics {
    
    Random random;
    
    public Combinatorics() {
    	this.random = new Random();
    }
    
    /**
     * Template method returning an element uniformly at random.
     * from a list.
     * @param <E>
     * @param list
     * @return
     * @throws IllegalArgumentException
     */
    public <E> E randomSample(ArrayList<E> list) throws IllegalArgumentException {
        return list.get(random.nextInt(list.size()));
    }
    
    
    /**
     * Template method returning an element at random with a biased 
     * distribution.
     * @param <E>
     * @param list
     * @return
     * @throws IllegalArgumentException
     */
    public <E> E randomSampleWithBias(ArrayList<E> list, double bias) throws IllegalArgumentException {
        double rand = random.nextDouble();
        double val = (Math.pow(rand, bias)) * list.size();
        int i = (int) Math.floor(val);
        return list.get(i);
    }
    
    
    /**
     * Return all possible pairs of elements from a collection.
     * (x,y) is equal to (y,x) and therefore not included.
     * If selfPair
     * 
     * @param <F>
     * @param collection
     * @param selfPair
     * @return
     */
    public <F> ArrayList<Pair<F>> allPairs(Collection <? extends F> collection, boolean selfPair) {

        ArrayList<Pair<F>> allPairs = new ArrayList<Pair<F>>();
        F[] array = (F[]) collection.toArray();
        if (selfPair) {
            for (int i = 0; i < array.length; ++i) {
                for (int j = i; j < array.length; ++j) {
                    allPairs.add(new Pair<F>(array[i], array[j]));
                }
            }
        } else {
            for (int i = 0; i < array.length; ++i) {
                for (int j = i + 1; j < array.length; ++j) {
                    allPairs.add(new Pair<F>(array[i], array[j]));
                }
            }
        }

        return allPairs;
    }
    
    
    /**
     * Return all possible pairs of the given element
     * and the given collection.
     * @param <F>
     * @param element
     * @param collection
     * @return
     */
    public <F> ArrayList<Pair<F>> allPairs(F element, Collection<? extends F> collection, boolean selfPair) {
        ArrayList<Pair<F>> allPairs = new ArrayList<Pair<F>>();
        if (selfPair) {
            for (F e : collection) {
            	allPairs.add(new Pair<F>(element, e));
            }
        } else {
            for (F e : collection) {
            	if (e != element) {
                	allPairs.add(new Pair<F>(element, e));
            	}
            }
        }

        return allPairs;
    }
    

    
    
    
    /**
     * Generic method returning a random pair.
     * @param <F>
     * @param list
     * @return
     */
    public  <F> Pair<F> randomPair(ArrayList<F> list) throws IllegalArgumentException {
    	if (list.size() <= 1) {
    		throw new IllegalArgumentException();
    	} else {
            F x = list.get(random.nextInt(list.size()));
            F y = list.get(random.nextInt(list.size()));
            while (x == y) {
               y = list.get(random.nextInt(list.size()));
            }
            return new Pair<F>(x,y);
    	}

    }
    
    
    /**
     * Return the binomial coefficient "n choose k".
     * @param n
     * @param k
     * @return
     */
    public static long binomial(long n, long k) {
        long result;
        result = (factorial(n) / (factorial(k) * factorial(n - k)));
        return result;
    }
    
    /**
     * Return the factorial n! of n.
     * @param n
     * @return
     */
    public static long factorial(long n) {
        if (n <= 1) {
            return 1;
        } else {
            return n* factorial(n-1);
        }
    }



}
