package de.uka.algo.generator.util;

import java.util.Collection;
import java.util.Random;


/**
 * Estimates a Gaussian distribution for a set of probabilities
 * and generates pseudo-random probabilities according to the distribution.
 * 
 * @author <christian.staudt @ ira.uka.de>
 *
 */
public class GaussianEstimator {
	
	private double mean;
	private double variance;
	private double stddev;
	
	private Random random;
	
	
	public GaussianEstimator(Collection<Double> c) {
		this.mean = this.calculateMean(c);
		this.variance = this.calculateVariance(c, this.mean);
		this.stddev = Math.sqrt(this.variance);
		this.random = new Random();
		
	}
	
	
	/**
	 * 
	 * @param mean
	 * @param variance
	 */
	public GaussianEstimator(double mean, double variance) {
		this.random = new Random();
		this.mean = mean;
		this.variance = variance;
		this.stddev = Math.sqrt(variance);
	}
	
	public double calculateMean(Collection<Double> c) {
		double sum = 0.0;
		for (double x : c) {
			sum += x;
		}
		return sum / c.size();
	}
	
	
	public double calculateVariance(Collection<Double> c, double mean) {
		Double sum = 0.0;
		for (double x : c) {
			sum += Math.pow((x - mean), 2);
		}
		double variance = sum / c.size();
		return variance;
	}
	
	synchronized public double next() {
		double x = -1;
		do {
			 x = this.stddev * this.random.nextGaussian() + this.mean;
		} while ((x < 0) || (x > 1));
		return x;
	}

}
