package de.uka.algo.generator.util;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Map;
import java.util.Random;


public class WeightedSelector<Element> {
	
	ArrayList<Double> bounds;
	ArrayList<Element> elements;
	double maxB;
	Random random;
	
	public WeightedSelector(Collection<Element> collection, Map<Element,Double> map) {
		assert collection.size() > 0;
		this.bounds = new ArrayList<Double>();
		this.elements = new ArrayList<Element>();
		double b = 0.0;
		for (Element e : collection) {
			this.elements.add(e);
			b = b + map.get(e);
			this.bounds.add(b);
		}
		this.maxB = b;
		this.random = new Random();
	}
	
	public Element select() {
		double x = this.random.nextDouble() * this.maxB;
		int index = this.binaryRangeSearch(x, this.bounds, 0, this.bounds.size());
		return this.elements.get(index);
	}
	
	
	/**
	 * Return index i for x so that x is in the interval (array[i-1],array[i]]
	 * using binary search.
	 * 
	 * @param x
	 * @param array
	 * @param low
	 * @param high
	 * @return
	 */
	public int binaryRangeSearch(double x, ArrayList<Double> array, int low, int high) {
		if (low > high) {
			return -1; // not found
		}

		int mid = (int) (low + high) / 2;
		
		if (array.get(mid) >= x) {
			if ((mid == 0) || (array.get(mid-1) < x)) { // x is in interval (array[mid-1],array[mid]]
				return mid;
			} else {
				return binaryRangeSearch(x, array, low, mid-1);
			}
		} else {
			if ((mid == array.size()-1) || (array.get(mid+1) >= x)) { // x is in interval (array[mid],array[mid+1]]
				return mid+1;
			} else { 
				return binaryRangeSearch(x, array, mid+1, high);
			}
		}


	}
}
