/* 
 * Integration: May 2, 2006 (katz)
 * Reviewed: May 2, 2006 (katz)
 *
 * Copyright 2004-2006.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
 */
package de.uka.algo.clustering;

import de.uka.algo.util.datastructures.parasitic.SlotData;


/**
 * A ClusterMap is a map from a clustering's clusters to a given type. Since a
 * ClusterMap store their data in Clusters, they provide a O(1) access.
 * 
 * @param <T> the value type
 * 
 * @see de.uka.algo.clustering.Cluster
 * @see de.uka.algo.clustering.Clustering
 *	 
 * @author Bastian Katz, katz@kit.edu
 */
public final class ClusterMap<T> extends SlotData<T> {

	/* ===== constructor ===== */

	/**
	 * Instantiates a new ClusterMap for the given Clustering. This can take up
	 * to O(k*l) for the number of clusters k and the number of ClusterMaps l.
	 * In a proper implementation, the amortized complexity should be in O(k).
	 * 
	 * @param clustering
	 */
	public ClusterMap(Clustering clustering) {
		super(clustering);
	}

	/* ===== map access ===== */
	
	/**
	 * Returns the object a cluster maps to, null if none.
	 * @param aCluster the cluster (key)
	 * @return the object a cluster maps to, null if none.
	 */
	public T get(Cluster aCluster) {
		return super.get(aCluster);
	}

	/**
	 * Sets the object a cluster maps to. A null value is allowed.
	 * @param aCluster the cluster (key)
	 */
	
	public void set(Cluster aCluster, T aValue) {
		super.set(aCluster, aValue);
	}
}
