/* 
 * Integration: May 4, 2006 (katz)
 * Reviewed: June 8, 2006 (katz)
 *
 * Copyright 2004-2006. --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
 */
package de.uka.algo.clustering;

import y.base.Edge;
import y.base.Node;
import y.base.NodeCursor;

/**
 * Abstract superclass for all listeners with a default event handler for all
 * events (and optionally some overridden event handlers for certain events).
 * 
 * @author Bastian Katz, mailto:katz@ira.uka.de
 */
public abstract class DefaultListener extends ClusteringListener {

	/**
	 * Constructor.
	 * 
	 * @param clustering
	 */
	protected DefaultListener(Clustering clustering) {
		super(clustering);
	}

	/**
	 * The event handler for any thrown event.
	 */
	protected abstract void defaultEvent();

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.uka.algo.clustering.ClusteringListener#newClusterEvent(boolean,
	 *      de.uka.algo.clustering.Cluster)
	 */
	protected void newClusterEvent(boolean notificationTime, Cluster c) {
		if (notificationTime == POST)
			defaultEvent();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.uka.algo.clustering.ClusteringListener#deleteEvent(boolean,
	 *      de.uka.algo.clustering.Cluster)
	 */
	protected void deleteEvent(boolean notificationTime, Cluster c) {
		if (notificationTime == POST)
			defaultEvent();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.uka.algo.clustering.ClusteringListener#addEvent(boolean,
	 *      de.uka.algo.clustering.Cluster, y.base.NodeCursor)
	 */
	protected void addEvent(boolean notificationTime, Cluster c, NodeCursor nc) {
		if (notificationTime == POST)
			defaultEvent();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.uka.algo.clustering.ClusteringListener#extractEvent(boolean,
	 *      de.uka.algo.clustering.Cluster, y.base.NodeCursor)
	 */
	protected void extractEvent(boolean notificationTime, Cluster c,
			NodeCursor nc) {
		if (notificationTime == POST)
			defaultEvent();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.uka.algo.clustering.ClusteringListener#resetEvent(boolean)
	 */
	protected void resetEvent(boolean notificationTime) {
		if (notificationTime == POST)
			defaultEvent();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.uka.algo.clustering.ClusteringListener#resetSingletonsEvent(boolean)
	 */
	protected void resetSingletonsEvent(boolean notificationTime) {
		if (notificationTime == POST)
			defaultEvent();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.uka.algo.clustering.ClusteringListener#resetLargeClusterEvent(boolean,
	 *      de.uka.algo.clustering.Cluster)
	 */
	protected void resetLargeClusterEvent(boolean notificationTime, Cluster c) {
		if (notificationTime == POST)
			defaultEvent();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.uka.algo.clustering.ClusteringListener#splitEvent(boolean,
	 *      de.uka.algo.clustering.Cluster, de.uka.algo.clustering.Cluster)
	 */
	protected void splitEvent(boolean notificationTime, Cluster a, Cluster b) {
		if (notificationTime == POST)
			defaultEvent();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.uka.algo.clustering.ClusteringListener#mergeEvent(boolean,
	 *      de.uka.algo.clustering.Cluster, de.uka.algo.clustering.Cluster)
	 */
	protected void mergeEvent(boolean notificationTime, Cluster a, Cluster b) {
		if (notificationTime == POST)
			defaultEvent();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.uka.algo.clustering.ClusteringListener#moveEvent(boolean,
	 *      de.uka.algo.clustering.Cluster, de.uka.algo.clustering.Cluster,
	 *      y.base.NodeCursor)
	 */
	protected void moveEvent(boolean notificationTime, Cluster a, Cluster b,
			NodeCursor nc) {
		if (notificationTime == POST)
			defaultEvent();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.uka.algo.clustering.ClusteringListener#edgeCreationEvent(y.base.Edge)
	 */
	protected void edgeCreationEvent(Edge newEdge) {
		defaultEvent();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.uka.algo.clustering.ClusteringListener#nodeCreationEvent(y.base.Node)
	 */
	protected void nodeCreationEvent(Node newNode) {
		defaultEvent();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.uka.algo.clustering.ClusteringListener#nodeRemovalEvent(boolean,
	 *      y.base.Node)
	 */
	protected void nodeRemovalEvent(boolean notificationTime, Node nodeToRemove) {
		if (notificationTime == POST)
			defaultEvent();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.uka.algo.clustering.ClusteringListener#edgeRemovalEvent(boolean,
	 *      y.base.Edge)
	 */
	protected void edgeRemovalEvent(boolean notificationTime, Edge edgeToRemove) {
		if (notificationTime == POST)
			defaultEvent();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.uka.algo.clustering.ClusteringListener#weightChangedEvent(y.base.Edge,
	 *      double, double)
	 */
	protected void weightChangedEvent(Edge e, double oldWeight, double newWeight) {
		defaultEvent();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.uka.algo.clustering.ClusteringListener#newWeightsEvent()
	 */
	protected void newWeightsEvent() {
		defaultEvent();
	}

}
