/* 
 * Integration: May 10, 2006
 *
 * Copyright 2004-2006.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
 */
package de.uka.algo.clustering.addon;

import y.base.Edge;
import y.base.EdgeCursor;
import y.base.Graph;
import y.base.NodeCursor;
import de.uka.algo.clustering.Cluster;
import de.uka.algo.clustering.Clustering;
import de.uka.algo.clustering.DefaultListener;
import de.uka.algo.util.GYCursor;

/**
 * This class represents the cluster editing set of a clustering. A cluster
 * editing set contains all edges that need to be removed or added to transfrom
 * the given graph into a clustergraph.
 * 
 * @author Daniel Delling
 * 
 */
public final class ClusterEditingSet extends DefaultListener {

	boolean isValid = false;

	int editingSetSize = 0;

	boolean[][] editingSet = null;

	Clustering cl = null;

	private boolean isDirected = false;

	/**
	 * Instantiates a new clustereditingset addon
	 * 
	 * @param clustering
	 *            the clustering to listen to
	 */
	protected ClusterEditingSet(Clustering clustering) {
		super(clustering);
		clustering.addListener(this);
		this.cl = clustering;
	}

	public static ClusterEditingSet getInstance(Clustering cl) {
		ClusterEditingSet ces = null;
		ces = (ClusterEditingSet) cl.getClusteringListener(staticID());
		if (ces == null) {
			return new ClusterEditingSet(cl);
		}
		return ces;
	}

	private void validate() {
		if (isValid) {
			return;
		}
		Graph g = cl.getGraph();
		editingSet = new boolean[g.N()][g.N()];
		isDirected = cl.getInterpretation().isDirected();
		editingSetSize = 0;

		// mark all possible pairs in the clusters
		for (GYCursor<Cluster> cc = cl.clusters(); cc.ok(); cc.next()) {
			Cluster c = cc.active();
			for (NodeCursor nc1 = c.members(); nc1.ok(); nc1.next()) {
				for (NodeCursor nc2 = c.members(); nc2.ok(); nc2.next()) {
					if (nc1.node() != nc2.node()) {
						editingSet[nc1.node().index()][nc2.node().index()] = true;
						editingSetSize++;
					}
				}
			}
		}

		// mark inter- and unmark intraclusteredges
		for (EdgeCursor ec = g.edges(); ec.ok(); ec.next()) {
			Edge e = ec.edge();
			if (cl.isInterClusterEdge(e)) { // mark interclusteredges
				editingSet[e.source().index()][e.target().index()] = true;
				editingSetSize++;
				if (!isDirected) {
					editingSet[e.target().index()][e.source().index()] = true;
					editingSetSize++;
				}
			} else { // unmark intraclusteredges
				editingSet[e.source().index()][e.target().index()] = false;
				editingSetSize--;
				if (!isDirected) {
					editingSet[e.target().index()][e.source().index()] = false;
					editingSetSize--;
				}
			}
		}

		isValid = true;
	}

	/**
	 * returns the cluster editing set represented by an unweighted boolean
	 * array
	 * 
	 * @return the editing set
	 */
	public boolean[][] getEditingSet() {
		validate();
		return editingSet;
	}

	/**
	 * returns the number of edges in the editing set
	 * 
	 * @return
	 */
	public int getEditingSetSize() {
		validate();
		if (isDirected) {
			return editingSetSize;
		} else {
			return editingSetSize / 2;
		}
	}

	@Override
	protected String id() {
		return staticID();
	}

	protected static String staticID() {
		return "ClusterEditingSet";
	}

	@Override
	protected void defaultEvent() {
		isValid = false;
	}

	@Override
	protected void policyChanged(boolean policy) {
		// TODO Auto-generated method stub
	}
}