/*
 * Created on Sep 1, 2004
 * Reviewed 1.2 (Jan 9, 2005)
 * 
 * Copyright 2004.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
 */
package de.uka.algo.clustering.algorithms;

import de.uka.algo.clustering.Clustering;

/**
 * A ClustererStage is a clusterer wrapping another clusterer, i.e. a clusterer
 * that makes some preprocessing, calls an optional core clusterer and does some
 * postprocessing afterwards.
 * 
 * @author Bastian Katz
 */
public abstract class Wrapper implements Algorithm {

    /** the core clusterer */
    private Algorithm core = null;

    /**
     * Instantiates a ClustererStage with the given core algorithm.
     * 
     * @param core
     *            the core algorithm (null for none)
     */
    public Wrapper(Algorithm core) {
        this.core = core;
    }

    /**
     * Passes a clustering to the core algorithm. Call this method when
     * implementing the <code> doCluster</code> method.
     * 
     * @param aClustering the clustering to pass to the core algorithm
     */
    protected void pass(Clustering aClustering) {
        if (core != null)
            core.run(aClustering);
    }
}