package de.uka.algo.clustering.algorithms.greedy;

import java.util.Stack;

import y.base.DataAcceptor;
import y.base.DataProvider;
import y.util.pq.DoubleObjectPQ;
import de.uka.algo.clustering.Cluster;
import de.uka.algo.clustering.ClusterMap;
import de.uka.algo.clustering.Clustering;
import de.uka.algo.clustering.algorithms.Algorithm;
import de.uka.algo.util.GYCursor;

public class Greedy implements Algorithm, DataAcceptor, DataProvider {
	Pair[][] pairs;

	private boolean fast;
	private PredictorFactory predFac;

	public Greedy(PredictorFactory predFac, boolean fast) {
		this.predFac = predFac;
		this.fast = fast;
	}

	public void runSlow(Clustering aClustering) {
		Predictor predictor = predFac.predictor(aClustering);
		while (true) {
			double step = 0.0;
			int index1 = 0;
			int index2 = 0;
			Cluster[] ar = aClustering.getClusterArray();
			for (int i = 0; i < ar.length; i++) {
				for (int k = i + 1; k < ar.length; k++) {
					double v = getChangeOnMerge(i, k, predictor, ar);
					if (v < step) {
						step = v;
						index1 = i;
						index2 = k;
					}
				}
			}
			if (step < 0.0) {
				aClustering.merge(ar[index1], ar[index2]);
			} else {
				break;
			}
		}
	}

	public void runFast(Clustering aClustering) {
		ClusterMap<Integer> numbers = new ClusterMap<Integer>(aClustering);
		int n = aClustering.size();
		Cluster[] clusters = new Cluster[n];
		Predictor predictor = predFac.predictor(aClustering);
		pairs = new Pair[aClustering.size()][aClustering.size()];
		DoubleObjectPQ pq = new DoubleObjectPQ((n * (n - 1)) / 2, this, this);
		for (GYCursor<Cluster> cc = aClustering.clusters(); cc.ok(); cc.next()) {
			Cluster c = cc.active();
			clusters[c.getIndex()] = c;
			numbers.set(c, c.getIndex());
		}

		for (int j = 0; j < n; j++) {
			for (int k = j + 1; k < n; k++) {
				Pair p = new Pair(j, k);
				pairs[j][k] = p;
				pq.add(p, getChangeOnMerge(p.a, p.b, predictor, clusters,
						numbers));
			}
		}
		while (!pq.isEmpty() && pq.getMinPriority() < 0) {
			Pair p = (Pair) pq.getMin();
			int kill = p.a;
			int merged = p.b;
			Cluster mergedCluster = aClustering.merge(clusters[p.a],
					clusters[p.b]);
			if (merged != numbers.get(mergedCluster)) {
				int buffer = merged;
				merged = kill;
				kill = buffer;
			}
			for (GYCursor<Cluster> curse = aClustering.clusters(); curse.ok(); curse
					.next()) {
				Cluster c = curse.active();
				int cIndex = numbers.get(c);
				if (cIndex < kill) {
					pq.remove(pairs[cIndex][kill]);
				} else {
					pq.remove(pairs[kill][cIndex]);
				}
			}
			Stack<Integer> s = new Stack<Integer>();
			for (GYCursor<Cluster> curse = aClustering.clusters(); curse.ok(); curse
					.next()) {
				s.add(numbers.get(curse.active()));
			}
			while (!s.isEmpty()) {
				int cIndex = s.pop();
				if (cIndex < merged) {
					Pair pp = pairs[cIndex][merged];
					pq.remove(pp);
					pq.add(pp, getChangeOnMerge(pp.a, pp.b, predictor,
							clusters, numbers));
				} else if (merged < cIndex) {
					Pair pp = pairs[merged][cIndex];
					pq.remove(pp);
					pq.add(pp, getChangeOnMerge(pp.a, pp.b, predictor,
							clusters, numbers));
				}
			}
		}
	}

	private double getChangeOnMerge(int a, int b, Predictor predictor,
			Cluster[] clusters, ClusterMap<Integer> numbers) {
		Predictor.MergeResult mr = predictor.getChangeOnMerge(clusters[a],
				clusters[b]);
		clusters[a] = mr.a;
		clusters[b] = mr.b;
		numbers.set(mr.a, a);
		numbers.set(mr.b, b);
		return -mr.value;
	}

	private double getChangeOnMerge(int a, int b, Predictor predictor,
			Cluster[] clusters) {
		Predictor.MergeResult mr = predictor.getChangeOnMerge(clusters[a],
				clusters[b]);
		clusters[a] = mr.a;
		clusters[b] = mr.b;
		return -mr.value;
	}

	private class Pair {
		int a;
		int b;
		Object data;
		Pair(int a, int b) {
			this.a = a;
			this.b = b;
		}
	}

	public void set(Object arg0, Object arg1) {
		((Pair) arg0).data = arg1;
	}

	public void setInt(Object arg0, int arg1) {
		((Pair) arg0).data = new Integer(arg1);
	}

	public void setDouble(Object arg0, double arg1) {
		((Pair) arg0).data = new Double(arg1);

	}

	public void setBool(Object arg0, boolean arg1) {
		((Pair) arg0).data = new Boolean(arg1);

	}

	public Object get(Object arg0) {
		return ((Pair) arg0).data;
	}

	public int getInt(Object arg0) {
		return (Integer) ((Pair) arg0).data;
	}

	public double getDouble(Object arg0) {
		return (Double) ((Pair) arg0).data;
	}

	public boolean getBool(Object arg0) {
		return (Boolean) ((Pair) arg0).data;
	}

	public void run(Clustering aClustering) {
		if (fast) runFast(aClustering);
		else runSlow(aClustering);

	}
}
