package de.uka.algo.clustering.comparison;

import de.uka.algo.clustering.Clustering;
import de.uka.algo.clustering.DefaultListener;
import de.uka.algo.clustering.addon.ClusterEditingSet;

/**
 * This class implements the counting pair sets according to two clusterings.
 * Each pair of nodes is classified whether they are in the same (or different)
 * cluster with respect to both clusterings
 * 
 * @author Daniel Delling
 * 
 */
public class ClusterEditingSetOperators {

	boolean valid = false;

	ClusterEditingSetOperatorListener cl1Listener;
	ClusterEditingSetOperatorListener cl2Listener;

	Clustering cl1;

	Clustering cl2;

	private int intersectionClusterEditingSize = 0;

	private int unionClusterEditingSize = 0;

	private ClusterEditingSetOperators(Clustering a, Clustering b) {
		cl1Listener = new ClusterEditingSetOperatorListener(a, b, this);
		cl2Listener = new ClusterEditingSetOperatorListener(b, a, this);
		cl1 = a;
		cl2 = b;
		
	}

	public static ClusterEditingSetOperators getClusterEditingSetOperator(Clustering a, Clustering b) {
		ClusterEditingSetOperatorListener l1 = (ClusterEditingSetOperatorListener) a
				.getClusteringListener(ClusterEditingSetOperatorListener.staticID(b));
		if (l1 != null) {
			return l1.parent;
		}
		return new ClusterEditingSetOperators(a, b);
	}

	private void validate() {
		if(valid){
			return;
		}
		boolean[][] ces1 = ClusterEditingSet.getInstance(cl1).getEditingSet(); 
		boolean[][] ces2 = ClusterEditingSet.getInstance(cl2).getEditingSet();
		
		for(int i = 0; i<cl1.getGraph().nodeCount(); i++){
			for(int j = 0; j<cl1.getGraph().nodeCount(); j++){
				if(ces1[i][j] == true){
					if(ces2[i][j] == true){
						intersectionClusterEditingSize++;
					}
					unionClusterEditingSize++;
				} else {
					if(ces2[i][j] == true){
						unionClusterEditingSize++;					
					}
				}
			}			
		}
		intersectionClusterEditingSize /= 2;
		unionClusterEditingSize /= 2;
		valid = true;
		
	}

	public int getIntersectionClusterEditingSize() {
		validate();
		return intersectionClusterEditingSize;
	}

	public int getUnionClusterEditingSize() {
		validate();
		return unionClusterEditingSize;
	}

	
	public static class ClusterEditingSetOperatorListener extends DefaultListener {

		ClusterEditingSetOperators parent;
		private Clustering opposite;

		protected ClusterEditingSetOperatorListener(Clustering clustering, Clustering opposite, ClusterEditingSetOperators parent) {
			super(clustering);
			this.parent = parent; 
			this.opposite = opposite;
			clustering.addListener(this);
		}

		@Override
		protected void defaultEvent() {
			parent.valid = false;
		}

		@Override
		protected void policyChanged(boolean policy) {
			// do nothing
		}

		@Override
		protected String id() {
			return staticID(opposite);
		}

		protected static String staticID(Clustering opposite) {
			return "ClusterEditingSetOperatorListener:"+opposite;
		}

	}




}
