/* 
 * Integration: May 4, 2006 (katz)
 *
 * Copyright 2004-2006.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
 */
package de.uka.algo.clustering.index;

import y.base.NodeCursor;
import de.uka.algo.clustering.Cluster;
import de.uka.algo.clustering.ClusterMap;
import de.uka.algo.clustering.Clustering;
import de.uka.algo.clustering.ClusteringListener;
import de.uka.algo.clustering.DefaultListener;
import de.uka.algo.util.GYCursor;
import de.uka.algo.util.GYList;

public abstract class ClusterBufferedIndex<T> extends DefaultListener implements
		Index {

	ClusterMap<T> info;

	private boolean valid = false;

	private double storedValue = 0.0;

	private double storedUnweightedValue = 0.0;

	protected ClusterBufferedIndex(Clustering clustering) {
		super(clustering);
		info = new ClusterMap<T>(clustering);
	}

	protected abstract T getData(Cluster c);

	protected abstract double value(GYCursor<T> infos);

	protected abstract double unweightedValue(GYCursor<T> infos);

	private void validate() {
		if (valid)
			return;
		GYList<T> list = new GYList<T>();
		for (GYCursor<Cluster> cc = clustering.clusters(); cc.ok(); cc.next()) {
			Cluster c = cc.active();
			T ci = info.get(c);
			if (ci == null) {
				ci = getData(c);
				info.set(c, ci);
			}
			list.add(ci);
		}
		storedValue = value(list.elements());
		storedUnweightedValue = unweightedValue(list.elements());
		valid = true;
	}

	private void invalidate(Cluster c) {
		info.set(c, null);
		valid = false;
	}

	public final double getValue() {
		validate();
		return storedValue;
	}

	public final double getUnweightedValue() {
		validate();
		return storedUnweightedValue;
	}

	@Override
	protected void defaultEvent() {
		valid = false;
		for (GYCursor<Cluster> cc = getClustering().clusters(); cc.ok(); cc
				.next()) {
			invalidate(cc.active());
		}
	}

	@Override
	public void addEvent(boolean notificationTime, Cluster c, NodeCursor nc) {
		if (notificationTime == ClusteringListener.POST) {
			invalidate(c);
			valid = false;
		}
	}

	@Override
	public void extractEvent(boolean notificationTime, Cluster c, NodeCursor nc) {
		if (notificationTime == ClusteringListener.POST) {
			invalidate(c);
			valid = false;
		}
	}

	@Override
	public void mergeEvent(boolean notificationTime, Cluster a, Cluster b) {
		if (notificationTime == ClusteringListener.POST) {
			invalidate(a);
			valid = false;
		}
	}

	@Override
	public void moveEvent(boolean notificationTime, Cluster a, Cluster b,
			NodeCursor nc) {
		if (notificationTime == ClusteringListener.POST) {
			invalidate(a);
			invalidate(b);
			valid = false;
		}
	}

	@Override
	public void disposeEvent() {
		info.dispose();
		super.disposeEvent();
	}

	@Override
	protected void policyChanged(boolean policy) {
		// do nothing
	}

	public String toString() {
		return id();
	}

}