package de.uka.algo.clustering.index;

import de.uka.algo.clustering.Clustering;
import de.uka.algo.clustering.ClusteringReference;

public class OperationIndex extends ClusteringReference implements Index {
	public static enum Operations {
		div, sub
	}

	private Index a;
	private Index b;
	private Operations operation;

	protected OperationIndex(Clustering cl, IndexFactory a, IndexFactory b,
			Operations operation) {
		super(cl);
		this.a = a.getIndex(cl);
		this.b = b.getIndex(cl);
		this.operation = operation;
	}

	public double getValue() {
		switch (operation) {
			case sub :
				return a.getValue() - b.getValue();
			case div :
				return a.getValue() / b.getValue();
		}
		return 0;
	}

	public double getUnweightedValue() {
		switch (operation) {
			case sub :
				return a.getUnweightedValue() - b.getUnweightedValue();
			case div :
				return a.getUnweightedValue() / b.getUnweightedValue();
		}
		return 0;
	}

	public static class Factory implements IndexFactory {

		private IndexFactory a;
		private IndexFactory b;
		private Operations operation;

		public Factory(IndexFactory a, IndexFactory b, Operations operation) {
			this.a = a;
			this.b = b;
			this.operation = operation;
		}

		public Index getIndex(Clustering cl) {
			return new OperationIndex(cl, a, b, operation);
		}
	}
}
