/*
 * Created on 14.10.2004
 *
 * Copyright 2004.  --- All rights resevered. 
 * Algorithmic Group, Karlsruhe, Germany.
 */
package de.uka.algo.clustering.index;

import de.uka.algo.clustering.Clustering;
import de.uka.algo.clustering.ClusteringReference;

/**
 * A UnweightedIndex is an Index returning the unweighted value of the
 * underlying index for both, unweighted and weighted case.
 * 
 * @author Bastian Katz
 */
public class UnweightedIndex extends ClusteringReference implements Index {

    private Index index = null;

    private UnweightedIndex(Clustering clustering, IndexFactory factory) {
        super(clustering);
        index = factory.getIndex(clustering);
    }

    public double getValue() {
        return index.getUnweightedValue();
    }

    public double getUnweightedValue() {
        return index.getUnweightedValue();
    }

	public String toString() {
		return "Unweighted("+index+")";
	}

    
    public static class Factory implements IndexFactory {

        private IndexFactory factory;

        public Factory(IndexFactory factory) {
            this.factory = factory;
        }

        @SuppressWarnings("synthetic-access")
		public Index getIndex(Clustering cl) {
            return new UnweightedIndex(cl, factory);
        }

        public UnweightedIndex getInstance(Clustering cl) {
            return (UnweightedIndex) getIndex(cl);
        }

        public String toString() {        
            return "Unweighted("+factory+")";
        }

    }

    
    
}