/* 
 * Integration: June 7, 2006 (katz)
 * Reviewed: /TODO code review (katz)
 *
 * Copyright 2004-2006. --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
 */

package de.uka.algo.clustering.index.modules;

import java.awt.BorderLayout;
import java.awt.Color;
import java.util.LinkedList;
import java.util.List;
import java.util.Vector;

import javax.swing.BoxLayout;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;

import y.option.OptionHandler;
import de.uka.algo.clustering.index.IndexRegistration;
import de.uka.algo.clustering.visualization.FacadeModule;
import de.uka.algo.util.ColorProvider;
import de.uka.algo.util.HTMLString;

/**
 * /TODO documentation (katz)
 * 
 * @author Bastian Katz, :katz@kit.edu
 */
public class IndexPopupModule extends FacadeModule {

	private static final int CLUSTERINGS = 6;

	private JFrame popupWindow;

	public IndexPopupModule() {
		super("Evaluate Indizes", "Bastian Katz", "", CLUSTERINGS, WEIGHTS_USE,
				GROUPS_KEEP, 1);
	
	}

	@SuppressWarnings("synthetic-access")
	@Override
	protected final void core() {

		popupWindow = new JFrame("Indizes");
		popupWindow.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		
	
		MyTableModel myTableModel = new MyTableModel();

		JTable p = new JTable(myTableModel);
		p.getColumn("Index").setPreferredWidth(300);
		for (int i = 1; i < p.getColumnCount(); i++) {
			p.getColumn(p.getColumnName(i)).setPreferredWidth(130);
		}
		p.doLayout();
		JPanel pp = new JPanel();
		pp.setLayout(new BoxLayout(pp, BoxLayout.Y_AXIS));
		pp.add(p.getTableHeader());
		pp.add(p);
		JScrollPane pane = new JScrollPane(pp);
		popupWindow.getContentPane().add(pane, BorderLayout.CENTER);
		popupWindow.pack();
		popupWindow.setVisible(true);
	}
	private class MyTableModel extends AbstractTableModel {
		private static final long serialVersionUID = -1091576109554578925L;
		private static final int HEADER_STATICTICS = 1;
		private String[][] tableEntries;
		private boolean weighted;

		private MyTableModel() {
			weighted = getChosenClustering(validVis().getFirst())
					.getInterpretation().isWeighted();
			tableEntries = new String[getRowCount()][getColumnCount()];

			// cluster sizes
			for (int j = 0; j < validVis().size(); j++) {
				tableEntries[0][0] = HTMLString.bold("clusters");
				tableEntries[0][j + 1] = ""
						+ getChosenClustering(validVis().get(j)).size();
			}

			for (int index = 0; index < IndexRegistration.getIndizes().size(); index++) {
				int indexRow = getIndexRow(index);
				tableEntries[indexRow][0] = IndexRegistration.getIndizes().get(
						index).toString();
				if (weighted)
					tableEntries[indexRow + 1][0] = "....unweighted";

				// determine maxima and minima, store maxima in list
				double[][] extrema = new double[weighted ? 2 : 1][2];
				for (double[] current : extrema) {
					current[0] = Double.MAX_VALUE;
					current[1] = Double.MIN_VALUE;
				}
				List<Integer> maxValueIndices = new Vector<Integer>();
				List<Integer> maxWeightedValueIndices = new Vector<Integer>();
				for (int j = 0; j < validVis().size(); j++) {
					double value = IndexRegistration.getIndizes().get(index)
							.getIndex(getChosenClustering(validVis().get(j)))
							.getValue();
					if (value > extrema[0][1]) {
						extrema[0][1] = value;
						maxValueIndices.clear();
						maxValueIndices.add(j);
					} else if (value == extrema[0][1]) {
						maxValueIndices.add(j);
					}
					extrema[0][0] = Math.min(value, extrema[0][0]);
					if (weighted) {
						value = IndexRegistration.getIndizes().get(index)
								.getIndex(
										getChosenClustering(validVis().get(j)))
								.getUnweightedValue();
						if (value > extrema[1][1]) {
							extrema[1][1] = value;
							maxWeightedValueIndices.clear();
							maxWeightedValueIndices.add(j);
						} else if (value == extrema[1][1]) {
							maxWeightedValueIndices.add(j);
						}
						extrema[1][0] = Math.min(value, extrema[1][0]);
					}
				}

				// fill table
				for (int j = 0; j < validVis().size(); j++) {
					double value = IndexRegistration.getIndizes().get(index)
							.getIndex(getChosenClustering(validVis().get(j)))
							.getValue();
					String v = String.format("%+1.8f", value);		
					
		
					Color valueColor = ColorProvider
							.interpolateHSV(
									ColorProvider.ColorScale.values()[getOptionHandler()
											.getEnum("color scale")],
									extrema[0][0], extrema[0][1], value);
					tableEntries[indexRow][j + 1] = (value == extrema[0][1]) ? HTMLString
							.boldColor("" + v, valueColor)
							: HTMLString.color("" + v, valueColor);
					if (weighted) {
						value = IndexRegistration.getIndizes().get(index)
								.getIndex(
										getChosenClustering(validVis().get(j)))
								.getUnweightedValue();
						v = String.format("%+1.8f", value);
						valueColor = ColorProvider
								.interpolateHSV(ColorProvider.ColorScale
										.values()[getOptionHandler().getEnum(
										"color scale")], extrema[1][0],
										extrema[1][1], value);
						tableEntries[indexRow + 1][j + 1] = (value == extrema[1][1]) ? HTMLString
								.boldColor("" + v, valueColor)
								: HTMLString.color("" + v, valueColor);
					}
				}
			}
		}

		private LinkedList<Integer> validVis() {
			LinkedList<Integer> result = new LinkedList<Integer>();
			for (int i = 0; i < CLUSTERINGS; i++) {
				if (getChosenVisualizations()[i].equals("No Input")) {
					continue;
				}
				result.addLast(new Integer(i));
			}
			return result;
		}

		private int getIndexRow(int index) {
			return (weighted ? 2 : 1) * index + HEADER_STATICTICS + 1;
		}

		public String getColumnName(int col) {
			if (col == 0)
				return "Index";
			return "" + getChosenVisualizations()[validVis().get((col - 1))];
		}

		public int getRowCount() {
			return (weighted ? 2 : 1) * IndexRegistration.getIndizes().size()
					+ 1 + HEADER_STATICTICS;
		}

		public int getColumnCount() {
			return validVis().size() + 1;
		}

		public Object getValueAt(int row, int col) {
			return tableEntries[row][col];
		}

		public boolean isCellEditable(int row, int col) {
			return false;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see y.module.YModule#getOptionHandler()
	 */
	@Override
	public OptionHandler createOptionHandler() {
		OptionHandler op = super.createOptionHandler();
		op.addEnum("color scale", ColorProvider.ColorScale.values(), 0);
		return op;
	}


}
