package de.uka.algo.clustering.io;

/*
 * Created on 16.10.2004
 *
 */
import java.util.LinkedList;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import y.base.Edge;
import y.base.EdgeMap;
import y.base.Graph;
import y.base.Node;
import y.base.NodeCursor;
import y.base.NodeList;
import y.base.NodeMap;
import y.view.Graph2D;
import y.view.LineType;
import de.uka.algo.clustering.Cluster;
import de.uka.algo.clustering.Clustering;
import de.uka.algo.graphs.GraphInterpretation;
import de.uka.algo.graphs.SimpleGraph;
import de.uka.algo.graphs.WeightedGraph;
import de.uka.algo.util.ColorProvider;

/**
 * This Class is used by the ClusteringIOHandler to read a file to a clustering.
 * 
 * @author Daniel Delling
 * 
 */

public class XMLHandler extends DefaultHandler {

	private Graph g;

	private Graph2D g2;

	private boolean graph2d = false;

	private NodeMap index = null;

	private EdgeMap costs = null;

	private Node n, s, t;

	private Node[] nodes = null;

	private LinkedList<Clustering> clusterings;

	private GraphInterpretation gi;

	private boolean weighted = false;

	private Node lastNode = null;

	private Edge lastEdge = null;

	private int currentClustering = 0;

	private LinkedList<NodeMap> clusters = null;
	private LinkedList<NodeMap> clusterNames = null;

	private String writeEdgeWeight = null;
	private String writeNodeLabel = null;

	private ColorProvider cp = null;

	public XMLHandler(Graph g) {
		this.g = g;
		if (g instanceof Graph2D) {
			this.g2 = (Graph2D) g;
			this.graph2d = true;
		}
		index = g.createNodeMap();
		cp = new ColorProvider();
	}

	protected Clustering getFirstClustering() {
		return clusterings.getFirst();
	}

	protected LinkedList<Clustering> getClustering() {
		return clusterings;
	}

	@Override
	public void startDocument() throws SAXException {
		super.startDocument();
		clusters = new LinkedList<NodeMap>();
		clusterNames = new LinkedList<NodeMap>();
		clusterings = new LinkedList<Clustering>();
	}

	public void startElement(String namespaceURI, String lName, // local name
			String qName, // qualified name
			Attributes attrs) throws SAXException {

		if (qName.equals("node")) {
			createNode(attrs);
		} else if (qName.equals("key")) {
			keyHandler(attrs);
		} else if (qName.equals("data")) {
			dataHandler(attrs);
		} else if (qName.equals("edge")) {
			createEdge(attrs);
		} else if (qName.equals("y:EdgeLabel")) {
			writeEdgeWeight = "";
		} else if (qName.equals("y:Fill")) {
			setColor(attrs);
		} else if (qName.equals("y:BorderStyle")) {
			setBorderColor(attrs);
		} else if (qName.equals("y:Geometry")) {
			setLocation(attrs);
		} else if (qName.equals("y:NodeLabel")) {
			writeNodeLabel = "";
		} else System.out.println(lName);
	}

	
	@Override
	public void endElement(String uri, String localName, String qName)
			throws SAXException {
		if (qName.equals("y:EdgeLabel")) {			
			costs.setDouble(lastEdge, Double.parseDouble(writeEdgeWeight));
			writeEdgeWeight = null;
		}
		if (qName.equals("y:NodeLabel")) {
			g2.setLabelText(lastNode, writeNodeLabel);
			writeNodeLabel = null;
		}
	}

	private void setBorderColor(Attributes attrs) {
		if (graph2d) {
			String color = attrs.getValue(2).substring(1);
			g2.getRealizer(n).setLineColor(cp.hexToColor(color));
			g2.getRealizer(n).setLineType(LineType.LINE_5);
		}
	}

	private void setColor(Attributes attrs) {
		if (graph2d) {
			String color = attrs.getValue(0).substring(1);
			g2.getRealizer(n).setFillColor(cp.hexToColor(color));
		}
	}

	private void setLocation(Attributes attrs) {
		if (graph2d) {
			g2.setCenter(lastNode, Double.parseDouble(attrs.getValue(0)),
					Double.parseDouble(attrs.getValue(1)));
		}
	}

	private void keyHandler(Attributes attrs) {
		if (attrs.getValue(0).substring(0, 2).equals("cl")) {
			clusters.add(g.createNodeMap());
			clusterNames.add(g.createNodeMap());
		} else if (attrs.getValue(0).equals("d1")) {
			costs = g.createEdgeMap();
			weighted = true;
		}

	}

	private void dataHandler(Attributes attrs) {
		if (attrs.getValue(0).substring(0, 2).equals("cl")) {
			clusters.get(currentClustering).setInt(lastNode,
					Integer.parseInt(attrs.getValue(1)));
			currentClustering++;
		}
		if (attrs.getValue(0).substring(0, 2).equals("na")) {
			clusterNames.get(currentClustering).set(lastNode,
					attrs.getValue(1));
		}
	}

	private void createNode(Attributes attrs) {
		int id = Integer.parseInt(attrs.getValue(0).substring(1));
		n = g.createNode();
		index.setInt(n, id);
		lastNode = n;
		currentClustering = 0;
	}

	private void createEdge(Attributes attrs) {
		// erste Kante?
		if (g.edgeCount() == 0) {
			nodes = new Node[g.nodeCount()];
			for (NodeCursor nc = g.nodes(); nc.ok(); nc.next()) {
				n = nc.node();
				nodes[index.getInt(n)] = n;
			}
		}
		s = nodes[Integer.parseInt(attrs.getValue(1).substring(1))];
		t = nodes[Integer.parseInt(attrs.getValue(2).substring(1))];
		lastEdge = g.createEdge(s, t);
	}

	@Override
	public void endDocument() throws SAXException {
		// TODO Auto-generated method stub
		super.endDocument();

		if (weighted) {
			gi = new WeightedGraph(g, costs);
		} else {
			gi = new SimpleGraph(g);
		}

		for (int i = 0; i < clusters.size(); i++) {
			Clustering cl = new Clustering(g, gi);

			// abgearbeitete Knoten
			int nodesDone = 0;
			// der Cluster, der bearbeitet wird
			int clusterIndex = 0;

			// fuer jeden Clusterindex werden hier die Knoten abgespeichert
			// diese werden nachher gemergt
			NodeList clusterNodes = new NodeList();

			NodeCursor nc = g.nodes();
			while (nodesDone < g.nodeCount()) {
				clusterNodes.clear();
				String clusterName = null;
				for (nc.toFirst(); nc.ok(); nc.next()) {
					// geh?rt der Knoten zu dem betrachteten Cluster
					if (clusters.get(i).getInt(nc.node()) == clusterIndex) {
						nodesDone++;
						clusterNodes.add(nc.node());
						clusterName = (String) clusterNames.get(i).get(nc.node());
					}
				}
				// merge alle Knoten der Liste
				if (clusterNodes.isEmpty() == false) {
					Cluster c = cl.newCluster(clusterNodes.nodes());
					if (clusterName!=null) c.setLabel(clusterName);
				}
				clusterIndex++;
			}
			clusterings.addLast(cl);
		}
	}

	
	/* note: labels may span over two chunks of input.*/
	public void characters(char ch[], int start, int length) {				
		if (writeEdgeWeight !=null) {			
			for (int i = start; i < (start + length); i++) {
				writeEdgeWeight += ch[i];
			}			
		}
		if (writeNodeLabel != null && g2!=null) {
			for (int i = start; i < (start + length); i++) {
				writeNodeLabel += ch[i];
			}
		}

	}


}