/*
 * created May 9, 2006
 *
 * Copyright 2006.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
 *
 */
package de.uka.algo.clustering.io.modules;

import java.io.File;
import java.util.LinkedList;

import javax.swing.JFileChooser;
import javax.swing.JFrame;

import y.geom.YPoint;
import y.view.Graph2D;
import de.uka.algo.clustering.Clustering;
import de.uka.algo.clustering.io.ClusteringIOHandler;
import de.uka.algo.clustering.layouts.Layouter;
import de.uka.algo.clustering.visualization.MultiVisualizingModule;

public class ReadModule extends MultiVisualizingModule {

	private static final int CLUSTERINGS = 3;
	
	public ReadModule() {
		super("Open Clustering", "Daniel Delling", "", 0, CLUSTERINGS, 0, false, false);
	}

	@Override
	protected Clustering[] coreRunMulti() {
		
	    JFileChooser chooser = new JFileChooser();
	    chooser.addChoosableFileFilter(new MyFilter());
	    
	    int returnVal = chooser.showOpenDialog(new JFrame());
	    String fileName = null;
	    if(returnVal == JFileChooser.APPROVE_OPTION) {
	       fileName = chooser.getSelectedFile().getAbsolutePath(); 
	    } else {
	    	return null;
	    }

	    ClusteringIOHandler io = new ClusteringIOHandler();
		Graph2D toCluster = getGraph2D();
		toCluster.clear();

		Clustering cl = null;
		
		LinkedList<Clustering> clusterings = io.read(toCluster, fileName);
		boolean hasEmbedding = true;
		YPoint first = toCluster.getCenter(toCluster.firstNode());
		YPoint last = toCluster.getCenter(toCluster.lastNode());
		if(first.x == last.x && first.y == last.y){
			hasEmbedding = false;			
		}
		
		cl = clusterings.get(0);
		if(!hasEmbedding){
			Layouter.getLayouters()[1].doLayout(cl);			
		}
		Clustering[] result = new Clustering[CLUSTERINGS];
		for (int i = 0; i<clusterings.size();i++) {
			result[i] = clusterings.get(i);
		}
		return result;
	}

	class MyFilter extends javax.swing.filechooser.FileFilter {
        public boolean accept(File file) {
            String filename = file.getName();
            return filename.endsWith(".graphml") || file.isDirectory();
        }
        public String getDescription() {
            return "*.graphml";
        }
    }		
}
