/* 
 * Integration: May 2, 2006 (katz)
 * Reviewed: July 18, 2008 (katz)
 *
 * Copyright 2004-2008.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
 */
package de.uka.algo.clustering.layouts;

import java.util.LinkedList;

import de.uka.algo.clustering.Clustering;

/**
 * A {@link Layouter} is a class providing a scheme to visualize a clustering by
 * node positions and/or edge routing.
 * <p>
 * To access an own implementation for visualization within yEd, call
 * {@code Layouter.add(new MyLayouter(params))} in your start-up code to add it
 * to the list of layouts that is used to populate menus.
 * 
 * @author Bastian Katz, bastian.katz@kit.edu
 */
public abstract class Layouter {

	/** the list of registered layouters */
	private static LinkedList<Layouter> layouterList = new LinkedList<Layouter>();

	/**
	 * Adds a given layouter to the list of layouters accessible from yEd.
	 * 
	 * @param aLayouter
	 *            the layouter to add.
	 */
	public static void addLayouter(Layouter aLayouter) {
		layouterList.add(aLayouter);
	}

	/**
	 * @return the list of currently registered layouts.
	 */
	public static Layouter[] getLayouters() {
		return layouterList.toArray(new Layouter[0]);
	}

	/**
	 * Layouts a clustering's graph.
	 * 
	 * @param aClustering
	 *            the input clustering. Must have a {@code Graph2D} as
	 *            underlying graph.
	 */
	public abstract void doLayout(Clustering aClustering);
}
