package de.uka.algo.clustering.operators;

import java.util.HashMap;
import java.util.Map;

import y.base.Node;
import y.base.NodeCursor;
import y.base.NodeList;
import de.uka.algo.clustering.Cluster;
import de.uka.algo.clustering.Clustering;
import de.uka.algo.util.GYCursor;

public class Product extends Operator {

	@Override
	public void run(Clustering operand, Clustering operator) {
		if (operand.getGraph() != operator.getGraph())
			throw new IllegalArgumentException("Graphs must be the same!");

		Map<Cluster, NodeList> clustermap;

		for (GYCursor<Cluster> clusters = operand.clusters(); clusters.ok(); clusters
				.next()) {
			clustermap = new HashMap<Cluster, NodeList>();
			Cluster currentCluster = (Cluster) clusters.current();
			for (NodeCursor members = currentCluster.members(); members.ok(); members
					.next()) {
				Node currentNode = (Node) members.current();
				Cluster currentCluster2 = operator.getCluster(currentNode);
				if (currentCluster2 == null) {
					// do nothing
				} else if (clustermap.containsKey(currentCluster2)) {
					clustermap.get(currentCluster2).add(currentNode);
				} else {
					clustermap.put(currentCluster2, new NodeList(currentNode));
				}
			}

			boolean fullsplit = false;
			for (NodeList currentList : clustermap.values()) {
				if (currentList.size() == currentCluster.size()) {
					fullsplit = true;
				} else {
					operand.split(currentCluster, currentList.nodes());
				}
			}
			if (!fullsplit) operand.delete(currentCluster);
		}
	}
}
