package de.uka.algo.clustering.visualization;

import y.base.NodeCursor;
import y.module.YModule;
import y.option.OptionHandler;
import y.view.Graph2D;
import y.view.hierarchy.HierarchyManager;
import de.uka.algo.clustering.Clustering;

public abstract class FacadeModule extends YModule {

    protected static final String SECTION = "yEd interface";

    private static final String CLUSTER = "Input Clustering";

    protected static final String LABELS = "Use Weights";

    public static final boolean GROUPS_KEEP = true;

    public static final boolean GROUPS_KILL = false;

    public static final int WEIGHTS_USE = 0;

    public static final int WEIGHTS_ASK = 1;

    public static final int WEIGHTS_IGNORE = 2;

    private Clustering grouping;

    private String[] chosenVisualizations;

    private Clustering[] chosenClusterings;
    
    private int handleWeights;

    protected boolean keepGroups;

	private Clustering defaultClustering;

	private int nonEmptyInputs;

    public FacadeModule(String arg0, String arg1, String arg2, int chosenInput,
            int handleWeights, boolean keepGroups, boolean allowEmptyInput) {
        this(arg0, arg1, arg2, chosenInput, handleWeights, keepGroups, allowEmptyInput?0:chosenInput);
    }

    public FacadeModule(String arg0, String arg1, String arg2, int chosenInput,
            int handleWeights, boolean keepGroups, int nonEmptyInputs) {
        super(arg0, arg1, arg2);
        this.chosenClusterings = new Clustering[chosenInput];
        this.chosenVisualizations = new String[chosenInput];
        this.handleWeights = handleWeights;
        this.keepGroups = keepGroups;
        this.nonEmptyInputs = nonEmptyInputs;
    }

    
    public Clustering getChosenClustering(int i) {
        return chosenClusterings[i];
    }

    protected abstract void core();

    @Override
    protected final void mainrun() {
        Graph2D graph = getGraph2D();
        OptionHandler op = getOptionHandler();

        boolean useweights;
        switch (handleWeights) {
        case WEIGHTS_USE:
            useweights = true;
            break;
        case WEIGHTS_IGNORE:
            useweights = false;
            break;
        case WEIGHTS_ASK:
            useweights = op.getBool(LABELS);
            break;
        default:
            useweights = false;
            break;
        }
        /* store hierarchy */
        grouping= new Grouping().extract(graph, useweights);

        /* kill hierarchy */
        HierarchyManager hierarchy = HierarchyManager.getInstance(graph);
        if (hierarchy != null) {
            for (NodeCursor c = graph.nodes(); c.ok(); c.next()) {
                if (!hierarchy.isNormalNode(c.node())) {
                    graph.removeNode(c.node());
                }
            }
        }

        defaultClustering = Graph2DFacade.getExtractors()[0].extract(graph, useweights);
        /* store input clusterings */
        for (int i = 0; i < chosenClusterings.length; i++) {
            int k = op.getEnum(inputName(i));
            Graph2DExtractor vis = getApplicableExtractors(i)[k];
            chosenVisualizations[i] = vis.toString();
            if (vis instanceof Grouping) {
            	chosenClusterings[i]  = grouping;
            } else {
            	chosenClusterings[i] = vis.extract(graph,
                        useweights);
            }
        }


        /* pass */
        core();

        /* restore hierarchy */
        if (!hierarchy.containsGroups() && keepGroups==GROUPS_KEEP) {
            new Grouping().fakeVisualize(grouping);
        }

        graph.fitGraph2DView();
        graph.updateViews();
    }

    protected String inputName(int i) {
        return CLUSTER + (i + 1);
    }

    public Graph2DExtractor getExtractor(int i) {
        int k = getOptionHandler().getEnum(inputName(i));
        return getApplicableExtractors(i)[k];
    }
    
    protected OptionHandler createOptionHandler() {
        OptionHandler op = new OptionHandler(SECTION);
        op.useSection(SECTION);
        for (int i = 0; i < chosenClusterings.length; i++) {
            op.addEnum(inputName(i), getApplicableExtractors(i),
                    defaultExtraction(i));
        }
        if (handleWeights == WEIGHTS_ASK) {
            op.addBool(LABELS, true);
        }
        return op;
    }
    public Clustering getDefaultClustering() {
        return defaultClustering;
    }

    public String[] getChosenVisualizations() {
    	return chosenVisualizations;
    }
    private int defaultExtraction(int i) {
//    	if (i>=nonEmptyInputs) {
//    		return 0;
//    	}
        if (i < getApplicableExtractors(i).length - 1)
            return i;
        else
            return 0;
    }
    
    @SuppressWarnings("synthetic-access")
	private Graph2DExtractor[] getApplicableExtractors(int i) {
    	if (i<nonEmptyInputs) {
    		return Graph2DFacade.getExtractors();
    	}
    	Graph2DExtractor[] external = Graph2DFacade.getExtractors();
    	Graph2DExtractor[] result = new Graph2DExtractor[external.length+1];
    	result[0] = new EmptyExtraction();
    	for (int j=0;j<external.length; j++) {
    		result[j+1] = external[j];
    	}
    	return result;    	
    }
    
    private class EmptyExtraction implements Graph2DExtractor {
		public Clustering extract(Graph2D graph, boolean useWeights) {
			return null;
		}    	
		@Override
		public String toString() {		
			return "No Input";
		}
    }
}
