package de.uka.algo.clustering.visualization;

import y.view.Graph2D;
import de.uka.algo.clustering.Clustering;

/**
 * A {@link Graph2DExtractor} is a class that can retrieve a clustering from a
 * given Graph2D object using Graph2D attributes. Classes implementing this
 * interface should be subclasses of Graph2DFacade. They can be registered there
 * to appear in all modules that operate on an input clustering.
 * 
 * @author Bastian Katz, bastian.katz@kit.edu
 */
public interface Graph2DExtractor {

	/**
	 * @param aGraph
	 *            an instance of {@link Graph2D}
	 * @param useWeights
	 *            flag to denote whether the interpretation of the graph should
	 *            extract edge labels as weights ({@code true}) or not ({@code true}).
	 *            Note that in the former case, empty or unparsable edge labels
	 *            default to "1".
	 * @return the clustering extracted according to the respective attribute.
	 */
	public abstract Clustering extract(Graph2D aGraph, boolean useWeights);
}
