package de.uka.algo.clustering.visualization;

import java.awt.Color;

import y.base.Node;
import y.base.NodeCursor;
import y.base.NodeList;
import y.view.Graph2D;
import y.view.hierarchy.HierarchyManager;
import de.uka.algo.clustering.Cluster;
import de.uka.algo.clustering.Clustering;
import de.uka.algo.graphs.GraphInterpretation;
import de.uka.algo.util.GYCursor;

public class Grouping extends Graph2DFacade implements
		Graph2DExtractor, Graph2DVisualizer {

	public void visualize(Clustering clustering, boolean round) {
		setEdgeLabels(clustering, round);
		fakeVisualize(clustering);
	}

	public Clustering extract(Graph2D graph, boolean useWeights) {
		Clustering result = new Clustering(graph, GraphInterpretation
				.getInterpretation(graph, useWeights));
		HierarchyManager hierarchy = graph.getHierarchyManager();
		for (NodeCursor nc = graph.nodes(); nc.ok(); nc.next()) {
			Node n = nc.node();
			if (hierarchy.isGroupNode(n)) {
				Cluster c = result.newCluster(hierarchy.getChildren(n));
				String name = graph.getLabelText(n);
				if (!name.equals("")) {
					c.setLabel(name);
				}
			}
		}
		result.pool2Singletons();
		return result;
	}

	public String toString() {
		return "Grouping";
	}

	public void fakeVisualize(Clustering clustering) {
		Graph2D graph = getGraph2D(clustering);
		HierarchyManager hierarchy = graph.getHierarchyManager();
		for (GYCursor<Cluster> c = clustering.clusters(); c.ok(); c.next()) {
			if (c.active().size() == 1)
				continue;
			Node g = hierarchy.createGroupNode(graph);
			if (c.active().hasLabel()) {
				graph.setLabelText(g, c.active().getLabel());
			}
			NodeList cluster = new NodeList();
			for (NodeCursor ccv = c.active().members(); ccv.ok(); ccv.next())
				cluster.add(ccv.node());
			hierarchy.groupSubgraph(cluster, g);
		}
		Color platin = new Color(222, 222, 222);
		for (NodeCursor c = graph.nodes(); c.ok(); c.next()) {
			if (hierarchy.isFolderNode(c.node())
					|| hierarchy.isGroupNode(c.node()))
				(graph.getRealizer(c.node())).setFillColor(platin);
		}
	}

}
