package de.uka.algo.clustering.visualization;

import y.option.OptionHandler;
import de.uka.algo.clustering.Clustering;
import de.uka.algo.clustering.layouts.Layouter;

public abstract class MultiVisualizingModule extends FacadeModule {

	private static final String LAYOUT = "layout";
	private static final String VIS_TECHNIQUE = "Output visualization";
	private int[] defaultVisualizations;
	
	private int defaultLayout;

	private boolean keepGroupsStore;

	private int outputCount;

	public MultiVisualizingModule(String arg0, String arg1, String arg2,
			int inputCount, int outputCount, int handleWeights,
			boolean keepGroups, boolean allowEmptyInput) {
		super(arg0, arg1, arg2, inputCount, handleWeights, keepGroups,
				allowEmptyInput);
		// TODO hack to restore 'keep groups' flag
		this.outputCount = outputCount;
		this.keepGroupsStore = keepGroups;
		defaultVisualizations = new int[outputCount];
		int applicableVisualizations = Graph2DFacade.getVisualizer().length;
		for (int i = 0; i<outputCount; i++) {
			defaultVisualizations[i] = i+1;
			if (defaultVisualizations[i]>applicableVisualizations-1) {
				defaultVisualizations[i] = 0;
			}
		}
		defaultLayout = 0;
	}

	public MultiVisualizingModule(String arg0, String arg1, String arg2,
			int inputCount, int outputCount, int handleWeights,
			boolean keepGroups, int nonEmptyInputs) {
		super(arg0, arg1, arg2, inputCount, handleWeights, keepGroups,
				nonEmptyInputs);
		// TODO hack to restore 'keep groups' flag
		this.outputCount = outputCount;
		this.keepGroupsStore = keepGroups;
		defaultVisualizations = new int[outputCount];
		int applicableVisualizations = Graph2DFacade.getVisualizer().length;
		for (int i = 0; i<outputCount; i++) {
			defaultVisualizations[i] = i+1;
			if (defaultVisualizations[i]>applicableVisualizations-1) {
				defaultVisualizations[i] = 0;
			}
		}
		defaultLayout = 0;
	}

	
	protected void setDefaultVisualization(int index, String id) {
		int testindex = 0;
		for (Graph2DVisualizer gv: Graph2DFacade.getVisualizer()) {
			if (gv.toString().equals(id)) {
				defaultVisualizations[index] = testindex;
			}
			testindex++;
		}	
	}

	protected void setDefaultLayouter(String className) {
		int index = 0;
		for (Layouter layouter: Layouter.getLayouters()) {
			if (layouter.toString().equals(className)) {
				defaultLayout = index;
			}
			index++;
		}
	}
	
	protected final void core() {
		keepGroups = keepGroupsStore;
		Clustering[] clusteringsToVisualize = coreRunMulti();
		getOptionHandler().useSection(SECTION);
		Layouter.getLayouters()[getOptionHandler().getEnum(LAYOUT)].doLayout(clusteringsToVisualize[0]);
		for (int i = 0; i < outputCount; i++) {
			if (clusteringsToVisualize[i] == null)
				continue;
			Graph2DVisualizer currentVisualizer = getVisualizer(i);
			if (currentVisualizer instanceof Grouping) {
				keepGroups = false;
			}
			currentVisualizer.visualize(clusteringsToVisualize[i], false);
		}
	}

	protected Graph2DVisualizer getVisualizer(int index) {
		OptionHandler op = getOptionHandler();
		op.useSection(SECTION);
		if (outputCount > 1) {
			return Graph2DFacade.getVisualizer()[op.getEnum(VIS_TECHNIQUE
					+ (index + 1))];
		}
		Graph2DVisualizer gv = Graph2DFacade.getVisualizer()[op.getEnum(VIS_TECHNIQUE)];
		System.out.println(gv);
		return gv;
	}

	protected abstract Clustering[] coreRunMulti();

	protected OptionHandler createOptionHandler() {
		OptionHandler op = super.createOptionHandler();
		op.useSection(SECTION);
		if (outputCount > 1) {
			for (int i = 0; i < outputCount; i++) {
				op.addEnum(VIS_TECHNIQUE + (i + 1), Graph2DFacade
						.getVisualizer(), defaultVisualizations[i]);
			}
		} else {
			op.addEnum(VIS_TECHNIQUE, Graph2DFacade.getVisualizer(),
					defaultVisualizations[0]);
		}
		op.addEnum(LAYOUT, Layouter.getLayouters(), defaultLayout);
		return op;
	}
}
