/*
 * Created on 09.08.2004
 * Reviewed 1.6 (Mar 26, 2005)
 *
 * Copyright 2004.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
 */
package de.uka.algo.graphs;

import y.base.Edge;
import y.base.EdgeCursor;
import y.base.EdgeMap;
import y.base.Graph;
import y.base.Node;

/**
 * A SimpleGraph is a graph that is undirected and has trivial, unchangeable
 * weights for all edges (1.0).
 * 
 * @author Bastian Katz
 */
public class SimpleGraph extends UndirectedGraph {

	/**
	 * Instantiates a SimpleGraph for a given Graph.
	 * 
	 * @param aGraph
	 *            the underlying graph
	 */
	public SimpleGraph(Graph aGraph) {
		super(aGraph);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.uka.algo.graphs.GraphInterpretation#getWeight(y.base.Edge)
	 */
	public double getWeight(Edge anEdge) {
		return (anEdge != null ? 1.0 : 0.0);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.uka.algo.graphs.GraphInterpretation#getPartialInterpretation(y.base
	 * .Graph, y.base.EdgeMap)
	 */
	public GraphInterpretation getPartialInterpretation(Graph subGraph,
			EdgeMap map) {
		return new SimpleGraph(subGraph);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.uka.algo.graphs.GraphInterpretation#getAdjacency(y.base.Node,
	 * y.base.Node)
	 */
	public double getAdjacency(Node source, Node target) {
		double result = 0;
		for (EdgeCursor ec = source.edges(); ec.ok(); ec.next()) {
			Edge anEdge = ec.edge();
			if (anEdge.opposite(source) == target) {
				result += 1;
			}
		}
		if (source == target)
			result /= 2;
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.uka.algo.graphs.GraphInterpretation#isWeighted()
	 */
	public boolean isWeighted() {
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.uka.algo.graphs.UndirectedGraph#getDegree(y.base.Node)
	 */
	@Override
	public final double getDegree(Node aNode) {
		// unfortunately, yfiles do not support counting loops once as usual for
		// undirected graphs.
		int i = 0;
		int j = 0;
		for (EdgeCursor ec = aNode.edges(); ec.ok(); ec.next()) {
			j++;
			if (ec.edge().opposite(aNode) != aNode)
				i++;
		}
		return (i + j) / 2;
	}
}
