/*
 * Created on Aug 28, 2004
 * Reviewed 1.2 (Mar 26, 2005)
 *
 * Copyright 2004.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
 */
package de.uka.algo.graphs;

import y.base.Edge;
import y.base.EdgeCursor;
import y.base.Graph;
import y.base.Node;

/**
 * An UndirectedGraph is a graph where edges have no direction, i.e. that are
 * sets of size two instead of tuples.
 * 
 * @author Bastian Katz
 */
public abstract class UndirectedGraph extends GraphInterpretation {

	/**
	 * Instantiates an UndirectedGraph for a given Graph.
	 * 
	 * @param aGraph
	 *            the underlying graph
	 */
	public UndirectedGraph(Graph aGraph) {
		super(aGraph);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.uka.algo.graphs.GraphInterpretation#isDirected()
	 */
	final public boolean isDirected() {
		return false;
	}

	/**
	 * Returns a node's degree. For undirected graphs, this means returning the
	 * sum of weights of all edges that are incident to the node (not counting
	 * loops twice!)
	 * 
	 * @param aNode
	 *            the node
	 * @return the node's degree
	 */
	public double getDegree(Node aNode) {
		double result = 0.0;
		for (EdgeCursor ec = aNode.outEdges(); ec.ok(); ec.next()) {
			result += getAdjacency(ec.edge().source(), ec.edge().target());
		}
		for (EdgeCursor ec = aNode.inEdges(); ec.ok(); ec.next()) {
			Edge edge = ec.edge();
			if (edge.source() != edge.target())
				result += getAdjacency(ec.edge().source(), ec.edge().target());
		}
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.uka.algo.graphs.GraphInterpretation#getDegree(y.base.Node)
	 */
	public final double getOutDegree(Node aNode) {
		return getDegree(aNode);
	}

	public final double getInDegree(Node aNode) {
		return getDegree(aNode);
	}
}
