package de.uka.algo.arxivGen;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.sql.Timestamp;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.StringTokenizer;

import de.uka.algo.io.Helper4GraphMLIO;

/**
 * @author robert
 * 
 */
public class NamelessReaderAndScheduler {

	/**
	 * @param args
	 * @throws IOException
	 */
	public static void main(String[] args) throws IOException {
		// sanity check
		if (args.length < 3) {
			System.out.println("Arguments: (String)filename of publist (int)timeOut in years (String)output filename)");
			return;
		}
		System.out.println("NamelessReaderAndScheduler called. Looking for stuff...");

		// global variable stuff
		boolean outPutTempGraphs = true;
//		boolean outPutTempGraphs = false;
		boolean paperPerTimeStep = true;
		String[] impacting = new String[3];
		int impact = 2;
		impacting[0] = "each edge has weight 1.0";
		impacting[1] = "each publication distributes weight 1.0";
		impacting[2] = "each author gets weight 1.0 per publication";
				
		// in file
		String inputFileName = args[0];
		// inputFileName = "/algoDaten/dynClustGraphs/arxiv/arxiv_2010-01-26_16-22-40.txt";
//		 inputFileName = "/algoDaten/dynClustGraphs/arxiv/gr-qc_complete.txt";
		// inputFileName = "/algoDaten/dynClustGraphs/arxiv/q-bio_complete.txt";
//		inputFileName = "/algoDaten/dynClustGraphs/arxiv/q-bio_all.txt";
//		inputFileName = "/algoDaten/dynClustGraphs/arxiv/cs_complete_all.txt";
//		inputFileName = "/algoDaten/dynClustGraphs/arxiv/gr-qc_complete.txt";
//		inputFileName = "/algoDaten/dynClustGraphs/arxiv/nucl-th_complete.txt";
//		inputFileName = "/algoDaten/dynClustGraphs/arxiv/astro-ph_all.txt";
//		inputFileName = "/algoDaten/dynClustGraphs/arxiv/nlin_all.txt";
//		inputFileName = "/algoDaten/dynClustGraphs/arxiv/hep_all.txt";
//		inputFileName = "/algoDaten/dynClustGraphs/arxiv/math-ph-complete.txt";
//		inputFileName = "/algoDaten/dynClustGraphs/arxiv/q-fin_all.txt";
//		inputFileName = "/algoDaten/dynClustGraphs/arxiv/stat_all.txt";
		File inputFile = new File(inputFileName);
		FileReader inputFileReader = new FileReader(inputFile);
		BufferedReader bufferedInputFileReader = new BufferedReader(inputFileReader);

		// timeout
		int timeOut = Integer.parseInt(args[1]);
//		timeOut = 2;

		// out file
		String outputFileName = args[2];
		// outputFileName = "/algoDaten/dynClustGraphs/arxiv/arxiv_2010-01-26_16-22-40_dynGraph_W2_2Y.txt";
//		outputFileName = inputFileName.substring(0, inputFileName.length() - 4) + "_dynGraph_W2_" + timeOut + "Y";
		if (paperPerTimeStep){
			outputFileName = outputFileName + "_ppts";
		}
		outputFileName = outputFileName + ".txt";
		File outputFile = new File(outputFileName);
		FileWriter outputFileWriter = new FileWriter(outputFile);
		BufferedWriter bufferedOutputFileWriter = new BufferedWriter(outputFileWriter);

		eventTimeOutScheduler scheduler = new eventTimeOutScheduler();
		scheduler.setTimeOutYears(timeOut);
		scheduler.setPaperPerTimeStep(paperPerTimeStep);

		fillScheduler(bufferedInputFileReader, scheduler, impacting, impact);
		emptyScheduler(scheduler, bufferedOutputFileWriter, outPutTempGraphs);
		statsuff(scheduler);

		bufferedInputFileReader.close();
		bufferedOutputFileWriter.close();
		inputFileReader.close();
		outputFileWriter.close();
	}

	private static void statsuff(eventTimeOutScheduler scheduler) {
		System.out.println(scheduler.stats());
		// HashMap<Integer, Double> nodeWeight = scheduler.getNodeWeights();
		// Map.Entry<Integer, Double> currentEntry;
		// list map of node weights
		// for (Iterator<Map.Entry<Integer, Double>> it = nodeWeight.entrySet().iterator();it.hasNext();){
		// currentEntry = it.next();
		// System.out.println(currentEntry.getKey() + " -> " + currentEntry.getValue());
		// }
		System.out.println("max n: " + scheduler.maxN() + ", and max m: " + scheduler.maxEc() + "(W: " + scheduler.maxEw()
				+ ")");
		System.out.println("stats at the end: n: " + scheduler.currentN() + ", and m: " + scheduler.currentEc() + "(W: "
				+ scheduler.currentEw() + ")");
		System.out.println("max compound: " + scheduler.getMaxCompound() + " yielding "
				+ Math.round(Math.ceil(Math.sqrt((double) scheduler.getMaxCompound()))) + " coauthors in one pub");
		System.out.println("Finished with timestep " + (scheduler.getCurrentTimeStep()-1));
		System.out.println("That was " + scheduler.getPaperCount() + "papers");
	}

	private static void emptyScheduler(eventTimeOutScheduler scheduler, BufferedWriter bufferedOutputFileWriter,
			boolean outPutTempGraphs) throws IOException {
		int often = 4;
		// scheduler.sortEvents();
		System.out.println("all sorted: " + scheduler.checkSorting());
		// init
		System.out.println("init scheduler usage");
		scheduler.init();
		// get them back
		System.out.println("emptying scheduler");
		String currentMsg;
		int counter = 0;
		int all = scheduler.size();
		while (scheduler.hasMoreEvents()) {
			LinkedList<String> msgs = scheduler.nextEvents();
			while (!msgs.isEmpty()) {
				currentMsg = msgs.poll();
				// System.out.print(currentMsg);
				bufferedOutputFileWriter.append(currentMsg);
			}
			++counter;
			if (((counter % (all / often)) == 0) && (outPutTempGraphs)) {
				System.out.print("outputting graph at " + counter + "...");
				outputGraph(scheduler, ("/algoDaten/dynClustGraphs/arxiv/tempGraphs/tmpDynGraphAt" + counter + ".graphml"));
				System.out.println(" done");
			}
		}
		if (outPutTempGraphs) {
			System.out.print("outputting final graph at " + counter + "...");
			outputGraph(scheduler, ("/algoDaten/dynClustGraphs/arxiv/tempGraphs/tmpDynGraphAt" + counter + ".graphml"));
			System.out.println(" done");
		}
	}

	private static void outputGraph(eventTimeOutScheduler scheduler, String filename) {
		try {
			Helper4GraphMLIO.writeGraphML(scheduler.getCurrentGraph(), filename);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private static void fillScheduler(BufferedReader bufferedInputFileReader, eventTimeOutScheduler scheduler,
			String[] impacting, int impact) throws IOException {
		// insert them
		System.out.println("filling scheduler using impacting mode: " + impacting[impact]);
		String currentLine;
		String currentID; // not used atm, only read
		String currentTimeString;
		Timestamp currentTimeStamp;
		String currentCat; // not used atm, only read
		String currentAuthors;
		int a = 1, b = 2, i = 0, multiples = 0;
		HashSet<String> processedIds = new HashSet<String>();
		double currentWeight = 1.0;
		while (bufferedInputFileReader.ready()) {
			currentLine = bufferedInputFileReader.readLine();
			currentLine = currentLine.replace("::", "§");
			StringTokenizer myTok = new StringTokenizer(currentLine, "§");
			if (currentLine.substring(0, 1).equals(">")) {
				System.out.println("end signal reached: " + currentLine);
				break;
			}
			if (myTok.countTokens() != 4) {
				System.out.println(currentLine.replace("§", "::") + " has wrong number of tokens");
//				new BufferedReader(new InputStreamReader(System.in)).readLine();
				continue;
			}
			currentID = myTok.nextToken().trim();
			if (currentID.equals("None")){
				continue;
			}
			if (processedIds.contains(currentID)) {
				++multiples;
				continue;
			}
			processedIds.add(currentID);
			currentTimeString = myTok.nextToken().trim();
			if (currentTimeString.equals("None")){
				continue;
			}
			currentTimeString = currentTimeString.replace("T", " ").replace("Z", " ");
			try {
				currentTimeStamp = Timestamp.valueOf(currentTimeString);
			} catch (Exception e) {
				System.out.println("We have an invalid timestamp, skipping entry " + currentTimeString);
				continue;
			}
			currentCat = myTok.nextToken();
			if (currentCat.equals("None")){
				continue;
			}
			currentAuthors = myTok.nextToken();
			if (currentAuthors.equals("None")){
				continue;
			}
			StringTokenizer myAuthorTok = new StringTokenizer(currentAuthors, ",");
			String[] currentAuthorTokens = new String[myAuthorTok.countTokens()];
			switch (impact) {
			case 0:
				currentWeight = 1.0;
				break;
			case 1:
				currentWeight = 2.0 / (double) (currentAuthorTokens.length * (currentAuthorTokens.length - 1));
				break;
			case 2:
				currentWeight = 1.0 / (double) (currentAuthorTokens.length - 1);
				break;
			default:
				break;
			}
			i = 0;
			while (myAuthorTok.hasMoreTokens()) {
				currentAuthorTokens[i++] = myAuthorTok.nextToken().trim();
			}
			for (int k = 0; k < currentAuthorTokens.length - 1; ++k) {
				for (int l = k + 1; l < currentAuthorTokens.length; ++l) {
					a = Integer.parseInt(currentAuthorTokens[k]);
					b = Integer.parseInt(currentAuthorTokens[l]);
					if (a == b) {
						System.out.println("Self-loop on author " + a + " detected, in pub " + currentID + " at "
								+ currentTimeStamp);
					} else {
						eventTimeOutScheduler.ScheduleEvent anEvent = new eventTimeOutScheduler.ScheduleEvent(a, b,
								currentTimeStamp, currentID, currentWeight);
						scheduler.enqueue(anEvent);
						// System.out.println(anEvent.verbose());
						// scheduler.almostSortedEnqueue(anEvent);
					}
				}
			}
		}
		System.out.println("used " + processedIds.size() + " papers, and ignored additional " + multiples
				+ " multiples thereof");
		System.out.println("sorting");
		scheduler.sortEvents();
		System.out.println("finished sorting");
	}

}
