package de.uka.algo.dynclustering.algos;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import y.base.Edge;
import y.base.Node;
import y.base.NodeCursor;
import de.uka.algo.clustering.Cluster;
import de.uka.algo.dynclustering.algos.base.AlgorithmFactory;
import de.uka.algo.dynclustering.algos.base.DynamicClusteringAlgorithm;
import de.uka.algo.dynclustering.graphs.DynamicGraph;
import de.uka.algo.dynclustering.util.YUtils;
import de.uka.algo.dynclustering.util.RunTimer.TimeUnit;


/**
 * Assigns nodes to clusters at random.
 * @author 
 *
 */
public class Randomly extends DynamicClusteringAlgorithm {
	
	protected Random rand;
	
	private static final String NAME = "Randomly";
	
	public static class Factory implements AlgorithmFactory {

		public DynamicClusteringAlgorithm getAlgorithm(DynamicGraph dynGraph) {
			return new Randomly(dynGraph);
		}


		public void setParams(List<Object> params) {
			// ignored
			
		}
		
	}

	public Randomly(DynamicGraph dynGraph) {
		super(dynGraph);
		this.rand = new Random();
	}

	@Override
	protected void addEvent(boolean notificationTime, Cluster to, NodeCursor nc) {
		// ignored

	}

	@Override
	protected void deleteEvent(boolean notificationTime, Cluster deletedCluster) {
		// ignored

	}

	@Override
	protected void edgeCreationEvent(Edge newEdge) {
		// ignored

	}

	@Override
	protected void edgeRemovalEvent(boolean notificationTime, Edge edgeToRemove) {
		// ignored

	}

	@Override
	protected void extractEvent(boolean notificationTime, Cluster from, NodeCursor nc) {
		// ignored

	}

	@Override
	protected void mergeEvent(boolean notificationTime, Cluster a, Cluster b) {
		// ignored

	}

	@Override
	protected void moveEvent(boolean notificationTime, Cluster from, Cluster to, NodeCursor nc) {
		// ignored

	}

	@Override
	protected void newClusterEvent(boolean notificationTime, Cluster newCluster) {
		// ignored

	}

	@Override
	protected void newWeightsEvent() {
		// ignored

	}

	@Override
	protected void nodeCreationEvent(Node newNode) {
		// ignored

	}

	@Override
	protected void nodeRemovalEvent(boolean notificationTime, Node nodeToRemove) {
		// ignored

	}

	@Override
	protected void policyChanged(boolean policy) {
		// ignored

	}

	@Override
	protected void resetEvent(boolean notificationTime) {
		// ignored

	}

	@Override
	protected void resetLargeClusterEvent(boolean notificationTime, Cluster c) {
		// ignored

	}

	@Override
	protected void resetSingletonsEvent(boolean notificationTime) {
		// ignored

	}

	@Override
	protected void splitEvent(boolean notificationTime, Cluster a, Cluster b) {
		// ignored

	}

	@Override
	protected void weightChangedEvent(Edge e, double oldWeight, double newWeight) {
		// ignored

	}

	public void onTimeStepEvent() {
		this.timer.start();
		//
		/* assign pool nodes with equal probability to an existing cluster or to a singleton */
		for (NodeCursor nc = this.clustering.poolNodes(); nc.ok(); nc.next()) {
			int k = this.clustering.clusterCount();
			int r = this.rand.nextInt(k+1);
			if (r == k) {
				this.clustering.newCluster(nc.node());
			} else {
				ArrayList<Cluster> clusterList = YUtils.gYCursor2ArrayList(this.clustering.clusters());
				this.clustering.add(clusterList.get(r), nc.node());
			}
		}
		// 
		this.timer.stop();
		
		// DEBUG
		System.out.println(">> " + this.id() + " done after " + this.timer.read(TimeUnit.MilliSeconds) + " [ms]");
		// DEBUG
	}
	
	public String toString() {
		return this.id();
	}

	@Override
	public String id() {
		return NAME + ":" +  this.uniqueID;
	}

}
