package de.uka.algo.dynclustering.algos;

import java.util.HashMap;
import java.util.List;

import y.base.Edge;
import y.base.Graph;
import y.base.Node;
import y.base.NodeCursor;
import de.uka.algo.clustering.Cluster;
import de.uka.algo.dynclustering.algos.base.AlgorithmFactory;
import de.uka.algo.dynclustering.algos.base.DynamicClusteringAlgorithm;
import de.uka.algo.dynclustering.graphs.DynamicGraph;
import de.uka.algo.dynclustering.graphs.GeneratedGraph;
import de.uka.algo.dynclustering.util.RunTimer.TimeUnit;


/**
 * This class poses as an algorithm but
 * reads the solution from the reference clustering output
 * by DCRGenerator.
 * 
 * @author Christian Staudt
 *
 */
public class Reference extends DynamicClusteringAlgorithm {
	
	public static class Factory implements AlgorithmFactory {

		public DynamicClusteringAlgorithm getAlgorithm(DynamicGraph dynGraph) {
			return new Reference(dynGraph);
		}

	}
	
	private static final String NAME = "Reference";
	
	protected HashMap<Integer,Cluster> clusterID2cluster;
	protected GeneratedGraph jGraph;

	public Reference(DynamicGraph dynGraph) {
		super(dynGraph);
		this.clusterID2cluster = new HashMap<Integer,Cluster>();
		this.jGraph = (GeneratedGraph) dynGraph;
	}

	@Override
	protected void addEvent(boolean arg0, Cluster arg1, NodeCursor arg2) {
		// ignored

	}

	@Override
	protected void deleteEvent(boolean arg0, Cluster arg1) {
		// ignored

	}

	@Override
	protected void edgeCreationEvent(Edge arg0) {
		// ignored

	}

	@Override
	protected void edgeRemovalEvent(boolean arg0, Edge arg1) {
		// ignored

	}

	@Override
	protected void extractEvent(boolean arg0, Cluster arg1, NodeCursor arg2) {
		// ignored

	}

	@Override
	protected void mergeEvent(boolean arg0, Cluster arg1, Cluster arg2) {
		// ignored

	}

	@Override
	protected void moveEvent(boolean arg0, Cluster arg1, Cluster arg2, NodeCursor arg3) {
		// ignored

	}

	@Override
	protected void newClusterEvent(boolean arg0, Cluster arg1) {
		// ignored

	}

	@Override
	protected void newWeightsEvent() {
		// ignored

	}

	@Override
	protected void nodeCreationEvent(Node arg0) {
		// ignored

	}

	@Override
	protected void nodeRemovalEvent(boolean arg0, Node arg1) {
		// ignored

	}

	@Override
	protected void policyChanged(boolean arg0) {
		// ignored

	}

	@Override
	protected void resetEvent(boolean arg0) {
		// ignored

	}

	@Override
	protected void resetLargeClusterEvent(boolean arg0, Cluster arg1) {
		// ignored

	}

	@Override
	protected void resetSingletonsEvent(boolean arg0) {
		// ignored

	}

	@Override
	protected void splitEvent(boolean arg0, Cluster arg1, Cluster arg2) {
		// ignored

	}

	@Override
	protected void weightChangedEvent(Edge arg0, double arg1, double arg2) {
		// ignored

	}

	public void onTimeStepEvent() {
		this.timer.start();
		//
		this.reCluster();
		//
		this.timer.stop();
		
		// DEBUG
		System.out.println(">> " + this.id() + " done after " + this.timer.read(TimeUnit.MilliSeconds) + " [ms]");
		// DEBUG
	}
	
	
	/**
	 * Reset to singletons and cluster according to the reference clustering
	 */
	protected void reCluster() {
		this.clustering.resetSingletons();
		this.clustering.pool2Singletons();
		this.clusterID2cluster.clear();
		Graph graph = this.clustering.getGraph();
		for (NodeCursor nc = graph.nodes(); nc.ok(); nc.next()) {
			Node u = nc.node();
			Cluster singleton = this.clustering.getCluster(u);
			int clusterID = this.jGraph.getClusterID(u);
			if (this.clusterID2cluster.containsKey(clusterID)) {
				Cluster cluster = this.clusterID2cluster.get(clusterID);
				Cluster merged = this.clustering.merge(singleton, cluster);
				this.clusterID2cluster.put(clusterID, merged);
			} else {
				this.clusterID2cluster.put(clusterID, singleton);
			}
		}
	}

	@Override
	public String toString() {
		return "Reference";
	}

	@Override
	public String id() {
		return NAME + ":" + this.uniqueID; 
	}

}
