package de.uka.algo.dynclustering.algos;


import y.base.Edge;
import y.base.Node;
import y.base.NodeCursor;
import de.uka.algo.clustering.Cluster;
import de.uka.algo.clustering.algorithms.newman.NewmanClusterer;
import de.uka.algo.clustering.algorithms.newman.internal.CoverageQInterface;
import de.uka.algo.clustering.algorithms.newman.internal.GenericAdditiveAdapter;
import de.uka.algo.clustering.algorithms.newman.internal.GenericAdditiveDeltaQMatrix;
import de.uka.algo.dynclustering.algos.base.AlgorithmFactory;
import de.uka.algo.dynclustering.algos.base.DynamicClusteringAlgorithm;
import de.uka.algo.dynclustering.graphs.DynamicGraph;
import de.uka.algo.dynclustering.util.RunTimer.TimeUnit;

/**
 * The static NewmanClusterer posing as a dynamic clustering algorithm.
 * 
 * @author Christian Staudt
 * 
 */
public class StaticNewman extends DynamicClusteringAlgorithm {

	protected NewmanClusterer newman;
	
	private static final String NAME = "StaticNewman";

	public static class Factory implements AlgorithmFactory {

		public StaticNewman getAlgorithm(DynamicGraph dynGraph) {
			return new StaticNewman(dynGraph);
		}

	};

	public StaticNewman(DynamicGraph dynGraph) {
		super(dynGraph);

		CoverageQInterface modI = new CoverageQInterface();
		GenericAdditiveAdapter ma = new GenericAdditiveAdapter(true, modI, new GenericAdditiveDeltaQMatrix(modI));

		this.newman = new NewmanClusterer(ma);

		this.newman.setCreatingDendro(false);
		this.newman.setStoppingOnPeak(true);
	}

	@Override
	protected void addEvent(boolean arg0, Cluster arg1, NodeCursor arg2) {
		// TODO Auto-generated method stub

	}

	@Override
	protected void deleteEvent(boolean arg0, Cluster arg1) {
		// TODO Auto-generated method stub

	}

	@Override
	protected void edgeCreationEvent(Edge arg0) {
		// TODO Auto-generated method stub

	}

	@Override
	protected void edgeRemovalEvent(boolean arg0, Edge arg1) {
		// TODO Auto-generated method stub

	}

	@Override
	protected void extractEvent(boolean arg0, Cluster arg1, NodeCursor arg2) {
		// TODO Auto-generated method stub

	}

	@Override
	protected void mergeEvent(boolean arg0, Cluster arg1, Cluster arg2) {
		// TODO Auto-generated method stub

	}

	@Override
	protected void moveEvent(boolean arg0, Cluster arg1, Cluster arg2, NodeCursor arg3) {
		// TODO Auto-generated method stub

	}

	@Override
	protected void newClusterEvent(boolean arg0, Cluster arg1) {
		// TODO Auto-generated method stub

	}

	@Override
	protected void newWeightsEvent() {
		// TODO Auto-generated method stub

	}

	@Override
	protected void nodeCreationEvent(Node arg0) {
		this.timer.start();
		this.clustering.newCluster(arg0);
		this.timer.stop();
	}

	@Override
	protected void nodeRemovalEvent(boolean arg0, Node arg1) {
		// TODO Auto-generated method stub

	}

	@Override
	protected void policyChanged(boolean arg0) {
		// TODO Auto-generated method stub

	}

	@Override
	protected void resetEvent(boolean arg0) {
		// TODO Auto-generated method stub

	}

	@Override
	protected void resetLargeClusterEvent(boolean arg0, Cluster arg1) {
		// TODO Auto-generated method stub

	}

	@Override
	protected void resetSingletonsEvent(boolean arg0) {
		// TODO Auto-generated method stub

	}

	@Override
	protected void splitEvent(boolean arg0, Cluster arg1, Cluster arg2) {
		// TODO Auto-generated method stub

	}

	@Override
	protected void weightChangedEvent(Edge arg0, double arg1, double arg2) {
		// TODO Auto-generated method stub

	}

	public void onTimeStepEvent() {
		this.timer.start();
		//
		if (!this.clustering.getGraph().isEmpty()) {
			this.newman.run(this.clustering);
		}
		//
		this.timer.stop();
		
		// DEBUG
		System.out.println(">> " + this.id() + " done after " + this.timer.read(TimeUnit.MilliSeconds) + " [ms]");
		// DEBUG
	}

	@Override
	public String toString() {
		return this.id();
	}

	@Override
	public String id() {
		return NAME + ":" + this.uniqueID;
 	}

}
