package de.uka.algo.dynclustering.algos;

import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Random;

import org.apache.commons.lang.IncompleteArgumentException;

import y.base.Edge;
import y.base.EdgeCursor;
import y.base.EdgeMap;
import y.base.Graph;
import y.base.Node;
import y.base.NodeCursor;
import y.base.NodeList;
import y.base.NodeMap;
import de.uka.algo.clustering.Cluster;
import de.uka.algo.clustering.ClusterMap;
import de.uka.algo.clustering.Clustering;
import de.uka.algo.dynclustering.algos.Blondel.NodeOrderStrategy;
import de.uka.algo.dynclustering.algos.base.AlgorithmFactory;
import de.uka.algo.dynclustering.algos.base.DynamicClusteringAlgorithm;
import de.uka.algo.dynclustering.algos.blondel.ClusterChangeListener;
import de.uka.algo.dynclustering.graphs.DynamicGraph;
import de.uka.algo.dynclustering.util.RunTimer.TimeUnit;
import de.uka.algo.graphs.GraphInterpretation;
import de.uka.algo.graphs.WeightedGraph;
import de.uka.algo.util.GYCursor;

public class TDStaticBlondel extends DynamicClusteringAlgorithm {

	public static class Factory implements AlgorithmFactory {
		

		public Blondel.NodeOrderStrategy nodeOrderStrategy;
		public Blondel.StoppingCriterion stoppingCriterion;
		public double alpha;

		public DynamicClusteringAlgorithm getAlgorithm(DynamicGraph dynGraph) {
			return new TDStaticBlondel(dynGraph, this.nodeOrderStrategy, this.stoppingCriterion, this.alpha);
		}

		public void setParams(List<Object> params) throws IncompleteArgumentException {
			for (Object param : params) {
				if (param instanceof Blondel.NodeOrderStrategy) {
					this.nodeOrderStrategy = (Blondel.NodeOrderStrategy) param;
				} else if (param instanceof Blondel.StoppingCriterion) {
					this.stoppingCriterion = (Blondel.StoppingCriterion) param;
				}
			}
			if ((this.nodeOrderStrategy == null)) {
				throw new IncompleteArgumentException("Required Parameters: " + "Blondel.NodeOrderStrategy");
			}
		}

	};

	public class HierarchyLevel {
		public HierarchyLevel(Graph aGraph, Clustering aClustering, Clustering oldClustering) {
			this.theGraph = aGraph;
			this.theClustering = aClustering;
			this.clusterToNextLevelNode = new ClusterMap<Node>(this.theClustering);
//			this.randWeights = aGraph.createEdgeMap(); // first one filled in addRandWeights
			this.randWeights = baseGraphRandWeights; //this one is the master base thingy
//			System.out.println(baseGraphRandWeights.getDouble(this.theGraph.edges().edge()));
//			System.out.println(this.randWeights.getDouble(this.theGraph.edges().edge()));
			levelToHierarchyLevel.put(levelToHierarchyLevel.size(), this);
//			nodeToWeight = new HashMap<Node, Integer>();
			survivingNodes = 0;
//			for (NodeCursor nc = aGraph.nodes(); nc.ok(); nc.next()) {
//				// andrea's new: set the nodes contribution to rand, and count overall survivors for rand normalization
//				if (oldClustering.getCluster(nc.node()) == null) {
//					nodeToWeight.put(nc.node(), 0);
//				} else {
//					nodeToWeight.put(nc.node(), 1);
//					++survivingNodes;
//				}
//			}
//			System.out.println("survivors: " + survivingNodes);
		}

		public HierarchyLevel(HierarchyLevel oldLevel) {
//			this.nodeToWeight = new HashMap<Node, Integer>();
			contractClustering(oldLevel, this);
			// this.theGraph = this.theClustering.getGraph();
			this.clusterToNextLevelNode = new ClusterMap<Node>(this.theClustering);
			levelToHierarchyLevel.put(levelToHierarchyLevel.size(), this);
		}

		public Graph theGraph;
		public Clustering theClustering;
		public ClusterMap<Node> clusterToNextLevelNode;
		public ClusterChangeListener oracle;
//		public HashMap<Node, Integer> nodeToWeight;
		EdgeMap randWeights;
	};

	private static final String NAME = "TDStaticBlondel";

	// parameters
	/** the chosen strategy, i.e., in which order are nodes asked */
	protected NodeOrderStrategy nodeOrderStrategy;

	protected Blondel.StoppingCriterion stoppingCriterion;

	// /** the chosen criterion for when to stop agglomeration */
	// protected StoppingCriterion stopCrit;
	//
	// /** the update policy, i.e., what is a bad node? */
	// protected UpdatePolicy updatePolicy; // needed in static case?
	//
	// algo variables
	protected NodeList initialBadNodesList;

	protected boolean isWeighted;
	protected double randNormFactor = 1.0;
	protected double bestInnerIncrease;
	protected Clustering oldClustering;
	protected Graph abstractedGraph;
	private Clustering outputClustering;
	protected int neighborHandShakes = 0;
	protected boolean isFirstGraph = true;
	protected int survivingNodes = 0;
	protected double alpha = 0.5;
	protected EdgeMap baseGraphRandWeights;
	protected int mIntersect = 0;

	protected double incrementalModularity;
	protected double incrementalRand;
	protected HashMap<Integer, HierarchyLevel> levelToHierarchyLevel = new HashMap<Integer, HierarchyLevel>();
	protected HashSet<Node> invalidNodes = new HashSet<Node>(4);

	public TDStaticBlondel(DynamicGraph dynGraph, NodeOrderStrategy nodeOrderStrategy,
			Blondel.StoppingCriterion stoppingCriterion, double alphaPar) {
		super(dynGraph);

		this.nodeOrderStrategy = nodeOrderStrategy;
		this.stoppingCriterion = stoppingCriterion;
		this.isFirstGraph = true;
		this.alpha = Math.max(Math.min(alphaPar, 1.0),0.0);
		isFirstGraph = true;
		
	}
	
	@Override
	public String toString() {
		return this.id();
	}

	@Override
	protected void addEvent(boolean notificationTime, Cluster to, NodeCursor nc) {
		// ignored
	}

	@Override
	protected void deleteEvent(boolean notificationTime, Cluster deletedCluster) {
		// ignored

	}

	@Override
	protected void edgeCreationEvent(Edge newEdge) {
		// ignored

	}

	@Override
	protected void edgeRemovalEvent(boolean notificationTime, Edge edgeToRemove) {
		// ignored

	}

	@Override
	protected void extractEvent(boolean notificationTime, Cluster from, NodeCursor nc) {
		// ignored

	}

	@Override
	protected void mergeEvent(boolean notificationTime, Cluster a, Cluster b) {
		// ignored

	}

	@Override
	protected void moveEvent(boolean notificationTime, Cluster from, Cluster to, NodeCursor nc) {
		// ignored

	}

	@Override
	protected void newClusterEvent(boolean notificationTime, Cluster newCluster) {
		// ignored

	}

	@Override
	protected void newWeightsEvent() {
		// ignored

	}

	@Override
	protected void nodeCreationEvent(Node newNode) {
		// this.timer.start();
		// this.clustering.newCluster(newNode);
		// this.timer.stop();
	}

	@Override
	protected void nodeRemovalEvent(boolean notificationTime, Node nodeToRemove) {
		// ignored

	}

	@Override
	protected void policyChanged(boolean policy) {
		// ignored

	}

	@Override
	protected void resetEvent(boolean notificationTime) {
		// ignored

	}

	@Override
	protected void resetLargeClusterEvent(boolean notificationTime, Cluster c) {
		// ignored

	}

	@Override
	protected void resetSingletonsEvent(boolean notificationTime) {
		// ignored

	}

	@Override
	protected void splitEvent(boolean notificationTime, Cluster a, Cluster b) {
		// ignored

	}

	@Override
	protected void weightChangedEvent(Edge e, double oldWeight, double newWeight) {
		// ignored

	}

	public void onTimeStepEvent() {
		this.timer.start();
		//
		if (isFirstGraph){
//			System.out.println("Clustering from scratch (no history)");
			
			if (oldClustering != null){
				oldClustering.disposeAllListeners();
				oldClustering.dispose();
			}

			if (baseGraphRandWeights != null){
				this.clustering.getGraph().disposeEdgeMap(baseGraphRandWeights);
			}
			
			oldClustering = new Clustering(this.clustering.getGraph());
			// oldClustering.resetSingletons();
//			theBaseListener = new BlondelBaseListener(aClustering);
//			aClustering.addListener(theBaseListener);
		}
		else {
//			System.out.println("Clustering with history");
			this.oldClustering.dispose();
			this.oldClustering = this.clustering.clone();
		}
				
		this.clustering.resetSingletons();
		outputClustering = this.clustering;
		this.init();
		//
		this.timer.stop();

		// DEBUG
		System.out.println(">> " + this.id() + " done after " + this.timer.read(TimeUnit.MilliSeconds) + " [ms]");
		// DEBUG
	}

	protected int[] permuteArrray(int[] order) {
		Random myRandom = new Random();
		int swapID;
		int tmp;
		for (int i = 0; i < order.length; ++i) {
			swapID = myRandom.nextInt(order.length);
			tmp = order[i];
			order[i] = order[swapID];
			order[swapID] = tmp;
		}
		return order;
	}

	/**
	 * Given the Hashmap Level->HierarchyLevel, the original clustering, and the currentClustering this procedure summarizes
	 * all arrays by iteratively substituting array values by the value found at that array index in the next higher array.
	 * Starting with singletons, in the end, this represents the final clustering. This works since new clusterings are built
	 * with oldCluster.index = newNode.index.
	 * 
	 * @param levelToClusteringVector
	 * @param outputClustering
	 * @param currentClustering
	 */
	protected void unfurlHierarchy(Clustering outputClustering) {
		Graph theGraph = outputClustering.getGraph();
		int n = theGraph.N();
		int[] nodeAssignment = new int[n];
		Cluster theCluster;
		Node theNode;
		// System.out.println("Unfurling nodes");
		for (NodeCursor nc = theGraph.nodes(); nc.ok(); nc.next()) {
			theNode = nc.node();
			for (int i = 0; i <= (levelToHierarchyLevel.size() - 2); i++) {
				theCluster = levelToHierarchyLevel.get(i).theClustering.getCluster(theNode);
				theNode = levelToHierarchyLevel.get(i).clusterToNextLevelNode.get(theCluster);
			}
			nodeAssignment[nc.node().index()] = theNode.index();
		}
		// System.out.println("Finalizing");
		if (outputClustering.poolNodes().ok()) {
			throw new RuntimeException("not ok.");
		}
		outputClustering.resetToClustering(nodeAssignment);
		if (outputClustering.poolNodes().ok()) {
			throw new RuntimeException("not ok.");
		}
	}

//	/**
//	 * TODO new!!! An indicator for extra edges used for computing changes to the rand index invoked by moves
//	 * 
//	 * @param a
//	 * @param b
//	 * @return 1 if both nodes were present and co-clustered in the previous clustering, 0 otherwise
//	 */
//	protected double getRandEdge(Node a, Node b) {
//		Cluster aCluster = previousClustering.getCluster(a);
//		if ((previousClustering.getCluster(a).equals(previousClustering.getCluster(b))) && (aCluster != null))
//			return 1.0;
//		else
//			return 0.0;
//	}

	/**
	 * Returns the sum of edge weights between aNode and the nodes of aCluster TODO andrea's new!!! modified as to include
	 * rand-values
	 * 
	 * @param aNode
	 * @param aCluster
	 * @return sum of edge weights between aNode and aCluster
	 */
	/**
	 * @param aNode
	 * @param aCluster
	 * @return
	 */
	protected double adjacencyToCluster(Node aNode, Cluster aCluster) {
		double weight = 0.0;
		Clustering aClustering = aCluster.getClustering();
		GraphInterpretation gi = aClustering.getInterpretation();
		// through all in and then all out edges...
		for (EdgeCursor ec = aNode.edges(); ec.ok(); ec.next()) {
			Edge theEdge = ec.edge();
			Node neighbor = theEdge.opposite(aNode);
			if (aCluster.equals(aClustering.getCluster(neighbor))) {
				weight += gi.getWeight(theEdge);
				// dont count self loops twice, only the egde weight counts...
				// new andrea's: non since encoded in edge weights
			}
		}
		// but yfiles puts a node twice in its own set of neighbors, in case of
		// a selfloop, thus subtract
		if (aCluster.equals(aClustering.getCluster(aNode))) {
			weight -= gi.getAdjacency(aNode, aNode);
			// new andrea's none, see above
		}
		// new andrea's: the malus induced by merges in general is subtracted in bestNeighbor below
		return weight;
	}

	/**
	 * calculates for a node, which of the clusters of all its neighbors is best to move to, in terms of modularity. Returns
	 * null if becoming a singleton is best. Returns himself if no improvement can be found -> old cluster TODO new!!! uses
	 * the rand-malus of merging in general
	 * 
	 * @param searchingNode
	 *            the node that is examining its neighborhood for better clusters
	 * @param newNeighbor
	 *            the neighbor which is in the cluster that is best for searchingNode to move to
	 * @return
	 */
	protected Node bestNeighbor(Node searchingNode, Clustering theClustering, EdgeMap randWeights) {
		double currentIncrease = 0.0;
		double currentModIncrease = 0.0;
		double currentRandIncrease = 0.0;
		double usedModIncrease = 0.0;
		double usedRandIncrease = 0.0;
		GraphInterpretation gi = theClustering.getInterpretation();
		Node currentNeighbor = null;
		Node newNeighbor = searchingNode; // this is the default - stay where you are
		Cluster oldCluster = theClustering.getCluster(searchingNode), newCluster;
		double covMinus, covPlus, expPlus, expMinus, otherRandDelta, ownRandDelta;
		double W = gi.getEdgeWeightSum();
		double selfLoopWeight = gi.getAdjacency(searchingNode, searchingNode);
		double nonSimpleWeight = gi.getDegree(searchingNode) + selfLoopWeight;
		HashSet<Cluster> seenClusters = new HashSet<Cluster>();

		// compute sum of randEdgeWeights to all neighboring clusters
		// selfloop randWeight is irrelevant and thus skipped
		HashMap<Cluster, Double> otherClusterToRandDelta = new HashMap<Cluster, Double>();
		ownRandDelta = 0;
		for (EdgeCursor ec = searchingNode.edges(); ec.ok(); ec.next()) {
			Edge theEdge = ec.edge();
			Node neighbor = theEdge.opposite(searchingNode);
			if (neighbor.equals(searchingNode))
				continue; // since selfloopsWeight is irrelevant for randWeights
			newCluster = theClustering.getCluster(neighbor);
			if (oldCluster.equals(newCluster)) {
				// cannot be a selfloop, otherwise we'd have skipped three lines above, so its a current intra edge
				ownRandDelta += randWeights.getDouble(theEdge);
			} else {
				if (!otherClusterToRandDelta.containsKey(newCluster))
					otherClusterToRandDelta.put(newCluster, 0.0);
				otherClusterToRandDelta.put(newCluster,	otherClusterToRandDelta.get(newCluster) + randWeights.getDouble(theEdge));
			}
		}
//		System.out.print("collecting movement Rand deltas: own = " + ownRandDelta);
//		for (Double value : otherClusterToRandDelta.values()){
//			System.out.print(", " + value);
//		}
//		System.out.println();

		for (NodeCursor nc = searchingNode.neighbors(); nc.ok(); nc.next()) {
			currentNeighbor = nc.node();
			newCluster = theClustering.getCluster(currentNeighbor);
			if (seenClusters.contains(newCluster))
				continue;
			seenClusters.add(newCluster);
			++neighborHandShakes;
			if (oldCluster.getClustering() == null) {
				// System.out.println("oldCluster Kaputt");
			}
			if (newCluster.getClustering() == null) {
				// System.out.println("newCluster Kaputt");
			}
			if (newCluster.equals(oldCluster))
				continue; // nothing to compute: staying increases by 0.0
			// self loops automatically in this term
			covMinus = adjacencyToCluster(searchingNode, oldCluster) / W;
			// self loops will contribute to future
			covPlus = (adjacencyToCluster(searchingNode, newCluster) + selfLoopWeight) / W;
			expPlus = (nonSimpleWeight * (theClustering.getBorderWeight(oldCluster) + 2
					* theClustering.getInnerWeight(oldCluster) - nonSimpleWeight))
					/ (2.0 * W * W);
			expMinus = (nonSimpleWeight * (theClustering.getBorderWeight(newCluster) + 2 * theClustering
					.getInnerWeight(newCluster))) / (2.0 * W * W);
			// new andrea's: address the global rand change mali and boni:
			otherRandDelta = otherClusterToRandDelta.get(newCluster);

			currentModIncrease = covPlus - covMinus + expPlus - expMinus;
			currentRandIncrease = (otherRandDelta - ownRandDelta) / (double)mIntersect;
			currentIncrease = (1.0-alpha) * currentModIncrease + alpha * currentRandIncrease;
			// new andrea's: add the local rand weight each old and new pair:
			// System.out.println(randNormFactor + " , " + currentIncrease);
			if (currentIncrease > bestInnerIncrease) {
				bestInnerIncrease = currentIncrease;
				newNeighbor = currentNeighbor;
				usedModIncrease = currentModIncrease;
				usedRandIncrease = currentRandIncrease;
			}
		}
		// isolated node -> singleton!
		// moving to another non-empty cluster does neither change modularity (no edges, no degree) nor randG (no edges).
		// thus, wherever it comes from (singleton or not), we now isolate it. 
		if (gi.getDegree(searchingNode) == 0) {
			newNeighbor = null;
			// System.out.print("i->s");
//			System.out.print("isolated node! -> singleton (no need to check)");
			usedModIncrease = 0.0;
			usedRandIncrease = 0.0;
		} else {
			// check goodness of becoming a simple singleton (might already have been one, but then with graphEdges)
			covMinus = adjacencyToCluster(searchingNode, oldCluster) / W;
			covPlus = selfLoopWeight / W;
			expPlus = (nonSimpleWeight * (theClustering.getBorderWeight(oldCluster) + 2
					* theClustering.getInnerWeight(oldCluster) - nonSimpleWeight))
					/ (2.0 * W * W);
			expMinus = 0.0;
			otherRandDelta = 0.0;
			// edgesRandMinus as above
			// new andrea's: address the global rand change malus for each old pair (no new pairs):
			currentModIncrease = covPlus - covMinus + expPlus - expMinus;
			currentRandIncrease = (otherRandDelta - ownRandDelta) / (double)mIntersect;
			currentIncrease = (1-alpha) * currentModIncrease + alpha * currentRandIncrease;
			if (currentIncrease > bestInnerIncrease) {
				bestInnerIncrease = currentIncrease;
				newNeighbor = null;
//				System.out.print("changing to / staying singleton");
				usedModIncrease = currentModIncrease;
				usedRandIncrease = currentRandIncrease;
				// System.out.print("->s");
			}
			// System.out.println();
		}
		incrementalModularity += usedModIncrease;
		incrementalRand += usedRandIncrease;
//		System.out.println("using for node " + searchingNode.index() + ": deltaMod=" + usedModIncrease + ", deltaRandG=" + usedRandIncrease + ", combined=" + bestInnerIncrease);
		return (newNeighbor);
	}

	/**
	 * iterates once through all nodes in the array and moves them to the cluster of their favorite neighbor, according to
	 * the function bestNeighbor
	 * 
	 * @param theClustering
	 * @return the maximum increase achieved in this iteration
	 */
	/**
	 * @param theClustering
	 * @return
	 */
	protected double bestNeighborsForEveryOne(HierarchyLevel hl, Node[] badNodes) {
		Clustering currentClustering = hl.theClustering;
		Graph currentGraph = hl.theGraph;

		for (Node aNode : badNodes) {
			if (!currentGraph.contains(aNode)) {
				System.out.println("aaaaaaaaaaaaaaahhhhhhhhhhhhhhhh");
			}
		}

		int n = badNodes.length;
		int[] order = orderNodes(currentGraph, badNodes);
		double maxIncrease = 0.0;
		for (int i = 0; i < n; ++i) {
			bestInnerIncrease = 0.0;
			Node currentNode = badNodes[order[i]];
			Node currentBestNeighbor = bestNeighbor(currentNode, currentClustering, hl.randWeights);
			if ((bestInnerIncrease > 1e-6) || currentNode.degree() == 0) {
				// System.out.println("changing things for node " + currentNode);
				Cluster oldCluster = currentClustering.getCluster(currentNode);
				currentClustering.extract(oldCluster, currentNode);
				// incrementalModularity += bestInnerIncrease; moved to inner function
				// System.out.println("Increase: " + bestInnerIncrease + "
				// currentBestNeighbor " + currentBestNeighbor);
				// new singleton (increase or not):
				if (currentBestNeighbor == null) {
					Cluster newCluster = currentClustering.newCluster(currentNode);
				} else { // normal case, migration:
					Cluster newCluster = currentClustering.getCluster(currentBestNeighbor);
					currentClustering.add(newCluster, currentNode);
					// System.out.println("migration from " + oldCluster + " to " + newCluster);
				}
			}
			if (bestInnerIncrease > maxIncrease) {
				maxIncrease = bestInnerIncrease;
			}
		}
		return (maxIncrease);
	}

	/**
	 * Given a Clustering, this procedure creates a new graph, with one node per given cluster (such that cluster.index =
	 * node.index!), and with edge weights corresponding to the sums of edge weights between clusters. This also creates
	 * self-loops. The graph is returned as a new clustering. Additionally this procedure takes a Hashmap of Arrays as an
	 * argument; it writes the clustering into a new array as node.index -> cluster index, and appends this array into the
	 * hashmap. The key of this entry is the level of hierarchy on which this clustering has been done, the original graph
	 * being level 0. TODO check andrea's new: higher level edges are augmented by randweights!
	 * 
	 * @param nodeToWeight
	 * 
	 * @param ering
	 *            input Clustering to be abstracted
	 * @return a graph with a node per input clustering, and appropriate weights
	 */
	protected void contractClustering(HierarchyLevel oldLevel, HierarchyLevel newLevel) {
		newLevel.theGraph = new Graph();
//		newLevel.nodeToWeight = new HashMap<Node, Integer>();
		newLevel.randWeights = newLevel.theGraph.createEdgeMap();
		GraphInterpretation theInterpretation = oldLevel.theClustering.getInterpretation();
		int k = oldLevel.theClustering.size();
		// prepare adjacency matrix
		double[][] adjMatrix = new double[k][k];
		double[][] randMatrix = new double[k][k];
		for (int i = 0; i < k; ++i) {
			for (int j = 0; j < k; ++j) {
				adjMatrix[i][j] = 0.0;
				randMatrix[i][j] = 0.0;
			}
		}
		// collect edge weights into matrix, indexed by clusterID
		Edge currentEdge = null;
		int c1, c2;
		for (EdgeCursor ec = oldLevel.theClustering.getGraph().edges(); ec.ok(); ec.next()) {
			currentEdge = ec.edge();
			c1 = oldLevel.theClustering.getCluster(currentEdge.source()).getIndex();
			c2 = oldLevel.theClustering.getCluster(currentEdge.target()).getIndex();
			adjMatrix[c1][c2] += theInterpretation.getWeight(currentEdge);
			randMatrix[c1][c2] += oldLevel.randWeights.getDouble(currentEdge);
		}
		// create k new nodes in new graph
		Node[] newNodes = new Node[k];
		for (int i = 0; i < k; ++i) {
			newNodes[i] = newLevel.theGraph.createNode();
		}

		// point from old cluster to new node, with clusterID = new node's
		// position in array newNodes[] atm.
//		System.out.println(oldLevel.theClustering.size());
		for (GYCursor<Cluster> cc = oldLevel.theClustering.clusters(); cc.ok(); cc.next()) {
//			System.out.println(oldLevel.clusterToNextLevelNode.toString());
//			System.out.println(oldLevel.clusterToNextLevelNode.get((Cluster) cc.current()));
			oldLevel.clusterToNextLevelNode.set((Cluster) cc.current(), newNodes[((Cluster) cc.current()).getIndex()]);
//			System.out.println(oldLevel.clusterToNextLevelNode.get((Cluster) cc.current()));
		}
		// create edges, indexed by newNodes[clusterID], with matrix weight
		double currentWeight = 0.0;
		double currentRand = 0.0;
		EdgeMap newGraphWeightsEdgeMap = newLevel.theGraph.createEdgeMap();
		for (int i = 0; i < k; ++i) {
			for (int j = i; j < k; ++j) {
				if (i == j) {
					currentWeight = adjMatrix[i][j];
					currentRand = randMatrix[i][j];
				} else {
					currentWeight = adjMatrix[i][j] + adjMatrix[j][i];
					currentRand = randMatrix[i][j] + randMatrix[j][i];
				}
				if ((currentWeight > 0.0) || (currentRand > 0.0)) {
					currentEdge = newLevel.theGraph.createEdge(newNodes[i], newNodes[j]);
					newGraphWeightsEdgeMap.setDouble(currentEdge, currentWeight);
					newLevel.randWeights.setDouble(currentEdge, currentRand);
				}
			}
		}
		// summarize old node weights
//		for (GYCursor<Cluster> cc = oldLevel.theClustering.clusters(); cc.ok(); cc.next()) {
//			int newWeight = 0;
//			for (NodeCursor nc = cc.active().members(); nc.ok(); nc.next()) {
//				newWeight += oldLevel.nodeToWeight.get(nc.node());
//			}
//			newLevel.nodeToWeight.put(oldLevel.clusterToNextLevelNode.get(cc.active()), newWeight);
//		}

		newLevel.theClustering = new Clustering(newLevel.theGraph, new WeightedGraph(newLevel.theGraph,
				newGraphWeightsEdgeMap));
		newLevel.theClustering.resetSingletons();
		newLevel.theGraph.disposeEdgeMap(newGraphWeightsEdgeMap);
//		System.out.println(newLevel.theGraph.getRegisteredEdgeMaps());
//		System.out.println(newLevel.theClustering.getGraph().contains(newLevel.theGraph.nodes().node()));
		return;
	}

	protected NodeList getInitialBadNodes() {
		return initialBadNodesList;
	}

	/**
	 * Calculate the clustering.
	 */
	protected void init() {
		// clear hierarchy levels and dispose of all kinds of persistent EdgeMaps
		for (int i = 0; i < levelToHierarchyLevel.size(); ++i) {
//			System.out.println("clearing at beginning, level " + i);
			if (i > 0){
				levelToHierarchyLevel.get(i).theGraph.disposeEdgeMap(levelToHierarchyLevel.get(i).randWeights);
				levelToHierarchyLevel.get(i).oracle.disposeMaps();
			}
			levelToHierarchyLevel.get(i).clusterToNextLevelNode.dispose();
			levelToHierarchyLevel.get(i).theClustering.removeClusteringListener(levelToHierarchyLevel.get(i).oracle);
			levelToHierarchyLevel.get(i).theClustering.disposeAllListeners();
			levelToHierarchyLevel.get(i).theClustering.dispose();
			if (i > 0){
				for (EdgeMap deletabeEdgeMap : levelToHierarchyLevel.get(i).theGraph.getRegisteredEdgeMaps()){
					levelToHierarchyLevel.get(i).theGraph.disposeEdgeMap(deletabeEdgeMap);
				}
			}
			System.out.print("EdgeMaps clinging to graph at level " + i + ": " + levelToHierarchyLevel.get(i).theGraph.getGraphListeners().hasNext() + ": ");
			System.out.print(levelToHierarchyLevel.get(i).theGraph.getRegisteredEdgeMaps().length + "\n");
		}
		this.levelToHierarchyLevel.clear();

		incrementalModularity = 0.0;
		incrementalRand = 0.0;
		neighborHandShakes = 0;
		GraphInterpretation gi = this.dynGraph.getInterpretation();
		
		for (NodeCursor nc = this.dynGraph.nodes(); nc.ok(); nc.next()) {
			incrementalModularity -= ((gi.getDegree(nc.node()) + gi.getAdjacency(nc.node(), nc.node())) * (gi.getDegree(nc
					.node()) + gi.getAdjacency(nc.node(), nc.node())));
		}
		incrementalModularity /= (4 * gi.getEdgeWeightSum() * gi.getEdgeWeightSum());

		double currentBestImprovement = 1.0;
		// andrea's new, artificially increase for letting things start
		int oldClusterCount = oldClustering.getGraph().N() + 1 + 1;
		int hierarchyLevel = 0;
		int it;
		Clustering levelZeroClustering = new Clustering(outputClustering.getGraph(), gi);
		levelZeroClustering.resetSingletons();
		
		// andrea's new
		if (!isFirstGraph) {
			// System.out.println(originalLevel.theClustering.toString());
//			randNormFactor = alpha * 2.0 / ((double) survivingNodes * ((double) survivingNodes - 1.0));
			mIntersect = 0;
			incrementalRand = 0;
			double currentBaseRandWeight = 0.0;
			for (EdgeCursor ec = outputClustering.getGraph().edges(); ec.ok(); ec.next()) {
//				System.out.print(currentLevel.theClustering.getInterpretation().getWeight(ec.edge()) + ",");
				currentBaseRandWeight = baseGraphRandWeights.getDouble(ec.edge());
				if (currentBaseRandWeight != 0.0){
					++mIntersect;
//					System.out.print("d");
					if (currentBaseRandWeight < -0.1){
						++incrementalRand;
//						System.out.print("r");
					}
				}
				else{
//					System.out.print("n");
				}
			}
			incrementalRand /= (double)mIntersect;
//			System.out.println("Using rand factor " + randNormFactor + ", and alpha " + alpha);
//			addRandWeights(currentLevel, oldClustering, originalLevel);
		} else {
//			randNormFactor = 0.0;
			baseGraphRandWeights = outputClustering.getGraph().createEdgeMap();
//			System.out.println("created an edgeMap !!" );
//			System.out.println(baseGraphRandWeights.getDouble(outputClustering.getGraph().edges().edge()));
			isFirstGraph = false;
			mIntersect = 1; //just to avoid div by zero
		}
//		System.out.println();	
		
		HierarchyLevel originalLevel = new HierarchyLevel(outputClustering.getGraph(), levelZeroClustering, oldClustering);
//		System.out.println("original level created: n = " + originalLevel.theGraph.N() + ", k = "
//				+ originalLevel.theClustering.clusterCount());
//		HierarchyLevel shadowLevel = new HierarchyLevel(originalLevel);
//		System.out.println("shadowLevel created: n = " + shadowLevel.theGraph.N() + ", k = "
//				+ shadowLevel.theClustering.clusterCount());
//		HierarchyLevel currentLevel = shadowLevel;
		HierarchyLevel currentLevel = originalLevel;
		
		// incremental rand is initiated as original clustering vs singletons, counting only survivors. we only need to count co-separations (due to singletons). Not doubly.
//		for (GYCursor<Cluster> cc = oldClustering.clusters(); cc.ok(); cc.next()) {
//			incrementalRand += ((survivingNodes - ((Cluster) cc.active()).size()) * ((Cluster) cc.active()).size() * randNormFactor * 0.5);
//		}
		
//		 System.out.println("shadow created: n = " + currentLevel.theGraph.N() + ", k = " +
//		 currentLevel.theClustering.clusterCount());
//		System.out.print("current graph edge weights : ");

//		System.out.println("Using rand factor 1/" + mIntersect + ", and alpha " + alpha);
		while (currentLevel.theClustering.size() < oldClusterCount) {
//			System.out.println("Level:" + hierarchyLevel++ + ", n:" + currentLevel.theClustering.getGraph().N() + ", m:"
//					+ currentLevel.theClustering.getGraph().E() + ", mod:" + incrementalModularity + ", " + ", randG:"
//					+ (incrementalRand) + ".");
			oldClusterCount = currentLevel.theClustering.size();
			currentBestImprovement = 1.0;
			it = 1;

			// while mod increase, migrate and merge nodes
			Node[] badNodes = currentLevel.theGraph.getNodeArray();
			while (currentBestImprovement > 1e-6) {
				currentBestImprovement = bestNeighborsForEveryOne(currentLevel, badNodes);
//				System.out.print("It:" + it++ + ", best increase:" + currentBestImprovement + " ... ");
			}
//			System.out.print("no more improvement, level done (" + currentLevel.theClustering.size() + " clusters). \n");

			currentLevel.oracle = new ClusterChangeListener(currentLevel);
			currentLevel.theClustering.addListener(currentLevel.oracle);

//			System.out.println("oldClusterCount = " + oldClusterCount + ", currentLevelCSize = " + currentLevel.theClustering.size());
			// when no more increase, possibly create next level
			if (currentLevel.theClustering.size() < oldClusterCount) {
//				System.out.println("creating level " + (levelToHierarchyLevel.size() - 1));
				currentLevel = new HierarchyLevel(currentLevel);
			}
		}
//		System.out.println("No more contractions, hierarchy done");
		unfurlHierarchy(outputClustering);
//		System.out.println("Finished at mod:" + incrementalModularity + " and rand: " + (incrementalRand)
//				+ ", with " + neighborHandShakes + " neighbors' handshakes.");
//		 System.out.println("===== Done =====");
		updatebaseGraphRandWeights();
		
		// again: clear hierarchy levels and dispose of all kinds of persistent EdgeMaps
//		levelZeroClustering.disposeAllListeners();
//		levelZeroClustering.dispose();
		for (int i = 0; i < levelToHierarchyLevel.size(); ++i) {
			if (i > 0){
				levelToHierarchyLevel.get(i).theGraph.disposeEdgeMap(levelToHierarchyLevel.get(i).randWeights);
				levelToHierarchyLevel.get(i).oracle.disposeMaps();
			}
			else {
//				for (EdgeMap someBaseGraphEdgeMap : levelToHierarchyLevel.get(i).theGraph.getRegisteredEdgeMaps()){
//					if (!someBaseGraphEdgeMap.equals(baseGraphRandWeights)){
//						System.out.println("gotcha! get lost!");
//						levelToHierarchyLevel.get(i).theGraph.disposeEdgeMap(someBaseGraphEdgeMap);
//					}
//				}
			}
			levelToHierarchyLevel.get(i).clusterToNextLevelNode.dispose();
			levelToHierarchyLevel.get(i).theClustering.removeClusteringListener(levelToHierarchyLevel.get(i).oracle);
			if (i > -1){
				levelToHierarchyLevel.get(i).theClustering.disposeAllListeners();
				levelToHierarchyLevel.get(i).theClustering.dispose();
			}
			if (i > 0){
				for (EdgeMap deletableEdgeMap : levelToHierarchyLevel.get(i).theGraph.getRegisteredEdgeMaps()){
					levelToHierarchyLevel.get(i).theGraph.disposeEdgeMap(deletableEdgeMap);
				}

				for (NodeMap deletableNodeMap : levelToHierarchyLevel.get(i).theGraph.getRegisteredNodeMaps()){
					levelToHierarchyLevel.get(i).theGraph.disposeNodeMap(deletableNodeMap);
				}
			}
//			System.out.print("GraphListeners clinging to graph at level " + i + ": " + levelToHierarchyLevel.get(i).theGraph.getGraphListeners().hasNext() + ": ");
//			System.out.print("EdgeMaps: " + levelToHierarchyLevel.get(i).theGraph.getRegisteredEdgeMaps().length + "\n");
//			System.out.print("NodeMaps :" + levelToHierarchyLevel.get(i).theGraph.getRegisteredNodeMaps().length + "\n");
		}
		this.levelToHierarchyLevel.clear();
		
		return;
	}

	private void updatebaseGraphRandWeights() {
		for (EdgeCursor ec = outputClustering.getGraph().edges(); ec.ok(); ec.next()){
			if (outputClustering.isIntraClusterEdge(ec.edge())){
				baseGraphRandWeights.setDouble(ec.edge(), 1.0);
//				System.out.print("i" + baseGraphRandWeights.getDouble(ec.edge()));
			}
			else{
				baseGraphRandWeights.setDouble(ec.edge(), -1.0);
//				System.out.print("x" + baseGraphRandWeights.getDouble(ec.edge()));
			}
		}
		System.out.println();
	}
	
	private void addRandWeights(HierarchyLevel currentLevel, Clustering oldClustering, HierarchyLevel originalLevel) {
		int additionalEdges = 0;
		double zeroAugWeight = 0.0;
		EdgeMap newRandEdgeMap = currentLevel.theGraph.createEdgeMap();
		EdgeMap zeroAugmentedEdgeMap = currentLevel.theGraph.createEdgeMap();
		HashSet<Edge> newZeroEdges = new HashSet<Edge>();
		// System.out.print("\nShadow starts out with " + currentLevel.theGraph.N() + "," + currentLevel.theGraph.E() +
		// " nodes,edges. W = " + currentLevel.theClustering.getOverAllWeight() + "\n");
		// WeightedGraph theWGraph = new WeightedGraph(currentLevel.theClustering.getInterpretation());
		// WeightedGraph theWGraph = (WeightedGraph)currentLevel.theGraph;
		for (GYCursor<Cluster> cc = oldClustering.clusters(); cc.ok(); cc.next()) {
			for (NodeCursor nc1 = cc.active().members(); nc1.ok(); nc1.next()) {
				Cluster c1 = originalLevel.theClustering.getCluster(nc1.node());
				Node newN1 = originalLevel.clusterToNextLevelNode.get(c1);
				for (NodeCursor nc2 = cc.active().members(); nc2.ok(); nc2.next()) {
					if (nc1.node().index() >= nc2.node().index())
						continue;
					Cluster c2 = originalLevel.theClustering.getCluster(nc2.node());
					Node newN2 = originalLevel.clusterToNextLevelNode.get(c2);
					Edge theEdge = newN1.getEdge(newN2);
					if (theEdge == null) {
						theEdge = currentLevel.theGraph.createEdge(newN1, newN2);
						// TODO serious, terrible, despicable hack, prevent weight zero edges from vanishing by having them
						// carry a weight aaalllmost zero
						zeroAugWeight = Double.MIN_NORMAL;
						++additionalEdges;
						newZeroEdges.add(theEdge);
					} else {
						zeroAugWeight = currentLevel.theClustering.getInterpretation().getWeight(theEdge);
					}
					newRandEdgeMap.setDouble(theEdge, 2.0 * randNormFactor);
					zeroAugmentedEdgeMap.setDouble(theEdge, zeroAugWeight);
					// System.out.print("(" + 2.0 * randNormFactor + "," + zeroAugWeight + ")");
					// System.out.print("w=" + theWeight + ",");
				}
			}
		}
		for (EdgeCursor ec = currentLevel.theGraph.edges(); ec.ok(); ec.next()) {
			if (!newZeroEdges.contains(ec.edge()))
				zeroAugmentedEdgeMap.setDouble(ec.edge(), currentLevel.theClustering.getInterpretation()
						.getWeight(ec.edge()));
		}
//		System.out.println(additionalEdges + " additional rand-edges (graph-weight=0, rand-weight=" + 2.0 * randNormFactor
//				+ ") created");

		currentLevel.theClustering = new Clustering(currentLevel.theGraph, new WeightedGraph(currentLevel.theGraph,
				zeroAugmentedEdgeMap));
		currentLevel.theClustering.resetSingletons();
		currentLevel.theGraph.disposeEdgeMap(zeroAugmentedEdgeMap);
		currentLevel.clusterToNextLevelNode = new ClusterMap<Node>(currentLevel.theClustering);
		currentLevel.randWeights = newRandEdgeMap;
		// System.out.print("\nShadow has been augmented to " + currentLevel.theGraph.N() + "," + currentLevel.theGraph.E() +
		// " nodes,edges. W = " + currentLevel.theClustering.getOverAllWeight() + "\n");
		return;
	}

	/**
	 * @param theGraph
	 *            the graph the nodes in the array are part of
	 * @param nodeArray
	 *            the nodes to be ordered
	 * @return an integer array which is a permutation of the 0...n-1 numbers
	 */
	protected int[] orderNodes(Graph theGraph, Node[] nodeArray) {
		int n = nodeArray.length;
		int[] order = new int[n];
		switch (nodeOrderStrategy) {
		case Array:
			for (int i = 0; i < n; ++i) {
				order[i] = i;
			}
			break;
		case InvArray:
			for (int i = 0; i < n; ++i) {
				order[i] = n - 1 - i;
			}
			break;
		case Random:
			for (int i = 0; i < n; ++i) {
				order[i] = i;
			}
			order = permuteArrray(order);
			break;
		default:
			System.out.println("Error, invalid node order strategy!");
			break;
		}

		return (order);
	}

	@Override
	public String id() {
		return NAME + "(" + this.nodeOrderStrategy + "," + this.stoppingCriterion + "," + this.alpha + "):" + this.uniqueID;
	}

}
