package de.uka.algo.dynclustering.algos.ilp;

import y.base.Graph;
import y.base.Node;
import y.base.NodeCursor;
import de.uka.algo.clustering.Cluster;
import de.uka.algo.clustering.Clustering;

public class PartialModularity implements PartialObjectiveFunction {

	Graph graph;
	Clustering clustering;
	
	
	public PartialModularity(Graph graph, Clustering C) {
		this.graph = graph;
		this.clustering = C;
	}
	
	public double getCoefficient(Node u, Node v) {
		double coefficient = -u.degree() * v.degree() / (2.0 * this.graph.edgeCount());
		// if there is an edge between u and v:
		if (u.getEdge(v) != null) {
			coefficient += 1.0;
		}
		
		return coefficient;
	}
	
	public double getCoefficient(Node u, Cluster c) {
		double coefficient = 0;
		for (NodeCursor nc = c.members(); nc.ok(); nc.next()) {
			Node w = nc.node();
			double part = -u.degree() * w.degree() / (2.0 * this.graph.edgeCount());
			if (u.getEdge(w) != null) {
				part += 1;
			}
			coefficient += part;
		}
		return coefficient;
	}
	
	public double getCoefficient(Cluster c1, Cluster c2) {
		double coefficient = 0;
		for (NodeCursor nc1 = c1.members(); nc1.ok(); nc1.next()) {
			Node u = nc1.node();
			for (NodeCursor nc2 = c2.members(); nc2.ok(); nc2.next()) {
				Node w = nc2.node();
				double part = -u.degree() * w.degree() / (2.0 * this.graph.edgeCount());
				if (u.getEdge(w) != null) {
					part += 1;
				}
				coefficient += part;
			}
		}
		return coefficient;
	}

}
