package de.uka.algo.dynclustering.algos.ilp;

import y.base.Node;
import y.base.NodeCursor;
import de.uka.algo.clustering.Cluster;
import de.uka.algo.clustering.Clustering;
import de.uka.algo.graphs.WeightedGraph;

public class PartialWeightedModularity implements PartialObjectiveFunction {
	
	WeightedGraph wGraph;
	Clustering clustering;
	
	public PartialWeightedModularity(WeightedGraph wGraph, Clustering C) {
		this.wGraph = wGraph;
		this.clustering = C;
	}

	public double getCoefficient(Node u, Cluster c) {
		double coefficient = 0;
		for (NodeCursor nc = c.members(); nc.ok(); nc.next()) {
			Node w = nc.node();
			double part = -wGraph.getDegree(u) * wGraph.getDegree(w) / (2.0 * wGraph.getEdgeWeightSum());;
			part += wGraph.getAdjacency(u, w);
			coefficient += part;
		}
		return coefficient;
	}

	public double getCoefficient(Node u, Node v) {
		double coefficient = -wGraph.getDegree(u) * wGraph.getDegree(v) / (2.0 * wGraph.getEdgeWeightSum());
		coefficient += wGraph.getAdjacency(u, v);
		return coefficient;
	}

	public double getCoefficient(Cluster c1, Cluster c2) {
		double coefficient = 0;
		for (NodeCursor nc1 = c1.members(); nc1.ok(); nc1.next()) {
			Node u = nc1.node();
			for (NodeCursor nc2 = c2.members(); nc2.ok(); nc2.next()) {
				Node w = nc2.node();
				double part = -wGraph.getDegree(u) * wGraph.getDegree(w) / (2.0 * wGraph.getEdgeWeightSum());;
				part += wGraph.getAdjacency(u, w);
				coefficient += part;
			}
		}
		return coefficient;
	}

}
