package de.uka.algo.dynclustering.eval;

import java.util.ArrayList;

import de.uka.algo.clustering.comparison.ComparatorFactory;
import de.uka.algo.clustering.comparison.graphbased.counting.FowlkesMallowsGraphExt;
import de.uka.algo.clustering.comparison.graphbased.counting.JaccardGraphExt;
import de.uka.algo.clustering.comparison.graphbased.counting.RandGraphExt;
import de.uka.algo.clustering.comparison.graphbased.informationTheory.FredJainGraphExt;
import de.uka.algo.clustering.comparison.graphbased.overlaps.MaximumMatchGraphExt;
import de.uka.algo.clustering.comparison.nodebased.counting.FowlkesMallows;
import de.uka.algo.clustering.comparison.nodebased.counting.Jaccard;
import de.uka.algo.clustering.comparison.nodebased.counting.Rand;
import de.uka.algo.clustering.comparison.nodebased.informationTheory.FredJain;
import de.uka.algo.clustering.comparison.nodebased.overlaps.MaximumMatch;
import de.uka.algo.clustering.index.Coverage;
import de.uka.algo.clustering.index.IndexFactory;
import de.uka.algo.clustering.index.InterClusterConductance;
import de.uka.algo.clustering.index.IntraClusterConductance;
import de.uka.algo.clustering.index.Modularity;
import de.uka.algo.clustering.index.OperationIndex;
import de.uka.algo.clustering.index.PerformanceConventional;
import de.uka.algo.clustering.index.Significance;
import de.uka.algo.dynclustering.algos.Blondel;
import de.uka.algo.dynclustering.algos.EOO;
import de.uka.algo.dynclustering.algos.Newman;
import de.uka.algo.dynclustering.algos.Newman2;
import de.uka.algo.dynclustering.algos.OneClustering;
import de.uka.algo.dynclustering.algos.PartialILP;
import de.uka.algo.dynclustering.algos.Randomly;
import de.uka.algo.dynclustering.algos.Reference;
import de.uka.algo.dynclustering.algos.Singleton;
import de.uka.algo.dynclustering.algos.StaticBlondel;
import de.uka.algo.dynclustering.algos.StaticNewman;
import de.uka.algo.dynclustering.algos.StaticNewman2;
import de.uka.algo.dynclustering.algos.base.AlgorithmFactory;
import de.uka.algo.dynclustering.algos.newman.DynBacktrackStrategy;
import de.uka.algo.dynclustering.algos.newman.DynStonePitStrategy;
import de.uka.algo.dynclustering.update.BoundedNeighborhoodStrategy;
import de.uka.algo.dynclustering.update.BreakUpStrategy;
import de.uka.algo.dynclustering.update.NeighborhoodStrategy;
import de.uka.algo.graphs.cut.ConductanceCutter.ConductanceStrategies;
import de.uka.algo.traversal.implementations.MultiSourceBFS;
import de.uka.algo.traversal.implementations.MultiSourcePrioritySearch;
import de.uka.algo.traversal.priorities.DegreePriority;
import de.uka.algo.traversal.priorities.DistanceDegreePriority;
import de.uka.algo.traversal.priorities.DistanceBeforeDegreePriority;
import de.uka.algo.traversal.priorities.TargetDistancePriority;
import de.uka.algo.traversal.priorities.StackPriority;

public class EvaluationFrontend {

	public ArrayList<AlgorithmFactory> selectedAlgos = new ArrayList<AlgorithmFactory>();
	public ArrayList<IndexFactory> selectedQIs = new ArrayList<IndexFactory>();
	public ArrayList<ComparatorFactory> selectedDMs = new ArrayList<ComparatorFactory>();

	private Evaluation eval;

	public String graphPath;

	/* algos */

	public void useStaticNewman(boolean flag) {
		if (flag) {
			this.selectedAlgos.add(new StaticNewman.Factory());
		}
	}
	
	public void useStaticNewman2(boolean flag) {
		if (flag) {
			this.selectedAlgos.add(new StaticNewman2.Factory());
		}
	}

	public void useNewman(boolean flag, int updateStrategy, int traversalStrategy, int priority,
			int traversalArg, double weightA, double weightB) {
		if (flag) {
			Newman.Factory factory = new Newman.Factory();
			if (updateStrategy == 1) {
				factory.strategy = new DynBacktrackStrategy();
			} else if (updateStrategy == 2) {
				factory.strategy = new DynStonePitStrategy();
			} else if (updateStrategy == 3) {
				factory.strategy = new BreakUpStrategy();
			} else if (updateStrategy == 4) {
				// bounded neighborhood
				if (traversalStrategy == 1) {
					// multi-source BFS
					factory.strategy = new BoundedNeighborhoodStrategy(new MultiSourceBFS(), traversalArg);
				} else if (traversalStrategy == 2) {
					// multi-source priority search
					if (priority == 1) {
						factory.strategy = new BoundedNeighborhoodStrategy(new MultiSourcePrioritySearch(new DegreePriority(weightA)), traversalArg);
					} else if (priority == 2) {
						// factory.strategy = new TraversalStrategy(new
						// MultiSourcePrioritySearch(new DistancePriority(null,
						// null)), traversalSize);
					} else if (priority == 3) {
						factory.strategy = new BoundedNeighborhoodStrategy(new MultiSourcePrioritySearch(new DistanceDegreePriority(weightA, weightB)),
								traversalArg);
					} else if (priority == 4) {
						factory.strategy = new BoundedNeighborhoodStrategy(new MultiSourcePrioritySearch(new DistanceBeforeDegreePriority()), traversalArg);
					} else if (priority == 5) {
						factory.strategy = new BoundedNeighborhoodStrategy(new MultiSourcePrioritySearch(new StackPriority()), traversalArg);
					}
				}
			} else if (updateStrategy == 5) {
				// neighborhood
				if (traversalStrategy == 1) {
					// multi-source BFS
					factory.strategy = new NeighborhoodStrategy(new MultiSourceBFS(), traversalArg);
				}
			}
			this.selectedAlgos.add(factory);

		}
	}
	
	
	public void useNewman2(boolean flag, int updateStrategy, int traversalStrategy, int priority,
			int traversalArg, double weightA, double weightB) {
		if (flag) {
			Newman2.Factory factory = new Newman2.Factory();
			if (updateStrategy == 1) {
				factory.strategy = new DynBacktrackStrategy();
			} else if (updateStrategy == 2) {
				factory.strategy = new DynStonePitStrategy();
			} else if (updateStrategy == 3) {
				factory.strategy = new BreakUpStrategy();
			} else if (updateStrategy == 4) {
				// bounded neighborhood
				if (traversalStrategy == 1) {
					// multi-source BFS
					factory.strategy = new BoundedNeighborhoodStrategy(new MultiSourceBFS(), traversalArg);
				} else if (traversalStrategy == 2) {
					// multi-source priority search
					if (priority == 1) {
						factory.strategy = new BoundedNeighborhoodStrategy(new MultiSourcePrioritySearch(new DegreePriority(weightA)), traversalArg);
					} else if (priority == 2) {
						// factory.strategy = new TraversalStrategy(new
						// MultiSourcePrioritySearch(new DistancePriority(null,
						// null)), traversalSize);
					} else if (priority == 3) {
						factory.strategy = new BoundedNeighborhoodStrategy(new MultiSourcePrioritySearch(new DistanceDegreePriority(weightA, weightB)),
								traversalArg);
					} else if (priority == 4) {
						factory.strategy = new BoundedNeighborhoodStrategy(new MultiSourcePrioritySearch(new DistanceBeforeDegreePriority()), traversalArg);
					} else if (priority == 5) {
						factory.strategy = new BoundedNeighborhoodStrategy(new MultiSourcePrioritySearch(new StackPriority()), traversalArg);
					}
				}
			} else if (updateStrategy == 5) {
				// neighborhood
				if (traversalStrategy == 1) {
					// multi-source BFS
					factory.strategy = new NeighborhoodStrategy(new MultiSourceBFS(), traversalArg);
				}
			}
			this.selectedAlgos.add(factory);

		}
	}

	public void useStaticBlondel(boolean flag, int nodeOrderStrategy, int stoppingCriterion) {
		if (flag) {
			StaticBlondel.Factory factory = new StaticBlondel.Factory();
			factory.nodeOrderStrategy = Blondel.NodeOrderStrategy.values()[nodeOrderStrategy - 1];
			factory.stoppingCriterion = Blondel.StoppingCriterion.values()[stoppingCriterion - 1];
			this.selectedAlgos.add(factory);
		}
	}

	public void useBlondel(boolean flag, int nodeOrderStrategy, int updatePolicy, int stoppingCriterion, 
			int updateStrategy, int traversalStrategy, int priority, int traversalArg, double weightA, double weightB) {
		if (flag) {
			Blondel.Factory factory = new Blondel.Factory();
			factory.nodeOrderStrategy = Blondel.NodeOrderStrategy.values()[nodeOrderStrategy - 1];
			factory.stoppingCriterion = Blondel.StoppingCriterion.values()[stoppingCriterion - 1];
			factory.updatePolicy = Blondel.UpdatePolicy.values()[updatePolicy - 1];
			
			// update strategy
			if (updateStrategy == 1) {
				factory.subsetStrategy = new BreakUpStrategy();
			} else if (updateStrategy == 2) {
				// bounded neighborhood
				if (traversalStrategy == 1) {
					// multi-source BFS
					factory.subsetStrategy = new BoundedNeighborhoodStrategy(new MultiSourceBFS(), traversalArg);
				} else if (traversalStrategy == 2) {
					// multi-source priority search
					if (priority == 1) {
						factory.subsetStrategy = new BoundedNeighborhoodStrategy(new MultiSourcePrioritySearch(new DegreePriority(weightA)), traversalArg);
					} else if (priority == 2) {
						// factory.subsetStrategy = new TraversalStrategy(new
						// MultiSourcePrioritySearch(new DistancePriority(null,
						// null)), traversalSize);
					} else if (priority == 3) {
						factory.subsetStrategy = new BoundedNeighborhoodStrategy(new MultiSourcePrioritySearch(new DistanceDegreePriority(weightA, weightB)),
								traversalArg);
					} else if (priority == 4) {
						factory.subsetStrategy = new BoundedNeighborhoodStrategy(new MultiSourcePrioritySearch(new DistanceBeforeDegreePriority()), traversalArg);
					} else if (priority == 5) {
						factory.subsetStrategy = new BoundedNeighborhoodStrategy(new MultiSourcePrioritySearch(new StackPriority()), traversalArg);
					}
				}
			} else if (updateStrategy == 3) {
				// neighborhood
				if (traversalStrategy == 1) {
					// multi-source BFS
					factory.subsetStrategy = new NeighborhoodStrategy(new MultiSourceBFS(), traversalArg);
				}
			}
			
			this.selectedAlgos.add(factory);
		}
	}

	public void usePartialILP(boolean flag, int solver, int objectiveFunction, int variant, int updateStrategy, 
			int traversalStrategy, int priority, int traversalSize, double weightA, double weightB) {
		if (flag) {
			PartialILP.Factory factory = new PartialILP.Factory();
			factory.solver = PartialILP.Solver.values()[solver - 1];
			factory.obj = PartialILP.Objective.values()[objectiveFunction - 1];
			System.out.println(factory.obj);
			if (updateStrategy == 1) {
				factory.strategy = new BreakUpStrategy();
			} else if (updateStrategy == 2) {
				factory.strategy = new BoundedNeighborhoodStrategy(new MultiSourceBFS(), traversalSize);
			}
			factory.variant = PartialILP.Variant.values()[variant - 1];
			this.selectedAlgos.add(factory);
		}
	}

	public void useRandomly(boolean flag) {
		if (flag) {
			this.selectedAlgos.add(new Randomly.Factory());
		}
	}

	public void useReference(boolean flag) {
		if (flag) {
			this.selectedAlgos.add(new Reference.Factory());
		}
	}

	public void useSingleton(boolean flag) {
		if (flag) {
			this.selectedAlgos.add(new Singleton.Factory());
		}
	}

	public void useOneClustering(boolean flag) {
		if (flag) {
			this.selectedAlgos.add(new OneClustering.Factory());
		}
	}

	public void useEOO(boolean flag, int strategy, int qi, int maxOps, boolean allowMerge, boolean allowShift, boolean allowSplit) {
		if (flag) {
			EOO.Factory factory = new EOO.Factory();
			// strategy
			factory.strategy = EOO.Strategy.values()[strategy - 1];
			// quality index
			if (qi == 1) {
				factory.quality = new Coverage.Factory();
			} else if (qi == 2) {
				factory.quality = new Modularity.Factory();
			}
			// rest
			factory.maxOps = maxOps;

			factory.allowMerge = allowMerge;
			factory.allowShift = allowShift;
			factory.allowSplit = allowSplit;

			this.selectedAlgos.add(factory);
		}
	}

	/* DMs */

	public void useRand(boolean flag, boolean flag2) {
		if (flag) {
			this.selectedDMs.add(new Rand.Factory());
		}
		if (flag2) {
			this.selectedDMs.add(new RandGraphExt.Factory());
		}
	}

	public void useJaccard(boolean flag, boolean flag2) {
		if (flag) {
			this.selectedDMs.add(new Jaccard.Factory());
		}
		if (flag2) {
			this.selectedDMs.add(new JaccardGraphExt.Factory());
		}
	}

	public void useFowlkesMallows(boolean flag, boolean flag2) {
		if (flag) {
			selectedDMs.add(new FowlkesMallows.Factory());
		}
		if (flag2) {
			selectedDMs.add(new FowlkesMallowsGraphExt.Factory());
		}
	}

	public void useFredJain(boolean flag, boolean flag2) {
		if (flag) {
			selectedDMs.add(new FredJain.Factory());
		}
		if (flag2) {
			selectedDMs.add(new FredJainGraphExt.Factory());
		}
	}

	public void useMaximumMatch(boolean flag, boolean flag2) {
		if (flag) {
			selectedDMs.add(new MaximumMatch.Factory());
		}
		if (flag2) {
			selectedDMs.add(new MaximumMatchGraphExt.Factory());
		}
	}

	/* QIs */

	public void useModularity(boolean flag) {
		if (flag) {
			selectedQIs.add(new Modularity.Factory());
		}
	}

	public void useCoverage(boolean flag) {
		if (flag) {
			selectedQIs.add(new Coverage.Factory());
		}
	}

	public void useSignificanceCoverage(boolean flag) {
		// this is equal to modularity
		if (flag) {
			selectedQIs.add(new Significance.Factory(Significance.Indizes.coverage, OperationIndex.Operations.sub));
		}
	}

	public void usePerformance(boolean flag) {
		if (flag) {
			selectedQIs.add(new PerformanceConventional.Factory());
		}
	}

	public void useSignificancePerformance(boolean flag) {
		if (flag) {
			selectedQIs.add(new Significance.Factory(Significance.Indizes.performance, OperationIndex.Operations.sub));
		}
	}

	public void useInterClusterConductance(boolean flag) {
		if (flag) {
			selectedQIs.add(new InterClusterConductance.Factory());
		}
	}

	public void useIntraClusterConductance(boolean flag) {
		if (flag) {
			selectedQIs.add(new IntraClusterConductance.Factory(ConductanceStrategies.Markovian));
		}
	}

	/* graph */

	public void setGraph(String path) {
		this.graphPath = path;
	}

	/* run */

	public Results run() {
		this.eval = new Evaluation();
		eval.importGraph(this.graphPath);
		if (this.graphPath.endsWith(".graphj")) {
			eval.importClusteringJournal(this.graphPath);
		}
		eval.registerAlgorithms(this.selectedAlgos);
		eval.registerQualityIndices(this.selectedQIs);
		eval.registerDistanceMeasures(this.selectedDMs);
		eval.run();
		Results results = eval.getResults();
		return results;
	}

	public Evaluation getEvaluation() {
		return this.eval;
	}

	/* */

}
