package de.uka.algo.dynclustering.eval;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;

import de.uka.algo.clustering.comparison.ComparatorFactory;
import de.uka.algo.clustering.comparison.graphbased.counting.FowlkesMallowsGraphExt;
import de.uka.algo.clustering.comparison.graphbased.counting.JaccardGraphExt;
import de.uka.algo.clustering.comparison.graphbased.counting.RandGraphExt;
import de.uka.algo.clustering.comparison.graphbased.informationTheory.FredJainGraphExt;
import de.uka.algo.clustering.comparison.graphbased.overlaps.MaximumMatchGraphExt;
import de.uka.algo.clustering.comparison.nodebased.counting.FowlkesMallows;
import de.uka.algo.clustering.comparison.nodebased.counting.Jaccard;
import de.uka.algo.clustering.comparison.nodebased.counting.Rand;
import de.uka.algo.clustering.comparison.nodebased.informationTheory.FredJain;
import de.uka.algo.clustering.comparison.nodebased.overlaps.MaximumMatch;
import de.uka.algo.clustering.index.Coverage;
import de.uka.algo.clustering.index.IndexFactory;
import de.uka.algo.clustering.index.InterClusterConductance;
import de.uka.algo.clustering.index.IntraClusterConductance;
import de.uka.algo.clustering.index.Modularity;
import de.uka.algo.clustering.index.OperationIndex;
import de.uka.algo.clustering.index.PerformanceConventional;
import de.uka.algo.clustering.index.Significance;
import de.uka.algo.dynclustering.algos.Blondel;
import de.uka.algo.dynclustering.algos.EOO;
import de.uka.algo.dynclustering.algos.Newman;
import de.uka.algo.dynclustering.algos.Newman2;
import de.uka.algo.dynclustering.algos.OneClustering;
import de.uka.algo.dynclustering.algos.PartialILP;
import de.uka.algo.dynclustering.algos.Randomly;
import de.uka.algo.dynclustering.algos.Reference;
import de.uka.algo.dynclustering.algos.Singleton;
import de.uka.algo.dynclustering.algos.StaticBlondel;
import de.uka.algo.dynclustering.algos.StaticNewman;
import de.uka.algo.dynclustering.algos.StaticNewman2;
import de.uka.algo.dynclustering.algos.base.AlgorithmFactory;
import de.uka.algo.dynclustering.algos.newman.DynBacktrackStrategy;
import de.uka.algo.dynclustering.algos.newman.DynStonePitStrategy;
import de.uka.algo.dynclustering.update.BoundedNeighborhoodStrategy;
import de.uka.algo.dynclustering.update.BreakUpStrategy;
import de.uka.algo.dynclustering.update.NeighborhoodStrategy;
import de.uka.algo.graphs.cut.ConductanceCutter.ConductanceStrategies;
import de.uka.algo.traversal.implementations.MultiSourceBFS;
import de.uka.algo.traversal.implementations.MultiSourcePrioritySearch;
import de.uka.algo.traversal.priorities.DegreePriority;
import de.uka.algo.traversal.priorities.DistanceDegreePriority;
import de.uka.algo.traversal.priorities.DistanceBeforeDegreePriority;
import de.uka.algo.traversal.priorities.TargetDistancePriority;
import de.uka.algo.traversal.priorities.StackPriority;

public class FakeEvaluationFrontend implements Serializable {

	
	/**
	 * 
	 */
	private static final long serialVersionUID = 7457791477802674924L;

	public List<HashMap<String, ArrayList<Object>>> algoStoreList = new LinkedList<HashMap<String, ArrayList<Object>>>();
	
	public HashMap<String, ArrayList<Object>> algoStore = new HashMap<String, ArrayList<Object>>();
	public HashMap<String, ArrayList<Object>> evalStore = new HashMap<String, ArrayList<Object>>();
	/* algos */

	public void useStaticNewman(boolean flag) {
		ArrayList<Object> list = new ArrayList<Object>();
		list.add(flag);
		algoStore.put("StaticNewman", list);
	}
	
	public void useStaticNewman2(boolean flag) {
		ArrayList<Object> list = new ArrayList<Object>();
		list.add(flag);
		algoStore.put("StaticNewman2", list);
	}

	public void useNewman(boolean flag, int updateStrategy, int traversalStrategy, int priority,
			int traversalArg, double weightA, double weightB) {
		ArrayList<Object> list = new ArrayList<Object>();
		list.add(flag);
		list.add(updateStrategy);
		list.add(traversalStrategy);
		list.add(priority);
		list.add(traversalArg);
		list.add(weightA);
		list.add(weightB);

		algoStore.put("Newman", list);
	}
	
	
	public void useNewman2(boolean flag, int updateStrategy, int traversalStrategy, int priority,
			int traversalArg, double weightA, double weightB) {
		ArrayList<Object> list = new ArrayList<Object>();
		list.add(flag);
		list.add(updateStrategy);
		list.add(traversalStrategy);
		list.add(priority);
		list.add(traversalArg);
		list.add(weightA);
		list.add(weightB);

		algoStore.put("Newman2", list);
	}

	public void useStaticBlondel(boolean flag, int nodeOrderStrategy, int stoppingCriterion) {
		ArrayList<Object> list = new ArrayList<Object>();
		list.add(flag);
		list.add(nodeOrderStrategy);
		list.add(stoppingCriterion);

		algoStore.put("StaticBlondel", list);
	}
	
	public void useTDStaticBlondel(boolean flag, int nodeOrderStrategy, int stoppingCriterion, double tDStaticBlondelAlpha) {
		ArrayList<Object> list = new ArrayList<Object>();
		list.add(flag);
		list.add(nodeOrderStrategy);
		list.add(stoppingCriterion);
		list.add(tDStaticBlondelAlpha);
		algoStore.put("TDStaticBlondel", list);
	}

	public void useBlondel(boolean flag, int nodeOrderStrategy, int updatePolicy, int stoppingCriterion, 
			int updateStrategy, int traversalStrategy, int priority, int traversalArg, double weightA, double weightB) {
		
		ArrayList<Object> list = new ArrayList<Object>();
		list.add(flag);
		list.add(nodeOrderStrategy);
		list.add(updatePolicy);
		list.add(stoppingCriterion);
		list.add(updateStrategy);
		list.add(traversalStrategy);
		list.add(priority);
		list.add(traversalArg);
		list.add(weightA);
		list.add(weightB);

		algoStore.put("Blondel", list);
	}

	public void usePartialILP(boolean flag, int solver, int objectiveFunction, int variant, int updateStrategy, 
			int traversalStrategy, int priority, int traversalSize, double weightA, double weightB) {
		
		ArrayList<Object> list = new ArrayList<Object>();
		list.add(flag);
		list.add(solver);
		list.add(objectiveFunction);
		list.add(variant);
		list.add(updateStrategy);
		list.add(traversalStrategy);
		list.add(priority);
		list.add(traversalSize);
		list.add(weightA);
		list.add(weightB);

		algoStore.put("PartialILP", list);
	}

	public void useRandomly(boolean flag) {
		ArrayList<Object> list = new ArrayList<Object>();
		list.add(flag);

		algoStore.put("Randomly", list);
	}

	public void useReference(boolean flag) {
		ArrayList<Object> list = new ArrayList<Object>();
		list.add(flag);

		algoStore.put("Reference", list);
	}

	public void useSingleton(boolean flag) {
		ArrayList<Object> list = new ArrayList<Object>();
		list.add(flag);

		algoStore.put("Singleton", list);
	}

	public void useOneClustering(boolean flag) {
		ArrayList<Object> list = new ArrayList<Object>();
		list.add(flag);

		algoStore.put("OneClustering", list);
	}

	public void useEOO(boolean flag, int strategy, int qi, int maxOps, boolean allowMerge, boolean allowShift, boolean allowSplit) {
		ArrayList<Object> list = new ArrayList<Object>();
		list.add(flag);
		list.add(strategy);
		list.add(qi);
		list.add(maxOps);
		list.add(allowMerge);
		list.add(allowShift);
		list.add(allowSplit);
		
		algoStore.put("EOO", list);
	}

	/* DMs */

	public void useRand(boolean flag, boolean flag2) {
		ArrayList<Object> list = new ArrayList<Object>();
		list.add(flag);
		list.add(flag2);
		evalStore.put("Rand", list);
	}

	public void useJaccard(boolean flag, boolean flag2) {
		ArrayList<Object> list = new ArrayList<Object>();
		list.add(flag);
		list.add(flag2);
		evalStore.put("Jaccard", list);
	}

	public void useFowlkesMallows(boolean flag, boolean flag2) {
		ArrayList<Object> list = new ArrayList<Object>();
		list.add(flag);
		list.add(flag2);
		evalStore.put("FowlkesMallows", list);
	}

	public void useFredJain(boolean flag, boolean flag2) {
		ArrayList<Object> list = new ArrayList<Object>();
		list.add(flag);
		list.add(flag2);
		evalStore.put("FredJain", list);
	}

	public void useMaximumMatch(boolean flag, boolean flag2) {
		ArrayList<Object> list = new ArrayList<Object>();
		list.add(flag);
		list.add(flag2);
		evalStore.put("MaximumMatch", list);
	}

	/* QIs */

	public void useModularity(boolean flag) {
		ArrayList<Object> list = new ArrayList<Object>();
		list.add(flag);
		evalStore.put("Modularity", list);
	}

	public void useCoverage(boolean flag) {
		ArrayList<Object> list = new ArrayList<Object>();
		list.add(flag);
		evalStore.put("Coverage", list);
	}

	public void useSignificanceCoverage(boolean flag) {
		// this is equal to modularity
		ArrayList<Object> list = new ArrayList<Object>();
		list.add(flag);
		evalStore.put("SignificanceCoverage", list);
	}

	public void usePerformance(boolean flag) {
		ArrayList<Object> list = new ArrayList<Object>();
		list.add(flag);
		evalStore.put("Performance", list);
	}

	public void useSignificancePerformance(boolean flag) {
		ArrayList<Object> list = new ArrayList<Object>();
		list.add(flag);
		evalStore.put("SignificancePerformance", list);
	}

	public void useInterClusterConductance(boolean flag) {
		ArrayList<Object> list = new ArrayList<Object>();
		list.add(flag);
		evalStore.put("InterClusterConductance", list);
	}

	public void useIntraClusterConductance(boolean flag) {
		ArrayList<Object> list = new ArrayList<Object>();
		list.add(flag);
		evalStore.put("IntraClusterConductance", list);
	}

	/* graph */

	public void setGraph(String path) {
		ArrayList<Object> list = new ArrayList<Object>();
		list.add(path);
		evalStore.put("Graph", list);
	}
	
	public void setTimeStepAggregation(int timeStepAggregation) {
		ArrayList<Object> list = new ArrayList<Object>();
		list.add(timeStepAggregation);
		evalStore.put("TimeStepAggregation", list);
	}

	
	/**
	 * Call if you want to use multiple instances of the same algorithm type.
	 */
	public void nextTurn() {
		this.algoStoreList.add(this.algoStore);
		this.algoStore = new HashMap<String, ArrayList<Object>>();
	}
	
	
}
