package de.uka.algo.dynclustering.graphs;

import de.uka.algo.dynclustering.util.HashSquareMatrix;
import de.uka.algo.generator.util.BidiHashMap;
import de.uka.algo.graphs.GraphInterpretation;
import de.uka.algo.graphs.GraphInterpretationListener;
import de.uka.algo.math.linalg.generic.MatrixAdapter;
import y.base.Edge;
import y.base.Graph;
import y.base.GraphEvent;
import y.base.GraphListener;
import y.base.Node;


/**
 * This class has 2 purposes:
 * - Couple an adjacency matrix of doubles to a graph. The matrix is automatically updated
 * on changes to the graph/ graph interpretation
 * - Maintain a stable mapping from node to index.
 * @author Christian Staudt
 *
 */
public class AdjacencyMatrixCoupler implements GraphListener, GraphInterpretationListener {

	int nodeIdCounter;
	BidiHashMap<Integer,Node> index2Node = new BidiHashMap<Integer,Node>();
	HashSquareMatrix matrix;
	Graph graph;
	GraphInterpretation graphInt;
	
	public AdjacencyMatrixCoupler(Graph graph, GraphInterpretation graphInt) {
		this.graph = graph;
		this.graphInt = graphInt;
		this.matrix = new HashSquareMatrix();
		
		// add as listener
		this.graph.addGraphListener(this);
		this.graphInt.addListener(this);
	}
	
	public void onGraphEvent(GraphEvent ev) {
		byte type = ev.getType();
		if (type == GraphEvent.NODE_CREATION) {
			int index = this.matrix.extend();
			this.index2Node.put(index, (Node) ev.getData());
		}
	}
	
	
	public void newWeightsEvent() {
		// TODO Auto-generated method stub
	}


	public void weightChangeEvent(Edge edge, double oldWeight, double newWeight) {
		// set symmetric weight
		this.matrix.set(this.getIndex(edge.source()), this.getIndex(edge.target()), newWeight);
		this.matrix.set(this.getIndex(edge.target()), this.getIndex(edge.source()), newWeight);
	}
	
	
	public int getIndex(Node v) {
		return this.index2Node.getKey(v);
	}
	
	public Node getNode(int i) {
		return this.index2Node.getValue(i);
	}
	
	public double get(Node u, Node v) {
		return this.matrix.get(this.index2Node.getKey(u), this.index2Node.getKey(v));
	}
	
	public MatrixAdapter getMatrix() {
		return this.matrix;
	}
	





}
