package de.uka.algo.dynclustering.graphs;

import de.uka.algo.graphs.GraphInterpretation;
import y.base.Edge;
import y.base.Graph;
import y.base.GraphEvent;
import y.base.Node;
import y.base.NodeCursor;

/**
 * A SuperGraphCoupler that adds an artificial sink to the supergraph. For all nodes v in the subgraph,
 * there exists an edge {s,v} in the supergraph.
 * @author Christian Staudt
 * @see de.uka.algo.dynclustering.graphs.SuperGraphCoupler
 *
 */
public class ArtificialSinkCoupler extends SuperGraphCoupler {
	
	Node s; // sink

	public ArtificialSinkCoupler(Graph subGraph, GraphInterpretation subGraphInterpretation) {
		super(subGraph, subGraphInterpretation);
		// TODO Auto-generated constructor stub
	}
	
	/**
	 * Create a sink node an connect it to all other nodes.
	 * @param w		weight of each edge {s,v}
	 * @return
	 */
	public Node addSink(Double alpha) {
		if (this.s == null) {
			this.s = this.superGraph.createNode();
			for (NodeCursor nc = this.superGraph.nodes(); nc.ok(); nc.next()) {
				Edge su = this.addEdge(this.s, nc.node());
				// set weight
				if (this.weighted) {
					this.superGraphWeights.setWeight(su, alpha);
				}
			}
			return this.s;
		} else throw new IllegalStateException("Sink already exists");
	}
	
	public Node getSink() {
		return this.s;
	}
	
	public void removeSink() {
		
	}
	
	public void onGraphEvent(GraphEvent ev) {
		// call super to write through changes
		super.onGraphEvent(ev);
		// additional actions
		if (ev.equals(GraphEvent.NODE_CREATION)) {
			// create edge to sink
			Node u_ = (Node) ev.getData();
			this.addEdge(this.s, this.super2subNodes.getKey(u_));
		} else if (ev.equals(GraphEvent.PRE_NODE_REMOVAL)) {
			// do nothing, node and edges have already been created
		}
	}

}
