package de.uka.algo.dynclustering.graphs;

import java.util.LinkedList;

import y.base.Node;

import de.uka.algo.graphs.GraphInterpretation;



/**
 *  DynamicGraph represents a dynamic graph as a sequence of 
 *  retrievable changes to a single y.base.Graph object.
 * 
 * @author christian.staudt @ ira.uka.de
 *
 */
public abstract class DynamicGraph extends y.base.Graph {
	
	protected LinkedList<DynamicGraphListener> listeners = new LinkedList<DynamicGraphListener>();
	protected LinkedList<DynamicGraphListener> waitingListeners = new LinkedList<DynamicGraphListener>();
	
	public enum GraphProperty {n, m, ClusterEvent}; // properties for evaluation results
	

	
	protected int t; // current time step
	
	
	public abstract boolean nextChange();
	
	public abstract boolean hasNextChange();
	
	/***
	 * @return	current time step
	 */
	public int t() {
		return this.t;
	}
	
	
	/**
	 * Notify all listeners of a new time step.
	 */
	public void nextTimeStep() {
		this.t += 1;
		for (DynamicGraphListener dgListener : this.listeners) {
			dgListener.onTimeStepEvent();
		}
	}
	
	
	
	/**
	 * Register a DynamicGraphListener
	 * @param dgListener
	 */
	public void addDynamicGraphListener(DynamicGraphListener dgListener) {
		this.listeners.add(dgListener);
	}
	
	/**
	 * Unregister a DynamicGraphListener
	 * @param dgListener
	 */
	public void removeDynamicGraphListener(DynamicGraphListener dgListener) {
		this.listeners.remove(dgListener);
	}
	

	public abstract GraphInterpretation getInterpretation();
	
	

	
}
