package de.uka.algo.dynclustering.graphs;

import java.io.BufferedReader;
import java.io.IOException;
import java.util.HashMap;

import de.uka.algo.generator.standalone.graph.journaling.GraphOperation;
import de.uka.algo.generator.util.Pair;
import de.uka.algo.graphs.GraphInterpretation;
import de.uka.algo.graphs.WeightedGraph;

import y.base.Edge;
import y.base.Node;

public class EmailGraph extends DynamicGraph {
	
	private WeightedGraph wGraph;
	
	private BufferedReader reader;
	private HashMap<String,Node> nodeMap = new HashMap<String,Node>();
	private HashMap<Pair<String>,Edge> edgeMap = new HashMap<Pair<String>,Edge>();
	private String nextLine;
	private int timeStepAggregation;
	private int aggregationCounter; 
	
	public EmailGraph(BufferedReader reader, int timeStepAggregation) {
		this.reader = reader;
		this.wGraph = new WeightedGraph(this, this.createEdgeMap());
		this.timeStepAggregation = timeStepAggregation;
		this.aggregationCounter = 0;
	}

	@Override
	public boolean hasNextChange() {
		try {
			return (this.nextLine = this.reader.readLine()) != null;
		} catch (IOException e) {
			e.printStackTrace();
			return false;
		}
	}

	@Override
	public boolean nextChange() {
		String[] split = this.nextLine.split(",");
		
		if (split[0].equals("createNode")) {
			String label = split[1];
			Node v = this.createNode();
			this.nodeMap.put(label, v);
		} else if (split[0].equals("deleteNode")) {
			String label = split[1];
			Node v = this.nodeMap.get(label);
			this.removeNode(v);
			this.nodeMap.remove(split[1]);
		} else if (split[0].equals("createEdge")) {
			String uLabel = split[1];
			String vLabel = split[2];
			Node u = this.nodeMap.get(uLabel);
			Node v = this.nodeMap.get(vLabel);
			Edge uv = this.createEdge(u, v);
			this.edgeMap.put(new Pair<String>(uLabel,vLabel), uv);
		} else if (split[0].equals("deleteEdge")) {
			String uLabel = split[1];
			String vLabel = split[2];
			Pair<String> uvLabel = new Pair<String>(uLabel, vLabel);
			Edge uv = this.edgeMap.get(uvLabel);
			this.removeEdge(uv);
			this.edgeMap.remove(uvLabel);
		} else if (split[0].equals("changeWeight")) {
			Edge uv = this.edgeMap.get(new Pair<String>(split[1],split[2]));
			this.wGraph.setWeight(uv, Double.parseDouble(split[3]));
		} else if (split[0].equals("timeStep")) {
			this.aggregationCounter++;
			if (this.aggregationCounter == this.timeStepAggregation) {
				this.aggregationCounter = 0;
				this.nextTimeStep();
				return true; // to signal next time step
			}
		}
		return false;
	}

	@Override
	public GraphInterpretation getInterpretation() {
		return this.wGraph;
	}
	
	public WeightedGraph getWeightedGraph() {
		return this.wGraph;
	}


}
