package de.uka.algo.dynclustering.graphs;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

import de.uka.algo.generator.standalone.graph.journaling.GraphJournal;
import de.uka.algo.generator.standalone.graph.journaling.GraphOperation;
import de.uka.algo.generator.util.Pair;
import de.uka.algo.graphs.GraphInterpretation;
import de.uka.algo.graphs.SimpleGraph;
import y.base.Node;
import y.base.Edge;

public class GeneratedGraph extends DynamicGraph {
	
	protected SimpleGraph sGraph;
	
	protected GraphJournal journal;
	protected ArrayList<Node> nodes = new ArrayList<Node>();
	protected HashMap<Pair<Integer>,Edge> edgeMap = new HashMap<Pair<Integer>,Edge>();
	protected Iterator<GraphOperation> jIter;
	
	protected HashMap<Node,Integer> node2ClusterID = new HashMap<Node,Integer>();
	
	public GeneratedGraph(GraphJournal journal) {
		this.sGraph = new SimpleGraph(this);
		this.journal = journal;
		this.jIter = this.journal.iterator();
		this.nodes.add(null); // first node at index 1
		this.t = 0;
 	}
	
	
	public boolean hasNextChange() {
		return this.jIter.hasNext();
	}
	
	/**
	 * Apply the next change to this graph
	 */
	public boolean nextChange() {
		
		GraphOperation op = this.jIter.next();
		if (op.type == GraphOperation.OpType.CreateNode) {
			this._createNode(op.arg0, op.arg1);
		} else if (op.type == GraphOperation.OpType.RemoveNode) {
			this._removeNode(op.arg0);
		} else if (op.type == GraphOperation.OpType.CreateEdge) {
			this._createEdge(op.arg0, op.arg1);
		} else if (op.type == GraphOperation.OpType.RemoveEdge) {
			this._removeEdge(op.arg0, op.arg1);
		} else if (op.type == GraphOperation.OpType.NextStep) {
			this.nextTimeStep();
			return true;
		} else {
			// these changes are not forwarded to the listeners
			if (op.type == GraphOperation.OpType.SetRefCluster) {
				int nodeID = op.arg0;
				int clusterID = op.arg1;
				Node u = this.nodes.get(nodeID);
				this.node2ClusterID.put(u, clusterID);
			}
			return this.nextChange();
		}
		return false;
		
	}


	/**
	 * Create node and add it to the node array.
	 */
	private void _createNode(int arg0, int arg1) {
		Node v = this.createNode();
		this.nodes.add(v);
		this.node2ClusterID.put(v, arg1);
	}
	
	/**
	 * Remove node from graph.
	 * @param id	node id
	 */
	private void _removeNode(int id) {
		Node v = this.nodes.get(id);
		this.nodes.set(id, null);
		this.removeNode(v);
	}
	
	private void _createEdge(int i, int j) {
		Node u = this.nodes.get(i);
		Node v = this.nodes.get(j);
		Edge e = this.createEdge(u,v);
		Pair<Integer> ij = new Pair<Integer>(i,j);
		this.edgeMap.put(ij, e);
		
	}
	
	private void _removeEdge(int i, int j) {
		Pair<Integer> ij = new Pair<Integer>(i,j);
		Edge e = this.edgeMap.get(ij);
		this.removeEdge(e);
		this.edgeMap.remove(ij);
	}


	@Override
	public GraphInterpretation getInterpretation() {
		return this.sGraph;
	}
	
	
	/**
	 * Get the reference clustering.
	 * @param u		a node
	 * @return		ID of the reference cluster the node is assigned to
	 */
	public int getClusterID(Node u) {
		return this.node2ClusterID.get(u);
	}


	
	
	
}
