package de.uka.algo.dynclustering.test;

import lpsolve.LpSolve;
import lpsolve.LpSolveException;


public class LPTest {

	public static void main(String[] args) {
		try {
			// set up LP with 2 variables (default:
			LpSolve lps = LpSolve.makeLp(0, 2);

			// set lower bound of first variable to 1.0 (default:0.0)
			lps.setLowbo(1, 1.0);
			// restrict first variable to integer values
			lps.setInt(1, true);
			// restrict first variable to binary values
			// lps.setBinary(1, true);

			// set upper bound of second variable to 2.0 (default: unbound)
			lps.setUpbo(2, 2.5);

			// set optimization to "maximize"
			lps.setMaxim();

			// set objective function to -x_1+2*x_2 (leading zero is
			// ignored, but necessary)
			double[] objFn = { 0.0, -1.0, 2.0 };
			lps.setObjFn(objFn);

			// add constraint x_1-x_2 >=0.0 (leading zero is ignored, but
			// necessary)
			double[] constraint = { 0.0, 1.0, -1.0 };
			lps.addConstraint(constraint, LpSolve.GE, 0.0);

			// verbosity level (standard=4)
			lps.setVerbose(4);

			// solve lp
			int status = lps.solve();

			// if solved successfully, print solution (no leading zeros?)
			if (status == 0) {
				double[] result = lps.getPtrVariables().clone();
				System.out.print("SOLUTION: ");
				for (int i = 0; i < result.length; i++) {
					System.out.print("x_" + (i + 1) + "=" + result[i] + "\t");
				}
				System.out.println();
			} else {
				System.out.println("LP ERRORCODE " + status);
			}
		} catch (LpSolveException e) {
			e.printStackTrace();
		}
	}
}