package de.uka.algo.dynclustering.update;

import java.util.HashSet;
import java.util.Map;

import de.uka.algo.clustering.Cluster;
import de.uka.algo.dynclustering.util.unionfind.UnionNodeForest;
import y.base.Edge;
import y.base.Node;

public class BacktrackStrategy extends UpdateStrategy {

	protected UnionNodeForest<Node> unf = new UnionNodeForest<Node>();
	protected HashSet<Cluster> untouchedClusters = new HashSet<Cluster>();
	
	private static final String NAME = "BacktrackStrategy";


	public void onEdgeCreation(Edge uv) {
		Node u = uv.source();
		Node v = uv.target();

		if (this.unf.sameSet(u, v)) {
			// intra-cluster edge
			this.unf.separate(u, v);
		} else {
			// inter-cluster edge
			this.unf.isolate(u);
			this.unf.isolate(v);
		}
	}

	public void onEdgeRemoval(Edge uv) {
		Node u = uv.source();
		Node v = uv.target();

		if (this.unf.sameSet(u, v)) {
			// intra-cluster edge
			this.unf.isolate(u);
			this.unf.isolate(v);
		} else {
			// inter-cluster edge
			// do nothing, new merge might be possible
		}
	}

	public void onNodeCreation(Node v) {
		this.unf.add(v);
	}

	public void onNodeRemoval(Node v) {
		this.unf.remove(v);
	}

	public String id() {
		return "Backtrack";
	}

	public void onMerge(Cluster c1, Cluster c2) {
		if (this.listening) {
			this.unf.union(c1.aMember(), c2.aMember());
		}
	}

	public void onSplit(Cluster a, Cluster b) {
		// TODO Auto-generated method stub

	}

	public void onWeightChange(Edge uv, double oldW, double newW) {
		Node u = uv.source();
		Node v = uv.target();
		
		if (newW > oldW) {
			// = edge creation
			if (this.unf.sameSet(u, v)) {
				// intra-cluster edge
				this.unf.separate(u, v);
			} else {
				// inter-cluster edge
				this.unf.isolate(u);
				this.unf.isolate(v);
			}
		} else {
			// = edge removal
			if (this.unf.sameSet(u, v)) {
				// intra-cluster edge
				this.unf.isolate(u);
				this.unf.isolate(v);
			} else {
				// inter-cluster edge
				// do nothing, new merge might be possible
			}
		}
		
	}
	
	
	public Map<Node,Node> getPartitionMap() {
		return this.unf.getPartitionMap();
	}
	

}
