package de.uka.algo.dynclustering.update;

import java.util.HashSet;
import java.util.Set;

import de.uka.algo.clustering.Cluster;
import de.uka.algo.traversal.MultiSourceTraversal;
import y.base.Edge;
import y.base.Node;

public class BoundedNeighborhoodStrategy extends SubsetStrategy {

	protected MultiSourceTraversal traversal;
	protected int k;
	
	private static final String NAME = "BoundedNeighborhood";
	
	
	/**
	 * 
	 * @param traversal		implementing multi-source graph traversal
	 * @param k				max. number of nodes that will be freed
	 */
	public BoundedNeighborhoodStrategy(MultiSourceTraversal traversal, int k) {
		super();
		this.traversal = traversal;
		this.k = k;
	}

	public void onEdgeCreation(Edge uv) {
		this.markAlong(uv);
	}

	public void onEdgeRemoval(Edge uv) {
		this.markAlong(uv);
	}

	public void onNodeCreation(Node u) {
		// ignored

	}

	public void onNodeRemoval(Node u) {
		if (this.nodes.contains(u)) {
			this.nodes.remove(u);
		}
	}
	
	
	protected void markAlong(Edge uv) {
		
		this.traversal.reset();
		
		Set<Node> S = new HashSet<Node>();
		S.add(uv.source());
		S.add(uv.target());
		
		this.dynGraph.t();
		for (traversal.from(S); traversal.ok() && (traversal.count() < this.k); traversal.next()) {
			if (!this.nodes.contains(traversal.node())) {
				this.nodes.add(traversal.node());
			} 
		}
		
		//DEBUG
//		System.out.println("marked nodes in this run: " + traversal.count());
//		System.out.println("marked nodes: " + this.nodes.size());
		// DEBUG
		
	}

	@Override
	public String id() {
		return NAME + "(" + this.traversal.toString() + "," + this.k + ")";
	}


	public void onMerge(Cluster c1, Cluster c2) {
		// ignored
		
	}

	public void onSplit(Cluster a, Cluster b) {
		// ignored
		
	}

	public void onWeightChange(Edge uv, double oldW, double newW) {
		this.markAlong(uv);
	}

}
