package de.uka.algo.dynclustering.update;

import de.uka.algo.clustering.Cluster;
import de.uka.algo.clustering.Clustering;
import de.uka.algo.graphs.GraphInterpretation;
import y.base.Edge;
import y.base.Node;
import y.base.NodeCursor;

public class BreakUpStrategy extends SubsetStrategy {

	private static final String NAME = "BreakUp";

	public void onEdgeCreation(Edge uv) {
		this.markClusters(uv);
	}

	public void onEdgeRemoval(Edge uv) {
		this.markClusters(uv);
	}

	public void onNodeCreation(Node u) {
		// ignored
	}

	public void onNodeRemoval(Node u) {
		if (this.nodes.contains(u)) {
			this.nodes.remove(u);
		}
	}


	protected void markClusters(Edge uv) {
		Node u = uv.source();
		Node v = uv.target();
		Clustering zeta = this.algo.getClustering();
		Cluster C0 = zeta.getCluster(u);
		Cluster C1 = zeta.getCluster(v);
		
		if (C0 != null) {
			for (NodeCursor nc = C0.members(); nc.ok(); nc.next()) {
				Node node = nc.node();
				this.nodes.add(node);
			}
		}
		if (C1 != null) {
			for (NodeCursor nc = C1.members(); nc.ok(); nc.next()) {
				this.nodes.add(nc.node());
			}
		}
	}

	public String id() {
		return NAME;
	}

	public void onMerge(Cluster c1, Cluster c2) {
		// ignored
	}

	public void onSplit(Cluster a, Cluster b) {
		// ignored
	}

	public void onWeightChange(Edge uv, double oldW, double newW) {
		this.markClusters(uv);
	}
}
