package de.uka.algo.dynclustering.update;

import java.util.HashSet;
import java.util.Set;

import de.uka.algo.clustering.Cluster;
import de.uka.algo.traversal.MultiSourceTraversal;
import y.base.Edge;
import y.base.Node;

public class NeighborhoodStrategy extends SubsetStrategy {

	protected MultiSourceTraversal traversal;
	protected int d;
	
	private static final String NAME = "Neighborhood";
	
	
	/**
	 * 
	 * @param traversal		implementing multi-source graph traversal
	 * @param d				max. distance
	 */
	public NeighborhoodStrategy(MultiSourceTraversal traversal, int d) {
		super();
		this.traversal = traversal;
		this.d = d;
	}

	public void onEdgeCreation(Edge uv) {
		this.markAlong(uv);
	}

	public void onEdgeRemoval(Edge uv) {
		this.markAlong(uv);
	}

	public void onNodeCreation(Node u) {
		// TODO Auto-generated method stub

	}

	public void onNodeRemoval(Node u) {
		if (this.nodes.contains(u)) {
			this.nodes.remove(u);
		}
	}
	
	
	
	protected void markAlong(Edge uv) {
		this.traversal.reset();
		
		Set<Node> S = new HashSet<Node>();
		S.add(uv.source());
		S.add(uv.target());
		
		for (traversal.from(S); traversal.ok() && (traversal.distance() <= this.d); traversal.next()) {
			if (!(this.nodes.contains(traversal.node()))) {
				this.nodes.add(traversal.node());
			}
		}
	}
	

	@Override
	public String id() {
		return NAME + "(" + this.traversal.toString() + "," + this.d + ")";
	}


	public void onMerge(Cluster c1, Cluster c2) {
		// TODO Auto-generated method stub
		
	}

	public void onSplit(Cluster a, Cluster b) {
		// TODO Auto-generated method stub
		
	}

	public void onWeightChange(Edge uv, double oldW, double newW) {
		this.markAlong(uv);
	}

}
