package de.uka.algo.dynclustering.util;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

public class CSVUtil {
	
	String sep; // separator
	ArrayList<String> buffer; // each element corresponds to a line in the file
	
	public CSVUtil() {
		this.buffer = new ArrayList<String>();
		this.sep = ","; // comma is the standard separator
	}
	
	public void setSeparator(String s) {
		this.sep = s;
	}
	
	public String getSeparator() {
		return this.sep;
	}
	
	/**
	 * Store a table of (i,value) where value = list[i]
	 * in the buffer.
	 * @param <T>	value type
	 * @param list	
	 */
	public <T> void enumerate(List<T> list) {
		int i = 0;
		for (T val : list) {
			this.buffer.add(i + this.sep + val);
		}
	}
	
	/**
	 * Add a collection of lines to the buffer.
	 * @param lines
	 */
	public void take(Collection<String> lines) {
		this.buffer.addAll(lines);
	}
	
	public void write(String path) {
		BufferedWriter writer = null;
		try {
			 writer = new BufferedWriter(new FileWriter(path));
			 
			 for (String line : this.buffer) {
				 writer.write(line);
				 writer.newLine();
			 }
			 
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			try {
				writer.flush();
				writer.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
			
		}
	}
	
	
	/**
	 * Reset the string buffer.
	 */
	public void reset() {
		this.buffer.clear();
	}

}
