package de.uka.algo.dynclustering.util;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;


/**
 * A  timer for runtime measurements.
 * @author Christian Staudt
 *
 */
public class RunTimer implements Serializable {

	long startTime;
	long elapsed;
	HashMap<Object,Long> saved;
	public enum TimeUnit {NanoSeconds, MicroSeconds, MilliSeconds, Seconds, Minutes};
	
	public RunTimer() {
		this.elapsed = 0;
		this.saved = new HashMap<Object,Long>();
	}
	
	/**
	 * Starts the timer and continues adding to the 
	 * elapsed time.
	 */
	public void start() {
		startTime = System.nanoTime();
	}
	
	
	/**
	 * Halts the timer and adds the time elapsed in this run
	 * to the total elapsed time. 
	 * @return	 total elapsed time
	 */
	public long stop() {
		long t = System.nanoTime() - this.startTime;
		this.elapsed += t;
		return this.elapsed;
	}
	
	
	/**
	 * Resets the total elapsed time to zero.
	 */
	public void reset() {
		this.elapsed = 0;
	}
	
	
	/**
	 * 
	 * @return	elapsed time in nanoseconds
	 */
	public long read() {
		return this.elapsed;
	}
	
	/**
	 * Return elapsed time measured by
	 * @param unit	time unit
	 * @return	elapsed time
	 */
	public double read(TimeUnit unit) {
		switch (unit) {
			case NanoSeconds:
				return (double) this.elapsed;
			case MicroSeconds:
				return (double) this.elapsed / Math.pow(10, 3);
			case MilliSeconds:
				return (double) this.elapsed / Math.pow(10, 6);
			case Seconds:
				return (double) this.elapsed / Math.pow(10, 9);
			case Minutes:
				return (double) this.elapsed / (Math.pow(10, 9) * 60);
		}
		return elapsed;
				
	}
	
	/**
	 * Save elapsed time.
	 */
	public void save(Object key) {
		this.saved.put(key, this.elapsed);
	}
	
	
}
