package de.uka.algo.dynclustering.util.unionfind;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

/**
 * Generic union find data structure using a hash map for
 * the parent pointers.
 * @author Christian Staudt
 *
 * @param <T>
 */
public class HashUnionFind<T> implements UnionFindImplementation<T> {
	
	protected int setCount;
	
	/**
	 * A mapping from an element to its parent. A root element is
	 * it's own parent.
	 */
	protected Map<T,T> parentMap; 

	public HashUnionFind() {
		// TODO: path compression
		this.parentMap = new HashMap<T,T>();
		this.setCount = 0;
	}
	
	/**
	 * Generic factory method - use it with type inference.
	 * @param <T>
	 * @param pathCompression
	 * 		switch path compression on or off
	 * @return
	 */
	public static <T> UnionFind<T> getInstance(boolean pathCompression) {
		return new UnionFind<T>(new HashUnionFind<T>(), pathCompression);
	}

	public T join(T x, T y) {
		// simply append in this order, no balancing strategy
		this.append(x, y);
		this.setCount--;
		return x; // as parent
	}
	
	public void append(T parent, T child) {
		this.parentMap.put(child, parent);
	}

	public T getParent(T x) {
		return this.parentMap.get(x);
	}

	public boolean isRoot(T anElement) {
		return this.parentMap.get(anElement).equals(anElement);
	}

	public void add(T x) {
		// add as root
		this.makeRoot(x);
	}

	public void makeRoot(T anElement) {
		this.parentMap.put(anElement, anElement);
		this.setCount++;
	}

	public int getSetCount() {
		return this.setCount;
	}

	public Set<T> getRoots() {
		// add all roots to a hash set
		Set<T> representatives = new HashSet<T>();
		for (T element : this.parentMap.keySet()) {
			if (this.isRoot(element)) {
				representatives.add(element);
			}
		}
		return representatives;
	}

	public boolean contains(T anElement) {
		return this.parentMap.containsKey(anElement);
	}

	public Set<T> getElements() {
		return this.parentMap.keySet();
	}

	public void remove(T anElement) {
		Set<T> children = new HashSet<T>();
		for (T e : this.parentMap.keySet()) {
			
		}
	}



}
