package de.uka.algo.dynclustering.util.unionfind;

import java.util.HashMap;
import java.util.Stack;

/**
 * This is an implementation of the union-find-backtrack data structure. Each
 * element has a stack which stores its union history.
 * 
 * @author Christian Staudt
 * 
 * @param <T>
 *            a type
 */
public class StackUnionFindBacktrack<T> extends HashUnionFind<T> implements UnionFindBacktrackImplementation<T> {

	/**
	 * Points to the element to which an element has been appended.
	 */
	protected HashMap<T, Stack<T>> stackMap;

	public StackUnionFindBacktrack() {
		super();
		this.stackMap = new HashMap<T, Stack<T>>();
	}

	/**
	 * Generic factory method - use it with type inference.
	 * 
	 * @param <T>
	 * @return
	 */
	public static <T> UnionFindBacktrack<T> getInstance() {
		return new UnionFindBacktrack<T>(new StackUnionFindBacktrack<T>());
	}

	public T split(T root) {
		Stack<T> stack = this.stackMap.get(root);
		if (stack.peek() != root) {
			T newRoot = stack.pop();
			this.makeRoot(newRoot);
			return newRoot;
		} else {
			// element has no union history
			return stack.peek();
		}
	}

	@Override
	public void add(T anElement) {
		this.makeRoot(anElement);
		Stack<T> stack = new Stack<T>();
		stack.push(anElement);
		this.stackMap.put(anElement, stack);
	}

	public void remove(T anElement) {
		if (this.isSingleton(anElement)) {

		} else {
			Stack<T> stack = this.stackMap.get(anElement);
			T firstChild = stack.pop();
			if (this.isRoot(anElement)) {
				this.makeRoot(firstChild);
			} else {
				T parent = this.getParent(anElement);
				Stack<T> parentStack = this.stackMap.get(parent);
				Stack<T> tempStack = new Stack<T>();
				// replace anElement with firstChild in the stack of parent
				while (!(parentStack.isEmpty())) {
					if (parentStack.peek().equals(anElement)) {
						parentStack.pop();
						tempStack.push(firstChild);
					} else {
						tempStack.push(parentStack.pop());
					}
				}
				while (!(tempStack.isEmpty())) {
					parentStack.push(tempStack.pop());
				}
				// 
			}
		}

		this.stackMap.remove(anElement);
		this.parentMap.remove(anElement);
	}
	

	@Override
	public void append(T parent, T child) {
		this.parentMap.put(child, parent);
		Stack<T> stack = this.stackMap.get(parent);
		stack.push(child);
	}

	public boolean isSingleton(T anElement) {
		if (this.isRoot(anElement)) {
			Stack<T> stack = this.stackMap.get(anElement);
			return stack.peek().equals(anElement);
		} else {
			// a singleton is always a root
			return false;
		}

	}

}
