package de.uka.algo.dynclustering.util.unionfind;

/**
 * Allows the unions that have been performed to be backtracked (undone).
 * 
 * @author Christian Staudt
 * 
 * @param <T>
 */
public class UnionFindBacktrack<T> extends UnionFind<T> implements UnionFindBacktrackInterface<T> {

	protected UnionFindBacktrackImplementation<T> castedImp; // equal super.imp;

	public UnionFindBacktrack(UnionFindBacktrackImplementation<T> imp) {
		super(imp, false); // path compression would break the structure
		this.castedImp = imp;
	}

	/**
	 * Backtracks one step in the union history of the set containing
	 * 
	 * @param anElement
	 * @return the new root produced
	 */
	public T backtrack(T anElement) {
		return this.castedImp.split(this.find(anElement));
	}

	/**
	 * Recursively backtrack in the set containing an element until the set is
	 * decomposed into singletons.
	 * 
	 * @param anElement
	 */
	public void backtrackCompletely(T anElement) {
		T rep = this.find(anElement);
		if (!(this.isSingleton(rep))) {
			T newRep = this.backtrack(rep);
			this.backtrackCompletely(newRep);
			this.backtrack(rep);
		}

	}

	/**
	 * Repeatedly splits the set of an element until the element is a
	 * representative.
	 * 
	 * @param anElement
	 */
	public void isolate(T anElement) {
		while (!this.isRepresentative(anElement)) {
			this.castedImp.split(this.find(anElement));
		}
	}

	public boolean isSingleton(T anElement) {
		// delegate to implementation
		return this.castedImp.isSingleton(anElement);
	}

}
