package de.uka.algo.dynclustering.util.unionfind;

import java.util.Set;

public interface UnionFindImplementation<T> {
	
	
	/**
	 * Add an element.
	 * @param anElement
	 */
	public abstract void add(T anElement);
	
	
	public abstract void remove(T anElement);

	/**
	 * Return true if the element is a root.
	 * @param anElement
	 * @return
	 */
	public abstract boolean isRoot(T anElement);

	/**
	 * Makes x the parent of y or vice-versa. 
	 * Allows subclasses to implement different balancing strategies.
	 * @param anElement
	 * @param anotherElement
	 * @return
	 * 		the parent 
	 */
	public abstract T join(T anElement, T anotherElement);

	/**
	 * Append child element to parent element.
	 * @param parent
	 * @param child
	 */
	public abstract void append(T parent, T child);

	/**
	 * Return the parent element of
	 * @param anElement
	 * 		an element
	 * @return
	 * 		the parent
	 */
	public abstract T getParent(T anElement);

	/**
	 * Makes an element a root by cutting out 
	 * the subtree rooted in this element.
	 * @param anElement
	 */
	public abstract void makeRoot(T anElement);
	
	
	/**
	 * Gets the current number of sets.
	 * @return
	 */
	public abstract int getSetCount();
	
	
	/**
	 * Gets the set of representatives.
	 * @return
	 */
	public abstract Set<T> getRoots();
	
	
	/**
	 * Check whether an element is contained in this data structure.
	 * @param anElement
	 * @return
	 */
	public abstract boolean contains(T anElement);
	
	
	public abstract Set<T> getElements();
	
	
	
}
