package de.uka.algo.traversal;

import java.util.PriorityQueue;

import de.uka.algo.traversal.priorities.TraversalPriority;



import y.base.Edge;
import y.base.EdgeCursor;
import y.base.Node;


/**
 * BFS-like graph traversal using a priority queue instead of a normal queue.
 * The priority queue has a comparator of the type TraversalPriority, which is
 * notified by PriorityTraversal.
 * 
 * @author Christian Staudt
 *
 */
public abstract class PriorityTraversal extends BFSLikeBase {

	protected TraversalPriority priority;

	
	public PriorityTraversal(TraversalPriority priority) {
		this.priority = priority;
		this.queue = new PriorityQueue<Node>(10, this.priority);
	}

	
	@Override
	protected Node dequeue() {
		Node u = this.queue.remove();
		this.priority.onDequeue(u);
		return u;
	}

	@Override
	protected void enqueue(Node u) {
		this.queue.add(u);
		this.priority.onEnqueue(u);
	}

	@Override
	protected void expand(Node u) {
		super.expand(u);
		this.priority.onExpand(u);
	}

	@Override
	protected void jumpTo(Node u) {
		this.priority.onJumpTo(u);
		super.jumpTo(u);
	}

	public void reset() {
		super.reset();
		this.priority.onReset();
	}


	
	@Override
	protected void traverse(Edge uv) {
		super.traverse(uv);
		this.priority.onTraverse(uv);	
	}
	
	
}
