package de.uka.algo.traversal.priorities;

import java.util.HashMap;
import java.util.Map;


import y.base.Edge;
import y.base.Node;


/**
 * Compares two nodes according to a distance function and their degree.
 * These attributes can be weighted. 
 * @author Christian Staudt
 *
 */
public class DistanceDegreePriority implements TraversalPriority {
	
	Map<Node,Integer> distance;
	double distW;
	double degW;
	
	
	/**
	 * 
	 * @param distanceWeight	a weighting factor for the distance value
	 * @param degreeWeight		a weighting factor for the degree value
	 */
	public DistanceDegreePriority(double distanceWeight, double degreeWeight) {
		this.distance = new HashMap<Node,Integer>();
		this.distW = distanceWeight;
		this.degW = degreeWeight;
	}

	public int compare(Node u, Node v) {
		double uVal = this.distW * this.distance.get(u) + this.degW * u.degree();
		double vVal = this.distW * this.distance.get(v) + this.degW * u.degree();
		if (uVal < vVal) {
			return -1;
		} else if (uVal == vVal) {
			return 0;
		} else {
			return 1;
		}
	}

	public void onDequeue(Node u) {
		// TODO Auto-generated method stub
		
	}

	public void onEnqueue(Node u) {
		// TODO Auto-generated method stub
		
	}

	public void onExpand(Node u) {
		// TODO Auto-generated method stub
		
	}

	
	public void onJumpTo(Node u) {
		/* visited without predecessor - this means that 
		 * u belongs to the source set
		 */
		this.distance.put(u, 0);
	}

	public void onReset() {
		this.distance.clear();
	}

	public void onTraverse(Edge uv) {
		// d(v) = d(u) + 1
		Node u = uv.source();
		Node v = uv.target();
		if (! this.distance.containsKey(v)) {
			int d = this.distance.get(u);
			this.distance.put(v, d + 1);
		}

	}
	
	public String toString() {
		return "DistanceDegree(" + distW + "," + degW + ")";
	}



}
