package de.uka.algo.traversal.priorities;

import java.util.HashMap;
import java.util.Map;

import de.uka.algo.traversal.WeightFunction;

import y.base.Edge;
import y.base.Node;
import y.base.NodeCursor;


/**
 * Prioritizes nodes according to the distance from the source. 
 * PrioritySearch(DistancePriorty) can be used to implement Dijkstra's Algorithm.
 * @author Christian Staudt
 *
 */
public class TargetDistancePriority implements TraversalPriority {
	
	Node target;
	Map<Node,Double> distance;
	WeightFunction omega;
	
	public TargetDistancePriority(Node target, WeightFunction omega) {
		this.target = target;
		this.omega = omega;
		this.distance = new HashMap<Node,Double>();
		for (NodeCursor nc = target.getGraph().nodes(); nc.ok(); nc.next()) {
			this.distance.put(nc.node(), Double.POSITIVE_INFINITY);
		}
	}

	public void onDequeue(Node u) {
		// TODO Auto-generated method stub

	}

	public void onEnqueue(Node u) {
		// TODO Auto-generated method stub

	}

	public void onExpand(Node u) {
		// TODO Auto-generated method stub

	}

	public void onJumpTo(Node u) {
		// initial nodes has distance zero
		this.distance.put(u, 0.0);
	}

	public void onReset() {
		// TODO Auto-generated method stub

	}

	public void onTraverse(Edge uv) {
		Node u = uv.source();
		Node v = uv.target();
		double uDistance = this.distance.get(u);
		this.distance.put(v, uDistance + omega.getWeight(uv));
	}

	public int compare(Node u, Node v) {
		double uPriority = this.distance.get(u);
		double vPriority = this.distance.get(v);
		if (uPriority <= vPriority) {
			return -1;
		} else if (uPriority == vPriority) {
			return 0;
		} else {
			return 1;
		}
	}

}
