package de.uka.algo.traversal.priorities;

import java.util.Comparator;

import y.base.Edge;
import y.base.Node;


/**
 * SearchPriority is a comparator of nodes which
 * can react to events occuring in graph traversal
 * in order to maintain the necessary data.
 * @author Christian Staudt
 *
 */
public interface TraversalPriority extends Comparator<Node> {
	
	/**
	 * Called when the search makes a visit
	 * @param v		going to node v
	 * @param u		coming from node u
	 */
	public void onTraverse(Edge uv);
	
	
	/**
	 * Called when the search visits	
	 * @param u		a node
	 * without reaching it via an edge.
	 */
	public void onJumpTo(Node u);
	
	/**
	 * Called when the search expands 
	 * @param u	a node
	 * i.e. visits all neighbors.
	 */
	public void onExpand(Node u);
	
	
	/**
	 * Called when 
	 * @param u		a node
	 * is enqueued.
	 */
	public void onEnqueue(Node u);
	
	
	/**
	 * Called when
	 * @param u		a node
	 * is dequeued.
	 */
	public void onDequeue(Node u);
	
	
	public void onReset();
	
}
