/*
 * Created on Aug 26, 2004
 * Reviewed 1.1 (Jan 8, 2005)
 *
 * Copyright 2004.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
 */
package de.uka.algo.clustering.algorithms;

import y.algo.GraphConnectivity;
import y.base.NodeList;
import de.uka.algo.clustering.Clustering;

/**
 * A ComponentClusterStage resets a clustering to one cluster for each connected
 * component.
 * 
 * @author Bastian Katz
 */
public class Components implements Algorithm {

    /**
     * Resets the given clustering such that every connected component forms one
     * cluster. Postcondition: All nodes are assigned to a cluster.
     * @param aClustering the clustering to reset
     */
    public void run(Clustering aClustering) {
        /* reset clustering (delete all clusters) */
        aClustering.reset();

        /* create clusters for all connected components */
        NodeList[] connectedComponents = GraphConnectivity
                .connectedComponents(aClustering.getGraph());
        for (int i = 0; i < connectedComponents.length; i++) {
            aClustering.newCluster(connectedComponents[i].nodes());
        }
    }
}