/*
 * created May 10, 2006
 *
 * Copyright 2006.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
 *
 */
package de.uka.algo.clustering.algorithms;

import java.util.Random;

import de.uka.algo.clustering.index.IndexFactory;

/**
 * This class implements a ElementalOperationOptimizer, allowing small decreases in quality.
 * The allowed loss of quality decreases with increasing number of iterations.
 * 
 * @author Daniel Delling
 * 
 */
public class ElementalOperationSimulatedAnnealing extends ElementalOperationOptimizer {

	private int temperature = 2;

	private Random ran = new Random();

	public ElementalOperationSimulatedAnnealing(IndexFactory quality, boolean shift, boolean merge,
			boolean split) {
		super(quality, shift, merge, split);
	}

	public ElementalOperationSimulatedAnnealing(IndexFactory quality) {
		super(quality);
	}

	@Override
	boolean accept(double a, double b) {
		boolean accept = a > b;
		if (!accept) {
			double p = 1 - Math.exp((1 - b + a) / -(double) temperature);
			accept |= ran.nextDouble() < p;
		}
		return accept;
	}

	@Override
	void afterIteration() {
		temperature *= 2;
	}

}
