package de.uka.algo.clustering.algorithms;

import java.util.Random;

import y.base.Edge;
import de.uka.algo.clustering.Cluster;
import de.uka.algo.clustering.Clustering;
import de.uka.algo.clustering.index.IndexFactory;
import de.uka.algo.clustering.operators.Product;
import de.uka.algo.util.GYCursor;

public class Genetic extends Object implements Algorithm {
	int pop = 25;
	int child = 300;
 
	IndexFactory fac = null;

	public Genetic(IndexFactory f) {
		fac = f;
	}
	
	public void run(Clustering aClustering) {
		Random r = new Random();
		Clustering[] population = new Clustering[pop];
		for (int i = 0; i < pop; i++) {
			population[i] = new Clustering(aClustering.getGraph(), aClustering
					.getInterpretation());
			population[i].resetSingletons();
		}
		double last = 0.0;
		double current = 0.0;
		double end = 3;
		do {
			Clustering[] childs = new Clustering[child];
			double[] vals = new double[child];
			for (int i = 0; i<pop; i++) {
				childs[i] = population[i];
				vals[i] = fac.getIndex(childs[i]).getValue();
			}
			last = vals[pop-1];
			for (int i = pop; i<2*pop; i++) {
				childs[i] = getChild(population[i-pop], population[i-pop]);
				vals[i] = fac.getIndex(childs[i]).getValue();
			}
			for (int i = 2*pop; i<child; i++) {
				childs[i] = getChild(population[r.nextInt(pop)], population[r.nextInt(pop)]);
				vals[i] = fac.getIndex(childs[i]).getValue();
			}
			for (int choose = 0; choose<pop; choose++) {
				int best=0;
				double value = -1000;
				for (int k = 0; k<child; k++) {
					if (childs[k]!=null && vals[k]>value) {
						best = k;
						value = vals[k];
					}
				}
				population[choose] = childs[best];
				childs[best] = null;
				current = vals[best];
			}			
			for (int i = 0;i<child; i++) {
				if (childs[i]!=null)
					childs[i].dispose();
			}
			end--;
			if (current>last) {
				end = 10;
			} 
		} while (end>0);
		aClustering.reset();
		for (GYCursor<Cluster> cc = population[0].clusters(); cc.ok();cc.next()) {
			aClustering.newCluster(cc.active().members());
		}
	}

	private Clustering getChild(Clustering clustering, Clustering clustering2) {
		Random r = new Random();
		Clustering result = clustering.clone();
		new Product().run(result, clustering2);
		for (int k = 0; k<r.nextInt((int) (result.size()/2.0));k++) {
			Edge e = clustering.getGraph().getEdgeArray()[r.nextInt(clustering.getGraph().E())];
			result.merge(e.source(), e.target());
		}
		return result;
	}
}
