/*
 * Created on May 10, 2006
 *
 * Copyright 2004-2006.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
 */
package de.uka.algo.clustering.algorithms;

import java.util.Random;

import y.base.NodeCursor;
import y.base.NodeList;
import de.uka.algo.clustering.Clustering;

/**
 * A RandomClusterer randomly assigns nodes to a specified number of
 * clusters, without paying attention to the edges.
 */
public class RandomClusterer implements Algorithm{

	private int minClustercount;
	private int maxClustercount;
	
	boolean selfAdjusting;
	
	public RandomClusterer() {
		this(-1);
		this.selfAdjusting = true;
	}
	
	/**
	 * 
	 * @param clustercount the number of clusters the clusterer should generate
	 */
	public RandomClusterer(int clustercount) {
		this(clustercount, clustercount);
	}
	
	/**
	 * the clustercount is picked random between minClustercount and maxClustercount everytime
	 * doCluster(Clustering aClustering) is called.
	 * @param minClustercount 
	 * @param maxClustercount
	 */
	public RandomClusterer(int minClustercount, int maxClustercount) {
		this.minClustercount = minClustercount;
		this.maxClustercount = maxClustercount;
		this.selfAdjusting = false;
	}
	
	
	public void run(Clustering aClustering){
		Random rnd = new Random();
		
		if(selfAdjusting) {
			int size = aClustering.getGraph().nodeCount();
			minClustercount = (int)Math.log10(size);
			maxClustercount = (int)Math.sqrt(size);
		}
		
		int clustercount = new Random().nextInt(maxClustercount-minClustercount+1)+minClustercount;
		
		aClustering.reset();
		
		NodeList[] nodelists = new NodeList[clustercount];
		
		for (int i=0; i<clustercount; i++){
			nodelists[i] = new NodeList();
		}
		
		NodeCursor nc = aClustering.getGraph().nodes();
		
		for(nc.toFirst(); nc.ok(); nc.next()){
			nodelists[rnd.nextInt(clustercount)].add(nc.node());
		}
		
		for (int i=0; i<clustercount; i++){
			if (!nodelists[i].isEmpty())
				aClustering.newCluster(nodelists[i].nodes());
		}
		
		
	}


}
