/*
 * Created on May 10, 2006
 *
 * Copyright 2004-2006.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
 */
package de.uka.algo.clustering.algorithms;

import java.util.Random;

import y.base.Node;
import de.uka.algo.clustering.Cluster;
import de.uka.algo.clustering.Clustering;

/**
 * A RandomPermutationClusterer randomly swaps nodes from one cluster to an
 * other according to a given clustering
 */
public class RandomPermutationClusterer implements Algorithm {

	static org.apache.log4j.Logger logger = org.apache.log4j.Logger
			.getLogger(RandomPermutationClusterer.class);
	
	static int globalCounter = 0;
	protected int localCounter = 0;
	
	private int permutationFactor = 2;

	/**
	 * @param permutationFactor
	 *          permutationFactor * n swaps are executed
	 */
	public RandomPermutationClusterer(int permutationFactor) {
		this.permutationFactor = Math.max(permutationFactor, 1);
		localCounter = globalCounter;
		++globalCounter;
	}

	public RandomPermutationClusterer() {
		this(2);
	}

	public void run(Clustering aClustering) {
		Random rnd = new Random();

		int n = aClustering.getGraph().nodeCount();

		int permutations = rnd.nextInt((permutationFactor - 1) * n + 1) + n;

		// logger.info(localCounter+" number of permutations is "+permutations);
		
		if (aClustering.clusterCount() == 1) {
			return;
		}
		

		int noChange = 0;
		
		//try as long as maximal permutations reached or 10 * n times no swap has been executed		
		while (permutations > 0 && noChange < 10*n) {
			int firstIndex = rnd.nextInt(n);
			int secondIndex = rnd.nextInt(n);
			// logger.info(globalCounter+" swapping nodes "+firstIndex+" and "+secondIndex);
			Node n1 = aClustering.getGraph().getNodeArray()[firstIndex];
			Node n2 = aClustering.getGraph().getNodeArray()[secondIndex];
			Cluster c1 = aClustering.getCluster(n1);
			Cluster c2 = aClustering.getCluster(n2);
			if (c1 != c2) {
				if (c1.size() > 1 && c2.size() > 1) {
					aClustering.move(c1, c2, n1);
					aClustering.move(c2, c1, n2);
				} else if (c1.size() == 1 && c2.size() > 1) {
					aClustering.move(c2, c1, n2);
					aClustering.move(c1, c2, n1);
				} else if (c1.size() > 1 && c2.size() == 1) {
					aClustering.move(c1, c2, n1);
					aClustering.move(c2, c1, n2);
				} else if (c1.size() == 1 && c2.size() == 1) {
					// logger.info("both are singeltons"); 
					// do nothing is both are singletons
				}
				permutations--;
				noChange = -1;
			}
			noChange++;
		}

	}

}
