/*
 * Created on Jul 6, 2004
 * Reviewed 1.6 (Jan 10, 2005)
 *
 * Copyright 2004.  --- All rights resevered. 
 * Algorithmic Group, Karlsruhe, Germany.
 */
package de.uka.algo.clustering.algorithms;

import y.base.EdgeCursor;
import y.base.EdgeList;
import de.uka.algo.clustering.Clustering;
import de.uka.algo.clustering.index.IndexFactory;
import de.uka.algo.graphs.linalg.GraphMatrix;
import de.uka.algo.graphs.linalg.MarkovianGraphMatrix;

/**
 * Implementation af a SingleLinkageClusterer, which sorts edges by their length
 * according to an n-dimensional spectral embedding of its nodes. 
 * 
 * @author Bastian Katz
 */
public class SpectralSingleLinkage extends SingleLinkage {

    /* the embedding dimension */
    private int dimension;

    /* the quality index to optimize */
    private IndexFactory quality;

    /**
     * Instantiates a new SpectralSingleLinkage clustering algorithm with a
     * embedding dimension and a quality index (by factory).
     * 
     * @param dimension
     *            the embedding dimension
     * @param quality
     *            the quality index (as factory)
     */
    public SpectralSingleLinkage(int dimension, IndexFactory quality) {
        this.dimension = dimension;
        this.quality = quality;
    }

    /**
     * Returns the edges sorted by length (ascending) in a spectral embedding.
     */
    @Override
    protected EdgeCursor sortedEdges(Clustering aClustering) {
        GraphMatrix gm = new MarkovianGraphMatrix(aClustering.getGraph(),
                aClustering.getInterpretation());
        EdgeList ret = gm.getSpectralEdgeList(dimension);
        return ret.edges();
    }

    /**
     * Returns the quality index
     * 
     * @return the quality index
     */
    @Override
    protected IndexFactory quality() {
        return quality;
    }
}