/**
 * Created on 17.11.2007
 */
package de.uka.algo.clustering.algorithms.betweenness.edgechoosing;

import java.util.Arrays;

import y.algo.Paths;
import y.base.Edge;
import y.base.EdgeList;
import y.base.EdgeMap;
import y.base.Graph;
import y.base.Node;
import de.uka.algo.clustering.Clustering;

/**
 * Hide the edges with the highest centrality until
 * a connected component splits up into two components
 * 
 * @author Ulrich Geilmann
 */
public class CutEdgeChooser implements EdgeChooser {

	/*
	 * @see de.uka.algo.clustering.betweenness.util.EdgeChooser#chooseEdge(y.base.Graph, y.base.EdgeMap, de.uka.algo.clustering.Clustering)
	 */
	public EdgeList chooseEdge(Graph graph, EdgeMap map, Clustering clustering) {
		Edge[] edges = graph.getEdgeArray();
		Arrays.sort (edges, new EdgeComparator(map));
		
		EdgeList l = new EdgeList();
		int i = edges.length - 1;
		while (i > -1) {
			Node v = edges[i].source();
			Node w = edges[i].target();
			graph.hide(edges[i]);
			l.add(edges[i]);
			if (v != w && Paths.findPath(graph, v, w, false).isEmpty())
				i = -1;
			else
				i--;
		}
		
		return l;
	}

}
