/**
 * Created on 12.11.2007
 */
package de.uka.algo.clustering.algorithms.betweenness.edgechoosing;

import y.base.Edge;
import y.base.EdgeCursor;
import y.base.EdgeList;
import y.base.EdgeMap;
import y.base.Graph;
import de.uka.algo.clustering.Clustering;

/**
 * Hide all edges with the highest centrality value
 * @author Ulrich Geilmann
 */
public class MaxEdgeChooser implements EdgeChooser {

	/*
	 * @see de.uka.algo.clustering.betweenness.util.EdgeChooser#chooseEdge(y.base.Graph, y.base.EdgeMap, de.uka.algo.clustering.Clustering)
	 */
	public EdgeList chooseEdge(Graph graph, EdgeMap map, Clustering clustering) {
		EdgeList l = new EdgeList();
		double max = Double.NEGATIVE_INFINITY;
		for (EdgeCursor cursor = graph.edges(); cursor.ok(); cursor.next()) {
			Edge e = cursor.edge();
			double v = map.getDouble(e);
			if (v > max) {
				l.clear();
				l.add(e);
				max = v;
			} else if (v == max) {
				l.add(e);
			}
		}
		for (EdgeCursor c = l.edges(); c.ok(); c.next())
			graph.hide(c.edge());
		return l;
	}

}
