/**
 * Created on 20.11.2007
 */
package de.uka.algo.clustering.algorithms.betweenness.edgechoosing;

import java.util.Arrays;

import y.base.Edge;
import y.base.EdgeList;
import y.base.EdgeMap;
import y.base.Graph;
import de.uka.algo.clustering.Clustering;

/**
 * Hide all edges whose centralities is above a defined percentage of the maximal centrality
 * @author Ulrich Geilmann
 *
 */
public class RelativeEdgeChooser implements EdgeChooser {
	
	/**
	 * @param p
	 * 	All edges with centrality > p*(maximal centrality) will be hidden
	 */
	public RelativeEdgeChooser (double p) {
		percentage = p;
	}
	private double percentage;

	/*
	 * @see de.uka.algo.clustering.betweenness.util.EdgeChooser#chooseEdge(y.base.Graph, y.base.EdgeMap, de.uka.algo.clustering.Clustering)
	 */
	public EdgeList chooseEdge(Graph graph, EdgeMap map, Clustering clustering) {
		Edge[] edges = graph.getEdgeArray();
		Arrays.sort (edges, new EdgeComparator(map));
		
		EdgeList l = new EdgeList();
		if (edges.length <= 0)
			return l;
		int i = edges.length - 1;
		double minChoose = percentage*map.getDouble(edges[i]);
		while (i >= 0) {
			if (map.getDouble(edges[i]) < minChoose)
				i = -1;
			else {
				graph.hide(edges[i]);
				l.add(edges[i]);
				i--;
			}
		}
		return l;	
	}

}
