/**
 * Created on 12.11.2007
 */
package de.uka.algo.clustering.algorithms.betweenness.edgechoosing;

import java.util.Arrays;

import y.base.Edge;
import y.base.EdgeList;
import y.base.EdgeMap;
import y.base.Graph;
import de.uka.algo.clustering.Clustering;

/**
 * Hide a fixed number of edges with the highest centralites
 * or a fixed fraction of the edges in the graph
 * 
 * @author Ulrich Geilmann
 */
public class SimpleEdgeChooser implements EdgeChooser {
	/**
	 * The number of edges to hide
	 */
	private int n;
	/**
	 * The fraction of edges to hide
	 */
	private double f;
	
	/**
	 * Whether we hide an absolute or relative number of edges
	 */
	private boolean fractional;
	
	/**
	 * @param n
	 * 	The number of edges to hide
	 */
	public SimpleEdgeChooser (int n) {
		this.n = n;
		fractional = false;
	}
	
	/**
	 * @param f
	 * 	The fraction of the edges in the graph to hide
	 */
	public SimpleEdgeChooser (double f) {
		this.f = f;
		fractional = true;
	}
	
	/*
	 * @see de.uka.algo.clustering.betweenness.util.EdgeChooser#chooseEdge(y.base.Graph, y.base.EdgeMap, de.uka.algo.clustering.Clustering)
	 */
	public EdgeList chooseEdge(Graph graph, EdgeMap map, Clustering clustering) {
		EdgeComparator comp = new EdgeComparator (map);
		Edge[] edges = graph.getEdgeArray();
		Arrays.sort (edges, comp);
		EdgeList l = new EdgeList();
		int m = n;
		if (fractional) {
			m = (int)(edges.length * f + 0.5);
			if (m <= 0) m = 1;
		}
		for (int i = edges.length-1; i >= 0 && l.size() < m; i--) {
			graph.hide(edges[i]);
			l.add(edges[i]);
		}
		return l;
	}

}
