/*
 * Created on 05.03.20049
 */
package de.uka.algo.clustering.algorithms.modules;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.WindowConstants;

import y.option.OptionHandler;
import y.view.Graph2D;
import de.uka.algo.clustering.Clustering;
import de.uka.algo.clustering.algorithms.Algorithm;
import de.uka.algo.clustering.algorithms.blondel.Blondel;
import de.uka.algo.clustering.algorithms.newman.internal.UpdateStrategy;
import de.uka.algo.clustering.visualization.Graph2DVisualizer;

/**
 * @author Robert Goerke
 */
public class BlondelModule extends AlgorithmModule {

	private static final String NODEORDER_STRATEGY = "order of nodes";
	private static final String STOPPING_CRITERION = "stopping criterion";
	private static final String UPDATE = "enable updater";
	private static final String UPDATE_POLICY = "update strategy";
	public BlondelModule() {super("Blondel", "Robert Goerke", "Clusters nodes using Blondels algorithm.", false, WEIGHTS_ASK);}

	@Override
	protected Algorithm algorithm() {
		OptionHandler op = getOptionHandler();
		op.useSection("Blondel");
		Blondel.NodeOrderStrategies nodeOrderStrategy = Blondel.NodeOrderStrategies.values()[op.getEnum(NODEORDER_STRATEGY)];
		Blondel.StoppingCriteria stoppingCriterion = Blondel.StoppingCriteria.values()[op.getEnum(STOPPING_CRITERION)];
		Blondel.UpdatePolicy updatePolicy = Blondel.UpdatePolicy.values()[op.getEnum(UPDATE_POLICY)];
		Blondel clusterer = new Blondel(nodeOrderStrategy,stoppingCriterion,updatePolicy);
//		System.out.println(getGraph2D());
		if(op.getBool(UPDATE)) {
			new UpdateWindow(getVisualizer(0),clusterer);
		}
		return clusterer;
	}

	@Override
	protected OptionHandler createOptionHandler() {
		OptionHandler op = new OptionHandler("");
		op.useSection("Blondel");
		op.addEnum(NODEORDER_STRATEGY, Blondel.NodeOrderStrategies.values(),1);
		op.addEnum(STOPPING_CRITERION, Blondel.StoppingCriteria.values(),0);
		op.addBool(UPDATE, true);
		op.addEnum(UPDATE_POLICY, Blondel.UpdatePolicy.values(), 0);
		op.addOptionHandler(super.createOptionHandler(), "");
		return op;
	}
	
	private class UpdateWindow extends JFrame implements ActionListener {
		private static final long serialVersionUID = 1L;
		private Blondel clusterer;
		private Graph2DVisualizer visualizer;
		
		public UpdateWindow(Graph2DVisualizer visualizer, Blondel theClusterer) {
			super("Clustering Updater");
			
			setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
			
			this.visualizer = visualizer;
			this.clusterer = theClusterer;
			
			JButton updateButton = new JButton("Update!");
			updateButton.addActionListener(this);
			add(updateButton);
			pack();
			setVisible(true);
		}
		
		public void actionPerformed(ActionEvent e) {
			updateClustering();
//			System.out.println(((Object)getGraph2D()).toString());
		}
		
		private void updateClustering() {
			Clustering theClustering = clusterer.refreshClustering();
			visualizer.visualize(theClustering, false);
	        ((Graph2D)theClustering.getGraph()).updateViews();
		}
	}
}
