/*
 * created May 8, 2006
 *
 * Copyright 2006.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
 *
 */
package de.uka.algo.clustering.algorithms.modules;

import y.option.OptionHandler;
import de.uka.algo.clustering.algorithms.Algorithm;
import de.uka.algo.clustering.algorithms.ElementalOperationMinimizer;
import de.uka.algo.clustering.algorithms.ElementalOperationOptimizer;
import de.uka.algo.clustering.algorithms.ElementalOperationSimulatedAnnealing;

public class ElementalOperationOptimizerModule extends IndexAlgorithmModule {

    private static final String[] EOO_MODI = {"optimization", "simulated annealing (opt)", "minimization"};

	public ElementalOperationOptimizerModule() {
		super("Elemental Operation Optimizer", "Daniel Delling", "", true, WEIGHTS_ASK);
	}


	@Override
	protected Algorithm algorithm() {
		OptionHandler op = getOptionHandler();
		op.useSection("EOO");
		int strategy = op.getEnum("type of optimization"); //0: opt, 1:simAnn, 2:min
		boolean merge = op.getBool("allow merging");
		boolean shift = op.getBool("allow shifting");
		boolean split = op.getBool("allow splitting of single nodes");
		int mo = op.getInt("maximal operations");
		if(strategy == 0){
			return new ElementalOperationOptimizer(getIndexFactory(), shift, merge, split);
		} else if(strategy == 1){
			return new ElementalOperationSimulatedAnnealing(getIndexFactory(), shift, merge, split);
		} else {
			return new ElementalOperationMinimizer(getIndexFactory(), shift, merge, split, mo);
		}
	}


	@Override
	protected OptionHandler createOptionHandler() {
		OptionHandler op = new OptionHandler("");
		op.useSection("EOO");
		op.addBool("allow merging", true);
		op.addBool("allow shifting", true);
		op.addBool("allow splitting of single nodes", true);
		op.addEnum("type of optimization", EOO_MODI, 0);
		op.addInt("maximal operations", 0);
		op.addOptionHandler(super.createOptionHandler(), "");
		return op;
	}

}
