/*
 * Created on Feb 20, 2006
 *
 * Copyright 2005.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
*/
package de.uka.algo.clustering.algorithms.modules;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.WindowConstants;

import y.base.NodeCursor;
import y.option.OptionHandler;
import y.view.Graph2D;
import y.view.hierarchy.HierarchyManager;
import de.uka.algo.clustering.Clustering;
import de.uka.algo.clustering.algorithms.Algorithm;
import de.uka.algo.clustering.algorithms.newman.NewmanClusterer;
import de.uka.algo.clustering.algorithms.newman.internal.CoverageQInterface;
import de.uka.algo.clustering.algorithms.newman.internal.GenericAdditiveAdapter;
import de.uka.algo.clustering.algorithms.newman.internal.GenericAdditiveDeltaQMatrix;
import de.uka.algo.clustering.algorithms.newman.internal.GenericQuotientAdapter;
import de.uka.algo.clustering.algorithms.newman.internal.GenericQuotientDeltaQMatrix;
import de.uka.algo.clustering.algorithms.newman.internal.MeasurementAdapter;
import de.uka.algo.clustering.algorithms.newman.internal.PerformanceQInterface;
import de.uka.algo.clustering.algorithms.newman.internal.QInterface;
import de.uka.algo.clustering.algorithms.newman.internal.QQInterface;
import de.uka.algo.clustering.algorithms.newman.internal.ResetUpdateStrategy;
import de.uka.algo.clustering.algorithms.newman.internal.UpdateStrategy;
import de.uka.algo.clustering.visualization.Graph2DExtractor;
import de.uka.algo.clustering.visualization.Graph2DVisualizer;

public class Newman2Module extends AlgorithmModule {
	private static final String STOPONPEAK = "stop on peak";
	private static final String CONSIDEREDGEWEIGTHS = "Consider Edge Weigths";
	private static final String DENDRO = "Create dendrogram";
	
	private static final String CLUSTERER = "Use which clusterer?";
	private static final String CL_DIFF_MOD = "DiffModularityAdapter";
	private static final String CL_DIFFMOD_REFACTORED = "DiffModularityAdapterRefactored";
	private static final String CL_MOD = "Modularity Adapter";
	private static final String CL_MOD_REFACTORED = "Modularity Adapter Refactored";
	private static final String CL_MOD_WITH_GENERIC_ADDITIVE = "MAR + Generic Additive";
	
	private static final String CL2 = "MAR mit Coverage";
	private static final String CL3 = "MAR mit Performance";
	
	private static final String CL4 = "MAR mit Coverage (Q)";
	private static final String CL5 = "MAR mit Performance (Q)";
	
	private static final String CLUSTERZAHLABBRUCH = "fixed clustercount (overrides ff)";
	private static final String CLUSTERZAHL = "number of clusters";
	
	private static final String UPDATE = "enable update of clustering";
	private static final String UPDATESTRATEGY = "update strategy";
	private static final String UPDATEINCOMPATIBILITYMESSAGE =
		"You have chosen the %s update strategy.\n" +
		"At present, it only works with the original modularity index\n" +
		"and without creating a dendrogram. Your settings will be adjusted.";
	
	
	public Newman2Module(){
		super("new Newman", "Jens Mueller", "Newman-like algorithms", true, WEIGHTS_ASK);
	}
		
	protected OptionHandler createOptionHandler() {
		OptionHandler op = new OptionHandler("");
		op.useSection("Newman");
		op.addBool(STOPONPEAK, true);
		//op.addBool(CONSIDEREDGEWEIGTHS, false);
		/* String[] args = {CL_DIFF_MOD, CL_DIFFMOD_REFACTORED, CL_MOD,
				CL_MOD_REFACTORED, CL_MOD_WITH_GENERIC_ADDITIVE}; */
		String[] args = {CL2, CL3, CL4, CL5};
		op.addEnum(CLUSTERER, args, 0);
		op.addBool(CLUSTERZAHLABBRUCH, false);
		op.addInt(CLUSTERZAHL, 1);
		op.addBool(DENDRO, false);
		op.useSection("Dynamics");
		op.addBool(UPDATE, false);
		op.addEnum(UPDATESTRATEGY, UpdateStrategy.getDescriptions(), 0);
		op.addOptionHandler(super.createOptionHandler(), "");
		return op;
	}

	@Override
	protected Algorithm algorithm() {
		OptionHandler op = getOptionHandler();
		
		boolean dendro = op.getBool(DENDRO);
		int clustermethod = op.getEnum(CLUSTERER);
		
		int usindex = op.getEnum(UPDATESTRATEGY);
		UpdateStrategy us = UpdateStrategy.create(usindex);
		if(us == null)
			us = new ResetUpdateStrategy();
		if(usindex > 0 && (op.getBool(DENDRO) || op.getEnum(CLUSTERER) != 0)) {
			String msg = String.format(UPDATEINCOMPATIBILITYMESSAGE,
					UpdateStrategy.getDescriptions()[usindex]);
			JOptionPane.showMessageDialog(null, msg);
			dendro = false;
			clustermethod = 0;
		}
		
		MeasurementAdapter ma = null;
		/* switch (op.getEnum(CLUSTERER)) {
		case 0:
			ma = new DiffModularityAdapter(considerEdgeWeights);
			break;
		case 1:
			ma = new DiffModularityAdapterRefactored(considerEdgeWeights);
			break;
		case 2:
			ma = new ModularityAdapter(considerEdgeWeights);
			break;
		case 3:
			ma = new ModularityAdapterRefactored(considerEdgeWeights);
			break;
		case 4:
			ma = new GenericAdapter(considerEdgeWeights);
			break;
		default:
			break;
		} */
		QInterface modI;
		QQInterface modIQ;
		
		switch (clustermethod) {
		case 0:
			modI = new CoverageQInterface();
			ma = new GenericAdditiveAdapter(true, modI,
					new GenericAdditiveDeltaQMatrix(modI));
			break;
		case 1:
			modI = new PerformanceQInterface();
			ma = new GenericAdditiveAdapter(true, modI,
					new GenericAdditiveDeltaQMatrix(modI));
			break;
		case 2:
			System.out.println("MAR case 2: modularity quotient");
			modIQ = new CoverageQInterface();
			ma = new GenericQuotientAdapter(true, modIQ,
					new GenericQuotientDeltaQMatrix(modIQ));
			break;
		case 3:
			System.out.println("MAR case 3: performance quotient");
			modIQ = new PerformanceQInterface();
			ma = new GenericQuotientAdapter(true, modIQ,
					new GenericQuotientDeltaQMatrix(modIQ));
			break;
		}
		
		NewmanClusterer clusterer = new NewmanClusterer(ma);
		if(op.getBool(UPDATE))
			clusterer.setUpdateStrategy(us);
		clusterer.setCreatingDendro(dendro);
		clusterer.setStoppingOnPeak(op.getBool(STOPONPEAK));
		if (op.getBool(CLUSTERZAHLABBRUCH))
			clusterer.setDesiredClusters(op.getInt(CLUSTERZAHL));
		
		if(op.getBool(UPDATE)) {
			new UpdateWindow(
					getGraph2D(),
					getExtractor(), 
					getVisualizer(0),					
					true,
					clusterer);
		}

		return clusterer;
	}

	private class UpdateWindow extends JFrame implements ActionListener {
		Graph2D graph;
		boolean useWeights;
		NewmanClusterer clusterer;
		private Graph2DExtractor extractor;
		private Graph2DVisualizer visualizer;
		
		public UpdateWindow(Graph2D graph, Graph2DExtractor extractor, Graph2DVisualizer visualizer, boolean useWeights, NewmanClusterer clusterer) {
			super("Clustering Updater");
			
			setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
			
			this.graph = graph;
			this.extractor = extractor;
			this.visualizer = visualizer;
			this.useWeights = useWeights;
			this.clusterer = clusterer;
			
			JButton button = new JButton("Update!");
			button.addActionListener(this);
			add(button);
			pack();
			setVisible(true);
		}
		
		public void actionPerformed(ActionEvent e) {
			update();
		}
		
		private void update() {
	        /* kill hierarchy */
	        HierarchyManager hierarchy = HierarchyManager.getInstance(graph);
	        if (hierarchy != null) {
	            for (NodeCursor c = graph.nodes(); c.ok(); c.next()) {
	                if (!hierarchy.isNormalNode(c.node())) {
	                    graph.removeNode(c.node());
	                }
	            }
	        }
			Clustering cl = extractor.extract(graph, useWeights);
			clusterer.run(cl); 
			visualizer.visualize(cl, false);
	        graph.updateViews();
		}
	}
}
