package de.uka.algo.clustering.algorithms.newman.internal;

import java.util.HashSet;
import java.util.Set;

import org.apache.log4j.Level;

import y.base.Edge;
import y.base.EdgeCursor;
import de.uka.algo.clustering.Cluster;
import de.uka.algo.clustering.Clustering;
import de.uka.algo.clustering.algorithms.newman.util.Converter;
import de.uka.algo.clustering.algorithms.newman.util.UnionFindBacktrack;
import de.uka.algo.graphs.GraphInterpretation;
import de.uka.algo.util.GYCursor;

public class BacktrackUpdateStrategy extends UpdateStrategy {
	static org.apache.log4j.Logger logger = org.apache.log4j.Logger
	.getLogger(BacktrackUpdateStrategy.class);

	UnionFindBacktrack unionHistory = null;
	
	final private int LAMBDA=3;

	@Override
	public void init(Clustering cl) {
		logger.setLevel(Level.INFO);
		unionHistory = new UnionFindBacktrack(cl.getGraph().N());
	}
	@Override
	public void prepareClustering(Clustering cl,
			GraphInterpretation oldGraph) {
		GraphDelta delta = new GraphDelta(oldGraph, cl.getInterpretation());
		Set<Integer> untouchedClusters = new HashSet<Integer>(cl.clusterCount());
		for(GYCursor<Cluster> cc = cl.clusters(); cc.ok(); cc.next()) {
			untouchedClusters.add(unionHistory.find(cc.active().aMember().index()));
		}
		for(EdgeCursor ec = delta.getGraph().edges(); ec.ok(); ec.next()) {
			Edge e = ec.edge();
			int n1 = e.source().index();
			int n2 = e.target().index();
			int c1 = unionHistory.find(n1);
			int c2 = unionHistory.find(n2);	


			if (delta.getWeight(e) > 0.0) {
				// edge *addition*
				
				if (c1 == c2) {
					// *intra*-cluster edge addition
					untouchedClusters.remove(c1);
					while(c1 == c2) {
						unionHistory.split(c1);
						c1 = unionHistory.find(n1);
						c2 = unionHistory.find(n2);
					}
					unionHistory.isolate(n1);
					unionHistory.isolate(n2);
				} else {
					// *inter*-cluster edge addition
					untouchedClusters.remove(c1);
					untouchedClusters.remove(c2);
					unionHistory.isolate(n1);
					unionHistory.isolate(n2);
				}
			} else {
				// step back randomly in untouched clusters without c1 and c2
				untouchedClusters.remove(c1);
				if (c1 != c2)
					untouchedClusters.remove(c2);
				int numClusters = untouchedClusters.size();
				if(numClusters > 0) {
					Integer [] clarray = new Integer[numClusters+LAMBDA];
					untouchedClusters.toArray(clarray);
					for(int l=0; l<LAMBDA; l++) {
						int cluster = clarray[(int)(Math.random()*numClusters)];
						int newcluster = unionHistory.split(cluster);
						if(newcluster >= 0) {
							clarray[numClusters++] = newcluster;
							untouchedClusters.add(newcluster);
						}
					}
				}
				if(c1 == c2) {
					while(c1 == c2) {
						unionHistory.split(c1);
						c1 = unionHistory.find(n1);
						c2 = unionHistory.find(n2);
					}
				} else {
					untouchedClusters.add(c1);
					untouchedClusters.add(c2);
				}
			}
		}
		Converter.UFB2Clustering(unionHistory, cl);
		logger.debug("after preparing clustering: " + cl.clusterCount() + " clusters left");
	}
	@Override
	public void mergingClusters(Cluster c1, Cluster c2) {
		unionHistory.union(unionHistory.find(c1.aMember().index()),
				unionHistory.find(c2.aMember().index()));
		
	}
}
