/*
 * Created on Feb 15, 2006
 *
 * Copyright 2005.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
*/
package de.uka.algo.clustering.algorithms.newman.internal;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;

import y.base.Edge;
import y.base.EdgeCursor;
import y.base.Graph;
import y.base.Node;
import y.base.NodeCursor;
import de.uka.algo.clustering.Cluster;
import de.uka.algo.clustering.Clustering;
import de.uka.algo.clustering.algorithms.newman.util.ClusterAdjacencyMatrix;
import de.uka.algo.clustering.algorithms.newman.util.FractionOfEdgeWeights;
import de.uka.algo.graphs.GraphInterpretation;
import de.uka.algo.util.GYCursor;
import de.uka.algo.util.datastructures.Pair;
import de.uka.algo.util.datastructures.Triple;


public class CoverageQInterface implements QQInterface {
	
	static org.apache.log4j.Logger logger = org.apache.log4j.Logger
			.getLogger(CoverageQInterface.class);
	
	private Clustering clustering;
	private boolean weighted;
	private FractionOfEdgeWeights frac;
	private ClusterAdjacencyMatrix adj;
	private GraphInterpretation interpretation;
	double degreeSum;
	double measurementQ;
	double measurementValue = 0.0;
	double measurementEValue = 0.0;
	boolean peakIsReached = false;
	
	private double edgeValue(Node n1, Node n2) {
		Edge e = edgeBetween(n1, n2);
		if (null == e)
			return 0.0;
		
		return weighted ? interpretation.getWeight(e) : 1.0;
	}
	
	/* (non-Javadoc)
	 * @see de.uka.algo.clustering.algorithms.newman2.util.ModularityLikeInterface#init(de.uka.algo.clustering.Clustering, de.uka.algo.clustering.algorithms.newman2.util.FractionOfEdgeWeights)
	 */
	public void init(Clustering c, FractionOfEdgeWeights frac, ClusterAdjacencyMatrix adj, boolean weighted){
		logger.info("Initializing Coverage modularity interface");

		this.clustering = c;
		//this.weighted = c.getInterpretation().isWeighted();
		this.weighted = weighted;
		logger.info("weighted: " + weighted);
		this.frac = frac;
		this.adj = adj;
		this.interpretation = c.getInterpretation();
		this.degreeSum = 2.0 * (weighted ? interpretation.getEdgeWeightSum() :
			clustering.getGraph().edgeCount());
		
		
		/* initialize the measurement
		 */
    	Graph graph = c.getGraph();
		measurementValue = 0.0;
		measurementEValue = 0.0;
		peakIsReached = false;
		
    	for (EdgeCursor ec = graph.edges(); ec.ok(); ec.next()) {
    		Node n1 = ec.edge().source(), n2 = ec.edge().target();
    		if (c.belongToSameCluster(n1, n2))
    			measurementValue += edgeValue(n1, n2);
    	}
    	measurementValue = 2.0 * measurementValue / degreeSum;
    	
    	for (GYCursor<Cluster> cc = c.clusters(); cc.ok(); cc.next()) {
    		double clusterDegreeFrac = frac.weightFracOf(cc.active()); 
    		measurementEValue += clusterDegreeFrac * clusterDegreeFrac;
    	}
    	
    	measurementQ = measurementValue-measurementEValue;
    	logger.info("Modularity nach init: " + measurementQ);
    	logger.info("Quality function: " + measurementValue);
    	logger.info("Expected value of QF: " + measurementEValue);
    	logger.info("Difference modularity: " + measurementQ);
	}
	
	/* (non-Javadoc)
	 * @see de.uka.algo.clustering.algorithms.newman2.util.ModularityLikeInterface#getInitialValue(de.uka.algo.clustering.Cluster, de.uka.algo.clustering.Cluster)
	 */
	public double getInitialDeltaQ(Cluster c1, Cluster c2){
		double v = getInitialDeltaQValue(c1, c2) - getInitialDeltaQEValue(c1, c2);
		logger.debug("initial deltaQ value of clusters "+c1.getIndex()+" and "
				+c2.getIndex()+": "+v);
		return v;
	}
	
	public double getNonInitialDeltaQNotConnected(Cluster c1, Cluster c2){
		double a_c1 = frac.weightFracOf(c1);
		double a_c2 = frac.weightFracOf(c2);
		//double newDeltaEValueMod = (a_c1+a_c2)*(a_c1+a_c2)-(a_c1*a_c1+a_c2*a_c2);
		// TODO: korrekte Formel?
		double newDeltaEValueMod = 2.0 * a_c1 * a_c2; 
		
		return -newDeltaEValueMod;
	}
	
	private Edge edgeBetween(Node anode, Node anode2) {
		for (EdgeCursor ec=anode.edges();ec.ok();ec.next())
			if ((ec.edge().target()==anode2)||(ec.edge().source()==anode2))return(ec.edge());
		return null;
	}
	
	
	/*
	 * (non-Javadoc)
	 * @see de.uka.algo.clustering.algorithms.newman2.neu.ModularityLikeInterface#getEValuePart(de.uka.algo.clustering.Cluster, de.uka.algo.clustering.Cluster)
	 */
	public double getEValuePart(Cluster c1, Cluster c2){
		return (2.0 * frac.weightFracOf(c1)*frac.weightFracOf(c2));
	}

	public boolean needsFullComputation() {
		return false;
	}

	public void updateModularity(double diff) {
		Logger logger = Logger.getLogger(this.getClass());
		logger.debug("Increasing value by " + diff + " to " + (measurementQ+diff));
		measurementQ += diff;
	}

	public double getModularity() {
		return measurementQ;
	}

	public Pair<Double, Boolean> getUpdatedDeltaQ(Cluster c1, Cluster c2, Cluster merged, Cluster cc, double deltaQ1, double deltaQ2) {
		double newDeltaQMod = 0;
		boolean dontStore = false;
		
		if(peakIsReached){
			newDeltaQMod = deltaQ1 + deltaQ2;
			return new Pair<Double, Boolean>(newDeltaQMod, dontStore);
		}
			
		
		if (adj.areAdjacent(cc, c1)) { // k adjacent to at least i
			if (adj.areAdjacent(cc, c2)) { // k adjacent to j too -> add stuff
				assert deltaQ1==deltaQ1;
				assert deltaQ2==deltaQ2;
				newDeltaQMod = deltaQ1 + deltaQ2; 
			}
			else { // k only adjacent to i
				assert deltaQ1==deltaQ1;
				newDeltaQMod = deltaQ1 - this.getEValuePart(cc, c2);
			}
		}
		else { // k not adjacent to i
			if (adj.areAdjacent(cc, c2)) { // k only adjacent to j
				assert deltaQ2==deltaQ2;
				newDeltaQMod = deltaQ2 - this.getEValuePart(cc, c1);
			}
			else { // neither i nor j adjacent to k
				/* in this case, i.e. the else-case, the value is
    			 * not computed.
    			 * 
    			 * Of course, it has changed, but is definitely negative.
    			 * 
    			 * So, as there are non-negative values in the matrix until
    			 * the peak is reached, the (negative) values for
    			 * non-connected pairs of clusters need not be computed
    			 * as long as the peak has not been reached.
    			 * 
    			 *  After the peak has been reached, a flag is set, and the
    			 *  computation takes place for these pairs, too.
    			 *  
        		 */
				
				
				/* peak not reached, otherwise the computation would have
				 * been done above.
				 */

				dontStore = true;
			}
		}
		return new Pair<Double, Boolean>(newDeltaQMod, dontStore);
	
	}

	public void setPeakIsReached() {
		peakIsReached = true;
	}

	public double getInitialDeltaQEValue(Cluster c1, Cluster c2) {
		return 2*frac.weightFracOf(c1)*frac.weightFracOf(c2);
	}

	public double getInitialDeltaQValue(Cluster c1, Cluster c2) {
		double crossingEdgesValue = 0.0;
		for (NodeCursor nc1 = c1.members(); nc1.ok(); nc1.next())
			for (NodeCursor nc2 = c2.members(); nc2.ok(); nc2.next()) {
				crossingEdgesValue += edgeValue(nc1.node(), nc2.node());
			}
		return 2.0 * crossingEdgesValue / degreeSum;
		 
	}

	public double getNonInitialDeltaEValueNotConnected(Cluster c1, Cluster c2) {
		double a_c1 = frac.weightFracOf(c1);
		double a_c2 = frac.weightFracOf(c2);
		//double newDeltaEValueMod = (a_c1+a_c2)*(a_c1+a_c2)-(a_c1*a_c1+a_c2*a_c2);
		// TODO: korrekte Formel?
		double newDeltaEValueMod = 2.0 * a_c1 * a_c2; 
		
		return newDeltaEValueMod;
	}

	public double getNonInitialDeltaValueNotConnected(Cluster c1, Cluster c2) {
		// 0 because disconnected
		return 0;
	}

	public void updateEValue(double diff) {
		Logger logger = Logger.getLogger(this.getClass());
		logger.info("Increasing value by " + diff + " to " + (measurementEValue+diff));
		measurementEValue += diff;
	}

	public void updateValue(double diff) {
		Logger logger = Logger.getLogger(this.getClass());
		logger.info("Increasing value by " + diff + " to " + (measurementValue+diff));
		measurementValue += diff;
	}

	public Pair<Double, Boolean> getUpdatedDeltaEValue(Cluster c1, Cluster c2, Cluster merged, Cluster cc, double deltaEValue1, double deltaEValue2) {
		double newDeltaEValueMod = 0;
		boolean dontStore = false;
		
		if(peakIsReached){
			newDeltaEValueMod = deltaEValue1 + deltaEValue2;
			return new Pair<Double, Boolean>(newDeltaEValueMod, dontStore);
		}
			
		if (adj.areAdjacent(cc, c1)) { // k adjacent to at least i
			if (adj.areAdjacent(cc, c2)) { // k adjacent to j too -> add stuff
				assert deltaEValue1==deltaEValue1;
				assert deltaEValue2==deltaEValue2;
				newDeltaEValueMod = deltaEValue1 + deltaEValue2; 
			}
			else { // k only adjacent to i
				assert deltaEValue1==deltaEValue1;
				newDeltaEValueMod = deltaEValue1 + this.getEValuePart(cc, c2);
			}
		}
		else { // k not adjacent to i
			if (adj.areAdjacent(cc, c2)) { // k only adjacent to j
				assert deltaEValue2==deltaEValue2;
				newDeltaEValueMod = deltaEValue2 + this.getEValuePart(cc, c1);
			}
			else { // neither i nor j adjacent to k
				/* in this case, i.e. the else-case, the value is
    			 * not computed.
    			 * 
    			 * Of course, it has changed, but is definitely negative.
    			 * 
    			 * So, as there are non-negative values in the matrix until
    			 * the peak is reached, the (negative) values for
    			 * non-connected pairs of clusters need not be computed
    			 * as long as the peak has not been reached.
    			 * 
    			 *  After the peak has been reached, a flag is set, and the
    			 *  computation takes place for these pairs, too.
    			 *  
        		 */
				
				
				/* peak not reached, otherwise the computation would have
				 * been done above.
				 */
				dontStore = true;
        	}
		}
		return new Pair<Double, Boolean>(newDeltaEValueMod, dontStore);
	}

	public Pair<Double, Boolean> getUpdatedDeltaValue(Cluster c1, Cluster c2, Cluster merged, Cluster cc, double deltaValue1, double deltaValue2) {
		double newDeltaValueMod = 0;
		boolean dontStore = false;

		if(peakIsReached){
			newDeltaValueMod = deltaValue1 + deltaValue2;
			return new Pair<Double, Boolean>(newDeltaValueMod, dontStore);
		}


		if (adj.areAdjacent(cc, c1)) { // k adjacent to at least i
			if (adj.areAdjacent(cc, c2)) { // k adjacent to j too -> add stuff
				assert deltaValue1==deltaValue1;
				assert deltaValue2==deltaValue2;
				newDeltaValueMod = deltaValue1 + deltaValue2; 
			}
			else { // k only adjacent to i
				assert deltaValue1==deltaValue1;
				newDeltaValueMod = deltaValue1; // deltaValue2 is 0
			}
		}
		else { // k not adjacent to i
			if (adj.areAdjacent(cc, c2)) { // k only adjacent to j
				assert deltaValue2==deltaValue2;
				newDeltaValueMod = deltaValue2; // deltaValue1 is 0
			}
			else { // neither i nor j adjacent to k
				/* in this case, i.e. the else-case, the value is
				 * not computed.
				 * 
				 * Of course, it has changed, but is definitely negative.
				 * 
				 * So, as there are non-negative values in the matrix until
				 * the peak is reached, the (negative) values for
				 * non-connected pairs of clusters need not be computed
				 * as long as the peak has not been reached.
				 * 
				 *  After the peak has been reached, a flag is set, and the
				 *  computation takes place for these pairs, too.
				 *  
				 */

				/* peak not reached, otherwise the computation would have
				 * been done above.
				 */

				dontStore = true;
			}
		}
		return new Pair<Double, Boolean>(newDeltaValueMod, dontStore);
	}

	public Triple<Double, Double, Boolean> getUpdatedValues(Cluster c1,
			Cluster c2, Cluster merged, Cluster cc,
			double deltaValue1, double deltaValue2,
			double deltaEValue1, double deltaEValue2) {
		
		double newDeltaQModZ = 0;
		double newDeltaQModN = 0;
		
		boolean dontStore = false;
		
		if(peakIsReached){
			newDeltaQModZ = deltaValue1 + deltaValue2;
			newDeltaQModN = deltaEValue1 + deltaEValue2;
			return new
				Triple<Double, Double, Boolean>(newDeltaQModZ, newDeltaQModN, dontStore);
		}
			
		
		if (adj.areAdjacent(cc, c1)) { // k adjacent to at least i
			if (adj.areAdjacent(cc, c2)) { // k adjacent to j too -> add stuff
				assert deltaValue1==deltaValue1;
				assert deltaValue2==deltaValue2;
				assert deltaEValue1==deltaEValue1;
				assert deltaEValue2==deltaEValue2;
				
				newDeltaQModZ = deltaValue1 + deltaValue2;
				newDeltaQModN = deltaEValue1 + deltaEValue2;
			}
			else { // k only adjacent to i
				assert deltaValue1==deltaValue1;
				assert deltaEValue1==deltaEValue1;
				
				newDeltaQModZ = deltaValue1;
				newDeltaQModN = deltaEValue1 + this.getEValuePart(cc, c2);
			}
		}
		else { // k not adjacent to i
			if (adj.areAdjacent(cc, c2)) { // k only adjacent to j
				assert deltaValue2==deltaValue2;
				assert deltaEValue2==deltaEValue2;
				
				newDeltaQModZ = deltaValue2;
				newDeltaQModN = deltaEValue2 + this.getEValuePart(cc, c1);
			}
			else { // neither i nor j adjacent to k
				/* in this case, i.e. the else-case, the value is
    			 * not computed.
    			 * 
    			 * Of course, it has changed, but is definitely negative.
    			 * 
    			 * So, as there are non-negative values in the matrix until
    			 * the peak is reached, the (negative) values for
    			 * non-connected pairs of clusters need not be computed
    			 * as long as the peak has not been reached.
    			 * 
    			 *  After the peak has been reached, a flag is set, and the
    			 *  computation takes place for these pairs, too.
    			 *  
        		 */
				
				
				/* peak not reached, otherwise the computation would have
				 * been done above.
				 */

				dontStore = true;
			}
		}
		return new Triple<Double, Double, Boolean>(newDeltaQModZ, newDeltaQModN, dontStore);
	}
	
	public double getEValue() {
		return measurementEValue;
	}

	public double getValue() {
		return measurementValue;
	}
}
