/*
 * Created on Feb 10, 2006
 *
 * Copyright 2005.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
*/
package de.uka.algo.clustering.algorithms.newman.internal;

import java.awt.Color;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Random;
import java.util.Set;

import y.base.Edge;
import y.base.Graph;
import y.base.Node;
import y.base.NodeCursor;
import y.base.NodeMap;
import y.io.YGFIOHandler;
import y.view.EdgeRealizer;
import y.view.Graph2D;
import y.view.NodeLabel;
import y.view.NodeRealizer;
import y.view.Port;
import yext.graphml.graph2D.GraphMLIOHandler;
import de.uka.algo.clustering.Cluster;
import de.uka.algo.clustering.ClusterMap;
import de.uka.algo.clustering.Clustering;
import de.uka.algo.clustering.ClusteringListener;
import de.uka.algo.clustering.DefaultListener;
import de.uka.algo.util.GYCursor;

/**
 * 
 * @author Jens Mueller
 *
 * enter comment here
 */
public class DendrogrammStorage {
	
	private Clustering clustering;
	private NodeMap nodeMap;
	private ClusterMap clusterIndizes;
	private ClusterMap clustersToMergeInfos;
	private Cluster[] indexToCluster;
	
	private class MergeInfo{
		public int cluster1;
		public int cluster2;
		public MergeInfo cl1; // null if single-node
		public MergeInfo cl2; //      --''--
		public double Q;
	}
	
	
	
	List<MergeInfo> merges = new LinkedList<MergeInfo>();
	
	ClusteringListener myListener;
	
	MergeInfo lastMergeBeforePeak;
	MergeInfo latestMerge;
	
	/**
	 * 
	 * @param cl clustering with which to initialize the store. Must be a singleton.
	 */
	public void init(Clustering cl){
		System.out.println("Initializing dendroStore");
		
		
		this.clustering = cl;
		Graph graph = cl.getGraph();
		this.nodeMap = graph.createNodeMap();
		int i = 0;
		for (NodeCursor nc = graph.nodes(); nc.ok(); nc.next()){
			Node n = nc.node();
			nodeMap.setInt(n, i);
			assert nodeMap.getInt(n) == i;
			i++;
		}
		
		this.indexToCluster = new Cluster[i];
		
		clusterIndizes = cl.createClusterMap();
		for (GYCursor<Cluster> cc = cl.clusters(); cc.ok(); cc.next()){
			Cluster c = cc.active();
			i = nodeMap.getInt(c.members().node());
			clusterIndizes.set(c, i);
			indexToCluster[i] = c;
		}
		
		clustersToMergeInfos = cl.createClusterMap();
		for (GYCursor<Cluster> cc = cl.clusters(); cc.ok(); cc.next()){
			assert clustersToMergeInfos.get(cc.active())==null;
			clustersToMergeInfos.set(cc.active(), null);
			assert clustersToMergeInfos.get(cc.active())==null;
		}
		
		myListener = new MyListener(cl);
		merges.clear();
		lastMergeBeforePeak = null;
		latestMerge = null;
	}
	
	public void merge(Cluster cl1, Cluster cl2, Cluster clNew){
		MergeInfo mi = new MergeInfo();
		
		if (clNew == cl1){
			mi.cluster1 = (Integer)clusterIndizes.get(cl1);
			mi.cluster2 = (Integer)clusterIndizes.get(cl2);
			
			mi.cl1 = (MergeInfo)clustersToMergeInfos.get(cl1);
			mi.cl2 = (MergeInfo)clustersToMergeInfos.get(cl2);
			
			System.out.print("");
		} else {
			assert false; // not reached
			
			mi.cluster2 = (Integer)clusterIndizes.get(cl1);
			mi.cluster1 = (Integer)clusterIndizes.get(cl2);
			
			mi.cl2 = (MergeInfo)clustersToMergeInfos.get(cl1);
			mi.cl1 = (MergeInfo)clustersToMergeInfos.get(cl2);
			System.out.print("");
		}
		
		merges.add(mi);
		latestMerge = mi;
		clustersToMergeInfos.set(clNew, mi);
		
	}
	
	public void detach(){
		clustering.removeClusteringListener(myListener);
	}
	
	private class MyListener extends DefaultListener{

		Cluster c1;
		Cluster c2;
		
		protected MyListener(Clustering clustering) {
			super(clustering);
			clustering.addListener(this);
		}

		@Override
		protected void mergeEvent(boolean notificationTime, Cluster a, Cluster b) {
			if (notificationTime == PRE){
				c1 = a;
				c2 = b;
			} else { // POST
				if (c1.size()!=0){
					merge(c1, c2, c1);
				} else {
					merge(c1, c2, c2);
				}
			}
		}

		@Override
		protected void defaultEvent() {
			// TODO Auto-generated method stub
			
		}

		@Override
		protected String id() {
			return "Dendro.inner";
		}

		@Override
		protected void policyChanged(boolean policy) {
			// TODO Auto-generated method stub
			
		}
	}
	
	public Graph2D createDendrogrammDiagonalEdges(){
		/*
		 * constants for layout
		 */
		final int distance_xy = 20;
		final int size_xy = 10;
		
		
		int nodeCount = clustering.getGraph().nodeCount();
		int clusterCount = clustering.clusterCount();
		if (clusterCount != 1)
			throw new RuntimeException("dendrogram not possible for " + clusterCount + " clusters.");
		
		Cluster cluster = clustering.clusters().active();
		
		List<Integer> clustersOrdered = new ArrayList<Integer>();
		
		/*
		 * Tiefensuche -> Einzel-Cluster in Liste eintragen.
		 */
		mergeDFSHelper((MergeInfo)clustersToMergeInfos.get(cluster), clustersOrdered);
		
		Graph2D g = new Graph2D();
		Node[] dendroNodes = new Node[nodeCount];
		
		
		/*
		 * Knoten für Einzelcluster erstellen
		 */
		int addedNodes = 0;
		for (Integer i : clustersOrdered) {
			Node node = g.createNode(distance_xy*addedNodes,0,i.toString());
			NodeRealizer nr = g.getRealizer(node);
			
			nr.setSize(size_xy, size_xy);
			// nr.setCenter(distance_xy*addedNodes, 0);
			nr.setFillColor(Color.RED);
			//nr.createNodeLabel().setText(i.toString());
			nr.getLabel().setFontSize(7);
			
			dendroNodes[i] = node;
			addedNodes++;
		}
		int height = 0;
		
		/*
		 * Merges abarbeiten
		 */
		Color nodeColor = Color.BLUE;
		for (MergeInfo mi: merges) {
			++height;
			Node node1 = dendroNodes[mi.cluster1];
			Node node2 = dendroNodes[mi.cluster2];
			
			double posx1 = g.getRealizer(node1).getCenterX();
			double posx2 = g.getRealizer(node2).getCenterX();
			String labelText = g.getLabelText(node1) + "," + g.getLabelText(node2);
			Node node = g.createNode(posx1/2+posx2/2, -height*distance_xy, labelText);
			NodeRealizer nr = g.getRealizer(node);
			nr.setSize(size_xy, size_xy);
			nr.setFillColor(nodeColor);
			// nr.setLabelText(new Double(mi.Q).toString());
			nr.getLabel().setFontSize(5);
			nr.setWidth(g.getSize(node1).width + g.getSize(node2).width);
			
			Edge e1 = g.createEdge(node1, node);
			Edge e2 = g.createEdge(node2, node);
			g.getRealizer(e1).setLineColor(Color.BLUE);
			g.getRealizer(e2).setLineColor(Color.BLUE);
			g.getRealizer(e1).setVisible(true);
			g.getRealizer(e2).setVisible(true);
			
			dendroNodes[mi.cluster1] = node;
			
			if (mi == lastMergeBeforePeak)
				nodeColor = Color.GREEN;
		}
		

		
		try {
			new YGFIOHandler().write(g, "dendro_diagonal.ygf");
			new GraphMLIOHandler().write(g, "dendro_diagonal.graphml");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		// TODO
		return null;
	}
	
	public Graph2D createDendrogrammRectangularEdgesAtSide(){
		/*
		 * constants for layout
		 */
		final int distance_xy = 20;
		final int size_xy = 10;
		
		
		int nodeCount = clustering.getGraph().nodeCount();
		int clusterCount = clustering.clusterCount();
		if (clusterCount != 1)
			throw new RuntimeException("dendrogram not possible for " + clusterCount + " clusters.");
		
		Cluster cluster = clustering.clusters().active();
		
		List<Integer> clustersOrdered = new ArrayList<Integer>();
		
		/*
		 * Tiefensuche -> Einzel-Cluster in Liste eintragen.
		 */
		mergeDFSHelper((MergeInfo)clustersToMergeInfos.get(cluster), clustersOrdered);
		
		Graph2D g = new Graph2D();
		Node[] dendroNodes = new Node[nodeCount];
		
		
		/*
		 * Knoten für Einzelcluster erstellen
		 */
		int addedNodes = 0;
		for (Integer i : clustersOrdered) {
			Node node = g.createNode(distance_xy*addedNodes,0,i.toString());
			NodeRealizer nr = g.getRealizer(node);
			
			nr.setSize(size_xy, size_xy);
			// nr.setCenter(distance_xy*addedNodes, 0);
			nr.setFillColor(Color.RED);
			//nr.createNodeLabel().setText(i.toString());
			nr.getLabel().setFontSize(7);
			
			dendroNodes[i] = node;
			addedNodes++;
		}
		int height = 0;
		
		
		/*
		 * Merges abarbeiten
		 */
		Color nodeColor = Color.BLUE;
		for (MergeInfo mi: merges) {
			++height;
			Node node1 = dendroNodes[mi.cluster1];
			Node node2 = dendroNodes[mi.cluster2];
			
			double posx1 = g.getRealizer(node1).getCenterX();
			double posx2 = g.getRealizer(node2).getCenterX();
			String labelText = g.getLabelText(node1) + "," + g.getLabelText(node2);
			Node node = g.createNode(posx1/2+posx2/2, -height*distance_xy, labelText);
			NodeRealizer nr = g.getRealizer(node);
			nr.setSize(size_xy, size_xy);
			nr.setFillColor(nodeColor);
			// nr.setLabelText(new Double(mi.Q).toString());
			nr.getLabel().setFontSize(5);
			nr.setWidth(g.getSize(node1).width + g.getSize(node2).width);
			
			Edge e1 = g.createEdge(node1, node);
			Edge e2 = g.createEdge(node2, node);
			g.getRealizer(e1).setLineColor(Color.BLUE);
			g.getRealizer(e2).setLineColor(Color.BLUE);
			g.getRealizer(e1).setVisible(true);
			g.getRealizer(e2).setVisible(true);
			
			/*
			 * Edges entsprechend dem Layout anordnen
			 */
			EdgeRealizer er = g.getRealizer(e1);
			er.setSourcePort(new Port(0, -size_xy/2));
			er.setTargetPort(new Port(-size_xy/2, 0));
			er.addPoint(g.getRealizer(node1).getCenterX(),g.getRealizer(node).getCenterY());
			
			er = g.getRealizer(e2);
			er.setSourcePort(new Port(0, -size_xy/2));
			//er.setTargetPort(new Port(size_xy/2, 0));
			er.addPoint(g.getRealizer(node2).getCenterX(),g.getRealizer(node).getCenterY());
			
			
			dendroNodes[mi.cluster1] = node;
			
			if (mi == lastMergeBeforePeak)
				nodeColor = Color.GREEN;
		}
		
		
		try {
			new YGFIOHandler().write(g, "dendro_side.ygf");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		// TODO
		return null;
	}
	
	
	
	
	
	
	
	
	
	
	public Graph2D createDendrogrammRectangularEdgesDown(){
		final Color baseClustersColor = Color.RED;
		final Color defaultClustersColor = Color.BLUE;
		final Color overPeakClusterColor = Color.GREEN;
		final Color peakClusterColor = Color.YELLOW;
		
		/*
		 * constants for layout
		 */
		final int distance_xy = 20;
		final int size_xy = 10;
		
		
		int nodeCount = clustering.getGraph().nodeCount();
		int clusterCount = clustering.clusterCount();
		if (clusterCount != 1)
			throw new RuntimeException("dendrogram not possible for " + clusterCount + " clusters.");
		
		Cluster cluster = clustering.clusters().active();
		
		List<Integer> clustersOrdered = new ArrayList<Integer>();
		
		/*
		 * Tiefensuche -> Einzel-Cluster in Liste eintragen.
		 */
		mergeDFSHelper((MergeInfo)clustersToMergeInfos.get(cluster), clustersOrdered);
		
		Graph2D g = new Graph2D();
		Node[] dendroNodes = new Node[nodeCount];
		
		
		/*
		 * Knoten für Einzelcluster erstellen
		 */
		int addedNodes = 0;
		for (Integer i : clustersOrdered) {
			Node node = g.createNode(distance_xy*addedNodes,0,i.toString());
			NodeRealizer nr = g.getRealizer(node);
			
			nr.setSize(size_xy, size_xy);
			// nr.setCenter(distance_xy*addedNodes, 0);
			nr.setFillColor(baseClustersColor);
			//nr.createNodeLabel().setText(i.toString());
			nr.getLabel().setFontSize(5);
			
			dendroNodes[i] = node;
			addedNodes++;
		}
		int height = 0;
		
		
		/*
		 * Merges abarbeiten
		 */
		Color nodeColor = defaultClustersColor;
		boolean peakIsReached = false;
		Set<Node> overPeakClusters = new HashSet<Node>();
		for (MergeInfo mi: merges) {
			++height;
			Node node1 = dendroNodes[mi.cluster1];
			Node node2 = dendroNodes[mi.cluster2];
			
			double posx1 = g.getRealizer(node1).getCenterX();
			double posx2 = g.getRealizer(node2).getCenterX();
			String labelText = g.getLabelText(node1) + "," + g.getLabelText(node2);
			Node node = g.createNode(posx1/2+posx2/2, -height*distance_xy, labelText);
			NodeRealizer nr = g.getRealizer(node);
			nr.setSize(size_xy, size_xy);
			nr.setFillColor(nodeColor);
			if(peakIsReached){
				overPeakClusters.add(node);
				if (!overPeakClusters.contains(node1))
					g.getRealizer(node1).setFillColor(peakClusterColor);
				if (!overPeakClusters.contains(node2))
					g.getRealizer(node2).setFillColor(peakClusterColor);
			}
			
			
			nr.getLabel().setFontSize(5);
			nr.setWidth(g.getSize(node1).width + g.getSize(node2).width);
			
			Edge e1 = g.createEdge(node1, node);
			Edge e2 = g.createEdge(node2, node);
			g.getRealizer(e1).setLineColor(Color.BLUE);
			g.getRealizer(e2).setLineColor(Color.BLUE);
			g.getRealizer(e1).setVisible(true);
			g.getRealizer(e2).setVisible(true);
			
			
			
			
			
			/*
			 * Edges entsprechend dem Layout anordnen
			 */
			EdgeRealizer er = g.getRealizer(e1);
			er.setSourcePort(new Port(0, -size_xy/2));
			er.setTargetPort(new Port(0, size_xy/2));
			er.addPoint(g.getRealizer(node1).getCenterX(), g.getRealizer(node).getCenterY()+size_xy*1.25);
			er.addPoint(g.getRealizer(node).getCenterX(), g.getRealizer(node).getCenterY()+size_xy*1.25);
			
			
			
			
			er = g.getRealizer(e2);
			er.setSourcePort(new Port(0, -size_xy/2));
			// er.setTargetPort(new Port(0, size_xy/2));
			er.addPoint(g.getRealizer(node2).getCenterX(), g.getRealizer(node).getCenterY()+size_xy*1.25);
			er.addPoint(g.getRealizer(node).getCenterX(), g.getRealizer(node).getCenterY()+size_xy*1.25);
			
			/*
			 * Q-Label plazieren
			 */
			double q = (double)((int)(mi.Q*10000))/10000.0;
			NodeLabel nl = new NodeLabel();
			nl.setVisible(true);
			nl.setText(Double.toString(q));
			nl.setFontSize(4);
			
			//nl.
			//nl.setOffset(0, size_xy*3.0/4.0);
			g.getRealizer(node).addLabel(nl);
			nl.setModel(NodeLabel.FREE);
			nl.setOffset(0, size_xy);
			//nl.set
			
			
			dendroNodes[mi.cluster1] = node;
			
			if (mi == lastMergeBeforePeak){
				nodeColor = overPeakClusterColor;
				peakIsReached = true;
			}
		}
		
		
		try {
			new YGFIOHandler().write(g, "dendro_down.ygf");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		
		return g;
	}
	
	
	private void mergeDFSHelper(MergeInfo mi, List<Integer> list){
		boolean one_first = new Random().nextInt(2)==0;
		
		
		if (one_first){
			if (mi.cl1 == null)
				list.add(mi.cluster1);
			else
				mergeDFSHelper(mi.cl1, list);
		}

		if (mi.cl2 == null)
			list.add(mi.cluster2);
		else
			mergeDFSHelper(mi.cl2, list);
		
		if (!one_first){
			if (mi.cl1 == null)
				list.add(mi.cluster1);
			else
				mergeDFSHelper(mi.cl1, list);
		}
		
	}
	
	public Clustering replay(int count, Clustering cl){
		if (count>this.merges.size())
			throw new IllegalArgumentException("Too many steps.");
		
		cl.resetSingletons();
		
		Cluster[] clusters = new Cluster[cl.clusterCount()];
		for (GYCursor<Cluster> cc = cl.clusters(); cc.ok(); cc.next()){
			Cluster c = cc.active();
			int i = nodeMap.getInt(c.members().node());
			clusters[i] = c;
		}
		
		Iterator<MergeInfo> iter = merges.iterator();
		for(int i=0; i<count; i++){
			MergeInfo mi = iter.next();
			clusters[mi.cluster1] = cl.merge(clusters[mi.cluster1], clusters[mi.cluster2]);
		}
		
		return cl;
	}
	
	public void setHasPeak(){
		this.lastMergeBeforePeak = latestMerge;
	}
	
	public void setLatestModularity(double Q){
		latestMerge.Q = Q;
	}
}
