/*
 * Created on Feb 10, 2006
 *
 * Copyright 2006.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
*/
package de.uka.algo.clustering.algorithms.newman.internal;

import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Logger;

import y.base.Graph;
import de.uka.algo.clustering.Cluster;
import de.uka.algo.clustering.Clustering;
import de.uka.algo.clustering.algorithms.newman.util.ClusterAdjacencyMatrix;
import de.uka.algo.clustering.algorithms.newman.util.FractionOfEdgeWeights;
import de.uka.algo.util.GYCursor;
import de.uka.algo.util.datastructures.Pair;

public class GenericAdditiveDeltaQMatrix implements GenericDeltaQMatrix {
	Map<Cluster, BoundedHeap<Cluster>> deltaMeasurementMatrix;
	BoundedHeap<Cluster> bestOfEachRow;
	
	Clustering clustering;
	ClusterAdjacencyMatrix adj;
	FractionOfEdgeWeights frac;
	boolean weighted;
	boolean peakIsReached = false;
	QInterface modInterface;
	boolean fullComputation;
	
	
	public GenericAdditiveDeltaQMatrix(QInterface modI){
		Logger logger = Logger.getLogger(this.getClass());
		
		this.modInterface = modI;
		fullComputation = modI.needsFullComputation();
		logger.info("Full computation at start: " + fullComputation);
	}
	
	/* (non-Javadoc)
	 * @see de.uka.algo.clustering.algorithms.newman2.neu.GenericDeltaQMatrix#init(de.uka.algo.clustering.Clustering, de.uka.algo.clustering.algorithms.newman2.alt.ClusterAdjacencyMatrix, de.uka.algo.clustering.algorithms.newman2.util.FractionOfEdgeWeights, boolean)
	 */
	public void init(Clustering cl, ClusterAdjacencyMatrix adj, FractionOfEdgeWeights frac, boolean weighted){
		Logger logger = Logger.getLogger(this.getClass());
		logger.info("Initializing matrix for quotient modularity.");

		peakIsReached = false;
		Graph graph = cl.getGraph();
		this.clustering = cl;
		this.adj = adj;
		this.frac = frac;
		// GraphInterpretation graphInt = cl.getInterpretation();
		this.weighted = weighted;
		
		
		modInterface.init(cl, frac, adj, weighted);
		
		
		double deltaij =0;
    	
    	deltaMeasurementMatrix = new HashMap<Cluster, BoundedHeap<Cluster>>();
    	
    	for (GYCursor<Cluster> i = clustering.clusters(); i.ok(); i.next()) {
    		Cluster curCl1 = i.active();
    		
    		BoundedHeap<Cluster> currentHeap = new BoundedHeap<Cluster>(clustering.size()-1);
    		
    		for (GYCursor<Cluster> j = clustering.clusters(); j.ok(); j.next()) {
    			Cluster curCl2 = j.active();
    			
    			if (curCl1.equals(curCl2)) continue;
    			
    			boolean Aij = adj.areAdjacent(curCl1, curCl2);
    			
    			if (!Aij && !fullComputation) continue;
    			/*
    			 * if the measurement supports it, only compute
    			 * Delta Q for connected clusters at the beginning.
    			 */
    			
    			
    			deltaij = modInterface.getInitialDeltaQ(curCl1, curCl2);
    			currentHeap.insertElement(deltaij,curCl2);
    		}
    		
    		/*
    		 * only store non-empty heaps
    		 */
    		
    		if (currentHeap.size()>0){
    			deltaMeasurementMatrix.put(curCl1, currentHeap);
    		}
    		
    	}
    	
        /*
         * initializebestOfEachRow():
         * 
         * Initializes the global heap, storing the best values of all cluster
         * heaps.
         * 
         */
    	
        bestOfEachRow = new BoundedHeap<Cluster>(clustering.size());
        rebuildGlobalHeap();
    	
	}
	
    /* (non-Javadoc)
	 * @see de.uka.algo.clustering.algorithms.newman2.neu.GenericDeltaQMatrix#merge(de.uka.algo.clustering.Cluster, de.uka.algo.clustering.Cluster, de.uka.algo.clustering.Cluster)
	 */
    public void merge(Cluster c1, Cluster c2, Cluster combinedCluster){
        Logger logger = Logger.getLogger(this.getClass());
    	
    	modInterface.updateModularity(this.getValue(c1, c2));

    	logger.info("Starting recomputation (merge of two clusters)");
    	BoundedHeap<Cluster> heapOfMergedCluster = new BoundedHeap<Cluster>(clustering.clusterCount()-1); //too large - only store adjacent clusters
    	// loop over all clusters i.e. heaps
    	for (GYCursor<Cluster> cc = clustering.clusters(); cc.ok(); cc.next()) {
    		Cluster currentCluster = cc.active();
    		// excluding those of the merge partners, they will be deleted
    		if (currentCluster.equals(c1) || currentCluster.equals(c2)) continue;
    		
    		
    		// the heap that is updated during this iteration
    		BoundedHeap<Cluster> currentHeap = deltaMeasurementMatrix.get(currentCluster);
    		
    		if (currentHeap != null) {
    			Pair<Double, Boolean> pair = 
    				modInterface.getUpdatedDeltaQ(c1, c2, combinedCluster, currentCluster,
    						currentHeap.valueOfElement(c1), currentHeap.valueOfElement(c2));
    			double newDeltaQMod = pair.getFirst();
    			boolean dontStore = pair.getSecond();
    			if (!dontStore){
        			// entries for both old merge partners are removed
        			currentHeap.removeElement(c1);
        			currentHeap.removeElement(c2);
        			currentHeap.insertElement(newDeltaQMod, combinedCluster);
        			heapOfMergedCluster.insertElement(newDeltaQMod, currentCluster);
        		}
    		}
    	}       
    	
    	
    	deltaMeasurementMatrix.remove(c1);
    	deltaMeasurementMatrix.remove(c2);
    	deltaMeasurementMatrix.put(combinedCluster, heapOfMergedCluster);
    	
    	/* rebuilds the global heap */

        bestOfEachRow.clear(); // tear down

        if (clustering.clusterCount() > 1){
        	for (GYCursor<Cluster> j = clustering.clusters(); j.ok(); j.next()){
        		Cluster clusterJ = j.active();
        		BoundedHeap<Cluster> heapForJ = deltaMeasurementMatrix.get(clusterJ);
        		if (heapForJ!=null && heapForJ.size()>0){
        			bestOfEachRow.insertElement(heapForJ.getMaximumValue(), clusterJ);
        		}
        	}
        }
    }
	
    /* (non-Javadoc)
	 * @see de.uka.algo.clustering.algorithms.newman2.neu.GenericDeltaQMatrix#setPeakIsReached()
	 */
    public void setPeakIsReached(){
    	peakIsReached = true;
    	modInterface.setPeakIsReached();
    }
    
    /* (non-Javadoc)
	 * @see de.uka.algo.clustering.algorithms.newman2.neu.GenericDeltaQMatrix#recomputeForAll()
	 */
    public void recomputeForAll(){
    	Logger logger = Logger.getLogger(this.getClass());
    	
    	for (GYCursor<Cluster> cc = clustering.clusters(); cc.ok(); cc.next()){
			Cluster c1 = cc.active();
			
			/* currentHeap holen
			 */
			BoundedHeap<Cluster> currentHeap;
			if (deltaMeasurementMatrix.containsKey(c1)){
				currentHeap  = deltaMeasurementMatrix.get(c1);
			} else {
				currentHeap = new BoundedHeap<Cluster>(clustering.clusterCount()-1);
				deltaMeasurementMatrix.put(c1, currentHeap);
			}
			
			/* 
			 * Only compute values for pairs not already in the matrix.
			 */
			
			for (GYCursor<Cluster> cc2 = clustering.clusters(); cc2.ok(); cc2.next()){
				Cluster c2 = cc2.active();
				if (c1 == c2) continue;
				if (currentHeap.existsElement(c2)) continue;
				
				{
					/* compute deltaQ(c1, c2) for disconnected
					 * c1 and c2
					 */
					
					if (adj.areAdjacent(c1, c2)){
						logger.error("Recomputation for adjacent clusters.");
						throw new RuntimeException("Something has gone terribly wrong ...");
					}
						
					currentHeap.insertElement(modInterface.getNonInitialDeltaQNotConnected(c1, c2), c2);
	    		} // block
			} // inner 'for' loop
		} // outer 'for' loop
		rebuildGlobalHeap();
    }
    
    /* (non-Javadoc)
	 * @see de.uka.algo.clustering.algorithms.newman2.neu.GenericDeltaQMatrix#isEmpty()
	 */
    public boolean isEmpty(){
    	return bestOfEachRow.size()==0;
    }
    
    /* (non-Javadoc)
	 * @see de.uka.algo.clustering.algorithms.newman2.neu.GenericDeltaQMatrix#getMaximumCluster1()
	 */
    public Cluster getMaximumCluster1(){
    	return bestOfEachRow.getMaximumElement();
    }
    
    /* (non-Javadoc)
	 * @see de.uka.algo.clustering.algorithms.newman2.neu.GenericDeltaQMatrix#getMaximumCluster2()
	 */
    public Cluster getMaximumCluster2(){
    	return deltaMeasurementMatrix.get(getMaximumCluster1()).getMaximumElement();
    }
    
    /* (non-Javadoc)
	 * @see de.uka.algo.clustering.algorithms.newman2.neu.GenericDeltaQMatrix#getMaximumValue()
	 */
    public double getMaximumValue(){
    	Logger logger = Logger.getLogger(this.getClass());
    	double maxValue = deltaMeasurementMatrix.get(getMaximumCluster1()).getMaximumValue();
    	// logger.info("best deltaQ: " + maxValue);
    	return maxValue;
    }
    
    /* (non-Javadoc)
	 * @see de.uka.algo.clustering.algorithms.newman2.neu.GenericDeltaQMatrix#getValue(de.uka.algo.clustering.Cluster, de.uka.algo.clustering.Cluster)
	 */
    public double getValue(Cluster c1, Cluster c2){
    	return deltaMeasurementMatrix.get(c1).valueOfElement(c2);
    }
    
    private void rebuildGlobalHeap(){
    	bestOfEachRow.clear(); // tear down

        if (clustering.clusterCount() > 1){
        	for (GYCursor<Cluster> j = clustering.clusters(); j.ok(); j.next()){
        		Cluster clusterJ = j.active();
        		BoundedHeap<Cluster> heapForJ = deltaMeasurementMatrix.get(clusterJ);
        		if (heapForJ!=null && heapForJ.size()>0){
        			bestOfEachRow.insertElement(heapForJ.getMaximumValue(), clusterJ);
        		}
        	}
        }
    }
    
}
