/*
 * Created on Feb 10, 2006
 *
 * Copyright 2006.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
*/
package de.uka.algo.clustering.algorithms.newman.internal;

import java.util.HashMap;
import java.util.Map;

import y.base.Graph;

import de.uka.algo.clustering.Cluster;
import de.uka.algo.clustering.ClusterMap;
import de.uka.algo.clustering.Clustering;
import de.uka.algo.clustering.algorithms.newman.util.ClusterAdjacencyMatrix;
import de.uka.algo.clustering.algorithms.newman.util.FractionOfEdgeWeights;
import de.uka.algo.util.GYCursor;
import de.uka.algo.util.datastructures.Triple;

public class GenericQuotientDeltaQMatrix implements GenericDeltaQMatrix {
	Map<Cluster, BoundedHeapOfClusterWithExtras> deltaMeasurementMatrix;
	
	BoundedHeap<Cluster> bestOfEachRow;
	
	Clustering clustering;
	ClusterAdjacencyMatrix adj;
	FractionOfEdgeWeights frac;
	boolean weighted;
	boolean peakIsReached = false;
	QQInterface modInterface;
	boolean fullComputation;
	
	/*
     * DEBUG only
     */
    ClusterMap cl_map;
	
	public GenericQuotientDeltaQMatrix(QQInterface modI){
		this.modInterface = modI;
		// fullComputation = modI.needsFullComputation();
		fullComputation = true; //temporary, for quotient (TODO)
	}
	
	/**
	 * 
	 * @param cl a (singleton) clustering
	 */
	public void init(Clustering cl, ClusterAdjacencyMatrix adj, FractionOfEdgeWeights frac, boolean weighted){
		Graph graph = cl.getGraph();
		this.clustering = cl;
		this.adj = adj;
		this.frac = frac;
		// GraphInterpretation graphInt = cl.getInterpretation();
		this.weighted = weighted;
		
		 /* DEBUG only :
         * fill cl_map
         */
        cl_map = cl.createClusterMap();
        for(GYCursor<Cluster> cc = cl.clusters(); cc.ok(); cc.next()){
        	System.out.println("Setting code for cluster: " + cc.active().aMember().hashCode());
        	cl_map.set(cc.active(), cc.active().aMember().hashCode());
        }
		
		
		modInterface.init(cl, frac, adj, weighted);
		
		System.out.println("initial modularity: " + modInterface.getValue() + "/" + modInterface.getEValue() + "="
				+ modInterface.getValue() / modInterface.getEValue());
		
		double deltaij;
		double deltaijZ, deltaijN;
    	
    	deltaMeasurementMatrix = new HashMap<Cluster, BoundedHeapOfClusterWithExtras>();
    	
    	for (GYCursor<Cluster> i = clustering.clusters(); i.ok(); i.next()) {
    		Cluster curCl1 = i.active();
    		
    		BoundedHeapOfClusterWithExtras currentHeap = new BoundedHeapOfClusterWithExtras(graph.N()-1);
    		//modInterface.init wurde aufgerufen!
    		double measurementValue = modInterface.getValue();
    		double measurementEValue = modInterface.getEValue();
    		
    		for (GYCursor<Cluster> j = clustering.clusters(); j.ok(); j.next()) {
    			Cluster curCl2 = j.active();
    			
    			if (curCl1.equals(curCl2)) continue;
    			
    			boolean Aij = adj.areAdjacent(curCl1, curCl2);
    			
    			if (!Aij && !fullComputation) continue;
    			/*
    			 * if the measurement supports it, only compute
    			 * Delta Q for connected clusters at the beginning.
    			 */
    			
    			
    			deltaijZ = modInterface.getInitialDeltaQValue(curCl1, curCl2);
    			deltaijN = modInterface.getInitialDeltaQEValue(curCl1, curCl2);
    			deltaij = (measurementValue + deltaijZ)/(measurementEValue + deltaijN);
    			
    			 /* DEBUG */
                System.out.println("initial value for " + (Integer)cl_map.get(i.active()) + "/" + (Integer)cl_map.get(j.active()) 
                		+ ": " + deltaijZ + "/" + deltaijN + "->"
                		+ deltaij);
    			
    			currentHeap.insertElement(deltaij, curCl2, deltaijZ, deltaijN);
    		}
    		
    		/*
    		 * only store non-empty heaps
    		 */
    		
    		if (currentHeap.size()>0){
    			deltaMeasurementMatrix.put(curCl1, currentHeap);
    		}
    		
    	}
    	
        /*
         * Initializes the global heap, storing the best values of all cluster
         * heaps.
         */
    	
        bestOfEachRow = new BoundedHeap<Cluster>(graph.N());
        rebuildGlobalHeap();
    	
	}

	public void merge(Cluster c1, Cluster c2, Cluster combinedCluster){
		// modInterface.updateModularity(this.getQ(c1, c2));
		// dieser Wert wird für Quotient nicht benutzt ...
		
		
		//System.out.println("nicht initiale Werte: " + modInterface.getValue() + "/" + modInterface.getEValue() + "="
		//		+ modInterface.getValue() / modInterface.getEValue());
		
		modInterface.updateValue(this.getValue(c1, c2));
		modInterface.updateEValue(this.getEValue(c1, c2));
		
		double measurementValue = modInterface.getValue();
		double measurementEValue = modInterface.getEValue();
		
		
		/* DEBUG only */
        int value1 = (Integer)cl_map.get(c1);
        int value2 = (Integer)cl_map.get(c2);
        cl_map.set(combinedCluster, value1 < value2 ? value1 : value2);
        System.out.println("merged " + value1 +  " and " + value2 + " -> " + (value1 < value2 ? value1 : value2));
        
		
		BoundedHeapOfClusterWithExtras heapOfMergedCluster = new BoundedHeapOfClusterWithExtras(clustering.clusterCount()-1); //too large - only store adjacent clusters
		// loop over all clusters i.e. heaps
		for (GYCursor<Cluster> cc = clustering.clusters(); cc.ok(); cc.next()) {
			Cluster currentCluster = cc.active();
			// excluding those of the merge partners, they will be deleted
			if (currentCluster.equals(c1) || currentCluster.equals(c2)) continue;


			// the heap that is updated during this iteration
			BoundedHeapOfClusterWithExtras currentHeap = deltaMeasurementMatrix.get(currentCluster);
			
			if (currentHeap == null) 
				continue;
			
			Triple<Double, Double, Boolean> triple
				= modInterface.getUpdatedValues(c1, c2,
						combinedCluster, currentCluster,
						currentHeap.getMeasurementValue(c1),
						currentHeap.getMeasurementValue(c2),
						currentHeap.getMeasurementEValue(c1),
						currentHeap.getMeasurementEValue(c2));
			
			double newDeltaValueMod = triple.getFirst();
			double newDeltaEValueMod = triple.getSecond();
			//boolean dontStore = triple.getThird();    <-- TODO
			boolean dontStore = false;
			double newDeltaQMod = (measurementValue+newDeltaValueMod)/(measurementEValue+newDeltaEValueMod);
			
			if (dontStore)
				System.out.println("meep!"); // debug
			
			if (!dontStore){
				// entries for both old merge partners are removed
				currentHeap.removeCluster(c1);
				currentHeap.removeCluster(c2);


				currentHeap.insertElement(newDeltaQMod, combinedCluster, newDeltaValueMod, newDeltaEValueMod);
				heapOfMergedCluster.insertElement(newDeltaQMod, currentCluster, newDeltaValueMod, newDeltaEValueMod);
			}

		}




		deltaMeasurementMatrix.remove(c1);
		deltaMeasurementMatrix.remove(c2);
		deltaMeasurementMatrix.put(combinedCluster, heapOfMergedCluster);

		/* tear down global heap */
		bestOfEachRow.clear(); // tear down

		/* recompute everything */
		for (GYCursor<Cluster> i = clustering.clusters(); i.ok(); i.next()){
			Cluster cl1 = i.active();
			if (deltaMeasurementMatrix.containsKey(cl1)){
				BoundedHeapOfClusterWithExtras currentHeap = deltaMeasurementMatrix.get(cl1);
				for (GYCursor<Cluster> j = clustering.clusters(); j.ok(); j.next()){
					Cluster cl2 = j.active();
					
					if(cl1 != cl2)
						if(currentHeap.existsElement(cl2)){
							double temp = currentHeap.valueOfCluster(cl2);
							double tempV = currentHeap.getMeasurementValue(cl2);
							double tempEV = currentHeap.getMeasurementEValue(cl2);
							
							double newValue = (tempV+measurementValue)/(tempEV+measurementEValue); 
							if (newValue != temp){
								currentHeap.removeCluster(cl2);
								currentHeap.insertElement(newValue, cl2, tempV, tempEV);
								// System.out.println("updated");
							} else {
								// System.out.println("not updated");
							}
							
							
							
						} else {
							throw new RuntimeException("Inner error");
						}
				}
			} else {
				throw new RuntimeException("Outer error");
			}
		}
		
		
		
		/* rebuilds the global heap */
		if (clustering.clusterCount() > 1){
			for (GYCursor<Cluster> j = clustering.clusters(); j.ok(); j.next()){
				Cluster clusterJ = j.active();
				BoundedHeapOfClusterWithExtras heapForJ = deltaMeasurementMatrix.get(clusterJ);
				if (heapForJ!=null && heapForJ.size()>0){
					bestOfEachRow.insertElement(heapForJ.getMaximumValue(), clusterJ);
				}
			}
		}
		
		System.out.println("neue Modularity: " + modInterface.getValue()/modInterface.getEValue());
		

	}
	
    public void setPeakIsReached(){
    	peakIsReached = true;
    }
    
    public void recomputeForAll(){
    	/*
		 * als nächstes: Modularity updaten
		 */
		
    	double measurementValue = modInterface.getValue();
		double measurementEValue = modInterface.getEValue();
		
    	
		for (GYCursor<Cluster> cc = clustering.clusters(); cc.ok(); cc.next()){
			Cluster c1 = cc.active();
			
			/* currentHeap holen
			 */
			BoundedHeapOfClusterWithExtras currentHeap;
			if (deltaMeasurementMatrix.containsKey(c1)){
				currentHeap  = deltaMeasurementMatrix.get(c1);
			} else {
				currentHeap = new BoundedHeapOfClusterWithExtras(clustering.clusterCount()-1);
				deltaMeasurementMatrix.put(c1, currentHeap);
			}
			
			/* 
			 * Only compute values for pairs not already in the matrix.
			 */
			
			for (GYCursor<Cluster> cc2 = clustering.clusters(); cc2.ok(); cc2.next()){
				Cluster c2 = cc2.active();
				if (c1 == c2) continue;
				if (currentHeap.existsElement(c2)) continue;
				
				{
					/* compute deltaQ(c1, c2) for disconnected
					 * c1 and c2
					 */
					
					if (adj.areAdjacent(c1, c2)){
						System.out.println("c1 und c2 sind adjazent!");
						throw new RuntimeException("Something has gone terribly wrong ...");
					}
					
					double valueMod = modInterface.getNonInitialDeltaValueNotConnected(c1, c2);
					double eValueMod = modInterface.getNonInitialDeltaEValueNotConnected(c1, c2); 
					double qMod = (measurementValue+valueMod)/(measurementEValue+eValueMod);
					
					currentHeap.insertElement(qMod, c2, valueMod, eValueMod);
	    			
				} // block
			} // inner 'for' loop
		} // outer 'for' loop
		rebuildGlobalHeap();
    }
    
    public boolean isEmpty(){
    	return bestOfEachRow.size()==0;
    }
    
    public Cluster getMaximumCluster1(){
    	return bestOfEachRow.getMaximumElement();
    }
    
    public Cluster getMaximumCluster2(){
    	return deltaMeasurementMatrix.get(getMaximumCluster1()).getMaximumCluster();
    }
    
    public double getMaximumValue(){
    	return deltaMeasurementMatrix.get(getMaximumCluster1()).getMaximumValue();
    }
    
    public double getQ(Cluster c1, Cluster c2){
    	return deltaMeasurementMatrix.get(c1).valueOfCluster(c2);
    }
    
    public double getValue(Cluster c1, Cluster c2){
    	return deltaMeasurementMatrix.get(c1).getMeasurementValue(c2);
    }
    
    public double getEValue(Cluster c1, Cluster c2){
    	return deltaMeasurementMatrix.get(c1).getMeasurementEValue(c2);
    }
    
    
    private void rebuildGlobalHeap(){
    	bestOfEachRow.clear(); // tear down

        if (clustering.clusterCount() > 1){
        	for (GYCursor<Cluster> j = clustering.clusters(); j.ok(); j.next()){
        		Cluster clusterJ = j.active();
        		BoundedHeapOfClusterWithExtras heapForJ = deltaMeasurementMatrix.get(clusterJ);
        		if (heapForJ!=null && heapForJ.size()>0){
        			bestOfEachRow.insertElement(heapForJ.getMaximumValue(), clusterJ);
        		}
        	}
        }
    }
    
}
