package de.uka.algo.clustering.algorithms.newman.internal;

import y.base.Edge;
import y.base.EdgeCursor;
import y.base.Node;
import de.uka.algo.graphs.GraphInterpretation;
import de.uka.algo.graphs.WeightedGraph;

/**
 * A graph that represents the difference between two graphs with common nodes.
 * 
 * The weight w(u,v) of an edge is the difference w_new(u,v) - w_old(u,v)
 * between the weight of the edge in the new and in the old graph.
 * 
 * @author maillard
 *
 */
class GraphDelta extends WeightedGraph {
	GraphDelta(GraphInterpretation oldgr, GraphInterpretation newgr) throws IllegalArgumentException{
		super(newgr);
		if(oldgr == null)
			return;
		
		if(oldgr.getGraph().N() != newgr.getGraph().N())
			throw new IllegalArgumentException();
		
		for(EdgeCursor oldec = oldgr.getGraph().edges(); oldec.ok(); oldec.next()) {
			double weightDiff = 0.0;
			Edge oldEdge = oldec.edge();
			weightDiff -= oldgr.getWeight(oldEdge);

			int si = oldEdge.source().index();
			int ti = oldEdge.target().index();
			Node s,t;
			try {
				s = graph.getNodeArray()[si];
				t = graph.getNodeArray()[ti];
			} catch (Exception ex) {
				throw new IllegalArgumentException();
			}

			if(!graph.containsEdge(s, t)) {
				Edge e = graph.createEdge(s,t);
				setWeight(e, weightDiff);
			}
			else
			{
				Edge myEdge = s.getEdgeTo(t);
				weightDiff += getWeight(myEdge);
				if(Double.compare(weightDiff, 0) == 0)
					graph.removeEdge(myEdge);
				else
					setWeight(myEdge, weightDiff);
			}
		}
	}
}
