/*
 * Created on Feb 15, 2006
 *
 * Copyright 2005.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
*/
package de.uka.algo.clustering.algorithms.newman.internal;

import org.apache.log4j.Logger;

import y.base.Edge;
import y.base.EdgeCursor;
import y.base.Graph;
import y.base.Node;
import y.base.NodeCursor;
import de.uka.algo.clustering.Cluster;
import de.uka.algo.clustering.Clustering;
import de.uka.algo.clustering.algorithms.newman.util.ClusterAdjacencyMatrix;
import de.uka.algo.clustering.algorithms.newman.util.FractionOfEdgeWeights;
import de.uka.algo.graphs.GraphInterpretation;
import de.uka.algo.util.datastructures.Pair;
import de.uka.algo.util.datastructures.Triple;


public class PerformanceQInterface implements QInterface, QQInterface {
	private Clustering clustering;
	private boolean weighted;
	// private FractionOfEdgeWeights frac;
	private GraphInterpretation interpretation;
	double edgeWeightSum;
	double edgeCount;
	double nodeCount;
	private ClusterAdjacencyMatrix adj;
	Graph graph;
	double N, M, W, edges;
	double measurementQ;
	double measurementValue = 0.0;
	double measurementEValue = 0.0;
	
	
	/* (non-Javadoc)
	 * @see de.uka.algo.clustering.algorithms.newman2.util.ModularityLikeInterface#init(de.uka.algo.clustering.Clustering, de.uka.algo.clustering.algorithms.newman2.util.FractionOfEdgeWeights)
	 */
	public void init(Clustering c, FractionOfEdgeWeights frac, ClusterAdjacencyMatrix adj, boolean weighted){
		Logger logger = Logger.getLogger(this.getClass());
		logger.info("Initializing Performance modularity interface");
		
		this.clustering = c;
		this.weighted = weighted;
		logger.info("weighted: " + weighted);
		// this.frac = frac;
		this.interpretation = c.getInterpretation();
		this.edgeCount=(double)clustering.getGraph().edgeCount();
		this.edgeWeightSum = interpretation.getEdgeWeightSum();
		this.nodeCount =(double)clustering.getGraph().nodeCount();
		this.adj = adj;
		this.graph = c.getGraph();
		this.N = graph.N();
		this.W = c.getInterpretation().getEdgeWeightSum();
		for (EdgeCursor ec = graph.edges(); ec.ok();ec.next())
	    	if (M < interpretation.getWeight(ec.edge()))
	    		M=interpretation.getWeight(ec.edge());
		this.edges = graph.edgeCount();
		
		initializeModularity();
		
	}
	
	/* (non-Javadoc)
	 * @see de.uka.algo.clustering.algorithms.newman2.util.ModularityLikeInterface#getInitialValue(de.uka.algo.clustering.Cluster, de.uka.algo.clustering.Cluster)
	 */
	public double getInitialDeltaQ(Cluster c1, Cluster c2){
		double deltaijValue, deltaijEValue, deltaij;
		double M = this.edges;
		/*
		 * Apparently, the "compute partially" optimization is not
		 * possible for perfomance modularity ...
		 */
		
		boolean Aij = adj.areAdjacent(c1, c2);
		// double ki = frac.weightFracOf(c1);
		// double kj = frac.weightFracOf(c2);
		double ki=c1.members().node().degree();
        double kj=c2.members().node().degree();
		
		
		if (!weighted){
    		deltaijValue =-1.0/(0.5*N*(N-1.0));
    		if (Aij) deltaijValue*=-1.0;  
            deltaijEValue = (2.0/M*ki*kj-2.0)/(N*(N-1.0)); // keep degrees
        } else {
    		double wij =0.0;
    		if (Aij) wij=(interpretation.getWeight(edgeBetween(c1.members().node(),c2.members().node())));
    		deltaijValue = (2.0* wij - M) / (0.5*N*(N-1.0)*M);
    		deltaijEValue = (2.0/W * weigthOf(c1.members().node()) * weigthOf(c2.members().node()) -2.0*M) / (N*(N-1.0)*M);
    	}

        deltaij = deltaijValue - deltaijEValue;
		
       return deltaij;
	}
	
	public double getNonInitialDeltaQNotConnected(Cluster c1, Cluster c2){
		throw new RuntimeException("Everything must be computed initially for Performance Modularity.");
	}
	
	private Edge edgeBetween(Node anode, Node anode2) {
		for (EdgeCursor ec=anode.edges();ec.ok();ec.next())
			if ((ec.edge().target()==anode2)||(ec.edge().source()==anode2))return(ec.edge());
		return null;
	}
	
	public double getEValuePart(Cluster c1, Cluster c2){
		//not reached
		throw new RuntimeException("should not be reached");
	}

	public boolean needsFullComputation() {
		return true;
	}
	
    private double weigthOf(Node anode) {
        double weight=0;
        for (EdgeCursor ec=anode.edges();ec.ok();ec.next())
            weight+=interpretation.getWeight(ec.edge());
        return weight;
    }

	public void updateModularity(double diff) {
		Logger logger = Logger.getLogger(this.getClass());
		logger.info("Increasing value by " + diff + " to " + (measurementValue+diff));
		measurementValue += diff;
	}
	
    private void initializeModularity() {
        Logger logger = Logger.getLogger(this.getClass());
    	
    	// double temp = 0.0;
        double m=graph.edgeCount();
        double n=graph.nodeCount();
        double ki=0.0;
        double modularityN=0.0;
        double modularityZ=0.0;
        double wi=0;
		double sumOfWvSquares =0.0;
		double sumOfKiSquares =0.0;
		// double myW=0.0;
		
		for (NodeCursor i = graph.nodes(); i.ok(); i.next()) {
			ki=i.node().degree();
			sumOfKiSquares += (double)(ki * ki);
			wi = weigthOf(i.node());
			sumOfWvSquares += wi*wi;;
		} 
		wi=0.0;
        
    	if (!weighted){
    		modularityN= 1.0+ (1.0/m * sumOfKiSquares - 2.0*m) / (n*(n-1.0));
    		modularityZ = 1.0-(m/(0.5*(n*(n-1.0)))); //performance
    	} else {
    		modularityN= (1.0/W* sumOfWvSquares + M*n*n - M*n -2.0*W) / (n*(n-1.0)*M);  //std nullHyp but weighted
    		modularityZ= 1.0 - W/(0.5*n*(n-1.0)*M);
    	}
        
        measurementQ = modularityZ-modularityN;
    	/*
    	 * TODO: Hier muß bekannt sein, ob Quotient oder Differenz ...
    	 * 
    	 * Erstmal ist die Annahme: Gesamtwert wird bei Quotient
    	 * nicht benutzt, also wird die Summe gespeichert.
    	 */
    	
    	this.measurementValue = modularityZ;
    	this.measurementEValue = modularityN;
    	
    	logger.info("Quality function: " + measurementValue);
    	logger.info("Expected value of QF: " + measurementEValue);
    	logger.info("Difference modularity: " + measurementQ);
    }

	public double getModularity() {
		return measurementQ;
		//throw new RuntimeException("not supported for quotient.");
	}

	public Pair<Double, Boolean> getUpdatedDeltaQ(Cluster c1, Cluster c2, Cluster merged, Cluster cc, double deltaQ1, double deltaQ2) {
		return new Pair<Double, Boolean>(deltaQ1+deltaQ2, false);
	}

	public void setPeakIsReached() {
		// peak does not matter here ...
		return;
	}

	public double getInitialDeltaQEValue(Cluster c1, Cluster c2) {
		double deltaijEValue;
		 double m = this.edges;
		/*
		 * Apparently, the "compute partially" optimization is not
		 * possible for perfomance modularity ...
		 */
		
		// boolean Aij = adj.areAdjacent(c1, c2);
		double ki=c1.members().node().degree();
        double kj=c2.members().node().degree();
		
		
		if (!weighted){
    		deltaijEValue = (2.0/m*ki*kj-2.0)/(N*(N-1.0)); // keep degrees
        } else {
    		deltaijEValue = (2.0/W * weigthOf(c1.members().node()) * weigthOf(c2.members().node()) -2.0*M) / (N*(N-1.0)*M);
    	}

		 /* DEBUG */
        /* System.out.println("--------");
        System.out.println("weighted: " + weighted);
        System.out.println("ki: " + ki);
        System.out.println("kj: " + kj);
        System.out.println("W: " + W);
        System.out.println("M: " + M);
        System.out.println("m: " + m);
		System.out.println("N: " + N);
		System.out.println("weigthOf(c1.members().node(): " + weigthOf(c1.members().node()));
		System.out.println("weigthOf(c2.members().node(): " + weigthOf(c2.members().node()));
		System.out.println("wi*wj: " + weigthOf(c1.members().node()) * weigthOf(c2.members().node()));
         */
		
        return deltaijEValue;
	}

	public double getInitialDeltaQValue(Cluster c1, Cluster c2) {
		double deltaijValue;
		//double M = this.edges; (TODO)
		/*
		 * Apparently, the "compute partially" optimization is not
		 * possible for perfomance modularity ...
		 */
		
		boolean Aij = adj.areAdjacent(c1, c2);
		//double ki=c1.members().node().degree();
        //double kj=c2.members().node().degree();
		
		double wij =0.0;
		
		if (!weighted){
    		deltaijValue =-1.0/(0.5*N*(N-1.0));
    		if (Aij) deltaijValue*=-1.0;  
        } else {
    		// double wij =0.0;
    		if (Aij) wij=(interpretation.getWeight(edgeBetween(c1.members().node(),c2.members().node())));
    		deltaijValue = (2.0* wij - M) / (0.5*N*(N-1.0)*M);
    	}

		 /* DEBUG */
        /* System.out.println("--------");
        System.out.println("weighted: " + weighted);
        System.out.println("Aij: " + Aij);
        System.out.println("W: " + W);
        System.out.println("M: " + M);
		System.out.println("N: " + N);
		System.out.println("wij: " + wij); */
		
		return deltaijValue;
	}

	public double getNonInitialDeltaEValueNotConnected(Cluster c1, Cluster c2) {
		throw new RuntimeException("Everything must be computed initially for Performance Modularity.");
	}

	public double getNonInitialDeltaValueNotConnected(Cluster c1, Cluster c2) {
		throw new RuntimeException("Everything must be computed initially for Performance Modularity.");
	}

	public Triple<Double, Double, Boolean> getUpdatedValues(Cluster c1, Cluster c2, Cluster merged, Cluster cc, double deltaValue1, double deltaValue2, double deltaEValue1, double deltaEValue2) {
		return new Triple<Double, Double, Boolean>(deltaValue1+deltaValue2, deltaEValue1+deltaEValue2, false);
	}

	public void updateEValue(double diff) {
		Logger logger = Logger.getLogger(this.getClass());
		logger.info("Increasing value by " + diff + " to " + (measurementEValue+diff));
		measurementEValue += diff;
	}

	public void updateValue(double diff) {
		Logger logger = Logger.getLogger(this.getClass());
		logger.info("Increasing value by " + diff + " to " + (measurementValue+diff));
		measurementValue += diff;
	}

	public double getEValue() {
		return measurementEValue;
	}

	public double getValue() {
		return measurementValue;
	}
}
