package de.uka.algo.clustering.algorithms.newman.internal;

import org.apache.log4j.Level;

import de.uka.algo.clustering.Cluster;
import de.uka.algo.clustering.Clustering;
import de.uka.algo.clustering.algorithms.newman.util.Converter;
import de.uka.algo.clustering.algorithms.newman.util.UnionFindBacktrack;
import de.uka.algo.graphs.GraphInterpretation;

public class RandomBacktrackUpdateStrategy extends UpdateStrategy {
	static org.apache.log4j.Logger logger = org.apache.log4j.Logger
	.getLogger(RandomBacktrackUpdateStrategy.class);

	UnionFindBacktrack unionHistory = null;
	
	final private int LAMBDA=10;

	@Override
	public void init(Clustering cl) {
		logger.setLevel(Level.INFO);
		unionHistory = new UnionFindBacktrack(cl.getGraph().N());
	}
	@Override
	public void prepareClustering(Clustering cl,
			GraphInterpretation oldGraph) {
		GraphDelta delta = new GraphDelta(oldGraph, cl.getInterpretation());
		int numAffEdges = delta.getGraph().E();
		int numClusters = cl.clusterCount();
		int [] clusters = new int[numClusters+LAMBDA*numAffEdges];
		for(int i=0; i<numClusters; i++) {
			clusters[i] = unionHistory.find(cl.getCluster(i).aMember().index());
		}
		for(int l=0; l<LAMBDA*numAffEdges; l++) {
			int cluster = clusters[(int)(Math.random()*numClusters)];
			int newcluster = unionHistory.split(cluster);
			if(newcluster >= 0) {
				clusters[numClusters++] = newcluster;
			}
		}
		Converter.UFB2Clustering(unionHistory, cl);
	}
	@Override
	public void mergingClusters(Cluster c1, Cluster c2) {
		unionHistory.union(unionHistory.find(c1.aMember().index()),
				unionHistory.find(c2.aMember().index()));
		
	}
}
