/*
 * Created on Feb 10, 2006
 *
 * Copyright 2005.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
*/
package de.uka.algo.clustering.algorithms.newman.util;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import y.base.Graph;
import y.base.NodeCursor;
import de.uka.algo.clustering.Cluster;
import de.uka.algo.clustering.Clustering;
import de.uka.algo.util.GYCursor;

public class ClusterAdjacencyMatrix {
	Map<Cluster, Set<Cluster>> clusterAdjacency;
	Clustering clustering;
	
	/**
	 * 
	 * @param cl a (not necessarily singleton) clustering
	 */
	public void init(Clustering cl){
		this.clustering = cl;
		
		// load factor 1.0 because the number of clusters is not going to
		// increase
		clusterAdjacency = new HashMap<Cluster, Set<Cluster>>(
				cl.clusterCount(), 1.0f);
		for (GYCursor<Cluster> cc = cl.clusters(); cc.ok(); cc.next()) {
			Set<Cluster> adjClusters = new HashSet<Cluster>();
			for (NodeCursor nc = cc.active().members(); nc.ok(); nc.next()) {
				for (NodeCursor adjnc = nc.node().neighbors(); adjnc.ok(); adjnc.next()) {
					adjClusters.add(cl.getCluster(adjnc.node()));
				}
			}
			clusterAdjacency.put(cc.active(), adjClusters);
		}
	}
	
	public void merge(Cluster c1, Cluster c2, Cluster combinedCluster){
    	Set<Cluster> adjCombinedCl = new HashSet<Cluster>(clustering.clusterCount()-1);
    	Set<Cluster> adjC1 = clusterAdjacency.get(c1);
    	Set<Cluster> adjC2 = clusterAdjacency.get(c2);
    	
    	for (GYCursor<Cluster> cc = clustering.clusters(); cc.ok(); cc.next()){
    		Cluster c = cc.active();
    		if (adjC1.contains(c) || adjC2.contains(c)){
    			adjCombinedCl.add(c);
    			Set<Cluster> adjCurrent = clusterAdjacency.get(c);
    			/*
    			 * delete just takes time ...
    			 */
    			// adjCurrent.remove(c1);
    			// adjCurrent.remove(c2);
    			// adjCurrent.put(combinedCluster, true);
    			adjCurrent.add(combinedCluster);
    		}
    	}
    	/*
    	 * delete only costs time ...
    	 */
    	// clusterAdjacency.remove(c1);
    	// clusterAdjacency.remove(c2);
    	clusterAdjacency.put(combinedCluster, adjCombinedCl);
	}
	
	public boolean areAdjacent(Cluster c1, Cluster c2) {
    	Set<Cluster> setForC1 = clusterAdjacency.get(c1);
    	return setForC1.contains(c2);
    }
	
}
