/*
 * Created on Feb 10, 2006
 *
 * Copyright 2005.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
*/
package de.uka.algo.clustering.algorithms.newman.util;

import java.util.HashMap;
import java.util.Map;

import y.base.EdgeCursor;
import y.base.Graph;
import y.base.Node;
import y.base.NodeCursor;
import de.uka.algo.clustering.Cluster;
import de.uka.algo.clustering.Clustering;
import de.uka.algo.graphs.GraphInterpretation;
import de.uka.algo.util.GYCursor;

public class FractionOfEdgeWeights {
	Map<Cluster, Double> fracOfClusterDeg;
	GraphInterpretation graphInt;
	
	public void init(Clustering cl, boolean weighted){
		// boolean weighted = cl.getInterpretation().isWeighted();
		Graph graph = cl.getGraph();
		int n = graph.N();
		this.graphInt = cl.getInterpretation();
		
		
		fracOfClusterDeg = new HashMap<Cluster, Double>(n);
		double degreeSum = 2.0 * (weighted ?
				graphInt.getEdgeWeightSum() : (double)graph.E());
		for (GYCursor<Cluster> cc = cl.clusters();cc.ok();cc.next()){
			Cluster currentCl = cc.active();
			double clusterDegree = 0.0;
			for (NodeCursor nc = currentCl.members(); nc.ok(); nc.next()) {
				double nodeDegree = (weighted ? weightOf(nc.node()) :
							(double)nc.node().degree());
				clusterDegree += nodeDegree;
			}
			fracOfClusterDeg.put(currentCl, clusterDegree / degreeSum);
		}
 	}
	
	private double weightOf(Node anode) {
        double weight=0;
        for (EdgeCursor ec=anode.edges();ec.ok();ec.next())
            weight+=graphInt.getWeight(ec.edge());
        return weight;
    }
	
	public double weightFracOf(Cluster cl){
		return fracOfClusterDeg.get(cl);
	}
	
	public void merge(Cluster c1, Cluster c2, Cluster combinedCluster){
		 double newFrac = fracOfClusterDeg.get(c1)+fracOfClusterDeg.get(c2);
	     fracOfClusterDeg.remove(c1);
	     fracOfClusterDeg.remove(c2);
	     fracOfClusterDeg.put(combinedCluster, newFrac);
	}

}
