/*
 * created May 4, 2006
 *
 * Copyright 2006.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
 *
 */
package de.uka.algo.clustering.comparison.graphbased.counting;

import de.uka.algo.clustering.Clustering;
import de.uka.algo.clustering.comparison.Comparator;
import de.uka.algo.clustering.comparison.ComparatorFactory;
import de.uka.algo.clustering.comparison.ConnectedCountingPairSet;

public class FowlkesMallowsGraphExt implements Comparator {

	private static final String ID = "Fowlkes-Mallows (g)";
	ConnectedCountingPairSet ccps;
	
	public FowlkesMallowsGraphExt(Clustering a, Clustering b){
		ccps = ConnectedCountingPairSet.getConnectedCountingPairSet(a,b);
	}
		
	public double getValue() {
		int e11 = ccps.getE11();
		int e10 = ccps.getE10();
		int e01 = ccps.getE01();
		if(e11+e01 != 0 && e11+e10 != 0){
			return 1-e11/Math.sqrt((e11+e10)*(e11+e01));
		} else if(e11 == 0){
			return 1;
		} else {
			return 0;
		}
	}

	
	@Override
	public String toString() {
		return ID;
	}

	public static class Factory implements ComparatorFactory {
		public Comparator getComparator(Clustering a, Clustering b) {
			return new FowlkesMallowsGraphExt(a, b);
		}		
		@Override
		public String toString() {
			return ID;
		}
	}

}
