/*
 * created May 5, 2006
 *
 * Copyright 2006.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
 *
 */
package de.uka.algo.clustering.comparison.graphbased.informationTheory;

import de.uka.algo.clustering.Clustering;
import de.uka.algo.clustering.comparison.Comparator;
import de.uka.algo.clustering.comparison.ComparatorFactory;
import de.uka.algo.clustering.comparison.MutualEdgeInformation;

public class VariationOfInformationGraphExt implements Comparator {
	
	private static final String ID = "Variation of Information n-norm(g)";
	private MutualEdgeInformation mei;
	private Clustering cl1;
	private Clustering cl2;

	public VariationOfInformationGraphExt(Clustering a, Clustering b){
		mei = MutualEdgeInformation.getMutualEdgeInformation(a,b);
		cl1 = a;
		cl2 = b;
	}
	
	public double getNonNormalizedValue() {
		return cl1.getEdgeEntropy()+cl2.getEdgeEntropy()-2*mei.getMutualEdgeInfo();
	}

	public double getValue() {
		return getNonNormalizedValue()/(Math.log10(cl1.getGraph().N())/Math.log10(2));
	}
	
	public double getClusterSizeNormalizedValue() {
		return getNonNormalizedValue()/(2*Math.log10(Math.max(cl1.clusterCount(),cl2.clusterCount()))/Math.log10(2));
	}
	
	@Override
	public String toString() {
		return ID;
	}

	public static class Factory implements ComparatorFactory {
		public Comparator getComparator(Clustering a, Clustering b) {
			return new VariationOfInformationGraphExt(a, b);
		}		
		@Override
		public String toString() {
			return ID;
		}
	}

}
