/*
 * created May 4, 2006
 *
 * Copyright 2006.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
 *
 */
package de.uka.algo.clustering.comparison.graphbased.overlaps;

import de.uka.algo.clustering.Cluster;
import de.uka.algo.clustering.Clustering;
import de.uka.algo.clustering.comparison.Comparator;
import de.uka.algo.clustering.comparison.ComparatorFactory;
import de.uka.algo.clustering.comparison.DegreeConfusionMatrix;
import de.uka.algo.util.GYCursor;

public class FMeasureGraphExt implements Comparator {

	private static final String ID = "F-Measure (g)";
	DegreeConfusionMatrix dcm;
	Clustering cl1;
	Clustering cl2;
	
	public FMeasureGraphExt(Clustering a, Clustering b){
		dcm = DegreeConfusionMatrix.getDegreeConfusionMatrix(a,b);
		cl1 = a;
		cl2 = b;
	}
	
	
	
	public double getValue() {
			
			double sum = 0;
			for (GYCursor<Cluster> cc1 = cl1.clusters(); cc1.ok(); cc1.next()) {
				Cluster c1 = cc1.active();
				double max = 0;
				int c1DegreeSum = 2*cl1.getInnerEdgeCount(c1)+cl1.getBorderEdgeCount(c1);
				for (GYCursor<Cluster> cc2 = cl2.clusters(); cc2.ok(); cc2.next()) {
					Cluster c2 = cc2.active();
					int c2DegreeSum = 2*cl2.getInnerEdgeCount(c2)+cl2.getBorderEdgeCount(c2);
					double f_ci_cj = 2*dcm.getIntersectionCardinality(c1,c2);
					f_ci_cj /= c1DegreeSum + c2DegreeSum ;
					if (f_ci_cj > max) max = f_ci_cj;
				}
				sum += c1DegreeSum * max;
			}
			
			return 1-sum/(2*cl1.getGraph().E());
			
		
	}


	@Override
	public String toString() {
		return ID;
	}

	public static class Factory implements ComparatorFactory {
		public Comparator getComparator(Clustering a, Clustering b) {
			return new FMeasureGraphExt(a, b);
		}		
		@Override
		public String toString() {
			return ID;
		}
	}
	public static class RevFactory implements ComparatorFactory {
		public Comparator getComparator(Clustering a, Clustering b) {
			return new FMeasureGraphExt(b, a);
		}		
		@Override
		public String toString() {
			return ID+" rev";
		}
	}

}
