/*
 * created May 4, 2006
 *
 * Copyright 2006.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
 *
 */
package de.uka.algo.clustering.comparison.graphbased.overlaps;

import java.util.HashSet;
import java.util.Set;

import de.uka.algo.clustering.Cluster;
import de.uka.algo.clustering.Clustering;
import de.uka.algo.clustering.comparison.Comparator;
import de.uka.algo.clustering.comparison.ComparatorFactory;
import de.uka.algo.clustering.comparison.DegreeConfusionMatrix;
import de.uka.algo.util.GYCursor;

public class MaximumMatchGraphExt implements Comparator {

	private static final String ID = "Maximum Match (g)";
	DegreeConfusionMatrix dcm;
	Clustering cl1;
	Clustering cl2;
	
	public MaximumMatchGraphExt(Clustering a, Clustering b){
		dcm = DegreeConfusionMatrix.getDegreeConfusionMatrix(a,b);
		cl1 = a;
		cl2 = b;
	}
	
	
	
	public double getValue() {
			Set<Cluster> clusters1 = new HashSet<Cluster>(cl1.clusterCount());
			for (GYCursor<Cluster> cc = cl1.clusters(); cc.ok(); cc.next()){
				clusters1.add(cc.active());
			}
			Set<Cluster> clusters2 = new HashSet<Cluster>(cl2.clusterCount());
			for (GYCursor<Cluster> cc = cl2.clusters(); cc.ok(); cc.next()){
				clusters2.add(cc.active());
			}
			
			int sum = 0;
			
			
			do{
				Cluster cluster1 = null;
				Cluster cluster2 = null;
				int max = -1;
				
				for (Cluster c1: clusters1){
					for (Cluster c2: clusters2){
						int m_c1_c2 = dcm.getIntersectionCardinality(c1, c2);
						if (m_c1_c2 > max){
							max = m_c1_c2;
							cluster1 = c1;
							cluster2 = c2;
						}
					}
				}
				
				sum += max;
				clusters1.remove(cluster1);
				clusters2.remove(cluster2);
				
			} while (!clusters1.isEmpty() && !clusters2.isEmpty());
			
			return 1-sum/(double)(2*cl1.getGraph().E());
		
	}

	@Override
	public String toString() {
		return ID;
	}

	public static class Factory implements ComparatorFactory {
		public Comparator getComparator(Clustering a, Clustering b) {
			return new MaximumMatchGraphExt(a, b);
		}		
		@Override
		public String toString() {
			return ID;
		}
	}

}
