/*
 * created May 4, 2006
 *
 * Copyright 2006.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
 *
 */
package de.uka.algo.clustering.comparison.graphbased.overlaps;

import de.uka.algo.clustering.Cluster;
import de.uka.algo.clustering.Clustering;
import de.uka.algo.clustering.comparison.Comparator;
import de.uka.algo.clustering.comparison.ComparatorFactory;
import de.uka.algo.clustering.comparison.DegreeConfusionMatrix;
import de.uka.algo.util.GYCursor;

public class MeilaHeckermanGraphExt implements Comparator {

	private static final String ID = "Meila-Heckerman (g)";
	DegreeConfusionMatrix dcm;
	Clustering cl1;
	Clustering cl2;
	
	public MeilaHeckermanGraphExt(Clustering a, Clustering b){
		dcm = DegreeConfusionMatrix.getDegreeConfusionMatrix(a,b);
		cl1 = a;
		cl2 = b;
	}
	
	
	
	public double getValue() {
		double mh = 0;
		for (GYCursor<Cluster> cc = cl1.clusters(); cc.ok(); cc.next()) {
			Cluster c = cc.active();
			mh += dcm.getMaximumIntersectionToCluster(c);
		}
		return 1-(mh/(2*cl1.getGraph().E()));
		
	}

	@Override
	public String toString() {
		return ID;
	}

	public static class Factory implements ComparatorFactory {
		public Comparator getComparator(Clustering a, Clustering b) {
			return new MeilaHeckermanGraphExt(a, b);
		}		
		@Override
		public String toString() {
			return ID;
		}
	}
	public static class RevFactory implements ComparatorFactory {
		public Comparator getComparator(Clustering a, Clustering b) {
			return new MeilaHeckermanGraphExt(b, a);
		}		
		@Override
		public String toString() {
			return ID+" rev";
		}
	}

}
