/*
 * created May 4, 2006
 *
 * Copyright 2006.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
 *
 */
package de.uka.algo.clustering.comparison.graphbased.overlaps;

import de.uka.algo.clustering.Cluster;
import de.uka.algo.clustering.Clustering;
import de.uka.algo.clustering.comparison.Comparator;
import de.uka.algo.clustering.comparison.ComparatorFactory;
import de.uka.algo.clustering.comparison.DegreeConfusionMatrix;
import de.uka.algo.util.GYCursor;

public class vanDongenGraphExt implements Comparator {

	private static final String ID = "van Dongen norm (g)";
	DegreeConfusionMatrix dcm;
	Clustering cl1;
	Clustering cl2;
	
	public vanDongenGraphExt(Clustering a, Clustering b){
		dcm = DegreeConfusionMatrix.getDegreeConfusionMatrix(a,b);
		cl1 = a;
		cl2 = b;
	}
	
	
	
	public double getNonNormalizedValue() {
		int sum1 = 0;
		int sum2 = 0;
		for (GYCursor<Cluster> cc = cl1.clusters(); cc.ok(); cc.next()) {
			Cluster c = cc.active();
			sum1 += dcm.getMaximumIntersectionToCluster(c);
		}
		for (GYCursor<Cluster> cc = cl2.clusters(); cc.ok(); cc.next()) {
			Cluster c = cc.active();
			sum2 += dcm.getMaximumIntersectionToCluster(c);
		}
		double n = cl1.getGraph().N();
		double m = cl1.getGraph().E();
		return 2*n-n/(2*m)*(sum1+sum2);
		
	}

	public double getValue() {
		return getNonNormalizedValue()/(2*cl1.getGraph().N());
	}

	@Override
	public String toString() {
		return ID;
	}

	public static class Factory implements ComparatorFactory {
		public Comparator getComparator(Clustering a, Clustering b) {
			return new vanDongenGraphExt(a, b);
		}		
		@Override
		public String toString() {
			return ID;
		}
	}

}
