/*
 * created May 4, 2006
 *
 * Copyright 2006.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
 *
 */
package de.uka.algo.clustering.comparison.nodebased.counting;

import de.uka.algo.clustering.Clustering;
import de.uka.algo.clustering.comparison.Comparator;
import de.uka.algo.clustering.comparison.ComparatorFactory;
import de.uka.algo.clustering.comparison.CountingPairSet;

/**
 * implements the adjusted Rand Measure:
 * 
 * 1-((n_11-t3)/(0.5*(t1+t2)-t3));
 * 
 * where t1 = n_11+n_10, t2=n_11+n_01, t3=2*t1*t2/(n(n-1))
 * 
 * @author Daniel Delling
 * 
 */
public class AdjustedRand implements Comparator {

	final static String ID = "adjusted Rand";

	CountingPairSet cps;

	public AdjustedRand(Clustering a, Clustering b) {
		cps = CountingPairSet.getCountingPairSet(a, b);
	}

	public double getValue() {
		double t1 = (cps.getN11() + cps.getN10());
		double t2 = (cps.getN11() + cps.getN01());
		double t3 = t1 * t2 / (cps.getNumberOfAllPairs());
		double divisor = 0.5 * (t1 + t2) - t3;
		if(divisor==0) {
			return 1;
		}
		return 1 - ((cps.getN11() - t3) / divisor);
	}

	@Override
	public String toString() {
		return ID;
	}

	public static class Factory implements ComparatorFactory {
		public Comparator getComparator(Clustering a, Clustering b) {
			return new AdjustedRand(a, b);
		}

		@Override
		public String toString() {
			return ID;
		}
	}
}
