/*
 * created May 4, 2006
 *
 * Copyright 2006.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
 *
 */
package de.uka.algo.clustering.comparison.nodebased.counting;

import de.uka.algo.clustering.Clustering;
import de.uka.algo.clustering.comparison.Comparator;
import de.uka.algo.clustering.comparison.ComparatorFactory;
import de.uka.algo.clustering.comparison.CountingPairSet;

/**
 * implements the Fowlkes-Mallows Measure:
 * 
 * 1-n11/Math.sqrt((n11+n10)*(n11+n01))
 * 
 * @author Daniel Delling
 *
 */

public class FowlkesMallows implements Comparator {

	final static String ID = "Fowlkes-Mallows";

	CountingPairSet cps;
	
	public FowlkesMallows(Clustering a, Clustering b){
		cps = CountingPairSet.getCountingPairSet(a,b);
	}
		
	public double getValue() {
		int n11 = cps.getN11();
		int n10 = cps.getN10();
		int n01 = cps.getN01();
		if(n11*(n01+n10) != 0){
			return 1-n11/Math.sqrt((n11+n10)*(n11+n01));
		} else if(n11 == 0){
			return 1;
		} else {
			return 0;
		}
	}

	
	@Override
	public String toString() {
		return ID;
	}

	public static class Factory implements ComparatorFactory {
		public Comparator getComparator(Clustering a, Clustering b) {
			return new FowlkesMallows(a, b);
		}		
		@Override
		public String toString() {
			return ID;
		}
	}
}
