/*
 * created May 4, 2006
 *
 * Copyright 2006.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
 *
 */
package de.uka.algo.clustering.comparison.nodebased.counting;

import de.uka.algo.clustering.Clustering;
import de.uka.algo.clustering.comparison.Comparator;
import de.uka.algo.clustering.comparison.ComparatorFactory;
import de.uka.algo.clustering.comparison.CountingPairSet;

/**
 * implements the Jaccard Measure:
 * 
 * 1-n_11/(n_11+n_10+n_01)
 * 
 * @author Daniel Delling
 *
 */
public class Jaccard implements Comparator {

	private static final String ID = "Jaccard";
	CountingPairSet cps;
	
	public Jaccard(Clustering a, Clustering b){
		cps = CountingPairSet.getCountingPairSet(a,b);
	}
		
	public double getValue() {
		if(cps.getNumberOfAllPairs() == cps.getN00()){
			return 0;
		} 
		return 1-(double)cps.getN11()/(double)(cps.getN11()+cps.getN10()+cps.getN01());					
	}

	
	@Override
	public String toString() {
		return ID;
	}

	public static class Factory implements ComparatorFactory {
		public Comparator getComparator(Clustering a, Clustering b) {
			return new Jaccard(a, b);
		}		
		@Override
		public String toString() {
			return ID;
		}
	}

}
