/*
 * created May 5, 2006
 *
 * Copyright 2006.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
 *
 */
package de.uka.algo.clustering.comparison.nodebased.informationTheory;

import de.uka.algo.clustering.Clustering;
import de.uka.algo.clustering.comparison.Comparator;
import de.uka.algo.clustering.comparison.ComparatorFactory;
import de.uka.algo.clustering.comparison.MutualInformation;

public class VariationOfInformation implements Comparator {
	
	private static final String ID = "Variation of Information n-norm";
	private MutualInformation mi;
	private Clustering cl1;
	private Clustering cl2;

	public VariationOfInformation(Clustering a, Clustering b){
		mi = MutualInformation.getMutualInformation(a,b);
		cl1 = a;
		cl2 = b;
	}
	
	public double getNonNormalizedValue() {
		return cl1.getEntropy()+cl2.getEntropy()-2*mi.getMutualInfo();
	}

	public double getValue() {
		return getNonNormalizedValue()/(Math.log10(cl1.getGraph().N())/Math.log10(2));
	}
	
	public double getClusterSizeNormalizedValue() {
		return getNonNormalizedValue()/(2*Math.log10(Math.max(cl1.clusterCount(),cl2.clusterCount()))/Math.log10(2));
	}
	
	@Override
	public String toString() {
		return ID;
	}

	public static class Factory implements ComparatorFactory {
		public Comparator getComparator(Clustering a, Clustering b) {
			return new VariationOfInformation(a, b);
		}		
		@Override
		public String toString() {
			return ID;
		}
	}

}
