/*
 * created May 4, 2006
 *
 * Copyright 2006.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
 *
 */
package de.uka.algo.clustering.comparison.nodebased.overlaps;

import de.uka.algo.clustering.Cluster;
import de.uka.algo.clustering.Clustering;
import de.uka.algo.clustering.comparison.Comparator;
import de.uka.algo.clustering.comparison.ComparatorFactory;
import de.uka.algo.clustering.comparison.ConfusionMatrix;
import de.uka.algo.util.GYCursor;

public class FMeasure implements Comparator {

	private static final String ID = "F-Measure";
	ConfusionMatrix cm;
	Clustering cl1;
	Clustering cl2;
	
	public FMeasure(Clustering a, Clustering b){
		cm = ConfusionMatrix.getConfusionMatrix(a,b);
		cl1 = a;
		cl2 = b;
	}
	
	
	
	public double getValue() {
			
			double sum = 0;
			for (GYCursor<Cluster> cc1 = cl1.clusters(); cc1.ok(); cc1.next()) {
				Cluster c1 = cc1.active();
				double max = 0;
				int cl1size = c1.size();
				for (GYCursor<Cluster> cc2 = cl2.clusters(); cc2.ok(); cc2.next()) {
					Cluster c2 = cc2.active();
					double f_ci_cj = 2*cm.getIntersectionCardinality(c1,c2);
					f_ci_cj /= cl1size + c2.size();
					if (f_ci_cj > max) max = f_ci_cj;
				}
				sum += cl1size * max;
			}
			
			return 1-sum/cl1.getGraph().N();
			
		
	}

	@Override
	public String toString() {
		return ID;
	}

	public static class Factory implements ComparatorFactory {
		public Comparator getComparator(Clustering a, Clustering b) {
			return new FMeasure(a, b);
		}		
		@Override
		public String toString() {
			return ID;
		}
	}

	public static class RevFactory implements ComparatorFactory {
		public Comparator getComparator(Clustering a, Clustering b) {
			return new FMeasure(b, a);
		}		
		@Override
		public String toString() {
			return ID+" rev";
		}
	}

}
