/*
 * created May 4, 2006
 *
 * Copyright 2006.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
 *
 */
package de.uka.algo.clustering.comparison.nodebased.overlaps;

import de.uka.algo.clustering.Cluster;
import de.uka.algo.clustering.Clustering;
import de.uka.algo.clustering.comparison.Comparator;
import de.uka.algo.clustering.comparison.ComparatorFactory;
import de.uka.algo.clustering.comparison.ConfusionMatrix;
import de.uka.algo.util.GYCursor;

public class MeilaHeckerman implements Comparator {

	private static final String ID = "Meila-Heckerman";
	ConfusionMatrix cm;
	Clustering cl1;
	Clustering cl2;
	
	public MeilaHeckerman(Clustering a, Clustering b){
		cm = ConfusionMatrix.getConfusionMatrix(a,b);
		cl1 = a;
		cl2 = b;
	}
	
	
	
	public double getValue() {
		double mh = 0;
		for (GYCursor<Cluster> cc = cl1.clusters(); cc.ok(); cc.next()) {
			Cluster c = cc.active();
			mh += cm.getMaximumIntersectionToCluster(c);
		}
		return 1-(mh/cl1.getGraph().N());
		
	}

	@Override
	public String toString() {
		return ID;
	}

	public static class Factory implements ComparatorFactory {
		public Comparator getComparator(Clustering a, Clustering b) {
			return new MeilaHeckerman(a, b);
		}		
		@Override
		public String toString() {
			return ID;
		}
	}
	public static class RevFactory implements ComparatorFactory {
		public Comparator getComparator(Clustering a, Clustering b) {
			return new MeilaHeckerman(b, a);
		}		
		@Override
		public String toString() {
			return ID+" rev";
		}
	}

}
