/*
 * created May 12, 2006
 *
 * Copyright 2006.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
 *
 */
package de.uka.algo.clustering.edgeweighters;

import y.base.Edge;
import y.base.EdgeCursor;
import y.base.EdgeMap;
import y.geom.YPoint;
import y.view.Graph2D;
import de.uka.algo.clustering.Clustering;
import de.uka.algo.clustering.generators.EmbeddedClusteredGraph;

public class DistanceWeighter extends EdgeWeighter {

	private double factor;

	public DistanceWeighter(double strechFactor) {
		this.factor = strechFactor;
	}

	public EdgeMap changeEdges(Clustering cl) {
		EdgeMap em = cl.getGraph().createEdgeMap();
		if (cl.getGraph() instanceof Graph2D) {
			Graph2D g = (Graph2D) cl.getGraph();
			double newWeight = 0.0;
			for (EdgeCursor ec = g.edges(); ec.ok(); ec.next()) {
				Edge e = ec.edge();
				newWeight = factor
						/ YPoint.distance(g.getCenter(e.source()), g
								.getCenter(e.target()));
				em.setDouble(e, newWeight);
			}
		} else if (cl instanceof EmbeddedClusteredGraph) {
			EmbeddedClusteredGraph cg = (EmbeddedClusteredGraph) cl;
			double newWeight = 0.0;
			for (EdgeCursor ec = cg.getGraph().edges(); ec.ok(); ec.next()) {
				Edge e = ec.edge();
				newWeight = factor
						/ YPoint.distance(cg.getPositionAsPoint(e.source()), cg
								.getPositionAsPoint(e.target()));
				em.setDouble(e, newWeight);
			}
		} else {
			throw new RuntimeException("no embedding found!");
		}
		return em;
	}

}
