/*
 * created May 12, 2006
 *
 * Copyright 2006.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
 *
 */
package de.uka.algo.clustering.edgeweighters;

import y.base.Edge;
import y.base.EdgeCursor;
import y.base.EdgeMap;
import de.uka.algo.clustering.Clustering;
import de.uka.algo.graphs.GraphInterpretation;

/**
 * a scaler scales the weights of the edges. You may scale interclusteredges
 * different than intraclusteredges
 * 
 * @author Daniel Delling
 * 
 */
public class Scaler extends EdgeWeighter {

	private double scaleInterValue;

	private double scaleIntraValue;

	/**
	 * a new scaler
	 * 
	 * @param scaleIntra
	 *            the value, intraclusteredges shall be scaled
	 * @param scaleInter
	 *            the value, interclusteredges shall be scaled
	 */
	public Scaler(double scaleIntra, double scaleInter) {
		this.scaleInterValue = scaleInter;
		this.scaleIntraValue = scaleIntra;
	}

	public EdgeMap changeEdges(Clustering cl) {
		EdgeMap em = cl.getGraph().createEdgeMap();
		double newWeight = 0.0;
		GraphInterpretation gi = cl.getInterpretation();
		for (EdgeCursor ec = cl.getGraph().edges(); ec.ok(); ec.next()) {
			Edge e = ec.edge();
			if (cl.isIntraClusterEdge(e)) {
				newWeight = gi.getWeight(e) * scaleIntraValue;
				em.setDouble(e, newWeight);
			}
			if (cl.isInterClusterEdge(e)) {
				newWeight = gi.getWeight(e) * scaleInterValue;
				em.setDouble(e, newWeight);
			}
		}
		return em;
	}

}
