/*
 * created Apr 28, 2006
 *
 * Copyright 2006.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
 *
 */
package de.uka.algo.clustering.generators.randomCluster;

import y.base.Graph;
import de.uka.algo.clustering.generators.utils.PartitionGenerator;

/**
 * this class implements a Generator creating clustergraphs with an
 * equiPartition
 * 
 * @author Daniel Delling
 * 
 */
public class EquiPartitionClusteredGraph extends RandomClusteredGraph {
	/**
	 * creates a RandomClustergraph with a equiPartition. the clusterSizes are
	 * picked equally random between minSize and maxSize
	 * 
	 * @param g
	 *            the given graph
	 * @param numberOfClusters
	 *            number of clusters
	 * @param minSize
	 *            minimum clusterSize
	 * @param maxSize
	 *            maximum clusterSize
	 * @param pIn
	 *            probability that an IntraclusterEdge is created
	 * @param pOut
	 *            probability that an InterclusterEdge is created
	 */
	public EquiPartitionClusteredGraph(Graph g, int numberOfClusters,
			int minSize, int maxSize, double pIn, double pOut) {
		super(g, PartitionGenerator.equiPartition(numberOfClusters, minSize,
				maxSize), pIn, pOut);
	}

	@Override
	public String getID() {
		return "EquiPartitionClusterGraph";
	}

}
