/*
 * Created Mar 8, 2006
 *
 * Copyright 2006.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
 */
package de.uka.algo.clustering.generators.randomCluster;

import y.base.Graph;
import de.uka.algo.clustering.generators.utils.PartitionGenerator;

/**
 * this class implements a Generator creating clustergraphs with an gaussian
 * partition
 * 
 * @author Daniel Delling
 * 
 */

public class GaussianClusteredGraph extends RandomClusteredGraph {

	/**
	 * creates a RandomClustergraph with a gaussian partition. The number of
	 * clusters is picked uniformly at random between log_10(n) and sqrt(n). the
	 * average of is nodes/number of clusters, the deviation is average/4.
	 * 
	 * @param g
	 *            the given graph
	 * @param nodes
	 *            number of clusters
	 * @param pIn
	 *            probability that an IntraclusterEdge is created
	 * @param pOut
	 *            probability that an InterclusterEdge is created
	 */
	public GaussianClusteredGraph(Graph g, int nodes, double pIn, double pOut) {
		super(g, PartitionGenerator.standardGaussPartition(nodes), pIn, pOut);
	}

	@Override
	public String getID() {
		return "GaussianClusterGraph";
	}

}
