/*
 * Created Mar 8, 2006
 *
 * Copyright 2006.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
 */
package de.uka.algo.clustering.generators.randomCluster;

import y.base.Graph;
import de.uka.algo.clustering.generators.utils.PartitionGenerator;

/**
 * this class implements a Generator creating clustergraphs with a multigaussian
 * partition
 * 
 * @author Daniel Delling
 * 
 */

public class MultiGaussianClusteredGraph extends RandomClusteredGraph {

	/**
	 * creates a RandomClustergraph with a multi gaussian partition.
	 * 
	 * @param g
	 *            the given graph
	 * @param numberOfClusters
	 *            number of clusters
	 * @param gaussAverage
	 *            averages of clustersizes
	 * @param gaussDeviation
	 *            deviations of clustersizes
	 * @param pIn
	 *            probability that an IntraclusterEdge is created
	 * @param pOut
	 *            probability that an InterclusterEdge is created
	 */
	public MultiGaussianClusteredGraph(Graph g, int numberOfClusters,
			int[] gaussAverage, int[] gaussDeviation, double pIn, double pOut) {
		super(g, PartitionGenerator.multiGaussPartition(numberOfClusters,
				gaussAverage, gaussDeviation), pIn, pOut);
	}

	@Override
	public String getID() {
		return "GaussianClusterGraph";
	}

}
