/*
 * Created Mar 8, 2006
 *
 * Copyright 2006.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
 */
package de.uka.algo.clustering.generators.randomCluster;

import y.base.Graph;
import de.uka.algo.clustering.generators.utils.PartitionGenerator;

/**
 * this class implements a Generator creating clustergraphs with a gaussian
 * partition. pOut is substituted by rho: the ratio of expected intra- and
 * interclusteredges
 * 
 * @author Daniel Delling
 * 
 */

public class SignificantGaussianClusteredGraph extends RandomClusteredGraph {

	/**
	 * creates a RandomClustergraph with a gaussian partition. The number of
	 * clusters is picked uniformly at random between log_10(n) and sqrt(n). the
	 * average of is nodes/number of clusters, the deviation is average/4.
	 * 
	 * @param g
	 *            the given graph
	 * @param nodes
	 *            number of clusters
	 * @param pIn
	 *            probability that an IntraclusterEdge is created
	 * @param rho
	 *            expected ratio of inter--cluster edges/intra--cluster edges
	 */
	public SignificantGaussianClusteredGraph(Graph g, int nodes, double pIn,
			double rho) {
		super(g);
		int[] partition = PartitionGenerator.standardGaussPartition(nodes);
		double pOutLocal = rho * pIn
				* ((double) nodes / (double) partition.length - 1)
				/ (nodes - (double) nodes / (double) partition.length);
		createGraph(partition, pIn, pOutLocal);
	}

	@Override
	public String getID() {
		return "SignificantGaussianClusterGraph";
	}

}
