/* 
 * Integration: June 8, 2006 (katz)
 * Reviewed: /TODO code review (katz)
 *
 * Copyright 2004-2006. --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
 */
package de.uka.algo.clustering.index;

import de.uka.algo.clustering.Clustering;
import de.uka.algo.clustering.ClusteringReference;

/**
 * @author Bastian Katz, mailto:katz@ira.uka.de
 */
public final class Coverage extends ClusteringReference implements Index {

	final static String INDEX_ID = "Coverage";
	
	Coverage(Clustering clustering) {
		super(clustering);
	}

	/* queries */

	/**
	 * Returns the clustering's coverage, i.e. the ratio of inner (covered)
	 * edges to outer (uncovered edges), represented by their weight.
	 * 
	 * @return the clustering's coverage
	 */
	public double getValue() {
		return 1 - (clustering.getOuterWeight() / clustering.getOverAllWeight());
	}

	/**
	 * Returns the clustering's coverage, i.e. the ratio of inner (covered)
	 * edges to outer (uncovered edges), represented by a uniform weight of 1.
	 * 
	 * @return the clustering's coverage
	 */
	public double getUnweightedValue() {
		return ((double) clustering.getInnerEdgeCount())
				/ clustering.getGraph().edgeCount();
	}

	@Override
	public String toString() {
		return INDEX_ID;
	}

	public static class Factory implements IndexFactory {
		public Index getIndex(Clustering cl) {
			return new Coverage(cl);
		}

		@Override
		public String toString() {
			return INDEX_ID;
		}
	}
}