package de.uka.algo.clustering.index;

import de.uka.algo.clustering.Cluster;
import de.uka.algo.clustering.Clustering;
import de.uka.algo.util.GYCursor;

public class InterClusterConductance extends ClusterBufferedIndex<double[]> {
	static final String INDEX_ID = "InterClusterConductance";
	
	private static final int UNWEIGHTED_INDEX = 0; 
	private static final int WEIGHTED_INDEX = 1; 
	
	InterClusterConductance(Clustering clustering) {
		super(clustering);
		clustering.addListener(this);
	}

	@Override
	protected double[] getData(Cluster aCluster) {
		double[] result = new double[2];

		/* the outer conductance, reused for both, weighted and unweighted */
		double phi = 0;

		if (aCluster.size() == 0 // per definitionem
				|| aCluster.size() == clustering.getGraph().nodeCount()) {
			phi = 1;
		} else if (getClustering().getBorderWeight(aCluster) == 0) { // trivial
			phi = 0;
		} else {
			double innerValue = getClustering().getInnerWeight(aCluster);
			innerValue *= 2;
			innerValue += getClustering().getBorderWeight(aCluster);

			double outerValue = getClustering().getOverAllWeight();
			outerValue *= 2;
			outerValue -= innerValue;
			if (innerValue < outerValue) {
				phi = getClustering().getBorderWeight(aCluster) / innerValue;
			} else {
				phi = getClustering().getBorderWeight(aCluster) / outerValue;
			}
		}
		result[0] = 1 - phi;

		/* unweighted */
		if (aCluster.size() == 0
				|| aCluster.size() == clustering.getGraph().nodeCount()) {
			phi = 1;
		} else if (getClustering().getBorderEdgeCount(aCluster) == 0) {
			phi = 0;
		} else {
			double innerValue = getClustering().getInnerEdgeCount(aCluster);
			innerValue *= 2;
			innerValue += getClustering().getBorderEdgeCount(aCluster);

			double outerValue = clustering.getGraph().edgeCount();
			outerValue *= 2;
			outerValue -= innerValue;

			if (innerValue < outerValue) {
				phi = getClustering().getBorderEdgeCount(aCluster)
						/ innerValue;
			} else {
				phi = getClustering().getBorderEdgeCount(aCluster)
						/ outerValue;
			}
		}
		result[1] = 1 - phi;
		return result;
	}
	
	protected double getValue(GYCursor<double[]> infos, int index) {
		double result = 1.0;
		for (; infos.ok(); infos.next()) {
			if (infos.active()[index]<result){
				result = infos.active()[index]; 
			}
		}
		return result;		
	}

	@Override
	protected double value(GYCursor<double[]> infos) {
		return getValue(infos, WEIGHTED_INDEX);
	}

	@Override
	protected double unweightedValue(GYCursor<double[]> infos) {
		return getValue(infos, UNWEIGHTED_INDEX);
	}

	@Override
	public String toString() {		
		return INDEX_ID;
	}


	public String id() {		
		return INDEX_ID;
	}

	
	public static class Factory implements IndexFactory {
		public Index getIndex(Clustering clustering) {
			InterClusterConductance icc = (InterClusterConductance) clustering.getClusteringListener(id());
			return (icc==null)?new InterClusterConductance(clustering):icc;
		}		
				
		public String toString() {		
			return INDEX_ID;
		}
		
		public String id() {		
			return INDEX_ID;
		}

	}	
}