/*
 * Created on 06.10.2004
 *
 * Copyright 2004.  --- All rights resevered. 
 * Algorithmic Group, Karlsruhe, Germany.
 */
package de.uka.algo.clustering.index;

import y.base.Edge;
import y.base.EdgeCursor;
import de.uka.algo.clustering.Cluster;
import de.uka.algo.clustering.Clustering;
import de.uka.algo.clustering.ClusteringReference;
import de.uka.algo.util.GYCursor;

/**
 * 
 * @author Bastian Katz
 */
public abstract class Performance extends ClusteringReference implements Index {
 
    /** maximum of edge weights */
    double maxWeight = 0;

    public double getMaxWeight() {
        return maxWeight;
    }

    protected Performance(Clustering clustering) {
        super(clustering);
        maxWeight = 0;

        for (EdgeCursor ec = clustering.getGraph().edges(); ec.ok(); ec.next()) {

            Edge anEdge = ec.edge();
            double weight = clustering.getInterpretation().getWeight(anEdge);

            if (maxWeight < weight) {
                maxWeight = weight;
            }
        }
    }
    
    public double weightPerformance(double m, double theta1,
            double theta2) {

        double result = 1;
        result-=theta1;
        result*=m;
        result*=clustering.getOuterEdgeCount();
        result+=clustering.getOuterWeight()*theta1;

        result-=clustering.getInnerWeight()*theta2;

        if (!clustering.getInterpretation().isDirected())
            result*=2;

        for (GYCursor<Cluster> cc = clustering.clusters(); cc.ok(); cc.next()) {
            Cluster c = cc.active();
            result+=c.size() * (c.size() - 1)*m;
        }
        long n = clustering.getGraph().nodeCount();
        return 1-(result/(n*(n - 1)*m));
    }

    public double unweightedPerformance(double theta2) {
        double result = clustering.getOuterEdgeCount()
                - clustering.getInnerEdgeCount()*theta2;
        if (!clustering.getInterpretation().isDirected())
            result*=2;
        for (GYCursor<Cluster> cc = clustering.clusters(); cc.ok(); cc.next()) {
            Cluster c = cc.active();
            result+=c.size() * (c.size() - 1);
        }
        long n = clustering.getGraph().nodeCount();
        return 1-(result/(n * (n - 1)));
    }

}