package de.uka.algo.clustering.index;

import de.uka.algo.clustering.Clustering;
import de.uka.algo.clustering.index.exspectation.ECoverage;
import de.uka.algo.clustering.index.exspectation.EPerformance;

public class Significance extends OperationIndex {
	public static enum Indizes {
		coverage, performance
	}
	private Indizes index;
	private Operations op;
	
	protected Significance(Clustering cl, Indizes index, Operations op) {
		super(cl, getIndex(index), getExspectation(index), op);
		this.index = index;
		this.op = op;
	}

	private static IndexFactory getExspectation(Indizes index) {
		switch (index) {
		case coverage:
			return new ECoverage.Factory();
		case performance:
			return new EPerformance.Factory();
		}
		return null;
	}

	private static IndexFactory getIndex(Indizes index) {
		switch (index) {
		case coverage:
			return new Coverage.Factory();
		case performance:
			return new PerformanceConventional
			.Factory();
		}
		return null;
	}

	public static class Factory implements IndexFactory {
		
		private Indizes index;
		private Operations operation;

		public Factory(Indizes index, Operations operation) {
			this.index = index;
			this.operation = operation;
		}
		
		public Index getIndex(Clustering cl) {
			return new Significance(cl, index, operation);
		}

		@Override
		public String toString() {
			return id(index, operation);
		}
	}

	@Override
	public String toString() {
		return id(index, op);
	}

	public static String id(Indizes index, Operations operation) {
		return "Significance("+index+", "+operation+")";
	}
	
}
