package de.uka.algo.clustering.index.exspectation;

import de.uka.algo.clustering.Cluster;
import de.uka.algo.clustering.Clustering;
import de.uka.algo.clustering.ClusteringReference;
import de.uka.algo.clustering.index.Index;
import de.uka.algo.clustering.index.IndexFactory;
import de.uka.algo.util.GYCursor;

public final class ECoverage extends ClusteringReference implements Index {

	private final static String INDEX_ID = "ExspectedCoverage";
	
	protected ECoverage(Clustering clustering) {
		super(clustering);
	}

	@Override
	public String toString() {
		return INDEX_ID;
	}

	public double getValue() {
		double edgeEnds = 0.0;
		for (GYCursor<Cluster> cc = getClustering().clusters(); cc.ok(); cc.next()) {
			Cluster c = cc.active();
			double helper = 2.0*clustering.getInnerWeight(c)+clustering.getBorderWeight(c);
			edgeEnds += helper*helper;
		}
		double edges = clustering.getOverAllWeight();
		return edgeEnds / (4.0*edges*edges);
	}

	public double getUnweightedValue() {
		double edgeEnds = 0.0;
		for (GYCursor<Cluster> cc = getClustering().clusters(); cc.ok(); cc.next()) {
			Cluster c = cc.active();
			double helper = 2.0*clustering.getInnerEdgeCount(c)+clustering.getBorderEdgeCount(c);
			edgeEnds += helper*helper;
		}
		double edges = clustering.getGraph().E();
		return edgeEnds / (4.0*edges*edges);
	}

	public static class Factory implements IndexFactory {
		public Index getIndex(Clustering cl) {
			return new ECoverage(cl);
		}

		@Override
		public String toString() {
			return INDEX_ID;
		}
	}

}