/*
 * Created on 20.09.2005
 *
 * Copyright 2005.  --- All rights resevered. 
 * Algorithmic Group, Karlsruhe, Germany.
 */
package de.uka.algo.clustering.index.exspectation;

import y.base.Edge;
import y.base.EdgeCursor;
import y.base.NodeCursor;
import de.uka.algo.clustering.Cluster;
import de.uka.algo.clustering.Clustering;
import de.uka.algo.clustering.ClusteringReference;
import de.uka.algo.clustering.index.Index;
import de.uka.algo.clustering.index.IndexFactory;
import de.uka.algo.clustering.index.PerformanceConventional;
import de.uka.algo.util.GYCursor;

/**
 * 
 * @author Robert Goerke
 */
public final class EPerformance extends ClusteringReference implements Index {
 
	final static String INDEX_ID = "Performance_Expectation";
	
    Index performance;
    
    /** maximum of edge weights */
    double maxWeight = 0;

    public double getMaxWeight() {
        return maxWeight;
    }

    EPerformance(Clustering clustering) {
        super(clustering);
        performance = new PerformanceConventional.Factory().getIndex(clustering);
        maxWeight = 0;

        for (EdgeCursor ec = clustering.getGraph().edges(); ec.ok(); ec.next()) {

            Edge anEdge = ec.edge();
            double weight = clustering.getInterpretation().getWeight(anEdge);

            if (maxWeight < weight) {
                maxWeight = weight;
            }
        }

    }
    
    public double weightEPerformance(double M) {
    	double totalWeight = getClustering().getOverAllWeight();
    	int N = clustering.getInterpretation().getGraph().N();
    	double result=0;
    	double currentWi=0;
    	double currentWv=0;
    	double sumOne=0;
    	double sumTwo=0;
        for (GYCursor<Cluster> cc = clustering.clusters(); cc.ok(); cc.next()) {
        	Cluster currentCluster = cc.active();
        	currentWi=0;
        	for (NodeCursor nc = currentCluster.members(); nc.ok(); nc.next()) {
        		currentWv=0;
        		for (EdgeCursor ec = nc.node().edges(); ec.ok(); ec.next()){
        			//if (clustering.getCluster(ec.edge().opposite(nc.node())).equals(currentCluster))
        			currentWv += clustering.getInterpretation().getWeight(ec.edge());
        		}
        		currentWi += currentWv;
        	}
        	sumOne += (currentWi * currentWi);
        	sumTwo += (currentCluster.size() * currentCluster.size());
        }
    	result = (sumOne/totalWeight + M*(N*N-sumTwo) - 2*totalWeight)/(M*N*(N-1));
    	return result;
    }

    public double unweightedEPerformance() {
    	int E = getClustering().getOuterEdgeCount()+getClustering().getInnerEdgeCount();
    	int N = clustering.getInterpretation().getGraph().N();
    	double result=0;
    	double currentAi=0;
    	double currentBi=0;
        for (GYCursor<Cluster> cc = clustering.clusters(); cc.ok(); cc.next()) {
        	Cluster currentCluster = cc.active();
        	currentAi=0;
        	currentBi=0;
        	for (NodeCursor nc = currentCluster.members(); nc.ok(); nc.next()) {
        		currentAi += nc.node().degree();
        		++currentBi;
        	}
        	result+=(currentAi*currentAi);
        	result-=(currentBi*currentBi);
        } 
    	result /= E;
    	result = (result - 2*E + N*N)/(N*(N-1)); 
    	return result;
    }
    
    public double getValue(double m) {
        return weightEPerformance(m);
    }

	public double getUnweightedValue() {
		return unweightedEPerformance();
	}

	public double getValue() {
        return getValue(getMaxWeight());
    }

    public static class Factory implements IndexFactory {
        public Index getIndex(Clustering cl) {
            return new EPerformance(cl);
        }
        
        @Override
		public String toString() {
            return INDEX_ID;
        }      
    }

    @Override
	public String toString() {
        return INDEX_ID;
    }
}
