/*
 * Created on May 28, 2008. 
 *  
 * Copyright 2008. --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany. 
 * 
 * Project: extension
 */
package de.uka.algo.clustering.linearprogram;

import lpsolve.LpSolve;
import lpsolve.LpSolveException;
import y.base.Graph;
import y.base.Node;
import de.uka.algo.clustering.Clustering;
import de.uka.algo.clustering.linearprogram.LPSolveAlgorithm;
import de.uka.algo.clustering.linearprogram.LPSolveGenerator;
import de.uka.algo.clustering.linearprogram.objective.ModularityObjectiveFunction;
import de.uka.algo.clustering.linearprogram.objective.WeightedModularityObjectiveFunction;
import de.uka.algo.graphs.GraphInterpretation;

/**
 * @author marco
 * 
 */
public class ExactModularityLPSolve extends LPSolveAlgorithm {

	boolean metricModel;
	boolean considerEdgeWeights;
	int tuningMode;

	public ExactModularityLPSolve(boolean metricModel, int tuningMode, boolean considerEdgeWeights) {
		this.metricModel = metricModel;
		this.considerEdgeWeights = considerEdgeWeights;
		this.tuningMode = tuningMode;
	}
	
	public ExactModularityLPSolve(boolean metricModel,  boolean considerEdgeWeights) {
		this.metricModel = metricModel;
		this.considerEdgeWeights = considerEdgeWeights;
		this.tuningMode = 0;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.uka.algo.clustering.linearprogram.LPSolveAlgorithm#createLP(y.base.Graph)
	 */
	@Override
	protected LpSolve createLP(Graph graph, GraphInterpretation graphInt) throws LpSolveException {
		LpSolve myLP = null;
		if (metricModel)
			myLP = LPSolveGenerator.makeMetricClustering(graph, true);
		else
			myLP = LPSolveGenerator.makeEquivalenceClustering(graph, true);
		GraphObjectiveFunction myObjectiveFunction = null;
		if (considerEdgeWeights)
			myObjectiveFunction = new WeightedModularityObjectiveFunction(graphInt);
		else 
			myObjectiveFunction = new ModularityObjectiveFunction(graph);
		LPSolveGenerator.addObjectiveFunction(myLP, graph,
				myObjectiveFunction, metricModel);
		return myLP;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.uka.algo.clustering.linearprogram.LPSolveAlgorithm#run(de.uka.algo.clustering.Clustering,
	 *      double[])
	 */
	@Override
	protected void run(Clustering clustering, double[] solution) {
		// get number of nodes
		int n = clustering.getGraph().nodeCount();
		// get ordering of nodes
		Node nodes[] = clustering.getGraph().getNodeArray();

		// reset clustering to singletons
		clustering.resetSingletons();

		// merges those nodes that have a distances of zero
		for (int i = 0; i < nodes.length; ++i)
			for (int j = i + 1; j < nodes.length; ++j) {
				int ij = LPSolveGenerator.getPos(i, j, n);
				if ((metricModel && solution[ij] <= 0.0)
						|| (!metricModel && solution[ij] >= 1.0)) {
					clustering.merge(nodes[i], nodes[j]);
				}
			}
	}

}
