/*
 * Created on Oct 23, 2008. 
 *  
 * Copyright 2008. --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany. 
 * 
 * Project: extension
 */
package de.uka.algo.clustering.linearprogram;

import ilog.concert.IloException;
import ilog.concert.IloNumVar;
import ilog.cplex.IloCplex;
import ilog.cplex.IloCplex.UnknownObjectException;

import java.util.Map;

import y.base.Graph;
import y.base.Node;
import de.uka.algo.clustering.Clustering;
import de.uka.algo.clustering.linearprogram.objective.ModularityObjectiveFunction;
import de.uka.algo.clustering.linearprogram.objective.WeightedModularityObjectiveFunction;
import de.uka.algo.graphs.GraphInterpretation;

/**
 * @author marco
 * 
 */
public class ExactModularityTunedCplex extends CplexAlgorithm {

	boolean metricModel;
	boolean considerEdgeWeights;
	int tuningMode;

	public ExactModularityTunedCplex(boolean metricModel, int tuningMode, boolean considerEdgeWeights) {
		this.metricModel = metricModel;
		this.considerEdgeWeights = considerEdgeWeights;
		this.tuningMode = tuningMode;
		
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.uka.algo.clustering.linearprogram.CplexAlgorithm#createLP(y.base.Graph,
	 *      java.util.Map)
	 */
	@Override
	protected IloCplex createLP(Graph graph, GraphInterpretation graphInt, 
			Map<String, IloNumVar> name2variable) throws IloException {
		name2variable.clear();

		IloCplex myLP = CplexGenerator.generateEdgeVariables(graph, true,
				name2variable);
		if (metricModel)
			myLP = CplexGenerator.addLazyMetricClusteringConstraints(myLP, graph,
					true, name2variable);
		else
			myLP = CplexGenerator.addEquivalenceClusteringConstraints(myLP,
					graph, true, name2variable);
		GraphObjectiveFunction myObjectiveFunction = null;
		if (considerEdgeWeights)
			myObjectiveFunction = new WeightedModularityObjectiveFunction(graphInt);
		else 
			myObjectiveFunction = new ModularityObjectiveFunction(graph);
		myLP = CplexGenerator.addObjectiveFunction(myLP, graph,	myObjectiveFunction, metricModel, name2variable);
		return myLP;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.uka.algo.clustering.linearprogram.CplexAlgorithm#run(de.uka.algo.clustering.Clustering,
	 *      ilog.cplex.IloCplex, java.util.Map)
	 */
	@Override
	protected void run(Clustering clustering, IloCplex cplex,
			Map<String, IloNumVar> name2variable) throws IloException {
		clustering.resetSingletons();
		// get an ordering of nodes
		Node nodes[] = clustering.getGraph().getNodeArray();

		for (int i = 0; i < nodes.length; ++i)
			for (int j = i + 1; j < nodes.length; ++j) {
				IloNumVar var = name2variable.get("X(" + i + "," + j + ")");
				try {
					double value = cplex.getValue(var);
					// System.out.println("X(" + i + "," + j + ") = "+value);
					if ((metricModel && value <= 0)
							|| (!metricModel && value >= 1.0))
						clustering.merge(nodes[i], nodes[j]);
				} catch (UnknownObjectException uoe) {
					// if an exception occurs, send a warning to the Logger
					StringBuffer cache = new StringBuffer();
					cache.append("could not read variable X(" + i + "," + j
							+ ") due to " + uoe.toString());
					for (StackTraceElement ste : uoe.getStackTrace())
						cache.append(ste.toString());
					logger.warn(cache.toString());
				}
			}

		// TODO Auto-generated method stub

	}

}
