/*
 * Created on May 05, 2008
 *
 * Copyright 2008.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
 */
package de.uka.algo.clustering.linearprogram;

import y.base.Node;

/**
 * This interface defines the signature to access the coefficients of an
 * (I)LP-objective function. Such a function is always defined as
 * 
 * sum over all ordered pairs (u, v) of c(u, v) * X(u, v),
 * 
 * where X(u, v) corresponds to the (binary) variable that presents, if node u
 * and v belong to the same cluster or not.
 * 
 * @author Marco Gaertler
 * 
 */
public interface GraphObjectiveFunction {

	/**
	 * Returns the coefficient c(u, v)
	 * 
	 * @param u node with smaller index 
	 * @param v node with larger index 
	 * @return associated coefficient 
	 */
	public double getCoefficient(Node u, Node v);

}
