/*
 * Created on May 05, 2008
 *
 * Copyright 2008.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
 */
package de.uka.algo.clustering.linearprogram.objective;

import y.base.Graph;
import y.base.Node;
import de.uka.algo.clustering.linearprogram.GraphObjectiveFunction;

/**
 * This class returns the (unweighted) modularity as objective function. 
 *   
 * @author Marco Gaertler 
 *
 */
public class ModularityObjectiveFunction implements GraphObjectiveFunction {

	/**
	 * Private reference to the used graph. 
	 */
	Graph myGraph; 
	
	/**
	 * Default constructor. A reference graph needs to be given. 
	 * @param graph reference graph 
	 */
	public ModularityObjectiveFunction(Graph graph){
		myGraph = graph;
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.uka.algo.clustering.linearprogram.GraphObjectiveFunction#getCoefficient(y.base.Node, y.base.Node)
	 */
	public double getCoefficient(Node u, Node v) {
		// initialize coefficient 
		double coefficient = -u.degree() * v.degree() / (2.0 * myGraph.edgeCount());
		// add constant value, if u and v are connected 
		if(u.getEdge(v) != null)
			coefficient += 1.0;
		// return coefficient 
		return coefficient;
	}

}
