/*
 * Created on May 05, 2008
 *
 * Copyright 2008.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
 */
package de.uka.algo.clustering.linearprogram.objective;

import y.base.Node;
import de.uka.algo.clustering.linearprogram.GraphObjectiveFunction;
import de.uka.algo.graphs.GraphInterpretation;

/**
 * This class returns the weighted modularity as objective function. 
 *   
 * @author Robert Goerke
 *
 */
public class WeightedModularityObjectiveFunction implements GraphObjectiveFunction {

	/**
	 * Private reference to the used graph. 
	 */
	GraphInterpretation myGraphInt; 
	
	/**
	 * Default constructor. A reference graph needs to be given. 
	 * @param graph reference graph 
	 */
	public WeightedModularityObjectiveFunction(GraphInterpretation graphInt){
		myGraphInt = graphInt;
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.uka.algo.clustering.linearprogram.GraphObjectiveFunction#getCoefficient(y.base.Node, y.base.Node)
	 */
	public double getCoefficient(Node u, Node v) {
		// initialize coefficient
		double coefficient = -myGraphInt.getDegree(u) * myGraphInt.getDegree(v) / (2.0 * myGraphInt.getEdgeWeightSum());
		// add constant value, if u and v are connected 
		coefficient += myGraphInt.getAdjacency(u, v);
//		if(u.getEdge(v) != null)
//			coefficient += myGraphInt.getWeight(u.getEdge(v));
		// return coefficient 
		return coefficient;
	}

}
