/*
 * Created on Feb 23, 2005
 *
 * Copyright 2004.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
 */
package de.uka.algo.clustering.visualization;

import java.awt.Color;
import java.util.HashMap;

import y.base.Node;
import y.base.NodeCursor;
import y.view.Graph2D;
import y.view.LineType;
import de.uka.algo.clustering.Cluster;
import de.uka.algo.clustering.Clustering;
import de.uka.algo.graphs.GraphInterpretation;
import de.uka.algo.util.ColorProvider;
import de.uka.algo.util.GYCursor;

/**
 * @author Bastian Katz
 * 
 */
public class BorderColorizer extends EdgeWeightVisualizer implements Graph2DExtractor {
	@Override
	public void visualize(Clustering clustering, boolean round) {
		super.visualize(clustering, round);
		Graph2D graph = getGraph2D(clustering);
		boolean trivial = clustering.size() == 1;
		ColorProvider randomColors = new ColorProvider();
		for (GYCursor<Cluster> cc = clustering.clusters(); cc.ok(); cc.next()) {
			Cluster clus = cc.active();
			Color m = trivial?ColorProvider.black:randomColors.randomColorRGB();
			for (NodeCursor nc = clus.members(); nc.ok(); nc.next()) {
				graph.getRealizer(nc.node()).setLineColor(m);
				if (!trivial) {
					graph.getRealizer(nc.node()).setLineType(LineType.LINE_5);					
				} else {
					graph.getRealizer(nc.node()).setLineType(LineType.LINE_1);
				}
			}
		}
		for (NodeCursor nc = clustering.poolNodes(); nc.ok(); nc.next()) {
			graph.getRealizer(nc.node()).setLineColor(Color.DARK_GRAY);
			graph.getRealizer(nc.node()).setLineType(LineType.LINE_5);
		}
		graph.getCurrentView().updateView();
	}

	@Override
	public String toString() {
		return "Border Colors";
	}
	
	public Clustering extract(Graph2D graph, boolean useWeights) {
		Clustering result = new Clustering(graph, GraphInterpretation.getInterpretation(graph, useWeights));
		HashMap<Color, Cluster> hm = new HashMap<Color, Cluster>();
		for (NodeCursor nc = graph.nodes(); nc.ok(); nc.next()) {
			Node n = nc.node();
			if (!graph.getHierarchyManager().isNormalNode(n)) continue;
			Color c = graph.getRealizer(n).getLineColor();
			Cluster cl = hm.get(c);
			if (cl == null) {
				cl = result.newCluster(n);
				hm.put(c, cl);
			} else {
				result.add(cl, n);
			}
		}
		result.pool2Singletons();
		return result;
	}

}
