/*
 * Created on Jul 17, 2008
 *
 * Copyright 2008.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
 */
package de.uka.algo.clustering.visualization;

import java.util.HashMap;

import y.base.Node;
import y.base.NodeCursor;
import y.view.Graph2D;
import de.uka.algo.clustering.Cluster;
import de.uka.algo.clustering.Clustering;
import de.uka.algo.graphs.GraphInterpretation;

/**
 * @author Bastian Katz
 * 
 */
public class Labeller extends EdgeWeightVisualizer {

	static String[] units = // Units 0 to 9
    { "", "I", "II", "III", "IV", "V", "VI", "VII", "VIII", "IX" };
 
  static String[] tens = // Tens 0 to 90
    { "", "X", "XX", "XXX", "XL", "L", "LX", "LXX", "LXXX", "XC" };
    
  static String[] hundreds = // HunDreDs 0 to 900
    { "", "C", "CC", "CCC", "CD", "D", "DC", "DCC", "DCCC", "Cm" };
 
  static String[] thousands = // ThousanDs 0 to 3000
    { "", "M", "MM", "MMM" };

  public static String toRoman( int n ) {
	    return thousands[ (n / 1000) ] +
	      hundreds[ (n / 100) % 10 ] +
	      tens[ (n / 10) % 10 ] +
	      units[ (n) % 10 ];
	  }
	
	public enum LabelType {letters, numbers, roman}

	private LabelType labelType;
	
	public Labeller(LabelType l) {
		super();
		this.labelType = l;
	}

	private String getLetter(int i) {
		String result = ""; 
		if (i>26) {
			result +=getLetter(i/26-1);
		}
		char x = (char) ('A'+((i-1)%26));
		result+=""+x;
		return result;
	}
	
	public String toString() {
		return "Node Labels - "+labelType;
	}
	
	@Override
	public void visualize(Clustering clustering, boolean round) {
		super.visualize(clustering, round);
		Graph2D graph = getGraph2D(clustering);
		int index = 0;
		for (Cluster cl : clustering.getClusterArray()) {
			if (cl.size()==1) {
				graph.setLabelText(cl.aMember(), "");
				continue;
			}
			String label = "";
			switch (labelType) {
			case letters:
				label+=getLetter(index+1);
				break;
			case numbers:
				label+=index;
				break;
			case roman:
				label+=toRoman(index+1);
				break;
			default:
				break;
			}

			for (NodeCursor nc = cl.members(); nc.ok(); nc.next()) {
				Node n = nc.node();
				graph.setLabelText(n, label);
			}
			index++;
		}		
		graph.getCurrentView().updateView();
	}

}
