/*
 * Created on Feb 23, 2005
 *
 * Copyright 2004.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
 */
package de.uka.algo.clustering.visualization;

import java.awt.Color;
import java.util.HashMap;

import y.base.Node;
import y.base.NodeCursor;
import y.view.Graph2D;
import de.uka.algo.clustering.Cluster;
import de.uka.algo.clustering.Clustering;
import de.uka.algo.graphs.GraphInterpretation;
import de.uka.algo.util.ColorProvider;
import de.uka.algo.util.GYCursor;

/**
 * @author Bastian Katz
 * 
 */
public class NodeColorizer extends EdgeWeightVisualizer implements Graph2DExtractor {

	@Override
	public void visualize(Clustering clustering, boolean round) {	
		super.visualize(clustering, round);
		Graph2D graph = getGraph2D(clustering);
		ColorProvider randomColors = new ColorProvider();
		for (GYCursor<Cluster> cc = clustering.clusters(); cc.ok(); cc.next()) {
			Cluster clus = cc.active();
			Color m = randomColors.randomColorRGB();
			for (NodeCursor nc = clus.members(); nc.ok(); nc.next()) {
				graph.getRealizer(nc.node()).setFillColor(m);
			}
		}
		for (NodeCursor nc = clustering.poolNodes(); nc.ok(); nc.next()) {
			graph.getRealizer(nc.node()).setFillColor(Color.DARK_GRAY);
		}
		if(graph.getCurrentView()!=null)
			graph.getCurrentView().updateView();
	}

	@Override
	public String toString() {
		return "Node Colors";
	}

	public Clustering extract(Graph2D graph, boolean useWeights) {
		Clustering result = new Clustering(graph, GraphInterpretation.getInterpretation(graph, useWeights));
		HashMap<Color, Cluster> hm = new HashMap<Color, Cluster>();
		for (NodeCursor nc = graph.nodes(); nc.ok(); nc.next()) {
			Node n = nc.node();
			if (!graph.getHierarchyManager().isNormalNode(n)) continue;
			Color c = graph.getRealizer(n).getFillColor();
			Cluster cl = hm.get(c);
			if (cl == null) {
				cl = result.newCluster(n);
				hm.put(c, cl);
			} else {
				result.add(cl, n);
			}
		}
		result.pool2Singletons();
		return result;
	}

}
