/*
 * Created on Feb 23, 2005
 *
 * Copyright 2004.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
 */
package de.uka.algo.clustering.visualization;

import java.util.HashMap;

import y.base.Node;
import y.base.NodeCursor;
import y.view.Graph2D;
import y.view.ShapeNodeRealizer;
import de.uka.algo.clustering.Cluster;
import de.uka.algo.clustering.Clustering;
import de.uka.algo.graphs.GraphInterpretation;

/**
 * @author Bastian Katz
 * 
 */
public class Shapifier extends EdgeWeightVisualizer implements Graph2DExtractor {

	public Shapifier() {
		super();
	}

	@Override
	public void visualize(Clustering clustering, boolean round) {
		super.visualize(clustering, round);
		Graph2D graph = getGraph2D(clustering);
		int type = 0;
		for (Cluster cl : clustering.getClusterArray()) {
			for (NodeCursor nc = cl.members(); nc.ok(); nc.next()) {
				Node n = nc.node();
				filter(((ShapeNodeRealizer)graph.getRealizer(n)), type);
			}
			type++;
		}		
		graph.getCurrentView().updateView();
	}

	/**
	 * @param shapeNodeRealizer
	 * @param type
	 */
	private void filter(ShapeNodeRealizer snr, int type) {
		switch (type) {
		case 0:
			snr.setShapeType(ShapeNodeRealizer.ELLIPSE);	
			break;
		case 1:
			snr.setShapeType(ShapeNodeRealizer.RECT);	
			break;
		case 2:
			snr.setShapeType(ShapeNodeRealizer.TRIANGLE);	
			break;
		case 3:
			snr.setShapeType(ShapeNodeRealizer.DIAMOND);	
			break;
		case 4:
			snr.setShapeType(ShapeNodeRealizer.HEXAGON);	
			break;
		case 5:
			snr.setShapeType(ShapeNodeRealizer.OCTAGON);	
			break;
		case 6:
			snr.setShapeType(ShapeNodeRealizer.PARALLELOGRAM);	
			break;
		case 7:
			snr.setShapeType(ShapeNodeRealizer.TRAPEZOID);	
			break;
		case 8:
			snr.setShapeType(ShapeNodeRealizer.TRAPEZOID_2);	
			break;
		case 9:
			snr.setShapeType(ShapeNodeRealizer.ROUND_RECT);	
			break;
		default:
			throw new RuntimeException(
					"can't visualize clusterings that many clusters by shapes.");
		}
		
	}

	@Override
	public String toString() {
		return "Node Shape";
	}

	public Clustering extract(Graph2D graph, boolean useWeights) {
		Clustering result = new Clustering(graph, GraphInterpretation
				.getInterpretation(graph, useWeights));

		HashMap<Byte, Cluster> hm = new HashMap<Byte, Cluster>();
		for (NodeCursor nc = graph.nodes(); nc.ok(); nc.next()) {
			Node n = nc.node();
			if (!graph.getHierarchyManager().isNormalNode(n))
				continue;
			Byte c = ((ShapeNodeRealizer) graph.getRealizer(n))
					.getShapeType();
			Cluster cl = hm.get(c);
			if (cl == null) {
				cl = result.newCluster(n);
				hm.put(c, cl);
			} else {
				result.add(cl, n);
			}
		}
		result.pool2Singletons();
		return result;
	}

}
