/**
 * Created on 29.11.2007
 */
package de.uka.algo.graphs.centralities.betweenness.sampling;

import java.util.Random;

import y.base.Graph;
import y.base.Node;
import y.base.NodeList;

/**
 * Chooses nodes by random. The number of nodes chosen is either a fixed number
 * or a fixed fraction of the nodes in the graph.
 * 
 * @author Ulrich Geilmann
 *
 */
public class RandomSampler implements Sampler {
	
	private Random random;
	private int n;
	private double f;
	
	/**
	 * Choose a fixed number of nodes
	 * @param n
	 * 	The number of nodes to choose
	 */
	public RandomSampler (int n) {
		random = new Random();
		this.n = n;
		f = -1;
	}
	
	/**
	 * Choose a fixed fraction of the nodes in the graph
	 * @param f
	 * 	The fraction of nodes to choose
	 */
	public RandomSampler (double f) {
		random = new Random();
		this.f = f;
		n = -1;
	}

	/*
	 * @see de.uka.algo.clustering.betweenness.util.sampling.Sampler#sample(y.base.Graph)
	 */
	public NodeList sample(Graph g) {
		/* Determine how many nodes to choose */
		int z = n;
		if (f != -1) {
			z = (int)(g.nodeCount() * f + 0.5);
			if (z <= 0)
				z = 1;
		}
		/* Choose nodes randomly */
		NodeList list = new NodeList();
		Node[] nodes = g.getNodeArray();
		int m = nodes.length;
		while (z > 0 && m > 0) {
			int c = random.nextInt(m);
			list.add(nodes[c]);
			/* Avoid choosing a node more than one time */
			nodes[c] = nodes[--m];
			z--;
		}
		
		return list;
	}

}
