/*
 * Created on 07.10.2004
 * Reviewed 1.4 (Mar 27, 2005)
 *
 * Copyright 2004.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
 */
package de.uka.algo.graphs.cut;

import y.base.NodeCursor;

/**
 * A ConductanceCut wraps the result of algorithms that find cuts in graphs with
 * an interpretation and calculate a conductance. That is, a ConductanceCut
 * contains a cursor iterating over a set of nodes and (one side of the cut,
 * inner nodes) and a conductance value.
 * 
 * @author Bastian Katz
 */
public class ConductanceCut extends Cut {

	/** the cut's conductance */
	private double conductance;

	/**
	 * Instantiates a new ConductanceCut wrapping a NodeCursor (w/o cloning it)
	 * and a conductance value
	 * 
	 * @param innerNodes the cut's inner nodes
	 * @param conductance the conductance value
	 */
	public ConductanceCut(NodeCursor innerNodes, double conductance) {
		super(innerNodes);
		this.conductance = conductance;
	}

	/**
	 * @return the conductance value.
	 */
	public double getConductance() {
		return conductance;
	}
}
