/*
 * Created on 07.10.2004
 * Reviewed 1.2 (Mar 27, 2005)
 *
 * Copyright 2004.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
 */
package de.uka.algo.graphs.cut;

import y.base.NodeCursor;

/**
 * A Cut wraps the result of algorithms finding a cut in a graph. That is, a Cut
 * contains a cursor iterating over a set of nodes and (one side of the cut,
 * inner nodes) provides additional information where needed.
 * 
 * @author Bastian Katz
 */
public class Cut {

	/** the 'inner nodes' of the cut */
	protected NodeCursor innerNodes;

	/**
	 * Instantiates a new Cut wrapping a NodeCursor w/o cloning it.
	 * 
	 * @param innerNodes
	 *            the cursor iteration over the cut's inner nodes
	 */
	public Cut(NodeCursor innerNodes) {
		this.innerNodes = innerNodes;
	}

	/**
	 * @return the inner nodes of the cut as node cursor. Note that the cursor
	 *         can point to an element different from its head if the cursor has
	 *         been changed concurrently.
	 */
	public NodeCursor getInnerNodes() {
		return innerNodes;
	}
}
