/*
 * Created on May 5, 2004
 * Reviewed 1.9 (Mar 29, 2005)
 *
 * Copyright 2004.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
 */
package de.uka.algo.graphs.linalg;

import java.util.Iterator;

import y.base.Graph;
import de.uka.algo.graphs.GraphInterpretation;
import de.uka.algo.math.linalg.generic.Cell2D;

/**
 * The class AdjacencyMatrix represents the adjacency matrix of a graph, where
 * the element in row v, column w is the adjacency of the node pair (v,w) as
 * defined in the respective interpretation.
 * 
 * @author Bastian Katz
 */
public class AdjacencyMatrix extends GraphMatrix {

	/**
	 * Instantiates a AdjacencyMatrix of the given underlying graph with an
	 * interpretation.
	 * 
	 * @param aGraph
	 *            the graph
	 * @param anInterpretation
	 *            the graph's interpretation
	 */
	public AdjacencyMatrix(Graph aGraph, GraphInterpretation anInterpretation) {
		super(aGraph, anInterpretation);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.uka.algo.math.linalg.MatrixAdapter#get(int, int)
	 */
	@Override
	public double get(int row, int column) {
		if (row >= graph.nodeCount() | column >= graph.nodeCount() | row < 0
				| column < 0)
			throw new IndexOutOfBoundsException();
		return interpretation.getAdjacency(row, column);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.uka.algo.math.linalg.matrix.MatrixAdapter#getFast(int, int)
	 */
	@Override
	public double getFast(int row, int column) {
		return interpretation.getAdjacency(row, column);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.uka.algo.math.linalg.MatrixAdapter#rows()
	 */
	@Override
	public int rows() {
		return graph.nodeCount();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.uka.algo.math.linalg.MatrixAdapter#columns()
	 */
	@Override
	public int columns() {
		return graph.nodeCount();
	}

	/* (non-Javadoc)
	 * @see de.uka.algo.math.linalg.matrix.MatrixAdapter#nonZeros()
	 */
	@Override
	public Iterator<Cell2D> nonZeros() {
		return getAdjacencies();
	}
}