/*
 * Created on May 16, 2004
 * Reviewed 1.2 (Mar 29, 2005)
 *
 * Copyright 2004.  --- All rights reserved. 
 * Algorithmic Group, Karlsruhe, Germany.
 */
package de.uka.algo.graphs.linalg;

import de.uka.algo.graphs.GraphInterpretation;
import de.uka.algo.graphs.SimpleGraph;
import y.base.Graph;

/**
 * A DegreeVector is a vector holding the degree of all nodes of a graph. As a
 * DegreeVector is very slow (to be up to date), it should be copied whenever
 * its get()-method is used excessively.
 * 
 * @author Bastian Katz
 */
public class DegreeVector extends GraphVector {

	/**
	 * Instantiates a DegreeVector of the given underlying graph where the n-th
	 * element is the degree of the n-th node as defined in the respective
	 * interpretation.
	 * 
	 * @param aGraph
	 *            the graph
	 * @param anInterpretation
	 *            the graph's interpretation
	 */
	public DegreeVector(Graph aGraph, GraphInterpretation anInterpretation) {
		super(aGraph, anInterpretation);
	}

	public DegreeVector(Graph graph) {
		this (graph, new SimpleGraph(graph));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.uka.algo.math.linalg.VectorAdapter#size()
	 */
	@Override
	public int size() {
		return graph.nodeCount();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.uka.algo.math.linalg.VectorAdapter#get(int)
	 */
	@Override
	public double get(int index) {
		if (index < 0 | index >= graph.nodeCount())
			throw new IndexOutOfBoundsException();
		return interpretation.getDegree(index);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.uka.algo.math.linalg.vectors.VectorAdapter#getFast(int)
	 */
	@Override
	public double getFast(int index) {
		return interpretation.getDegree(index);
	}
}