/**
 * Created on Okt 31, 2007
 */

package de.uka.algo.graphs.traverse.bfs;

import y.base.Edge;
import y.base.Graph;
import y.base.Node;
import de.uka.algo.graphs.traverse.bfs.BFS.EdgeType;

/**
 * Providing the root, traverse and done methods
 * which are called during the BFS
 * 
 * Implement this interface and use BFS.addModule() to make
 * the BFS call your methods
 * 
 * @author Ulrich Geilmann
 *
 */
public interface BFSModule {
	
	/**
	 * Do all initialization here.
	 * This method is called once the module is added to the BFS-Object
	 * and whenever <code>BFS.initAll()</code> is called, which should be
	 * executed every time when the results of previous searches have to
	 * be reset.
	 * @param g
	 * 	The graph the search will be performed in 
	 */
	public abstract void init ( Graph g );
	
	/**
	 * This method is called once at the beginning of the search
	 * @param s
	 * 	The root-node
	 */
	public abstract void root ( Node s );
	
	/**
	 * This method is called for every Edge once the search runs over it
	 * @param v
	 * 	The source node
	 * @param v_bfs
	 * 	The bfs-layer of the source node
	 * @param e
	 * 	The edge itself
	 * @param marking
	 * 	The marking of the Edge
	 * @param w
	 * 	The target node
	 * @param w_bfs
	 * 	The bfs-layer of the target node
	 */
	public abstract void traverse ( Node v, int v_bfs, Edge e, EdgeType marking, Node w, int w_bfs );
	
	/**
	 * This method is called once after the search has been finished
	 * @param s
	 * 	The root-node
	 */
	public abstract void done ( Node s );
	
	/**
	 * Cleaning up - this method is called by <code>BFS.cleanUp()</code>
	 * Do all cleaning up here.
	 */
	public abstract void cleanUp();
	
}
