package yed;

import de.uka.algo.clustering.comparison.ComparatorRegistration;
import de.uka.algo.clustering.comparison.ComparatorRegistration.ComparatorTypes;
import de.uka.algo.clustering.comparison.graphbased.EditingSetQuotient;
import de.uka.algo.clustering.comparison.graphbased.counting.AdjustedRandGraphExt;
import de.uka.algo.clustering.comparison.graphbased.counting.FowlkesMallowsGraphExt;
import de.uka.algo.clustering.comparison.graphbased.counting.JaccardGraphExt;
import de.uka.algo.clustering.comparison.graphbased.counting.RandGraphExt;
import de.uka.algo.clustering.comparison.graphbased.informationTheory.FredJainGraphExt;
import de.uka.algo.clustering.comparison.graphbased.informationTheory.StrehlGoshGraphExt;
import de.uka.algo.clustering.comparison.graphbased.informationTheory.VariationOfInformationGraphExt;
import de.uka.algo.clustering.comparison.graphbased.overlaps.FMeasureGraphExt;
import de.uka.algo.clustering.comparison.graphbased.overlaps.MaximumMatchGraphExt;
import de.uka.algo.clustering.comparison.graphbased.overlaps.MeilaHeckermanGraphExt;
import de.uka.algo.clustering.comparison.graphbased.overlaps.vanDongenGraphExt;
import de.uka.algo.clustering.comparison.nodebased.counting.AdjustedRand;
import de.uka.algo.clustering.comparison.nodebased.counting.FowlkesMallows;
import de.uka.algo.clustering.comparison.nodebased.counting.Jaccard;
import de.uka.algo.clustering.comparison.nodebased.counting.Rand;
import de.uka.algo.clustering.comparison.nodebased.informationTheory.FredJain;
import de.uka.algo.clustering.comparison.nodebased.informationTheory.StrehlGosh;
import de.uka.algo.clustering.comparison.nodebased.informationTheory.VariationOfInformation;
import de.uka.algo.clustering.comparison.nodebased.overlaps.FMeasure;
import de.uka.algo.clustering.comparison.nodebased.overlaps.MaximumMatch;
import de.uka.algo.clustering.comparison.nodebased.overlaps.MeilaHeckerman;
import de.uka.algo.clustering.comparison.nodebased.overlaps.vanDongen;
import de.uka.algo.clustering.comparison.qualitybased.QualityDistance;
import de.uka.algo.clustering.index.Coverage;
import de.uka.algo.clustering.index.IndexRegistration;
import de.uka.algo.clustering.index.InterClusterConductance;
import de.uka.algo.clustering.index.IntraClusterConductance;
import de.uka.algo.clustering.index.PerformanceConventional;
import de.uka.algo.clustering.index.Significance;
import de.uka.algo.clustering.index.OperationIndex.Operations;
import de.uka.algo.clustering.index.Significance.Indizes;
import de.uka.algo.clustering.layout.Circle;
import de.uka.algo.clustering.layout.NoLayout;
import de.uka.algo.clustering.layout.SpringEmbedder;
import de.uka.algo.clustering.layouts.Layouter;
import de.uka.algo.clustering.visualization.BorderColorizer;
import de.uka.algo.clustering.visualization.Graph2DFacade;
import de.uka.algo.clustering.visualization.Grouping;
import de.uka.algo.clustering.visualization.LabelExtractor;
import de.uka.algo.clustering.visualization.Labeller;
import de.uka.algo.clustering.visualization.LargeClusterExtractor;
import de.uka.algo.clustering.visualization.NoVisualization;
import de.uka.algo.clustering.visualization.NodeColorizer;
import de.uka.algo.clustering.visualization.Shapifier;
import de.uka.algo.clustering.visualization.SingletonExtractor;
import de.uka.algo.graphs.cut.ConductanceCutter;

public class ExtensionLauncher {
	public static void main(String[] args) {
		Layouter.addLayouter(new NoLayout());
		Layouter.addLayouter(new Circle());
		Layouter.addLayouter(new SpringEmbedder());
		Graph2DFacade.addVisualization(new NoVisualization());
		Graph2DFacade.addVisualization(new NodeColorizer());
		Graph2DFacade.addVisualization(new Grouping());
		Graph2DFacade.addVisualization(new BorderColorizer());
		Graph2DFacade.addVisualization(new Shapifier());
		Graph2DFacade.addVisualization(new Labeller(Labeller.LabelType.letters));
		Graph2DFacade.addVisualization(new Labeller(Labeller.LabelType.numbers));
		Graph2DFacade.addVisualization(new Labeller(Labeller.LabelType.roman));
		Graph2DFacade.addVisualization(new LabelExtractor());
		Graph2DFacade.addVisualization(new SingletonExtractor());
		Graph2DFacade.addVisualization(new LargeClusterExtractor());
		
		
		IndexRegistration.addIndex(new Coverage.Factory());
		IndexRegistration.addIndex(new PerformanceConventional.Factory());
		IndexRegistration.addIndex(new InterClusterConductance.Factory());
		IndexRegistration.addIndex(new IntraClusterConductance.Factory(ConductanceCutter.ConductanceStrategies.Markovian));
		IndexRegistration.addIndex(new IntraClusterConductance.Factory(ConductanceCutter.ConductanceStrategies.Laplacian));
		IndexRegistration.addIndex(new Significance.Factory(Indizes.coverage, Operations.sub));
		IndexRegistration.addIndex(new Significance.Factory(Indizes.coverage, Operations.div));
		IndexRegistration.addIndex(new Significance.Factory(Indizes.performance, Operations.sub));
		IndexRegistration.addIndex(new Significance.Factory(Indizes.performance, Operations.div));
		
		ComparatorRegistration.addComparator(new QualityDistance.FactoryModularity(), ComparatorTypes.qualityBased);
		ComparatorRegistration.addComparator(new AdjustedRand.Factory(), ComparatorTypes.nodeBasedCounting);
		ComparatorRegistration.addComparator(new FowlkesMallows.Factory(), ComparatorTypes.nodeBasedCounting);
		ComparatorRegistration.addComparator(new Jaccard.Factory(), ComparatorTypes.nodeBasedCounting);
		ComparatorRegistration.addComparator(new Rand.Factory(), ComparatorTypes.nodeBasedCounting);
		ComparatorRegistration.addComparator(new FMeasure.Factory(), ComparatorTypes.nodeBasedOverlap);
		ComparatorRegistration.addComparator(new FMeasure.RevFactory(), ComparatorTypes.nodeBasedOverlap);
		ComparatorRegistration.addComparator(new MaximumMatch.Factory(), ComparatorTypes.nodeBasedOverlap);
		ComparatorRegistration.addComparator(new MeilaHeckerman.Factory(), ComparatorTypes.nodeBasedOverlap);
		ComparatorRegistration.addComparator(new MeilaHeckerman.RevFactory(), ComparatorTypes.nodeBasedOverlap);
		ComparatorRegistration.addComparator(new vanDongen.Factory(), ComparatorTypes.nodeBasedOverlap);
		ComparatorRegistration.addComparator(new FredJain.Factory(), ComparatorTypes.nodeBasedInformationTheoretic);
		ComparatorRegistration.addComparator(new StrehlGosh.Factory(), ComparatorTypes.nodeBasedInformationTheoretic);
		ComparatorRegistration.addComparator(new VariationOfInformation.Factory(), ComparatorTypes.nodeBasedInformationTheoretic);
		ComparatorRegistration.addComparator(new AdjustedRandGraphExt.Factory(), ComparatorTypes.graphBasedCounting);
		ComparatorRegistration.addComparator(new FowlkesMallowsGraphExt.Factory(), ComparatorTypes.graphBasedCounting);
		ComparatorRegistration.addComparator(new JaccardGraphExt.Factory(), ComparatorTypes.graphBasedCounting);
		ComparatorRegistration.addComparator(new RandGraphExt.Factory(), ComparatorTypes.graphBasedCounting);
		ComparatorRegistration.addComparator(new FMeasureGraphExt.Factory(), ComparatorTypes.graphBasedOverlap);
		ComparatorRegistration.addComparator(new FMeasureGraphExt.RevFactory(), ComparatorTypes.graphBasedOverlap);	
		ComparatorRegistration.addComparator(new MaximumMatchGraphExt.Factory(), ComparatorTypes.graphBasedOverlap);
		ComparatorRegistration.addComparator(new MeilaHeckermanGraphExt.Factory(), ComparatorTypes.graphBasedOverlap);
		ComparatorRegistration.addComparator(new MeilaHeckermanGraphExt.RevFactory(), ComparatorTypes.graphBasedOverlap);
		ComparatorRegistration.addComparator(new vanDongenGraphExt.Factory(), ComparatorTypes.graphBasedOverlap);
		ComparatorRegistration.addComparator(new FredJainGraphExt.Factory(), ComparatorTypes.graphBasedInformationTheoretic);
		ComparatorRegistration.addComparator(new StrehlGoshGraphExt.Factory(), ComparatorTypes.graphBasedInformationTheoretic);
		ComparatorRegistration.addComparator(new VariationOfInformationGraphExt.Factory(), ComparatorTypes.graphBasedInformationTheoretic);
		ComparatorRegistration.addComparator(new EditingSetQuotient.Factory(), ComparatorTypes.userDefined);
		ApplicationLauncher.main(args);
	}
}
